/**
* Drag Object : make dom object draggable by www.youngpup.net
*/
jsgtk.domDrag = {

	// The current element being dragged
	obj : null,
	
	/**
	* init() method 				: The initalization function for the drag object
	* @param {object} o 			: The element to act as the drag handle
	* @param {object} oRoot 		: The element to be dragged, if not specified, the handle will be the element dragged.
	* @param {integer} minX 		: min x of coordinate allowed for the element
	* @param {integer} maxX 		: max x of coordinate allowed for the element
	* @param {integer} minY 		: min y of coordinate allowed for the element
	* @param {integer} maxY 		: max y of coordinate allowed for the element
	* @param {boolean} bSwapHorzRef : Toggle the horizontal coordinate system
	* @param {boolean} bSwapVertRef : Toggle the vertical coordinate system
	* @param {function} fxMapper	: Functions for mapping x and y coordinates to others
	* @param {function} fyMapper 	: Functions for mapping x and y coordinates to others
	*/
	init : function(o, oRoot, minX, maxX, minY, maxY, hDisable, vDisable, bSwapHorzRef, bSwapVertRef, fXMapper, fYMapper){

		// Watch for the drag event to start
		o.onmousedown = jsgtk.domDrag.start;

		// Figure out which coordinate system is being used
		o.hmode	= bSwapHorzRef ? false : true ;
		o.vmode	= bSwapVertRef ? false : true ;

		// Figure out which element is acting as the draggable 'handle'
		o.root = oRoot && oRoot != null ? oRoot : o ;

		 // Initalize the specified coordinate system
		if (o.hmode  && isNaN(parseInt(o.root.style.left  ))) o.root.style.left   = "0px";
		if (o.vmode  && isNaN(parseInt(o.root.style.top   ))) o.root.style.top    = "0px";
		if (!o.hmode && isNaN(parseInt(o.root.style.right ))) o.root.style.right  = "0px";
		if (!o.vmode && isNaN(parseInt(o.root.style.bottom))) o.root.style.bottom = "0px";		

		// Look to see if the user provided  min/max x/y coordinates
		o.minX	= typeof minX != 'undefined' ? minX : null;
		o.minY	= typeof minY != 'undefined' ? minY : null;
		o.maxX	= typeof maxX != 'undefined' ? maxX : null;
		o.maxY	= typeof maxY != 'undefined' ? maxY : null;

		o.minX	= hDisable ? 0 : o.minX;
		o.maxX	= hDisable ? 0 : o.maxX;
		o.minY	= vDisable ? 0 : o.minY;
		o.maxY	= vDisable ? 0 : o.maxY;

		// Check for any specified x and y coordinate mappers
		o.xMapper = fXMapper ? fXMapper : null;
		o.yMapper = fYMapper ? fYMapper : null;

		// Add shells for all the user-defined functions
		o.root.onDragStart	= new Function();
		o.root.onDragEnd	= new Function();
		o.root.onDrag		= new Function();
	},

	start : function(e){

		// Figure out the object that's being dragged
		var o = jsgtk.domDrag.obj = this;

		// Normalize the event object
		e = jsgtk.domDrag.fixE(e);

		// Get the current x and y coordinates
		var y = parseInt(o.vmode ? o.root.style.top  : o.root.style.bottom);
		var x = parseInt(o.hmode ? o.root.style.left : o.root.style.right );

		// Call the user's function with the current x and y coordinates
		o.root.onDragStart(x, y, e);

		// Remember the starting mouse position
		o.lastMouseX	= e.clientX;
		o.lastMouseY	= e.clientY;

		// If we're using the CSS coordinate system
		if (o.hmode) {

			// set the min and max coordiantes, where applicable
			if (o.minX != null)	o.minMouseX	= e.clientX - x + o.minX;
			if (o.maxX != null)	o.maxMouseX	= o.minMouseX + o.maxX - o.minX;

		// Otherwise, we're using a traditional mathematical coordinate system
		} else {
			if (o.minX != null) o.maxMouseX = -o.minX + e.clientX + x;
			if (o.maxX != null) o.minMouseX = -o.maxX + e.clientX + x;
		}

		// If we're using the CSS coordinate system
		if (o.vmode) {
	
			 // set the min and max coordiantes, where applicable
			if (o.minY != null)	o.minMouseY	= e.clientY - y + o.minY;
			if (o.maxY != null)	o.maxMouseY	= o.minMouseY + o.maxY - o.minY;

		// Otherwise, we're using a traditional mathematical coordinate system
		} else {
			if (o.minY != null) o.maxMouseY = -o.minY + e.clientY + y;
			if (o.maxY != null) o.minMouseY = -o.maxY + e.clientY + y;
		}

		// Watch for 'dragging' and 'drag end' events
		document.onmousemove	= jsgtk.domDrag.drag;
		document.onmouseup		= jsgtk.domDrag.end;

		return false;
	},

	// A function to watch for all movements of the mouse during the drag event
	drag : function(e){

		// Normalize the event object
		e = jsgtk.domDrag.fixE(e);

		// Get our reference to the element being dragged
		var o = jsgtk.domDrag.obj;

		// Get the position of the mouse within the window
		var ey	= e.clientY;
		var ex	= e.clientX;

		// Get the current x and y coordinates
		var y = parseInt(o.vmode ? o.root.style.top  : o.root.style.bottom);
		var x = parseInt(o.hmode ? o.root.style.left : o.root.style.right );
		var nx, ny;

		// If a minimum X position was set, make sure it doesn't go past that
		if (o.minX != null) ex = o.hmode ? Math.max(ex, o.minMouseX) : Math.min(ex, o.maxMouseX);

		// If a maximum X position was set, make sure it doesn't go past that
		if (o.maxX != null) ex = o.hmode ? Math.min(ex, o.maxMouseX) : Math.max(ex, o.minMouseX);

		// If a minimum Y position was set, make sure it doesn't go past that
		if (o.minY != null) ey = o.vmode ? Math.max(ey, o.minMouseY) : Math.min(ey, o.maxMouseY);

		// If a maximum Y position was set, make sure it doesn't go past that
		if (o.maxY != null) ey = o.vmode ? Math.min(ey, o.maxMouseY) : Math.max(ey, o.minMouseY);

		// Figure out the newly translated x and y coordinates
		nx = x + ((ex - o.lastMouseX) * (o.hmode ? 1 : -1));
		ny = y + ((ey - o.lastMouseY) * (o.vmode ? 1 : -1));

		// and translate them using an x or y mapper function (if provided)
		if (o.xMapper) nx = o.xMapper(y);
		else if (o.yMapper)	ny = o.yMapper(x);

		// Set the new x and y coordinates onto the element
		jsgtk.domDrag.obj.root.style[o.hmode ? "left" : "right"] = nx + "px";
		jsgtk.domDrag.obj.root.style[o.vmode ? "top" : "bottom"] = ny + "px";

		// and remember the last position of the mouse
		jsgtk.domDrag.obj.lastMouseX	= ex;
		jsgtk.domDrag.obj.lastMouseY	= ey;

		// Call the user's onDrag function with the current x and y coordinates
		jsgtk.domDrag.obj.root.onDrag(nx, ny, e);
		return false;
	},

	// Function that handles the end of a drag event
	end : function(){

		// No longer watch for mouse events (as the drag is done)
		document.onmousemove = null;
		document.onmouseup   = null;

		// Call our special onDragEnd function with the x and y coordinates
		// of the element at the end of the drag event
		jsgtk.domDrag.obj.root.onDragEnd( 
			parseInt(jsgtk.domDrag.obj.root.style[jsgtk.domDrag.obj.hmode ? "left" : "right"]), 
			parseInt(jsgtk.domDrag.obj.root.style[jsgtk.domDrag.obj.vmode ? "top" : "bottom"])
		);

		// No longer watch the object for drags
		jsgtk.domDrag.obj = null;
	},

	// A function for normalizes the event object
	fixE : function(e){

		// If no event object exists, then this is IE, so provide IE's event object
		if (typeof e == 'undefined') e = window.event;

		// If the layer properties aren't set, get the values from the equivalent
		// offset properties
		if (typeof e.layerX == 'undefined') e.layerX = e.offsetX;
		if (typeof e.layerY == 'undefined') e.layerY = e.offsetY;
		return e;
	}
};
