jsgtk.Event = {
	
	/**
	 * Add a new event listener to a dom object The event will be fired when the browser event occurs.
	 * @param {object} domObj
	 * @param {String} type
	 * @param {Function} callback
	 * @return {object} - handle used to remove listeners
	 */
	addDomListener: function(domObj, type, callback){
		if(domObj.addEventListener){
			domObj.addEventListener(type, callback, false);
		} else {
			domObj.attachEvent("on" + type, callback);
		}
		var handler = {
			obj: domObj,
			type: type,
			callback: callback
		};
		
		return handler;
	},
	
	/**
	 * Add a new event listener to a standard object this event can be fired using the trigger() method.
	 * @param {object} obj - Scope the callback will be called in
	 * @param {String} type
	 * @param {Function} callback
	 * @return {object} - handle used to remove listeners
	 */
	addListener: function(obj, type, callback){
		if(!obj.listeners) obj.listeners = {};
		if(!obj.listeners[type]) obj.listeners[type] = [];
		obj.listeners[type].push(callback);
		return {
			obj: obj,
			type: type,
			callback: callback,
			isDomObj: false
		};
	},

	/**
	 * Remove an event listener which was added using addListener or addDomListener.
	 * @param {object} handler
	 */
	removeListener: function(handler){		
		if(handler.obj.removeEventListener){
			handler.obj.removeEventListener(handler.type, handler.callback, false);
		} else {
			handler.obj.detachEvent("on" + handler.type, handler.callback);
		}
	},

	/**
	 * Re-register event handler which was removed
	 * @param {object} handler
	 */
	registerHandler: function(handler){
		if(handler.obj.addEventListener){
			handler.obj.addEventListener(handler.type, handler.callback, false);
		} else {
			handler.obj.attachEvent("on" + handler.type, handler.callback);
		}			
		return handler;
	},
	
		/**
	 * Trigger an oject event which has been added using addListener().
	 * @param {object} obj
	 * @param {String} type
	 * @param {Anything} arguments - the arguments can be overloaded to pass additional items
	 */
	trigger: function(obj, type){ 
		if(obj.listeners && obj.listeners[type]){
			var _args = [];
			for(var i=2; i< arguments.length; i++){
				_args[i - 2]= arguments[i];
			}					
			for(var i = 0; i < obj.listeners[type].length; i++){
				obj.listeners[type][i].apply(obj,_args);
			}				
		}				
	},
	
	/**
	* Cancell bubbling phase of event
	* @param {object} e - event object
	*/
	stopBubble: function(e){
        if ( e && e.stopPropagation ) e.stopPropagation();
        else window.event.cancelBubble = true;
	},
	
	stopDefault: function(e){
		if ( e && e.preventDefault )
		     e.preventDefault();
		else
		     window.event.returnValue = false;
    	return false;
	}
};
