var MylabsSvgVml = function(posX, posY, width, height){
    var self = this;
    this._width = width;
    this._height = height;
    this._posX = posX;
    this._posY = posY;

    this._node = MylabsSvgVml.type == "SVG" ? this._renderSVG(posX, posY, width, height) : this._renderVML(posX, posY, width, height);
};

MylabsSvgVml.prototype = {
    _renderSVG: function(posX, posY, width, height){
        var node = document.createElementNS("http://www.w3.org/2000/svg", "svg:svg");

        node.setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns:xlink", "http://www.w3.org/1999/xlink");
        this._setAttributes(node, {
            "width" : width + "px",
            "height" : height + "px",
            "viewBox": posX + " " + posY + " " + width + " " + height
        });

//        var defs = this._createDefs();        
//        node.appendChild(defs);
        
        return node;
    },
    
    _renderVML: function(posX, posY, width, height){
        document.namespaces.add("rvml", "urn:schemas-microsoft-com:vml"); 
        document.createStyleSheet().addRule(".rvml", "behavior:url(#default#VML)");
        
        var group = document.createElement("rvml:group");
        group.className = "rvml";
        var groupStyle = group.style;
        groupStyle.position = "absolute";
        groupStyle.width = width + "px";
        groupStyle.height = height + "px";
        groupStyle.zIndex = "1000";

        group.setAttribute("coordsize" , width + "," + height);
        
        return group;
    },
    
    _setAttributes: function(dom, args){
        for(property in args){
            dom.setAttribute(property, args[property]);
        }
    },
    
    getNode: function(){
        return this._node;
    },
    
    path: function(pathString){
        if(MylabsSvgVml.type == "SVG"){
            var path = document.createElementNS("http://www.w3.org/2000/svg", "path");
            this._node.appendChild(path);
            path.setAttribute("d", pathString);    

            return {
                attr: function(prop, value){
                    path.setAttribute(prop, value);
                }
            };
        }
        else{
            var shape = document.createElement("rvml:shape");
            shape.className = "rvml";
            this._setAttributes(shape, {
                "path": pathString,
                "filled" : "t",
                "stroked" : "t",
                "coordsize": this._width + "," + this._height
            });
            
            shape.style.width = this._width + "px";
            shape.style.height = this._height + "px";
            
            var fill = document.createElement("rvml:fill");
            fill.className = "rvml";
            fill.setAttribute("type" , "solid");
            
            var stroke = document.createElement('rvml:stroke');
            stroke.className = "rvml";
            stroke.setAttribute("opacity" , "1");
            
            shape.appendChild(fill);
            shape.appendChild(stroke);
            
            this._node.appendChild(shape);
                   
            return {
                attr: function(prop, value){
                    if(prop == "stroke")
                        prop = "strokecolor";
                    if(prop == "fill")
                        prop = "fillcolor";

                    shape.setAttribute(prop, value);
                }
            };
        }
    },

    rect: function(round){
        if(MylabsSvgVml.type == "SVG"){
            var rectangle = document.createElementNS("http://www.w3.org/2000/svg", "rect");
            this._node.appendChild(rectangle);
            rectangle.setAttribute("x", this._posX);
            rectangle.setAttribute("y", this._posY);
            rectangle.setAttribute("width", this._width);
            rectangle.setAttribute("height", this._height);
            rectangle.setAttribute("rx", round);
            rectangle.setAttribute("ry", round);           
            
            return {
                attr: function(prop, value){
                    rectangle.setAttribute(prop, value);
                }
            };
        }
        else{
            var rectangle = document.createElement("rvml:roundrect");
            rectangle.className = "rvml";
            var rectStyle = rectangle.style;
            rectStyle.width = this._width + "px";
            rectStyle.height = this._height + "px";
            
            rectangle.setAttribute("arcsize", round/Math.min(this._width, this._height));
            
            var fill = document.createElement("rvml:fill");
            fill.className = "rvml";
            fill.setAttribute("type" , "solid");
            
            var stroke = document.createElement('rvml:stroke');
            stroke.className = "rvml";
            stroke.setAttribute("opacity" , "1");
            
            rectangle.appendChild(fill);
            rectangle.appendChild(stroke);
            
            this._node.appendChild(rectangle);
            
            return{
                attr: function(prop, value){
                    if(prop == "stroke")
                        prop = "strokecolor";
                    if(prop == "fill")
                        prop = "fillcolor";

                    rectangle.setAttribute(prop, value);
                }
            };
        }
    },
    
    remove: function(){
        this._node.parentNode.removeChild(this._node);
    }
};
    
MylabsSvgVml.type = (window.SVGAngle || document.implementation.hasFeature("http://www.w3.org/TR/SVG11/feature#BasicStructure", "1.1") ? "SVG" : "VML");
