describe("gtk.Accordion", {
	"before_each": function(){
		g = {};
		g.accordion = new gtk.Accordion();
		g.children = g.accordion._children;
		
		g.buttonMock = mock(gtk.Button);
		when(g.buttonMock.getNode()).thenReturn(document.createElement("div"));
		when(g.buttonMock.setParent(g.accordion));
		anyInstance("gtk.Button", g.buttonMock);
		
		g.buttonLabel = mock(gtk.Label);
		when(g.buttonLabel.getNode()).thenReturn(document.createElement("div"));
		when(g.buttonLabel.setParent(g.accordion));
		anyInstance("gtk.Label", g.buttonLabel);
	},
	"after_each": function(){
		g = {};
		verify();
	},
	"appendChild(), should add a new accordion": function(){  
	    // Test when two children added
	    g.accordion.appendChild("Accordion 1", g.buttonMock);
	    g.accordion.appendChild("Accordion 2", g.buttonLabel);
	    value_of(g.children.length).should_be(2);
	    value_of(g.accordion._activeIndex).should_be_null();
	    value_of(g.accordion.getMinHeightComposite()).should_be(44);
	    value_of(g.accordion.getMinWidthComposite()).should_be(200);
	},
	"activate(), should activate its content and update its _activeIndex": function(){
	    g.accordion.appendChild("Accordion 1", g.buttonMock);
	    g.accordion.appendChild("Accordion 2", g.buttonLabel);
	    g.accordion.activate(1);
	    value_of(g.accordion._activeIndex).should_be(1);
	    value_of(g.accordion._children[1]["childNodeHolder"]).should_not_be_null();
	    value_of(g.accordion._children[1]["childNodeHolder"].style.display).should_be("block");
	    value_of(g.accordion._children[0]["childNodeHolder"].style.display).should_be("none");
	},
	"activate(), should not activate any tab if index out of range": function(){
	    g.accordion.appendChild("Accordion 1", g.buttonMock);
	    g.accordion.appendChild("Accordion 2", g.buttonLabel);
	    g.accordion.activate(20);
	    value_of(g.accordion._activeIndex).should_be_null();
	    value_of(g.accordion._children[0]["childNodeHolder"]).should_not_be_null();
	    value_of(g.accordion._children[0]["childNodeHolder"].style.display).should_be("none");
	    value_of(g.accordion._children[1]["childNodeHolder"].style.display).should_be("none");
	},
	"deActivate(), should deactivate its content and update its _activeIndex": function(){
	    g.accordion.appendChild("Accordion 1", g.buttonMock);
	    g.accordion.appendChild("Accordion 2", g.buttonLabel);
	    g.accordion.activate(1);
	    g.accordion.deActivate(1);
	    value_of(g.accordion._activeIndex).should_be_null();
	    value_of(g.accordion._children[1]["childNodeHolder"]).should_not_be_null();
	    value_of(g.accordion._children[0]["childNodeHolder"].style.display).should_be("none");
	    value_of(g.accordion._children[1]["childNodeHolder"].style.display).should_be("none");
	},
	"'activated' signal": function(){
		g.isEmitted = false;
		g.accordion.appendChild("Accordion 1", g.buttonMock);
		g.accordion.activate(1);
		g.accordion.connect("activated", function(){
			g.isEmitted = true;
		});

		var clickEvent = document.createEvent ("Event");
		clickEvent.initEvent ("click", true, false);
		g.accordion._children[0]["headerNodeHolder"].dispatchEvent (clickEvent);
		
		value_of(g.isEmitted).should_be_true();
	},
	"'deactivated' signal": function(){
		g.isEmitted = false;
		g.accordion.appendChild("Accordion 1", g.buttonMock);
		
		g.accordion.connect("deactivated", function(){
			g.isEmitted = true;
		});

		var clickEvent = document.createEvent ("Event");
		clickEvent.initEvent ("click", true, false);
		g.accordion._children[0]["headerNodeHolder"].dispatchEvent (clickEvent);
		
		value_of(g.isEmitted).should_be_true();
	},
	"getMinHeightComposite()": function(){
		g.accordion.resetMinHeight(300);
		value_of(g.accordion.getMinHeightComposite()).should_be(300);
	},
	"getMinHeightComposite()": function(){
		g.accordion.resetMinHeight();
		value_of(g.accordion.getMinHeightComposite()).should_be_undefined();
	},
	"getMinWidthComposite()": function(){
		g.accordion.resetMinWidth();
		value_of(g.accordion.getMinWidthComposite()).should_be_undefined();
	},
	"getMinWidthComposite()": function(){
		g.accordion.resetMinWidth(500);
		value_of(g.accordion.getMinWidthComposite()).should_be(500);
	},
	"resizeWidthComposite()": function(){
		g.accordion.resizeWidthComposite();
		value_of(g.accordion._width).should_be(200);
	},
	"resizeWidthComposite()": function(){
		g.accordion.resizeWidthComposite(150);
		value_of(g.accordion._width).should_be(200);
	},
	"resizeWidthComposite()": function(){
		g.accordion.resizeWidthComposite(300);
		value_of(g.accordion._width).should_be(300);
	},
	"resizeHeightComposite()": function(){
		g.accordion.resizeHeightComposite();
		value_of(g.accordion._height).should_be(30);
	},
	"resizeHeightComposite()": function(){
		g.accordion.resizeHeightComposite(10);
		value_of(g.accordion._height).should_be(30);
	},
	"resizeHeightComposite()": function(){
		g.accordion.resizeHeightComposite(50);
		value_of(g.accordion._height).should_be(50);
	}
});
