describe("gtk.Button", {
	"before_each": function(){
		g = {};
		g.button = new gtk.Button();
		
		g.imageMock = mock(gtk.Image);
		when( g.imageMock.getMinWidthComposite()).thenReturn(100);
		when(g.imageMock.getNode()).thenReturn(document.createElement("div"));
		when(g.imageMock.getMinHeightComposite()).thenReturn(1);
		when(g.imageMock.getChildNode()).thenReturn(document.createElement("div"));
		anyInstance("gtk.Image", g.imageMock);
		
		g.labelMock = mock(gtk.Label);
		when( g.labelMock.getText()).thenReturn("gtk.Label");
		when( g.labelMock.getNode()).thenReturn(document.createElement("div"));
		when( g.labelMock.getMinHeightComposite()).thenReturn(16);
		when( g.labelMock.getMinWidthComposite()).thenReturn(16);
		anyInstance("gtk.Label", g.labelMock);
	},
	"after_each": function(){
		g = {};
		verify();
	},
	"'click' signal": function(){
		g.isEmitted = false;
	
		g.button.connect("clicked", function(){
			g.isEmitted = true;
		});
	
		var clickEvent = document.createEvent ("Event");
		clickEvent.initEvent ("click", true, false);
		g.button.getNode().dispatchEvent (clickEvent);
		
		value_of(g.isEmitted).should_be_true();
	},
	
	"'enterkeypressed' signal": function(){
		g.isEmitted = false;
	
		g.button.connect("enterkeypressed", function(){
			g.isEmitted = true;
		});
	
		var clickEvent = document.createEvent ("Event");
		clickEvent.initEvent ("keyup", true, false);
		g.button.getNode().dispatchEvent (clickEvent);
		
		value_of(g.isEmitted).should_be_true();
	},
	
	"'mouseover' signal": function(){
		g.isEmitted = false;
	
		g.button.connect("mouseover", function(){
			g.isEmitted = true;
		});
	
		var clickEvent = document.createEvent ("Event");
		clickEvent.initEvent ("mouseover", true, false);
		g.button.getNode().dispatchEvent (clickEvent);
		
		value_of(g.isEmitted).should_be_true();
	},
	"'mouseout' signal": function(){
		g.isEmitted = false;
	
		g.button.connect("mouseout", function(){
			g.isEmitted = true;
		});
	
		var clickEvent = document.createEvent ("Event");
		clickEvent.initEvent ("mouseout", true, false);
		g.button.getNode().dispatchEvent (clickEvent);
		
		value_of(g.isEmitted).should_be_true();
	},
	"setImage()": function(){
		g.button.setImage(g.imageMock);
		value_of(g.button.getImage()).should_be(g.imageMock);
		value_of(g.button.getMinWidthComposite()).should_be(115);
		value_of(g.button.getMinHeightComposite()).should_be(24);
		
		value_of(g.button._imageDom.firstChild).should_be(g.imageMock.getNode());
	},
	"setTextLabel()": function(){
		g.button.setTextLabel("this is gtk.Label");
		value_of(g.button.getLabel()).should_be(g.labelMock);
		value_of(g.button.getMinWidthComposite()).should_be(31);
		value_of(g.button.getMinHeightComposite()).should_be(24);
	},
	"setAsImage()": function(){
		g.button.setAsImage(g.imageMock);
		value_of(g.button._domObj.className).should_match("gtk_button_with_image_shape");
		value_of(g.button.getMinWidthComposite()).should_be(100);
		value_of(g.button.getMinHeightComposite()).should_be(1);
	},
	"showLoading()": function(){
		g.isClicked = false;
		
		g.button.showLoading();
		value_of(g.button._loading).should_not_be_null();
	
		g.button.connect("clicked", function(){
			g.isClicked = true;
		});
		
		var clickEvent = document.createEvent ("Event");
		clickEvent.initEvent ("click", true, false);
		g.button.getNode().dispatchEvent (clickEvent);
		
		value_of(g.isClicked).should_be_false();
	},
	"hideLoading()": function(){
		g.isClicked = false;
		
		g.button.showLoading();
		g.button.hideLoading();
		value_of(g.button._loading).should_be_null();
	
		g.button.connect("clicked", function(){
			g.isClicked = true;
		});
		
		var clickEvent = document.createEvent ("Event");
		clickEvent.initEvent ("click", true, false);
		g.button.getNode().dispatchEvent (clickEvent);
		
		value_of(g.isClicked).should_be_true();
	},
	"setTabindex()": function(){
		g.button.setTabindex(12);
		value_of(g.button.getNode().tabIndex).should_be(12);
	},
	"disable()": function(){
		g.button.disable();
		value_of(g.button._isEnable).should_be_false;
		value_of(g.button.getNode().className).should_match("gtk_button_disabled");
		value_of(g.button._gradient.className).should_match("gradisable");
	},
	"enable()": function(){
		g.button.enable();
		value_of(g.button._isEnable).should_be_false;
		value_of(g.button.getNode().className).should_match("gtk_button");
		value_of(g.button._gradient.className).should_match("gtk_button_gradient");
	},
	"resizeWidthComposite()": function(){
		g.button.resizeWidthComposite(100);
		value_of(g.button.getWidth()).should_be(100);
	},
	"resizeHeightComposite()": function(){
		g.button.resizeHeightComposite(100);
		value_of(g.button.getHeight()).should_be(100);
	}
});
