describe('gtk_image', {
	'before_each': function(){
		g = {};			
		g.image = new gtk.Image();
	},
	'after_each': function(){
		g = {};
	},
	"setFromFile()": function(){
		g.image.setFromFile(null, 25, 27);
		g.src = g.image._dom.src;
		value_of(g.src).should_match(g.image._defaultImage);
		
		g.image.setFromFile("foo.png", 25, 27);
		g.src = g.image._dom.src;
		value_of(g.src).should_match("foo.png");
		value_of(g.image.getMinWidthComposite()).should_be(25);
		value_of(g.image.getMinHeightComposite()).should_be(27);
	},
	"'click' signal": function(){
		g.isEmitted = false;
	
		g.image.connect("clicked", function(){
			g.isEmitted = true;
		});
	
		var clickEvent = document.createEvent ("Event");
		clickEvent.initEvent ("click", true, false);
		g.image._dom.dispatchEvent (clickEvent);
		
		value_of(g.isEmitted).should_be_true();
	},
	"'mouseout' signal": function(){
		g.isEmitted = false;
	
		g.image.connect("mouseout", function(){
			g.isEmitted = true;
		});
	
		var clickEvent = document.createEvent ("Event");
		clickEvent.initEvent ("mouseout", true, false);
		g.image._dom.dispatchEvent (clickEvent);
		
		value_of(g.isEmitted).should_be_true();
	},
	"'mouseover' signal": function(){
		g.isEmitted = false;
	
		g.image.connect("mouseover", function(){
			g.isEmitted = true;
		});
	
		var clickEvent = document.createEvent ("Event");
		clickEvent.initEvent ("mouseover", true, false);
		g.image._dom.dispatchEvent (clickEvent);
		
		value_of(g.isEmitted).should_be_true();
	},
	"'closed' signal": function(){
		g.isEmitted = false;
	
		g.image.connect("closed", function(){
			g.isEmitted = true;
		});
	
		g.image.enableDelete();
	
		var clickEvent = document.createEvent ("Event");
		clickEvent.initEvent ("click", true, false);
		g.image.closeDom.dispatchEvent (clickEvent);
		
		value_of(g.isEmitted).should_be_true();
	},
	"enable()": function(){
		g.image.enable();
		value_of(g.image._disableLayer.style.display).should_be("none");
	},
	"disable()": function(){
		g.image.disable();
		value_of(g.image._disableLayer.style.display).should_be("block");
	},
	"resetWidth()": function(){
		g.image.resetWidth();
		// Test if no width parameter passed
		value_of(g.image.getMinWidthComposite()).should_be(16);
		value_of(g.image._dom.width).should_be(16);
	
		// Test if width parameter passed
		g.image.resetWidth(200);
		value_of(g.image.getMinWidthComposite()).should_be(200);
		value_of(g.image._dom.width).should_be(200);
	},
	"resetHeight()": function(){
		g.image.resetHeight();
		// Test if no height parameter passed
		value_of(g.image.getMinHeightComposite()).should_be(16);
		value_of(g.image._dom.height).should_be(16);
	
		// Test if height parameter passed
		g.image.resetHeight(55);
		value_of(g.image.getMinHeightComposite()).should_be(55);
		value_of(g.image._dom.height).should_be(55);
	},
	"resetSize()": function(){
		// Test if no parameter passed
		g.image.resetSize();
		value_of(g.image.getMinHeightComposite()).should_be(16);
		value_of(g.image.getMinWidthComposite()).should_be(16);
		value_of(g.image._dom.height).should_be(16);
		value_of(g.image._dom.width).should_be(16);
		
		// Test if only height passed
		g.image.resetSize(null, 100);
		value_of(g.image.getMinHeightComposite()).should_be(100);
		value_of(g.image.getMinWidthComposite()).should_be(16);
		value_of(g.image._dom.height).should_be(100);
		value_of(g.image._dom.width).should_be(16);
		
		// Test if only width passed
		g.image.resetSize(100, null);
		value_of(g.image.getMinHeightComposite()).should_be(100);
		value_of(g.image.getMinWidthComposite()).should_be(100);
		value_of(g.image._dom.height).should_be(100);
		value_of(g.image._dom.width).should_be(100);
		
		// Test if only height passed
		g.image.resetSize(100, 50);
		value_of(g.image.getMinHeightComposite()).should_be(50);
		value_of(g.image.getMinWidthComposite()).should_be(100);
		value_of(g.image._dom.height).should_be(50);
		value_of(g.image._dom.width).should_be(100);
	},
	"resetDefaultSize()": function(){
		// Test if no image set
		g.image.resetDefaultSize();
		value_of(g.image._dom.height).should_be(16);
		value_of(g.image._dom.width).should_be(16);
		value_of(g.image.getMinHeightComposite()).should_be(16);
		value_of(g.image.getMinWidthComposite()).should_be(16);
		
		// Test if a real image is set
		g.image.setFromFile("http://www.google.com.kh/intl/en_com/images/srpr/logo1w.png", 25, 27);
		g.image.resetDefaultSize();
		value_of(g.image._dom.height).should_be(95);
		value_of(g.image._dom.width).should_be(275);
		value_of(g.image.getMinHeightComposite()).should_be(95);
		value_of(g.image.getMinWidthComposite()).should_be(275);
	},
	"getChildNode()": function(){
		g.image.getChildNode();
		value_of(g.image._dom.src).should_match(g.image._defaultImage);
	}
});
