describe("gtk.vbox",{
	"before_each": function() {
		g = {};
		g.vbox = new gtk.VBox(true, 5);
	},
	"after_each": function(){
		g = {};
		verify();;
	},
	'new gtk.VBOX(true, 5), will make _minWidth and _minHeight default to 100': function(){
		value_of(g.vbox._minWidth).should_be(100);
		value_of(g.vbox._minHeight).should_be(100);
		value_of(g.vbox._domObj.className).should_be('widget container box vbox')
	},
	'setSizeRequest(), will make _minWidth to 200, _minHeight to 300, isMinWidthChanged to true and _isMinHeightChanged to true': function(){
		g.vbox.setSizeRequest(200, 300);
		value_of(g.vbox._minWidth).should_be(200);
		value_of(g.vbox._minHeight).should_be(300);
		value_of(g.vbox._isMinWidthChanged).should_be_true();
		value_of(g.vbox._isMinHeightChanged).should_be_true();
	},
	"setSpace(50), g.vbox.spacing will be 50": function(){
		var widgetStub1 = new gtk.VBox(true, 5);
		g.vbox.packStart(widgetStub1, true, true, 1);
		value_of(g.vbox.getSpace()).should_be(5);
		g.vbox.setSpace(50);
		value_of(g.vbox.getSpace()).should_be(50);
	},
	"when I call packStart() passing a child as an argument, then I will get child's information calling queryChildPacking()" : function() {
		//stub a child widget
		g.child = {
			sizeRequest: function(){return [100, 100]},
			_domObj: Dom.create({'tag': 'div'}),
			emit: function(){},
			connect: function(){}
		};
		g.vbox.packStart(g.child, true, true, 10);
		var child_info = g.vbox.queryChildPacking(g.child);
		value_of(child_info.expand).should_be(true);
		value_of(child_info.fill).should_be(true);
		value_of(child_info.padding).should_be(10);
		value_of(child_info.packType).should_be(gtk.PACK_START);
	},
	"when I call packEnd() passing a child as an argument, then I will get child's information calling queryChildPacking()" : function() {
		//stub a child widget
		g.child = {
			sizeRequest: function(){return [100, 100]},
			_domObj: Dom.create({'tag': 'div'}),
			emit: function(){},
			connect: function(){}
		};
		g.vbox.packEnd(g.child, false, false, 10);
		var child_info = g.vbox.queryChildPacking(g.child);
		value_of(child_info.expand).should_be(false);
		value_of(child_info.fill).should_be(false);
		value_of(child_info.padding).should_be(10);
		value_of(child_info.packType).should_be(gtk.PACK_END);
	},
	"when I call packStartDefaults() passing a child as an argument, then I will get child's information calling queryChildPacking()" : function() {
		//stub a child widget
		g.child = {
			sizeRequest: function(){return [100, 100]},
			_domObj: Dom.create({'tag': 'div'}),
			emit: function(){},
			connect: function(){}
		};
		g.vbox.packStartDefaults(g.child);
		var child_info = g.vbox.queryChildPacking(g.child);
		value_of(child_info.expand).should_be(true);
		value_of(child_info.fill).should_be(true);
		value_of(child_info.padding).should_be(0);
		value_of(child_info.packType).should_be(gtk.PACK_START);
	},
	"when I call packEndDefaults() passing a child as an argument, then I will get child's information calling queryChildPacking()" : function() {
		//stub a child widget
		g.child = {
			sizeRequest: function(){return [100, 100]},
			_domObj: Dom.create({'tag': 'div'}),
			emit: function(){},
			connect: function(){}
		};
		g.vbox.packEndDefaults(g.child);
		var child_info = g.vbox.queryChildPacking(g.child);
		value_of(child_info.expand).should_be(true);
		value_of(child_info.fill).should_be(true);
		value_of(child_info.padding).should_be(0);
		value_of(child_info.packType).should_be(gtk.PACK_END);
	},
	"when I call setHomogeneous() passing Homogeneous as an argument, then I will get child's information calling getHomogeneous()" : function() {
		//stub a child widget
		g.child = {
			sizeRequest: function(){return [100, 100]},
			_domObj: Dom.create({'tag': 'div'}),
			emit: function(){},
			connect: function(){}
		};
		g.vbox.packStartDefaults(g.child);
		
		g.vbox.setHomogeneous(true);
		var box_info = g.vbox.getHomogeneous();
		value_of(box_info).should_be(true);
		
		g.vbox.setHomogeneous(false);
		box_info = g.vbox.getHomogeneous();
		value_of(box_info).should_be(false);
	},
	/*******************RIVATE METHODS *******************/
	'_createElement(), will return dom which contains three children': function(){
		var dom = g.vbox._createElement(gtk.PACK_START);
		value_of(dom.className).should_be('PACK_START');
		value_of(dom.childNodes.length).should_be(3);
		
		var dom = g.vbox._createElement(gtk.PACK_END);
		value_of(dom.className).should_be('PACK_END');
	},
	'_createSpace(), will return dom which is space': function(){
		var dom = g.vbox._createSpace();
		value_of(typeof dom).should_be('object');
	},
	'_getChildHeight, will return the height of child widget plus its top and bottom padding': function(){
		var widgetStub = { child: {
				sizeRequest: function(){return [10, 10]}
			},
			padding: 10
		};
		var height = g.vbox._getChildHeight(widgetStub);
		value_of(height).should_be(30);
	},
	'_resizeElement(), will resize the children of element': function(){
		var dom = g.vbox._createElement(gtk.PACK_START);
		var domObj = Dom.create({"tag": "div"});
		g.vbox._resizeElement(dom, domObj, 10, 10, 2);
		
		value_of(dom.childNodes[0].style.height).should_be("2px");
		value_of(dom.childNodes[0].style.width).should_be("10px");
		
		value_of(dom.childNodes[1].style.height).should_be('6px');
		value_of(dom.childNodes[1].style.width).should_be("10px");
		
		value_of(domObj.style.height).should_be('6px');
		value_of(domObj.style.width).should_be("10px");
		
		value_of(dom.childNodes[2].style.height).should_be('2px');
		value_of(dom.childNodes[2].style.width).should_be('10px');
	},
	'_getMaxChildSize(), return the max width and height of children': function(){
		var widgetStub1 = { 
			child: {sizeRequest: function(){return [100, 100]}},
			padding: 10
		};
		var widgetStub2 = { 
			child: {sizeRequest: function(){return [200, 200]}},
			padding: 10
		};
		g.vbox._children = [widgetStub1, widgetStub2];
		var size = g.vbox._getMaxChildSize();
		value_of(size[0]).should_be(200);
		value_of(size[1]).should_be(220);
	},
	'resizeContainerHeight(), will resize the height of g.vbox._domObj and g.vbox._minHeight': function(){
		g.vbox._resizeContainerHeight(300);
		value_of(g.vbox._minHeight).should_be(300);
		value_of(g.vbox._domObj.style.height).should_be('300px');
	},
	'resizeContainerWidth(), will resize the width of g.vbox._domObj and g.vbox._minWidth': function(){
		g.vbox._resizeContainerWidth(300);
		value_of(g.vbox._minWidth).should_be(300);
		value_of(g.vbox._domObj.style.width).should_be('300px');
	},
	'_resizeWidgetsHomo(width, height)': function(){
		var widgetStub1 = { 
			child: {
				sizeRequest: function(){return [100, 100]},
				_domObj: Dom.create({'tag': 'div'})
			},
			padding: 10,
			fill: true,
			element: g.vbox._createElement(gtk.PACK_START),
			space: g.vbox._createSpace()
		};
		var widgetStub2 = { 
			child: {
				sizeRequest: function(){return [200, 200]},
				_domObj: Dom.create({'tag': 'div'})
			},
			padding: 10,
			fill: false,
			element: g.vbox._createElement(gtk.PACK_START),
			space: g.vbox._createSpace()
		};
		g.vbox._children = [widgetStub1, widgetStub2];
		g.vbox._resizeWidgetsHomo(300, 300);
		
		value_of(widgetStub1.width).should_be(300);
		value_of(widgetStub1.height).should_be(280);
		value_of(widgetStub1.element.style.height).should_be("300px");
		value_of(widgetStub1.element.style.width).should_be("300px");
		
		value_of(widgetStub2.width).should_be(300);
		value_of(widgetStub2.height).should_be(200);
		value_of(widgetStub2.element.style.height).should_be("300px");
		value_of(widgetStub2.element.style.width).should_be("300px");
		
		value_of(widgetStub2.space.style.height).should_be("5px");
		value_of(widgetStub2.space.style.width).should_be("300px");
	},
	'_getWidgetsSizeNoHomo(), return the max width and height of vbox': function(){
		var widgetStub1 = { 
			child: {sizeRequest: function(){return [100, 100]}},
			padding: 10
		};
		var widgetStub2 = { 
			child: {sizeRequest: function(){return [200, 200]}},
			padding: 10
		};
		g.vbox._children = [widgetStub1, widgetStub2];
		var size = g.vbox._getWidgetsSizeNoHomo();
		value_of(size[0]).should_be(200);
		value_of(size[1]).should_be(345);
	},
	'_resizeWidgetsNoHomo(width, height)': function(){
		var widgetStub1 = { 
			child: {
				sizeRequest: function(){return [100, 100]},
				_domObj: Dom.create({'tag': 'div'})
			},
			padding: 10,
			fill: true,
			expand: true,
			element: g.vbox._createElement(gtk.PACK_START),
			space: g.vbox._createSpace()
		};
		var widgetStub2 = { 
			child: {
				sizeRequest: function(){return [200, 200]},
				_domObj: Dom.create({'tag': 'div'})
			},
			padding: 10,
			fill: false,
			expand: true,
			element: g.vbox._createElement(gtk.PACK_START),
			space: g.vbox._createSpace()
		};
		var widgetStub3 = { 
			child: {
				sizeRequest: function(){return [200, 200]},
				_domObj: Dom.create({'tag': 'div'})
			},
			padding: 10,
			fill: false,
			expand: false,
			element: g.vbox._createElement(gtk.PACK_START),
			space: g.vbox._createSpace()
		};
		g.vbox._children = [widgetStub1, widgetStub2, widgetStub3];
		
		var size = g.vbox._getWidgetsSizeNoHomo();
		//Resize container when the height of all elements is bigger then the container
		g.vbox._resizeContainerHeight(size[1]);
		g.vbox._resizeContainerWidth(size[0]);
		g.vbox._resizeWidgetsNoHomo(size[0], size[1]);
		
		value_of(widgetStub1.width).should_be(200);
		value_of(widgetStub1.height).should_be(100);
		value_of(widgetStub1.element.style.height).should_be("120px");
		value_of(widgetStub1.element.style.width).should_be("200px");
		
		value_of(widgetStub2.width).should_be(200);
		value_of(widgetStub2.height).should_be(200);
		value_of(widgetStub2.element.style.height).should_be("220px");
		value_of(widgetStub2.element.style.width).should_be("200px");
		
		value_of(widgetStub3.width).should_be(200);
		value_of(widgetStub3.height).should_be(200);
		value_of(widgetStub3.element.style.height).should_be("220px");
		value_of(widgetStub3.element.style.width).should_be("200px");
		
		value_of(widgetStub2.space.style.height).should_be("5px");
		value_of(widgetStub2.space.style.width).should_be("200px");
	},
	'_getLastPackEndChild(), return the child which type is gtk.PACK_END': function(){
		var widgetStub1 = { 
			child: {
				sizeRequest: function(){return [100, 100]},
				_domObj: Dom.create({'tag': 'div'})
			},
			padding: 10,
			fill: true,
			expand: true,
			element: g.vbox._createElement(gtk.PACK_START),
			space: g.vbox._createSpace(),
			packType: gtk.PACK_START
		};
		var widgetStub2 = { 
			child: {
				sizeRequest: function(){return [200, 200]},
				_domObj: Dom.create({'tag': 'div'})
			},
			padding: 10,
			fill: false,
			expand: true,
			element: g.vbox._createElement(gtk.PACK_END),
			space: g.vbox._createSpace(),
			packType: gtk.PACK_END
		};
		g.vbox._children = [widgetStub1, widgetStub2];
		g.vbox._getLastPackEndChild();
		value_of(g.vbox._getLastPackEndChild()).should_be(widgetStub2);
	},
	'_appendPackChild(), pack the child the g.vbox._domObj according its packtype': function(){
		var widgetStub1 = { 
			child: {
				sizeRequest: function(){return [100, 100]},
				_domObj: Dom.create({'tag': 'div'})
			},
			padding: 10,
			fill: true,
			expand: true,
			element: g.vbox._createElement(gtk.PACK_START),
			space: g.vbox._createSpace(),
			packType: gtk.PACK_START
		};
		var widgetStub2 = { 
			child: {
				sizeRequest: function(){return [200, 200]},
				_domObj: Dom.create({'tag': 'div'})
			},
			padding: 10,
			fill: false,
			expand: true,
			element: g.vbox._createElement(gtk.PACK_END),
			space: g.vbox._createSpace(),
			packType: gtk.PACK_END
		};
		
		g.vbox._appendPackChild(widgetStub2.element, widgetStub2.space);
		g.vbox._children.push(widgetStub2);
		
		g.vbox._appendPackChild(widgetStub1.element, widgetStub1.space);
		g.vbox._children.push(widgetStub1)

		value_of(g.vbox._domObj.childNodes[0].className).should_be('PACK_START');
		value_of(g.vbox._domObj.childNodes[2].className).should_be('PACK_END');
	},
	/************************** INTEGRATION TEST **********************/
	'_resizeChildren(), will emit "resize" signal to children widgets': function(){
		var vbox1 = new gtk.VBox(true, 10);
		var vbox2 = new gtk.VBox(true, 10);
		
		vbox1.connect('resize', function(width, height){
			value_of(width).should_be(100);
			value_of(height).should_be(100);
		});
		g.vbox.packStart(vbox1, true, true, 1);
		
		vbox2.connect('resize', function(width, height){
			value_of(width).should_be(100);
			value_of(height).should_be(100);
		});
		g.vbox.packStart(vbox2, true, true, 1);
	},
	'_connectChildrenSignal(), will emit "sizerequest" to vbox': function(){
		var vbox1 = new gtk.VBox(true, 10);
		
		g.vbox.connect('sizerequest', function(width, height){
			value_of(width).should_be(100);
			value_of(height).should_be(102);
		});
		g.vbox.packStart(vbox1, true, true, 1);
	}
});
