describe('jsgtk.Util', {
	'before_each': function(){
		g = {};
		g.util = jsgtk.Util;
	},
	'after_each': function(){
		g = {};
	},
	"clone()": function(){
		g.object = {'foo': 'foo'};
		value_of(g.util.clone(g.object)).should_be(g.object);
	},
	"isMatched()": function(){
		value_of(g.util.isMatched("bar", "bar")).should_be_true();
	},
	"arrayRemoveElement()": function(){
		g.array = [1,2,3,4,5];
		g.util.arrayRemoveElement(g.array, 2);
		value_of(g.array.length).should_be(4);
	},
	"arraySwapElements()": function(){
		g.array = [1,2,3,4,5];
		g.util.arraySwapElements(g.array, 1, 5);
		value_of(g.array[0]).should_be(5);
		value_of(g.array[4]).should_be(1);
	},
	"getWindowHeight()": function(){
		value_of(g.util.getWindowHeight()).should_be(window.innerHeight)
	},
	"getWindowWidth()": function(){
		value_of(g.util.getWindowWidth()).should_be(window.innerWidth)
	},
	"calculateTextWidthHeight()": function(){
		value_of(g.util.calculateTextWidthHeight("foo bar")[0]).should_be(49);
		value_of(g.util.calculateTextWidthHeight("foo bar")[1]).should_be(21);
	},
	"trim()": function(){
		value_of(g.util.trim(" foo bar foo ")).should_be("foo bar foo ");
	},
	"getStyle()": function(){
		g.element = document.createElement("DIV");
		g.element.style.height = "1px";
		value_of(g.util.getStyle(g.element, "height")).should_be("1px")
	},
	"getHeight()": function(){
		g.element = document.createElement("DIV");
		g.element.style.height = "1px";
		value_of(g.util.getHeight(g.element)).should_be("1")
	},
	'resetCSS()': function(){
		g.element = document.createElement("DIV");
		g.element.style.height = "1px";
		g.util.resetCSS(g.element, {height: "2px"});
		value_of(g.element.style.height).should_be("2px");
	},
	"restoreCSS()": function(){
		g.element = document.createElement("DIV");
		g.element.style.height = "1px";
		g.oldStyle = g.util.resetCSS(g.element, {height: "2px"});
		value_of(g.element.style.height).should_be("2px");

		g.util.resetCSS(g.element, g.oldStyle);
		value_of(g.element.style.height).should_be("1px");
	},
	"fullHeight": function(){
		g.element = document.createElement("div");
		g.element.style.height = "1px";
		value_of(g.util.fullHeight(g.element)).should_be(1);
		
		g.element = document.createElement("div");
		
			g.subElement = document.createElement("div");
			g.subElement.style.height = "100px";
			g.element.appendChild(g.subElement);
			
		value_of(g.util.fullHeight(g.element)).should_be(100);
	},
	"fullWidth()": function(){
		g.element = document.createElement("div");
		g.element.style.width = "5px";
		value_of(g.util.fullWidth(g.element)).should_be(5);
		
		g.element = document.createElement("div");
			g.element2 = document.createElement("div");
			g.element2.style.width = "120px";
		g.element.appendChild(g.element2);
		document.body.appendChild(g.element);
		g.element.style.width = "10px";
		value_of(g.util.fullWidth(g.element)).should_be(10);	
	},
	"pageX()": function(){
		g.element = document.createElement("div");
		g.element.style.left = "10px";
		g.element.style.position = "absolute";
		document.body.appendChild(g.element);
		value_of(g.util.pageX(g.element)).should_be(10);
	},
	"pageY()": function(){
		g.element = document.createElement("div");
		g.element.style.left = "7px";
		g.element.style.position = "fixed";
		document.body.appendChild(g.element);
		value_of(g.util.pageX(g.element)).should_be(7);
	},
	"getPageSize()": function(){
		value_of(g.util.getPageSize()[0]).should_be(window.innerWidth);
		value_of(g.util.getPageSize()[1]).should_be(window.innerHeight);
		value_of(g.util.getPageSize()[2]).should_be(window.innerWidth);
		value_of(g.util.getPageSize()[3]).should_be(window.innerHeight);
	},
	"debug()": function(){
		value_of(function(){
			g.util.debug("error");
		}).should_throw();
	},
	"removeArrayElements()": function(){
		var arr = [5, 3, 5, 8, 6];
		g.util.removeArrayElements(arr, 1, 2)
		value_of(arr.length).should_be(3);
		value_of(arr).should_be([5, 8, 6]);
	},
	"getScrollX()": function(){
		value_of(g.util.getScrollX()).should_be(document.body.scrollLeft);
	},
	"getScrollY()": function(){
		value_of(g.util.getScrollY()).should_be(document.body.scrollTop);
	}
});

