/**
 * @constructor
 */
gobject = {
	_signals: [],
	_signalCount: 0,
	
	/**
	* The gobject.signalNew() registers specified object type to a specified signal_name
	* @param  {string}  signalName  : a string containing the name of the signal
    * @param  {object}  type 		: the object type that the signal is associated with
	* @return {integer} signalId    :a unique integer signal ID
	*/
	signalNew: function(signalName, type) {
		var signal = {};
		if(this.signalLookup(signalName, type)) throw("RuntimeError: this signal is already binned to the type");
		this._signalCount++;
		signal[signalName] = {'type': type, 'id': this._signalCount};
		this._signals.push(signal);
		return this._signalCount;
	},

	/**
	* find the specified the signal that is associated to the object type
	* @param {string} name       : the name of a signal
	* @param {object} type       : object type
    * @return {iteger} or {null} : the integer id of a signal supported by type or 0.
	*/
	signalLookup: function(name, type) {
		for(var i = this._signals.length; i--;){
			if(this._signals[i][name] && this._signals[i][name].type === type ) return this._signals[i][name].id;
			if(this._signals[i][name] && type instanceof this._signals[i][name].type ) return this._signals[i][name].id;
		}
		return null;
	}
};
