/**
 * Base class of all widgets, handle signal binding and triggering
 * @constructor
 */
gobject.GObject = function(){
	this.props = {};
	this._signalHandlers = {};	

	/**
	 * Trigger the the registered signal
	 * @param {String} detailedSignal The name of signal
	 * @param {Arguments} arguments value that will be used as argument by handler
	 */
	this.emit = function(detailedSignal){
		if(this._signalHandlers[detailedSignal]){
			var args = [];
			for(var i=1; i<arguments.length; i++) args.push(arguments[i]);		
			for(var i=0; i<this._signalHandlers[detailedSignal].length; i++){
				this._signalHandlers[detailedSignal][i].handler.apply(this, args.concat(this._signalHandlers[detailedSignal][i].args));
			}				
		}
	};
	
	/**
	 * Listen to the signal being triggered
	 * @param {String} detailedSignal The name of signal
	 * @param {Function} handler function to be executed when the signal is triggered
	 * @return The id of handler
	 */
	this.connect = function(detailedSignal, handler){
		this._signalCheck(detailedSignal);
		if(!this._signalHandlers[detailedSignal]){
			this._signalHandlers[detailedSignal] = [];
			this._handlerId++;
		}
		var args = [];
		var handlerId = new UUID().id;
		for(var i=2; i<arguments.length; i++) args.push(arguments[i]);
		this._signalHandlers[detailedSignal].push({'handler': handler, 'args': args, id: handlerId});
		return handlerId;
	};
    
    /**
     * Disconnect the function from the signal
     * @param {String} signal handlerId id
     */
	this.disconnect = function(handlerId){
		for(var el in this._signalHandlers){
			var handlers = this._signalHandlers[el];
			for(var i = 0, len = handlers.length; i < len; i++){
				if(handlers[i].id === handlerId) {
					handlers.splice(i, 1);
					break;
				}
			}
		}
	};
	
	/** @private */
	this._signalCheck = function(detailedSignal){
		var signal = gobject.signalLookup(detailedSignal, this);
		if(signal) return signal;
		throw("Undefined signal name: " + detailedSignal);
	};

    /** @private */
	this._removeSignal = function(handlerId){
		for (var el in this._signalHandlers){
			if(el == handlerId) delete this._signalHandlers.handlerId;		
		}
	};
};
