/**
* @constructor
* @base gtk.Widget
* @param {String} Label text to be displayed by in the middle of button
*/
gtk.CheckButton = function(label){
    gtk.Widget.apply(this);

	/** 'checked' signal*/
	this.CHECKBUTTON_CHECKED = 'checked';
	/** 'unchecked' signal*/
	this.CHECKBUTTON_UNCHECKED = "unchecked";
	/** 'focused' signal*/
	this.CHECKBUTTON_FOCUSED = "focused";
	/** 'blurred' signal*/
	this.CHECKBUTTON_BLURRED = "blurred";

    this._isChecked   	= false;
    this._imageSize   	= 16;
    this._space       	= 7;
    this._imageClass  	= "gtk_check_button_img";
    this._disabledModal	= null;
        
    this._labelWidget = new gtk.Label(label);
    
    /** @private */
    this._render = function(){
        this._domObj.className += " gtk_check_button";
            this._checkImgLabelContainer = jsgtk.Dom.create({"append": this._domObj, "tag": "div", "class": "gtk_check_button_img_label_container"});
                this._checkImage = jsgtk.Dom.create({"append": this._checkImgLabelContainer, "tag": "div", "className": "gtk_check_button_img"});                
                this._checkLabel = jsgtk.Dom.create({"append": this._checkImgLabelContainer, "tag": "div", "class": "gtk_check_button_label"});
                this._checkLabel.appendChild(this._labelWidget.getNode());
                
        this._width  = this._minWidth  = this._imageSize + this._labelWidget.getMinWidthComposite() + this._space;
        this._height = this._minHeight = this._labelWidget.getMinHeightComposite();
    };
    this._render();
    
    this._initEvent();
    
    this.resizeWidthComposite(this._width);
    this.resizeHeightComposite(this._height);
};

gtk.CheckButton.prototype = new gtk.Widget();
gtk.CheckButton.prototype.constructor = gtk.CheckButton;

/** @private */
gtk.CheckButton.prototype._initEvent = function(){
    var self = this;
	var jsgtkEvent = jsgtk.Event;
	
	this._mouseupHandler = jsgtkEvent.addDomListener(this._domObj, 'mouseup', function(){
	    if(!self._isChecked) self.setChecked();
        else self.setUnchecked();
	});
	this._mouseoverHandler = jsgtkEvent.addDomListener(this._checkImgLabelContainer, 'mouseover', function(){
		if(self._isChecked) self._checkImage.className = "gtk_check_button_img gtk_check_button_checked_hover";
		else self._checkImage.className = "gtk_check_button_img gtk_check_button_unchecked_hover";
	});
	
	this._mouseoutHandler = jsgtkEvent.addDomListener(this._checkImgLabelContainer, 'mouseout', function(){
		if(self._isChecked) self._checkImage.className = "gtk_check_button_img gtk_check_button_checked";
		else   self._checkImage.className = "gtk_check_button_img";
	});
	
	jsgtkEvent.addDomListener(this._domObj, "focus", function(){
		self.emit("focused");
    });
    
    jsgtkEvent.addDomListener(this._domObj, "blur", function(){
    	self.emit("blurred");
    });
    
    this._keyupHandler = jsgtkEvent.addDomListener(this._domObj, 'keyup', function(e){
	    if(e.keyCode == gtk.ENTER_KEY){
           if(!self._isChecked) self.setChecked();
           else self.setUnchecked();
	    }
	});
};

/**
 * Method to set tab index value
 */
gtk.CheckButton.prototype.setTabindex = function(index){
    this._domObj.tabIndex = index;
};

/**
 * set default checked on check button
 */
gtk.CheckButton.prototype.setChecked = function(){
    this._checkImage.className = this._imageClass +" gtk_check_button_checked";
    this._isChecked = true;
    this.emit("checked", this._labelWidget.getText());
};

/**
 * uncheck the checked
 */
gtk.CheckButton.prototype.setUnchecked = function(){
    this._checkImage.className ="gtk_check_button_img gtk_check_button_unchecked";
	this._isChecked = false;
	this.emit("unchecked", this._labelWidget.getText());
};

/**
 * return state of checkbutton
 * @return {boolean}
 */
gtk.CheckButton.prototype.getCheckedState = function(){
    return this._isChecked;
};

/**
 * All functionalities are disabled
 */
gtk.CheckButton.prototype.disable = function(){
	var jsgtkEvent = jsgtk.Event;
	jsgtkEvent.removeListener(this._mouseupHandler);
	jsgtkEvent.removeListener(this._keyupHandler);
	jsgtkEvent.removeListener(this._mouseoutHandler);
	jsgtkEvent.removeListener(this._mouseoverHandler);
	this._labelWidget.setAttribute({color: "#6F6F6F"});
};

/**
 * All functionalities are enabled
 */
gtk.CheckButton.prototype.enable = function(){
	var jsgtkEvent = jsgtk.Event;
	jsgtkEvent.registerHandler(this._mouseupHandler);
	jsgtkEvent.registerHandler(this._keyupHandler);
	jsgtkEvent.registerHandler(this._mouseoutHandler);
	jsgtkEvent.registerHandler(this._mouseoverHandler);
	this._labelWidget.setAttribute({color: ""});
};

/** @private */
gtk.CheckButton.prototype._resizeHeight = function(height){
    this._height = height;
    this._domObj.style.height = height + "px";
        this._checkImgLabelContainer.style.height = this._minHeight + "px";
};

/** @private */
gtk.CheckButton.prototype._resizeWidth = function(width){
    this._width = width;
    this._domObj.style.width = width + "px";
        this._checkImgLabelContainer.style.width = width + "px";
};

/** @private */
gtk.CheckButton.prototype.resizeWidthComposite = function(width){
    var minWidth = this._minWidth < width ? width : this._minWidth;
    this._resizeWidth(minWidth);
};

/** @private */
gtk.CheckButton.prototype.resizeHeightComposite = function(height){
    var minHeight = this._minHeight < height ? height : this._minHeight;
    this._resizeHeight(minHeight);
    this._positionCheckBox();
};

/** @private */
gtk.CheckButton.prototype._positionCheckBox = function(height){
    this._checkImgLabelContainer.style.top = this._height / 2 - this._minHeight / 2 + "px";
};

/** Signal in CheckButton widget: checked */
gobject.signalNew("checked", gtk.CheckButton, null, null, []);
gobject.signalNew("unchecked", gtk.CheckButton, null, null, []);
gobject.signalNew("focused", gtk.CheckButton, null, null, []);
gobject.signalNew("blurred", gtk.CheckButton, null, null, []);
