/**
 * ComboBox is a class to create a combobox dropdown list that contains many optional choices
 * The combobox dropdown list stores data as model from gtk.TreeStore.
 * @see gtk.TreeStore
 * @constructor
 * @base gtk.Widget
 */
gtk.ComboBox = function(){
    gtk.Widget.apply(this);
	
	/** 'clicked' signal */
	this.COMBOBOX_CLICKED = "clicked";
	/** 'changed' signal */
	this.COMBOBOX_CHANGED = "changed";	
    
    this._minWidth = this._width = 150;
    this._minHeight = this._height = 30;
    
	this._isToogled = false;
	this._vBarWidth = 25;
	this._isDropDownCreated = false;
	this._selectedItem;

	this._render = function(){
	    this._domObj.className += " gtk_combobox";
			this._entryHolder = jsgtk.Dom.create({tag: "div", append: this._domObj, className: "gtk_combobox_input_holder"});
				this._entry = jsgtk.Dom.create({tag: "input", append: this._entryHolder, className: "gtk_combobox_input"});

		    this._gradient = jsgtk.Dom.create({"append": this._domObj, "tag": "div", "class": "gtk_button_gradient"});		

			this._activeLayer = jsgtk.Dom.create({"append": this._domObj, "tag": "div", "class": "gtk_combobox_active_layer"});

			this._labelContainer = jsgtk.Dom.create({'append': this._domObj, 'tag': 'div','class': 'gtk_combobox_label_container'});
    			var table = jsgtk.Dom.create({append: this._labelContainer, tag: "table"});
	    			var tr = jsgtk.Dom.create({append: table, tag: "tr"});
	    				var td = jsgtk.Dom.create({append: tr, tag: "td"});
		    				this._labelDom = jsgtk.Dom.create({"append": td, "text": "- Please Select -", "tag": "div", 'class': 'gtk_combobox_label'});

			var arrowImge = jsgtk.Dom.create({'append': this._domObj, 'tag': 'div','class': 'gtk_combobox_image_container'});
            arrowImge.style.width = this._vBarWidth + 'px';
    			var table = jsgtk.Dom.create({append: arrowImge, tag: "table"});
	    			var tr = jsgtk.Dom.create({append: table, tag: "tr"});
	    				var td = jsgtk.Dom.create({append: tr, tag: "td"});
	    					jsgtk.Dom.create({append: td, tag: "div", "class": "gtk_combobox_image"});

			this.__dropDown = jsgtk.Dom.create({tag: "div", append: document.body, className: "gtk_combobox_holder"});
			jsgtk.Dom.hideElement(this.__dropDown);
	};

	this._render();
	this._initEvent();
	this._dropDownRendere = new gtk.SmartEntryDropDownRendere(this);
	this._controller = new gtk.ComboBoxController(this);
};

gtk.ComboBox.prototype = new gtk.Widget();
gtk.ComboBox.prototype.constructor = gtk.ComboBox;

/**
 * A method to to set model onto the widget
 * @param {Object} model The object model value from gtk.TreeStore 
 */
gtk.ComboBox.prototype.setModel = function(model){
	this._controller.setModel(model);
};

/**
 * A method to set selected item by any value member
 * @param {String} valueMember The name of value member assigned
 * @param {String or Integer} value The field value in accordance with its value member
 */
gtk.ComboBox.prototype.setSelectedItemBy = function(valueMember, value){
    if(value) this.emit("_valueMembersearched", valueMember, value);
};

/**
 * A method to get/return object of selected item
 * @return {Object}, The object value of the selected item
 * @type Object
 */
gtk.ComboBox.prototype.getSelectedItem = function(){
    return this._selectedItem;
};

/**
 * A method to set its tab index number
 * @param {Interger} index The tab index value to be set
 */
gtk.ComboBox.prototype.setTabindex = function(index){
    this._domObj.tabIndex = index;
};

/** @private */
gtk.ComboBox.prototype._addEventMouseWheel = function(scrollObj){
    var self = this;
    jsgtk.Event.addDomListener(scrollObj, 'scroll', function(e){
	    self._deactivate();
	}, false);
};

/**
 * A method to set displaying error or not
 * @param {Boolean} isOnError The boolean value to be passed: true is to show error, false is to hide
 */
gtk.ComboBox.prototype.showError = function(isOnError){
	this._showError = (typeof isOnError !== 'undefined' && typeof isOnError === 'boolean') ? isOnError : true;
    
    if(this._showError) this._domObj.className += " gtk_combobox_on_error";
    else this._domObj.className = "gtk_widget gtk_combobox";
};

/**
 * A method to get/return string value of selected item
 * @return {String} The text value of selected item
 * @type String
 */
gtk.ComboBox.prototype.getActiveText = function(){
    return this._labelDom.innerHTML;
};

/**
 * A method to reload or refresh its contents
 */
gtk.ComboBox.prototype.refresh = function(){
    this._isDropDownCreated = false;
    this._deactivate();
    this._clearDropDown();
	this._labelDom.innerHTML = "- Please Select -";
};

/** @private */
gtk.ComboBox.prototype._setSelectedItem = function(value){
    if(this._selectedItem === value) return;
    this._selectedItem = value;
    this._labelDom.innerHTML = value["display"];
    this.emit("changed", value);
};

/** @private */
gtk.ComboBox.prototype._renderDropDown = function(children, parentNode){
	var rowNodes = this._dropDownRendere._constructRowNodes(children, parentNode);
	this._appendDropDown(rowNodes);
    this._isDropDownCreated = true;
};

/** @private */
gtk.ComboBox.prototype._clearDropDown = function(){
	this.__dropDown.innerHTML = "";
};

/** @private */
gtk.ComboBox.prototype._handleRowNodeSelected = function(){
	var selectedRowNode = this._dropDownRendere.getSelectedRowNode();
	this.emit("_itemselected", selectedRowNode.id);
	this._deactivate();
};

/** @private */
gtk.ComboBox.prototype.__bindInputEvent = function(){
	var jsgtkEvent = jsgtk.Event, self = this;
	var timeoutId;
	this.__inputKeyupId = jsgtkEvent.addDomListener(this._entry, "keydown", function(e){

		switch(e.keyCode){
			case gtk.DOWN_ARROW_KEY:
				self.__fireDownArrowKey();
				return;
			case gtk.UP_ARROW_KEY:
				self.__fireUpArrowKey();
				return
			case gtk.ENTER_KEY:
				self.__fireEnterKey();
				return;
			case gtk.TAB_KEY:
				self.__fireEnterKey();
				jsgtk.Event.stopDefault(e);
				return;
		}
			
		if(self._entry.value){
		
			clearTimeout(timeoutId);
			
			timeoutId = setTimeout(function(){

                var text = self._entry.value;
                var selectedRowNode = self._dropDownRendere.getSelectedRowNode();
				
			    if(selectedRowNode) {
                    var isMatched = self._compareText(selectedRowNode.innerHTML, text);
                    if(!isMatched) self._handleSearch(text);
                }else self._handleSearch(text);
			}, 200);
		}
	});
};

/** @private */
gtk.ComboBox.prototype._handleSearch = function(text){
    var lastCharacter = text[text.length - 1];
    this.emit("_search", lastCharacter);
    this._entry.value = lastCharacter;
};

/** @private */
gtk.ComboBox.prototype._initEvent = function(){
	var self = this;

	jsgtk.Event.addDomListener(this._domObj, "mousedown", function(e){
		if(self._isToogled)
		    self._deactivate();
		else
            self._activate();
                 
	    if(!self._isDropDownCreated) {
	        self.emit("clicked");
	        self._showLoading();
	    }

		jsgtk.Event.stopBubble(e);
		jsgtk.Event.stopDefault(e);
	});

	jsgtk.Event.addDomListener(this.__dropDown, "mousedown", function(e){
		var node = e.target;
		if(jsgtk.Util.isMatched("gtk_smart_entry_drop_down_element", node.className)){
			self._handleRowNodeSelected();
		}
		jsgtk.Event.stopBubble(e);
	});
	
	jsgtk.Event.addDomListener(this.__dropDown, "mouseover", function(e){
		var node = e.target;
		if(jsgtk.Util.isMatched("gtk_smart_entry_drop_down_element", node.className))
			self._dropDownRendere.hilightRowNode(node);
	});
	
	jsgtk.Event.addDomListener(this._entry, "blur", function(e){
		self._deactivate();
		jsgtk.Event.stopBubble(e);
	});

	this.__bindInputEvent();
};

/** @private */
gtk.ComboBox.prototype._compareText = function(text1, text2){
    var re = new RegExp("^" + text2.toLowerCase());
    var result = text1.toLowerCase().match(re);
    return result;
};

/** @private */
gtk.ComboBox.prototype._hilightRow = function(path){
	this._dropDownRendere.hilightRowById(path)
};

/** @private */
gtk.ComboBox.prototype.setFocus = function(){
	this._dropDownRendere._deHilightSelectedRow();
	this._entry.focus();
}

/** @private */
gtk.ComboBox.prototype.lostFocus = function(){
	this._entry.blur();
};

/** @private */
gtk.ComboBox.prototype._appendDropDown = function(dropDown){
	this._hideLoading();
	this.__dropDown.appendChild(dropDown);
};

/** @private */
gtk.ComboBox.prototype._showLoading = function(){
	this.__dropDown.className += " gtk_combobox_loading";
};
/** @private */
gtk.ComboBox.prototype._hideLoading = function(){
	this.__dropDown.className = "gtk_combobox_holder";
};
/** @private */
gtk.ComboBox.prototype.__removeInputEvent = function(){
	var jsgtkEvent = jsgtk.Event;
	if(this.__inputKeypressId) jsgtkEvent.removeListener(this.__inputKeypressId);
	if(this.__inputKeyupId) jsgtkEvent.removeListener(this.__inputKeyupId);
};

/** @private */
gtk.ComboBox.prototype.__fireDownArrowKey = function(){
	this._dropDownRendere.hilightNextRowNode();
};

/** @private */
gtk.ComboBox.prototype.__fireUpArrowKey = function(){
	this._dropDownRendere.hilightPreviousRowNode();
};

/** @private */
gtk.ComboBox.prototype.__fireEnterKey = function(){
	if(this._dropDownRendere.getSelectedRowNode()){
		this._handleRowNodeSelected();
	
		this._dropDownRendere._deHilightSelectedRow();
	}
};

/** @private */
gtk.ComboBox.prototype._setPosition = function(){
    var top = jsgtk.Util.pageY(this._domObj);
    var left = jsgtk.Util.pageX(this._domObj);
    var header = parseInt(this._domObj.style.height);
    var scrollY = this._getNearestScrollerY();
           
    this.__dropDown.style.top = top + header + 2 - scrollY + "px";
    this.__dropDown.style.left = left + 1 + "px";
};

/** @private */
gtk.ComboBox.prototype._getNearestScrollerY = function(){
    var scrollY = 0;
    var parentNode = this._domObj.parentNode;
    while(parentNode.parentNode != null){
        if(/gtk_scroller_base/.test(parentNode.className)){
            scrollY = parentNode.scrollTop;
            this._addEventMouseWheel(parentNode);
            break;
        }
        parentNode = parentNode.parentNode;
    }
    return scrollY;
};

/** @private*/
gtk.ComboBox.prototype._activate = function(){
	this._isToogled = true;
	this.setFocus();
	jsgtk.Dom.showElement(this.__dropDown);
    jsgtk.Dom.showElement(this._activeLayer);
    this._domObj.className += " gtk_combobox_active";
	this._setPosition();
};

/** @private*/
gtk.ComboBox.prototype._deactivate = function(){
	this._isToogled = false;
	this.lostFocus();
	jsgtk.Dom.hideElement(this.__dropDown);
    jsgtk.Dom.hideElement(this._activeLayer);
    this._domObj.className = "gtk_widget gtk_combobox";
};

/** @private */
gtk.ComboBox.prototype.resizeWidthComposite = function(width){
	this._width = width;
	this._domObj.style.width = width + 'px';
		this._labelContainer.style.width = width - this._vBarWidth - 9 + "px";
		this.__dropDown.style.width = width + "px";
};

/** @private */
gtk.ComboBox.prototype.resizeHeightComposite = function(height){
	this._height = height;
	this._domObj.style.height = height + 'px';
};

gobject.signalNew("_rendersubnodes", gtk.ComboBox, null, null, []);
gobject.signalNew("_setselectedvalues", gtk.ComboBox, null, null, []);
gobject.signalNew("_search", gtk.ComboBox, null, null, []);
gobject.signalNew("_itemselected", gtk.ComboBox, null, null, []);
gobject.signalNew("_valueMembersearched", gtk.ComboBox, null, null, []);
gobject.signalNew("changed", gtk.ComboBox, null, null, []);
gobject.signalNew("clicked", gtk.ComboBox, null, null, []);
