/** 
 * ComboBoxController is a core class of gtk.ComboBox to control all logic matters
 * to perform any specific action.
 * @constructor
 * @see gtk.ComboBox
 */
gtk.ComboBoxController = function(view){
	this._model = null;
	var self = this;

	this._render = function(model, parentNode){
		var mashedUpdata = this.__mashupModel(model.getChildren());
		view._renderDropDown(mashedUpdata, parentNode);
	};

	view.connect("_rendersubnodes", function(path, parentNode){
		var childModel = self._model.getChildByPath(path);
		self._render(childModel, parentNode);
	});	
	
	view.connect("_search", function(value){
		var model = self._model.search(value);
		var path = model ? model.getPath() : "";
		view._hilightRow(path);
	});
	
	view.connect("_itemselected", function(path){
	    var childModel = self._model.getChildByPath(path);
	    view._setSelectedItem(childModel.getValue())
	});
	
	view.connect("_valueMembersearched", function(valueMember, value){
	    var childModel = self._model._searchByValueMember(valueMember, value);
	    view._setSelectedItem(childModel.getValue())
	});
};

/** @private */
gtk.ComboBoxController.prototype.setModel = function(model){
	this._model = model;
	this._render(this._model, null);
};

/** @private */
gtk.ComboBoxController.prototype.__mashupModel = function(nodes){
    var data = [], node, hasChild;
    for(var i = 0, len = nodes.length; i < len; i++){
        node = nodes[i];
        hasChild = node.getChildren().length  > 0;
        data.push({"display": node.getValue()["display"], "hasChildNodes": hasChild, "path": node.getPath(), "valueObject": node.getValue()});
    }
    return data;
};
