//Class created to help DropDownViewLabel
/** @private */
gtk.DropDownView = function(){
    gtk.Widget.apply(this);
    
    var self = this;
    this._minWidth = this._width = 20;
    this._minHeight = this._height = 16;
    this.__entryWidth = 20;

	this._selectedItem = null;
    this._index = 0;
    
    /** @private */
	this._render = function(){
		var jsgtkDom = jsgtk.Dom;
		this._domObj = jsgtkDom.create({tag:"div", className: "gtk_drop_down"});	    
			this.__holder = jsgtkDom.create({tag: "div", append: this._domObj, className: "gtk_drop_down_entry_holder"});
				this.__entryLabel = jsgtkDom.create({tag: "span", append: this.__holder, className: "gtk_drop_down_display_label"});
				this.__entry = jsgtkDom.create({tag: "input", append: this.__holder, className: "gtk_drop_down_entry"});
			this.__dropDown = jsgtkDom.create({tag: "div", append: this._domObj, className: "gtk_drop_down_holder"});
	};	

	this._render();
};

gtk.DropDownView.prototype = new gtk.Widget();
gtk.DropDownView.prototype.constructor = gtk.DropDownView;

/** @private */
gtk.DropDownView.prototype.constructView = function(model, label){
	this._model = model;
	new gtk.DropDownController(this, model, label);
};

//call by controller
/** @private */
gtk.DropDownView.prototype._renderDropDown = function(children, parent, count){
	var len = children.length;
	count = count || 0;
	var docFragment = parent || document.createDocumentFragment();

	for(var i = 0; i < len; i++){

		var child = children[i];
		var isAncestor = child.getPath().length == 2;
		if(child.hasNode){
			var childNode = jsgtk.Dom.create({tag: "div", append: docFragment, text: child.getValue().value, className: "gtk_drop_down_group"});
			childNode.style.paddingLeft = 15 * ((child.getPath().length/2)-1) + "px";
			var grandChildren = child.getChildren();
			this._renderDropDown(grandChildren, docFragment, count++);
		}
		else{
			var element = jsgtk.Dom.create({tag: "div", append: docFragment, text: child.getValue().value, className: "gtk_drop_down_element"});
			element.style.paddingLeft = 15 * ((child.getPath().length/2)-1) + "px";
			element.id = child.getPath();
		}
	}
	return docFragment;
};

/** @private */
gtk.DropDownView.prototype._updateSize = function(parentWidth){
	this.__parentWidth = parentWidth;
	this.__dropDown.style.width = parentWidth + "px";
};

/** @private */
gtk.DropDownView.prototype._appendDropDown = function(dropDown){	
	this.__dropDown.appendChild(dropDown);
	this.__hideDropDown();
	var self = this;
	
	jsgtk.Event.addDomListener(this.__dropDown, "mouseover", function(e){
		var node = e.target;
		if(node.id){
			self.__deHilightRow(self.__selectedRow);
			self.__selectedRow = node;
			node.className += " gtk_drop_down_selected";
		}
	});
	
	jsgtk.Event.addDomListener(this.__dropDown, "click", function(e){
		var node = e.target;
		if(/gtk_drop_down_element/.test(node.className)){
			self.__entry.value = node.innerHTML;
			tag = self.__createTag();

			var child = self._model.getChildByPath(node.id);
			tag.object = child.getValue();
			self.__dropDown.style.display = "none";
		}
		
		jsgtk.Event.stopBubble(e);
	});
};

/** @private */
gtk.DropDownView.prototype.__deHilightRow = function(selectedRow){
	if(selectedRow)
		selectedRow.className = "gtk_drop_down_element";
};

/** @private */
gtk.DropDownView.prototype._setDisplayedLabel = function(label){
	this.__entryLabel.innerHTML = label;
	this.__updateHolderWidth(this.__entryLabel);
	this.__bindInputEvent();
};

/** @private */
gtk.DropDownView.prototype.__updateHolderWidth = function(entryLabel){
	var tempDom = entryLabel.cloneNode(true);
	this.__entryWidth = jsgtk.Util.fullWidth(tempDom);
};

/** @private */
gtk.DropDownView.prototype.__bindInputEvent = function(){
	var self = this;
	var entry = this.__entry;
	var tempSpan = document.createElement("span");
	var width, value;
	
	jsgtk.Event.addDomListener(entry, "keypress", function(){
		value = entry.value;
	});
		
	jsgtk.Event.addDomListener(entry, "keyup", function(e){
		switch(e.keyCode){
			case gtk.DOWN_ARROW_KEY:
				self.__showDropDown();
				self.__fireDownArrowKey();
				return;
			case gtk.UP_ARROW_KEY:
				self.__showDropDown();
				self.__fireUpArrowKey();
				return;
			case gtk.ENTER_KEY:
				self.__fireEnterKey();
				self.emit("_enterkeypressed", self);
				return;
			case gtk.BACKSPACE_KEY:
				if(!value){
					self.emit("_backspacekeypressed", self);
				}
				break;
			case gtk.DELETE_KEY:
				if(!value){
					self.emit("_deletekeypressed", self);
				}
		}
		if(entry.value){
			self.__hideLabel();
			self.__showDropDown();
			tempSpan.innerHTML = entry.value;
            tempWidth = self.__calculateTextWidth(tempSpan) + self.__entryWidth - 2;
            width = (tempWidth) < self.__parentWidth - 2? tempWidth + "px" : self.__parentWidth - 2 + "px";
            entry.style.width = width;
            self.__holder.style.width = width;
			self.emit("_search", entry.value);
		}else{
			self.__showLabel();
			self.__hideDropDown();
		}
	});
	
	jsgtk.Event.addDomListener(this.__entryLabel, "click", function(e){
		entry.focus();
	});
};

/** @private */
gtk.DropDownView.prototype.__calculateTextWidth = function(dom){
    var tempDom = dom.cloneNode(true);
    tempDom.style.display = "none";
    document.body.appendChild(tempDom);
    return jsgtk.Util.fullWidth(tempDom);
};

/** @private */
gtk.DropDownView.prototype.__fireDownArrowKey = function(){
	this.__deHilightRow(this.__selectedRow);
	var nextNode = this.__getNextSelectableNode(this.__selectedRow);
	this.__hilightElement(nextNode);
};

/** @private */
gtk.DropDownView.prototype.__fireUpArrowKey = function(){
	if(this.__selectedRow) this.__deHilightRow(this.__selectedRow);
	var previousNode = this.__getPreviousSelectableNode(this.__selectedRow) || this.__selectedRow;
	this.__hilightElement(previousNode);
};

/** @private */
gtk.DropDownView.prototype.__fireEnterKey = function(){
	if(this.__selectedRow){
		this.__entry.value = this.__selectedRow.firstChild.nodeValue;
		var tag = this.__createTag();
		var child = this._model.getChildByPath(this.__selectedRow.id);
		tag.object = child.getValue();
		this.__hideDropDown();
	}
};

/** @private */
gtk.DropDownView.prototype._setTag = function(obj){
	var createdTag = this.__holder.childNodes[2];
	if(createdTag) this.__holder.removeChild(createdTag);
	var tag = this.__createTag(obj.value);
	tag.object = obj;
};

/** @private */
gtk.DropDownView.prototype.__createTag = function(value){
	var tagValue = value || this.__entry.value;
	if(tagValue){
        var self = this;
        var tag = jsgtk.Dom.create({"tag": "div", "className": "gtk_drop_down_tag_holder", append: this.__holder});
        	var spanTag  = jsgtk.Dom.create({"tag": "span", "append": tag , "text": tagValue});
        	var closeButton = jsgtk.Dom.create({"tag": "div", append: tag , "className": "gtk_drop_down_close"});
        
        this.__holder.style.width = "";
        this._domObj.style.width = "";
        
        jsgtk.Event.addDomListener(closeButton, "click", function(e){
            self.__holder.removeChild(self.__holder.lastChild);
            self.__showEntry();
            self.__showLabel();
            self.__entry.value = "";
            self.__entry.focus();
        }, false);

        this.__entry.style.width = this.__entryWidth + "px";
        this.__hideEntry();
        this.__entryLabel.style.display = "none";
        return tag;
    }
};

/** @private */
gtk.DropDownView.prototype.__getNextSelectableNode = function(currentNode){
	var currentNode = currentNode || this.__dropDown.firstChild;
	var nextNode = currentNode.nextSibling || currentNode;
	if(/gtk_drop_down_element/.test(nextNode.className)){
		return nextNode;
	}
	else{
		var node = this.__getNextSelectableNode(nextNode);
		return node;
	}
};

/** @private */
gtk.DropDownView.prototype.__hilightElement = function(selectedRow, index){
	this.__selectedRow = selectedRow;
	this.__selectedRow.className += " gtk_drop_down_selected";
	var index = index || this.__getSelectedNodeIndex();
	this.__scrollToHilightedRow(index);
};

/** @private */
gtk.DropDownView.prototype.__getSelectedNodeIndex = function(){
	var tempNodes = this.__dropDown.cloneNode(true).childNodes;
	var selectedNode = this.__selectedRow;
	var found = false;
	for(var i = 0, len = tempNodes.length; i < len; i++){
		if(/gtk_drop_down_selected/.test(tempNodes[i].className))
			break;
	}
	return i;
};

/** @private */
gtk.DropDownView.prototype.__getPreviousSelectableNode = function(currentNode){
	var currentNode = currentNode;
	var previousNode = currentNode? currentNode.previousSibling : this.__dropDown.lastChild;
	if(!previousNode) return;
	if(/gtk_drop_down_element/.test(previousNode.className)){
		return previousNode;
	}
	else{
		var node = this.__getPreviousSelectableNode(previousNode);
		return node;
	}
};

/** @private */
gtk.DropDownView.prototype._hilightRow = function(path){
	var resultDom = null;
	var tempDom = this.__dropDown.cloneNode(true).childNodes;
	var len = tempDom.length;
	var found = false;
	for(var i = 0; i < len; i++){
		if(tempDom[i].id == path){
			found = true;
			break;	
		}
	}
    
    if(found){
    	this.__deHilightRow(this.__selectedRow);
    	this.__hilightElement(this.__dropDown.childNodes[i], i);
    }
};

/** @private */
gtk.DropDownView.prototype._getSelected = function(){
	return this.__selected;
};

/** @private */
gtk.DropDownView.prototype._deleteTag = function(){
	if(this.__holder.childNodes.length == 3){
		this.__holder.removeChild(this.__holder.lastChild);
		this.__selected = false;
		this.__showEntry();
		this.__showLabel();
		this.__entry.value = "";
		this.__entry.focus();
	}
};

/** @private */
gtk.DropDownView.prototype._selectTag = function(){
	if(this.__holder.childNodes.length == 3){
		this.__holder.lastChild.className += " gtk_drop_down_view_selected";
		this.__selected = true;
	}
};

/** @private */
gtk.DropDownView.prototype._getItem = function(){
	if(this.__holder.childNodes[2]) this.__holder.childNodes[2].object;
};

/** @private */
gtk.DropDownView.prototype._setFocus = function(){
	if(/gtk_drop_down_tag_holder/.test(this.__holder.lastChild.className)) return;
	this.__showEntry();
	this.__entry.focus();
};

/** @private */
gtk.DropDownView.prototype.__scrollToHilightedRow = function(positionFromTop){
    this.__dropDown.scrollTop = positionFromTop * 16 - 70;
};

/** @private */
gtk.DropDownView.prototype._hiddenEntry = function(){
	this.__entry.style.visibility = "hidden";
}

/** @private */
gtk.DropDownView.prototype.__hideDropDown = function(){
	this.__dropDown.style.display = "none";
};

/** @private */
gtk.DropDownView.prototype.__showDropDown = function(){
	this.__dropDown.style.display = "";
};

/** @private */
gtk.DropDownView.prototype.__showEntry = function(){
	this.__entry.style.display = "";
	this.__entry.style.visibility = "";
};

/** @private */
gtk.DropDownView.prototype.__hideEntry = function(){
	this.__entry.style.display = "none";
};

/** @private */
gtk.DropDownView.prototype.__showLabel = function(){
	var entryLabelStyle = this.__entryLabel.style;
	entryLabelStyle.visibility = "";
	entryLabelStyle.display = "";
};
/** @private */
gtk.DropDownView.prototype.__hideLabel = function(){
	this.__entryLabel.style.visibility = "hidden";
};
/** @private */
gtk.DropDownView.prototype._getDom = function(){
	return this._domObj;
};
/** @private */
gtk.DropDownView.prototype._getItem = function(){
	var item = this.__holder.firstChild.object;
	return item;
};

/** DropDown signal:  */
gobject.signalNew("_search", gtk.DropDownView, null, null, []);
gobject.signalNew("_enterkeypressed", gtk.DropDownView, null, null, []);
gobject.signalNew("_backspacekeypressed", gtk.DropDownView, null, null, []);
gobject.signalNew("_deletekeypressed", gtk.DropDownView, null, null, []);
