/**
 * A single line text entry field
 * @constructor
 * @param {String} maxLength The maximum length of the entry
 */
gtk.Entry = function(maxLength){
	gtk.Widget.apply(this);
	
	/** 'keypressed' signal */
	this.ENTRY_KEYPRESSED = "keypressed";
	/** 'enterkeypressed' signal */
	this.ENTRY_ENTERKEYPRESSED = "enterkeypressed";
	/** 'focused' signal */
	this.ENTRY_FOCUSED = "focused";
	/** 'blurred' signal */
	this.ENTRY_BLURRED = "blurred";
	/** 'changed' signal */
	this.ENTRY_CHANGED = "changed";
	/** 'clicked' signal */
	this.ENTRY_CLICKED = "clicked";
	
    var self 				= this;	
	this._dom 				= null;
	this._text 				= "";
	this._maxLength 		= maxLength || "";
	this._width 			= this._minWidth = 60;
	this._height 			= this._minHeight = 18;
	this._widthChars;
	this._span;
	this._visible 			= true;
	this._entryFirstValue 	= "";
	this._showError 		= true;
	
	/** @private */ 
	this._render = function(){
		var _self = this;
		this._domObj.className += " gtk_entry";
			this._span = jsgtk.Dom.create({append: this._domObj, tag: "span"});
				this._dom = jsgtk.Dom.create({append: this._span, tag: "input", "className":"gtk_entry_input", type: "text", style:"border:0;"});
			
        this.resize(this._width, this._height);

	};	

	this._render();
	
	this._initEvent();
};

gtk.Entry.prototype = new gtk.Widget();
gtk.Entry.prototype.constructor = gtk.Entry;

/** @private */
gtk.Entry.prototype._initEvent = function(){
	var self = this;
	var jsgtkEvent = jsgtk.Event;
	
	jsgtkEvent.addDomListener(this._dom, "keyup", function(e){
        if(e.keyCode === gtk.ENTER_KEY){
        	self.emit("enterkeypressed", self._dom.value, e);
        	return;
        } 
        if(self._dom.value != self._entryFirstValue){
			self.emit("changed", self._dom.value);
        }
        
        self.emit("keypressed", self._dom.value, e);
	    jsgtkEvent.stopBubble(e);
	}, false);

	jsgtkEvent.addDomListener(this._span, "click", function(){
	    self._dom.focus();
	    self.emit("clicked", self._dom.value);
	});

	jsgtkEvent.addDomListener(this._dom, "keypress", function(){
	    self._entryFirstValue = self._dom.value;
	});

	jsgtkEvent.addDomListener(this._dom, "focus", function(e){
	    self.emit("focused", self._dom.value, e);
	    self._domObj.style.border = "1px groove #F9CF06";
	    if(self._label) self._label.style.display = "none";    	
	});
	
	jsgtkEvent.addDomListener(this._dom, "blur", function(e){
	    self.emit("blurred", self._dom.value, e);
	    self._domObj.style.border = "";
	    if(self._label) if(!self._dom.value) self._label.style.display = "";
	}, false);
};


/**
 * Focus the cursor in the entry
 */
gtk.Entry.prototype.setFocus = function(){
    this._dom.focus();   
};

/**
 * Method to show error or not
 * @param {Boolean} isOnError to show error, false to hide
 */
gtk.Entry.prototype.showError = function(isOnError){
	this._showError = (typeof isOnError !== 'undefined' && typeof isOnError === 'boolean') ? isOnError : true;
    if(this._showError) this._domObj.className += " gtk_entry_onerror";
    else this._domObj.className = "gtk_widget gtk_entry";
};

/**
 * Method to set tab index value
 * @param {Integer} index integer value to be set
 */
gtk.Entry.prototype.setTabindex = function(index){
    this._dom.tabIndex = index;
};
/**
 * @depricate
 */
gtk.Entry.prototype.setTextSelectable = function(isSelectable){
    if(isSelectable){
		this.activateTextSelectable();
		console.log("gtk.Entry.setTextSelectable() will be depricated, please use gtk.Entry.activateTextSelectable()")
	}else{
		this.deactivateTextSelectable();
		console.log("gtk.Entry.setTextSelectable() will be depricated, please use gtk.Entry.deactivateTextSelectable()")
	}
};

/** make gtk.Entry selectabe*/
gtk.Entry.prototype.activateTextSelectable = function(){this._dom.className = ""};

/** make gtk.Entry unselectable*/
gtk.Entry.prototype.deactivateTextSelectable = function(){this._dom.className = "gtk_entry_non_selectable"};

/**
 * set the entry border to be visible or invisible
 * @param {Boolean} isBorderVisible If true, the entry border is visible. If false, the entry border is invisible
 */
gtk.Entry.prototype.setBorderVisible = function(isBorderVisible){
    this._domObj.style.borderColor = isBorderVisible ? "1px solid grey" : "transparent";
	console.log("gtk.Entry.setBorderVisible() will be depricated, please use gtk.Entry.showBorder() or gtk.Entry.hideBorder()");
};

gtk.Entry.prototype.showBorder = function(){this._domObj.style.borderColor = "1px solid grey"};
gtk.Entry.prototype.hideBorder = function(){this._domObj.style.borderColor = "transparent"};

/**
 * Sets the "text" property to the value of text. The string in text replaces the current contents of the entry.
 * @param {String} text A string to use as the new contents of the entry
 */
gtk.Entry.prototype.setText = function(text){
	this._text = text;
	this._dom.value = text === undefined ? "" : this._text;

	if(this._label) this._removeLabel();

	this.emit("changed");
};

gtk.Entry.prototype._removeLabel = function(){
	this._span.removeChild(this._label);
	this._label = null;
};

/**
 * Returns the value of the "text" property which is a string containing the contents of the entry.
 * @return {String} text the content of the entry as a string
 */
gtk.Entry.prototype.getText = function(){
	return this._dom.value;
};

/**
 * sets the "width-chars" property to the value of n_char
 * @param {interger} nChars Width in chars
 */
gtk.Entry.prototype.setWidthChars = function(nChars){
 	this._widthChars = nChars;
 	this._updateWidthHeight(nChars);
    this._resizeWidth(this._width);
};
 
/**
 * gets the value of the "widthChars" property which is set by the setWidthChars() method
 * @return {integer} number of chars to request space for
 */
gtk.Entry.prototype.getWidthChars = function(){
	return this._widthChars;
};

/**
 * sets the "max-length" property to the value of max
 * @param {interger} max The maximum length of the entry. The value passed in will be clamped to the range 0-65536.
 */
gtk.Entry.prototype.setMaxLength = function(max){
 	this._maxLength = max;
 	this._dom.setAttribute("maxlength", max);
};
 
/**
 * retrieves the value of the "max-length" property that specifies the maximum allowed length of the text in the entry.
 * @return {integer} The maximum allowed number of characters
 */
gtk.Entry.prototype.getMaxLength = function(){
	return this._maxLength;
};

/**
 * sets the "visibility" property to the value of visible. If visible is True  the contents of the entry are displayed as plain text. If visible is False, the contents are obscured by replacing the characters with the invisible character
 * @param {boolean} visible If False the contents are obscured using the "invisible-char"
 */
gtk.Entry.prototype.setVisibility = function(visible){
 	this._visible = visible;
 	this._visible? this._dom.setAttribute("type", "text"): this._dom.setAttribute("type", "password");
};
 
/**
 * retrieves the value of the "visibility" property that determines whether the text in entry is visible.
 * @return {boolean} True if the text is currently visible
 */
gtk.Entry.prototype.getVisibility = function(){
	return this._visible;
};

/**
 * Retrieves the current length of the text in entry
 * @return {interger} Return the current number of characters in GtkEntry, or 0 if there are none.
 */
gtk.Entry.prototype.getTextLength = function(){
	var charLength = this._dom.value.length;
	return charLength;
};

/** @private */
gtk.Entry.prototype._updateWidthHeight = function(size){
	var input = this._dom.cloneNode(true);
	if(size){
		var inputStyle = input.style;
		input.setAttribute("size", size);
		inputStyle.width = "";   
	}
	document.body.appendChild(input);
	this._width = input.clientWidth;
    this._minWidth = this._width;
	document.body.removeChild(input);
};

/**
 * Set the displayed label in Entry. When the entry is focus, the label will be remove from entry
 * @param {String} label The label to be displayed in entry
 */
gtk.Entry.prototype.setDisplayLabel = function(label){
	if(this._label) this._label.innerHTML = label || "";
    else this._label = jsgtk.Dom.create({"tag": "span", "append": this._span, "className": "gtk_entry_label", "text": label});
};

/**
 * Method to select text in entry
 */
gtk.Entry.prototype.select = function(){
    this._dom.select();
};

/**
* Entry will be enabled or disabled
* @param {boolean} isEnabled
*/
gtk.Entry.prototype.setEnable = function(isEnabled){
	isEnabled ? this.enable() : this.disable();
};

/**
 *  enable gtk.Entry
 */
gtk.Entry.prototype.enable = function(){
    this._dom.disabled = false;
    this._domObj.style.backgroundColor = "";
    this._dom.style.backgroundColor = "";
};

/**
 * disable gtk.Entry
 */
gtk.Entry.prototype.disable = function(){
    this._dom.disabled = true;
    this._dom.style.width = parseInt(this._domObj.style.width) - 4  + "px";
    this._domObj.style.backgroundColor = "#EDECEC";
    this._dom.style.backgroundColor = "#EDECEC";
};

/** @private */
gtk.Entry.prototype.getMinWidthComposite = function(){
	return this._minWidth;
};

/** @private */
gtk.Entry.prototype.getMinHeightComposite = function(){
	return this._minHeight;
};

/** @private */
gtk.Entry.prototype.resetMinHeightComposite = function(){
	var minSize = this.getSizeRequest();
	this.setHeight(minSize[1]);
};

/** @private */
gtk.Entry.prototype.resetMinWidthComposite = function(){
	var minSize = this.getSizeRequest();
	this.setWidth(minSize[0]);
};

/** @private */
gtk.Entry.prototype.resize = function(width, height){
    this._resizeWidth(width);
    this._resizeHeight(height);
};

/** @private */
gtk.Entry.prototype._resizeWidth = function(width){
	this._domObj.style.width = width - 2 + "px";
	this._dom.style.width = width - 8 + 'px';
	this._width = width;
};
/** @private */
gtk.Entry.prototype._resizeHeight = function(height){
   	this._domObj.style.height = height + "px";
   	this._dom.style.height = height - 2 + "px";
	this._span.style.lineHeight = height + "px";
	this._height = height;
};

/** @private */
gtk.Entry.prototype._resizeHeightByFont = function(height){
    this._height = this._minHeight = height;
   	this._domObj.style.height = height + "px";
   	this._dom.style.height = height - 2 + "px";
	this._span.style.lineHeight = height + "px";
};

/**
 * Reset height by font size
 * @param {Interger} size The size of font in pixel
 */
gtk.Entry.prototype.resizeEntryByFont = function(size){
    this._dom.style.fontSize = size + "px";
	var tmpNode = this._dom.cloneNode(true);
	var tempStyle = tmpNode.style;
	tempStyle.fontSize = size + "px";
	tempStyle.width = "";
	tempStyle.height = "";
	var height = jsgtk.Util.fullHeight(tmpNode);
    this._resizeHeightByFont(height);
};

/** @private */
gtk.Entry.prototype.resizeWidthComposite = function(width){
    this._width = width;
	this._resizeWidth(width);
};

/** @private */
gtk.Entry.prototype.resizeHeightComposite = function(height){
    this._height = height;
	this._resizeHeight(height);
};

gobject.signalNew("keypressed", gtk.Entry, null, null, []);
gobject.signalNew("enterkeypressed", gtk.Entry, null, null, []);
gobject.signalNew("focused", gtk.Entry, null, null, []);
gobject.signalNew("blurred", gtk.Entry, null, null, []);
gobject.signalNew("changed", gtk.Entry, null, null, []);
gobject.signalNew("clicked", gtk.Entry, null, null, []);
