//Class created to help DropDownViewLabel
/** @private */
gtk.EntryTag = function(label){
	var self = this;
	gtk.Widget.apply(this);
    this._minWidth = this._width = 20;
    this._minHeight = this._height = 16;
    this._entryWidth = 30;

	this._selectedItem = null;
    this._index = 0;
    
    /** @private */
	this._render = function(){
		var jsgtkDom = jsgtk.Dom;
		this._domObj = jsgtkDom.create({tag:"div", className: "gtk_entry_tag"});
		this._domObj.id = new UUID().id;	    
			this.__entryLabel = jsgtkDom.create({tag: "span", append: this._domObj, className: "gtk_entry_tag_display_label", text: label});
			this.__entry = jsgtkDom.create({tag: "input", append: this._domObj, className: "gtk_entry_tag_input"});
	};

	this._render();
	this.__updateDomWidth();
	this.__initEvent();
};

gtk.EntryTag.prototype = new gtk.Widget();
gtk.EntryTag.prototype.constructor = gtk.EntryTag;

/** @private */
gtk.EntryTag.prototype._getDom = function(){
	return this._domObj;
};

/** @private */
gtk.EntryTag.prototype.__updateDomWidth = function(){
	var tempDom = this.__entryLabel.cloneNode(true);
//	document.body.appendChild(tempDom);
	this.__entryWidth = jsgtk.Util.fullWidth(tempDom);
	this._domObj.style.width = this.__entryWidth + 2 + "px";
	this.__entry.style.width = this.__entryWidth + "px";
};

/** @private */
gtk.EntryTag.prototype.__initEvent = function(){
	this.__bindLabelEvent();
	this.__bindEntryEvent();
};

/** @private */
gtk.EntryTag.prototype.__bindLabelEvent = function(){
	var self = this;
	jsgtk.Event.addDomListener(this.__entryLabel, "click", function(){
		self._setFocus();
	});
};

/** @private */
gtk.EntryTag.prototype._setFocus = function(){
	if(/gtk_entry_tag_holder/.test(this._domObj.className)) return;
	this.__showEntry();
	this.__entry.focus();
};

/** @private */
gtk.EntryTag.prototype.__bindEntryEvent = function(){
	var jsgtkEvent = jsgtk.Event;
	var self = this;
	var entry = this.__entry;
	var tempSpan = document.createElement("span");
	var value;
	
	jsgtkEvent.addDomListener(entry, "blur", function(){
		if(!entry.value){
			return;
		} 
		self.__hideEntry();
		self.__hideLabel();
		self.__createTag();
	});
	
	jsgtkEvent.addDomListener(this.__entry, "keypress", function(){
		value = entry.value;
	});
	
	jsgtkEvent.addDomListener(this.__entry, "keyup", function(e){
		switch(e.keyCode){
			case gtk.ENTER_KEY:
				self.__fireEnterKey();
				self.emit("_enterkeypressed", self);
				break;
			case gtk.BACKSPACE_KEY:
				if(!value) self.emit("_backspacekeypressed", self);
				break;
			case gtk.DELETE_KEY:
				if(!value) self.emit("_deletekeypressed", self);
		}
				
		if(entry.value){
			tempSpan.innerHTML = entry.value;
            tempWidth = self._calculateTextWidth(tempSpan) + self._entryWidth - 2;
            width = (tempWidth) < self.__parentWidth - 2? tempWidth : self.__parentWidth - 2;
            entry.style.width = width + "px";
            if(parseInt(self._domObj.style.width) < width)
            	self._domObj.style.width = width + "px";
			self.__hideLabel();
		}
		else	
			self.__showLabel();	
	});
};

/** @private */
gtk.EntryTag.prototype._calculateTextWidth = function(dom){
    var tempDom = dom.cloneNode(true);
    tempDom.style.display = "none";
    document.body.appendChild(tempDom);
    return jsgtk.Util.fullWidth(tempDom);
};

/** @private */
gtk.EntryTag.prototype.__fireEnterKey = function(){
	this.__hideEntry();
	this.__hideLabel();
	this.__createTag();
};

/** @private */
gtk.EntryTag.prototype._setTag = function(obj){
	var lastChild = this._domObj.childNodes[2];
	if(lastChild) this._domObj.removeChild(lastChild);
	var tag = this.__createTag(obj.value);
	tag.object = obj;
};

/** @private */
gtk.EntryTag.prototype._updateSize = function(parentWidth){
	this.__parentWidth = parentWidth;
};

/** @private */
gtk.EntryTag.prototype.__createTag = function(value){
	var tagValue = value || this.__entry.value;
	if(tagValue){
        var self = this;
        var lastChild = this._domObj.lastChild;
        if(/gtk_entry_tag_holder/.test(lastChild.className))
        	this._domObj.removeChild(lastChild);
        
    	var tag = jsgtk.Dom.create({"tag": "div", "className": "gtk_entry_tag_holder"});
		var spanTag  = jsgtk.Dom.create({"tag": "span", "append": tag , "text": tagValue});
		var closeButton = jsgtk.Dom.create({"tag": "div", append: tag , "className": "gtk_entry_tag_close"});

		this._domObj.appendChild(tag);

		jsgtk.Event.addDomListener(closeButton, "click", function(e){
			self._domObj.removeChild(tag);
			self._domObj.style.width = self.__entryWidth + 5 + "px";
			self.__showEntry();
			self.__entry.value = "";
			self.__entry.focus();
			self.__showLabel();
		});

		this.__entry.style.width = this.__entryWidth + "px";
		this._domObj.style.width = this.__entryWidth + 5 + "px";
		this.__hideEntry();
		this.__entryLabel.style.display = "none";
		this.__updateHolderWidth(tag);
        return tag;
    }
};

/** @private */
gtk.EntryTag.prototype.__updateHolderWidth = function(tag){
	var tempDom = tag.cloneNode(true);
	document.body.appendChild(tempDom);
	var width = jsgtk.Util.fullWidth(tempDom);
	if(width > this.__parentWidth) width = this.__parentWidth - 5;
	this._domObj.style.width = width + 5 + "px";
};

/** @private */
gtk.EntryTag.prototype._getSelected = function(){
	return this.__selected;
};

/** @private */
gtk.EntryTag.prototype._deleteTag = function(){
	if(this._domObj.childNodes.length == 3){
		var lastChild = this._domObj.lastChild;
		this._domObj.removeChild(lastChild);
		this.__selected = false;
		this._domObj.style.width = this.__entryWidth + 3 + "px";
		this.__showEntry();
		this.__showLabel();
		this.__entry.value = "";
		this.__entry.focus();
	}
};

/** @private */
gtk.EntryTag.prototype._selectTag = function(){
	if(this._domObj.childNodes.length == 3){
		var lastChild = this._domObj.lastChild;
		lastChild.className += " gtk_entry_tag_selected";
		this.__selected = true;
	}
};

/** @private */
gtk.EntryTag.prototype._hiddenEntry = function(){
	this.__entry.style.visibility = "hidden";
};

/** @private */
gtk.EntryTag.prototype.__hideEntry = function(){
	this.__entry.style.display = "none";
};

/** @private */
gtk.EntryTag.prototype.__showEntry = function(){
	this.__entry.style.display = "";
	this.__entry.style.visibility = "";
};

/** @private */
gtk.EntryTag.prototype.__hideLabel = function(){
	this.__entryLabel.style.visibility = "hidden";
};

/** @private */
gtk.EntryTag.prototype.__showLabel = function(){
	this.__entryLabel.style.visibility = "";
	this.__entryLabel.style.display = "";
};

/** @private */
gtk.EntryTag.prototype._getValue = function(){
	var lastChild = this._domObj.lastChild;
	if(/gtk_entry_tag_holder/.test(lastChild.className))
		return lastChild.firstChild.firstChild.nodeValue;
	else
		return "";
};

/** @private */
gtk.EntryTag.prototype._getItem = function(){
	if(this._domObj.childNodes.length == 3)
		return {value: this._domObj.lastChild.firstChild.firstChild.nodeValue}		
};

/** EntryTag signal:  */
gobject.signalNew("_enterkeypressed", gtk.EntryTag, null, null, []);
gobject.signalNew("_backspacekeypressed", gtk.EntryTag, null, null, []);
gobject.signalNew("_deletekeypressed", gtk.EntryTag, null, null, []);

