/**
 * Form is a class to create form widget
 * @param {Object} attributes The attributes of theform
 * @constructor
 * @base gtk.Widget
 */
gtk.Form = function(attributes){
    gtk.Widget.apply(this);
    
    this._child = null;
    
    /** @private */
    this._render = function(){
        this._domObj.className += " gtk_form";
        this._form = jsgtk.Dom.create({"tag":"form", "append": this._domObj});
        this.setAttribute(attributes);
    };
    
    this._render();
};

gtk.Form.prototype = new gtk.Widget();
gtk.Form.prototype.constructor = gtk.Form;

/**
 * A method to set the attributes to form
 * @param {Object} attributes The attributes to be set as form's attributes
 */
gtk.Form.prototype.setAttribute = function(attributes){
    var dom = this._form;
    for(var property in attributes){
        dom.setAttribute(property, attributes[property]);
    }
};

/**
 * A method to add a widget to Form
 * @param {Widget} widget A child widget to be added to Form
 */
gtk.Form.prototype.add = function(widget){
	if(!this._child){
		this._child = widget;
		if(widget._domObj) this._form.appendChild(widget.getNode());
		
		var size = this._getChildSize(widget);
		this.resize(size[0], size[1]);
	}
};

/** @private */
gtk.Form.prototype.resize = function(width, height){
    this._resizeWidth(width);
    this._resizeHeight(height);  
    
    if(this._child){
        this._child.resizeWidthComposite(width);
        this._child.resizeHeightComposite(height);
    }
};

/** @private */
gtk.Form.prototype._resizeWidth = function(width){
	this._width = width;
	this._domObj.style.width = this._width + 'px';
};

/** @private */
gtk.Form.prototype._resizeHeight = function(height){
	this._height = height;
	this._domObj.style.height = this._height - 2 + 'px';
};

/** @private */
gtk.Form.prototype._getChildSize = function(widget){
    var width    = widget.getWidth();
    var height   = widget.getHeight();

    return [width, height];
};

/**
 * A method to get the minimum width of Form
 * @return The minimum width of Form
 * @type Integer
 */
gtk.Form.prototype.getMinWidthComposite = function(){
    this._minWidth = this._child ? this._child.getMinWidthComposite() : this._width;
	return this._minWidth;
};

/**
 * A method to get the minimum height of Form
 * @return The minimum height of Form
 * @type Integer
 */
gtk.Form.prototype.getMinHeightComposite = function(){
    this._minHeight = this._child ? this._child.getMinHeightComposite() : this._height;
	return this._minHeight;
};

/**
 * A method to resize the width of Form
 * @param {Interger} width The length of width to be resize.
 */
gtk.Form.prototype.resizeWidthComposite = function(width){
	var minWidthComposite = this.getMinWidthComposite();
	var minWidth = width > minWidthComposite ? width : minWidthComposite;
	this.resize(minWidth, this._height);
};

/**
 * A method to resize the height of Form
 * @param {Interger} height The length of width to be resize.
 */
gtk.Form.prototype.resizeHeightComposite = function(height){
	var minHeightComposite = this.getMinHeightComposite();
	var minHeight = height > minHeightComposite ? height : minHeightComposite;
	this.resize(this._width, minHeight);
};
