/**
 * A class to create a homogeneous horizontal box or container
 * @constructor
 * @param {Integer}, an integer spacing value
 * @param {Integer}, an integer padding value
 */
gtk.HBoxHomogeneous = function(spacing, padding) {
	gtk.HBox.call(this, spacing, padding);
	
	/** @private */
	this._render = function(){
	    var domObj = this._domObj;
		domObj.className += " box_homogeneous";
	};
	
	this._render();
};

gtk.HBoxHomogeneous.prototype = new gtk.HBox();
gtk.HBoxHomogeneous.prototype.constructor = gtk.HBoxHomogeneous;

/** @private */
gtk.HBoxHomogeneous.prototype.resize = function(width, height){
	this._resizeHeight(height);
	this._resizeWidth( width);
};

/** @private */
gtk.HBoxHomogeneous.prototype._handleFilledElement = function(child, width){
    child["childWidget"].getNode().style.left = child["padding"] + "px";
    child["childWidget"].resizeWidthComposite(width - child["padding"] * 2);
};

/** @private */
gtk.HBoxHomogeneous.prototype._handleNoneFilledElement = function(child, width){
    var childMinWidth = child["childWidget"].getMinWidthComposite();
    var left = (width - childMinWidth) / 2;
    child["childWidget"].getNode().style.left = left + "px";
    child["childWidget"].resizeWidthComposite(childMinWidth);
};

/** @private */
gtk.HBoxHomogeneous.prototype._resizeChildrenWidth = function(width){    
    var child;
    for(var i = 0, len = this._children.length; i < len; i++){
        child = this._children[i];
        child["space"].style.width = this._spacing + "px";
        child["space"].style.left = this._padding + "px";
        child["element"].style.width = width + "px";
        child["element"].style.left = this._padding + "px";

        if(child["fill"]) this._handleFilledElement(child, width);
        else this._handleNoneFilledElement(child, width);
    }
    this._hideLastSpace();
};

/** @private */
gtk.HBoxHomogeneous.prototype._getMaxChildWidth = function(){
    var maxChildWidth = this._children[0]["childWidget"].getMinWidthComposite() + (this._children[0]["padding"] * 2);
    var childWidth;
    for(var i = 1, len = this._children.length; i < len; i++){
        childWidth = this._children[i]["childWidget"].getMinWidthComposite() + (this._children[i]["padding"] * 2);
        maxChildWidth = maxChildWidth < childWidth ? childWidth : maxChildWidth;
    }
    return maxChildWidth;
};

/** 
 * Method to get/return minimum composite width
 * @return {Integer}, the minimum composite width
**/
gtk.HBoxHomogeneous.prototype.getMinWidthComposite = function(){
    var childrenWidth = this._getMaxChildWidth() * this._children.length;
    var spacings      = this._spacing * (this._children.length - 1);
    var paddings      = this._padding * 2;
	var minWidth 	  = childrenWidth + spacings +paddings;
	minWidth		  = this._minWidth < minWidth ? minWidth : this._minWidth; 
    return  minWidth;
};

/** 
 * Method to resize composite width of the widget
 * @param {Integer}, the composite width to be set
**/
gtk.HBoxHomogeneous.prototype.resizeWidthComposite = function(width){
    var maxChildWidth = this._getMaxChildWidth();
    var maxWidth      = this.getMinWidthComposite();
    if(width > maxWidth){
        var paddings      = this._padding * 2;
        var offsetWidth = width - maxWidth - paddings;
        var extraWidth  = offsetWidth / this._children.length;
        this._resizeWidth(width);
        this._resizeChildrenWidth(maxChildWidth + extraWidth);
    }else{
        this._resizeWidth(maxWidth);
        this._resizeChildrenWidth(maxChildWidth);
    }
};
