/**
 * A class to create non homogeneous horizontal box or container
 * @constructor
 * @param {Integer}, an integer spacing value
 * @param {Integer}, an integer padding value
 */
gtk.HBoxNoneHomogeneous = function(spacing, padding) {
    gtk.HBox.call(this, spacing, padding);
	this._render = function(){
	    var domObj = this._domObj;
		domObj.className += " hbox_none_homogeneous";
	};
	
	this._render();
};

gtk.HBoxNoneHomogeneous.prototype = new gtk.HBox();
gtk.HBoxNoneHomogeneous.prototype.constructor = gtk.HBoxNoneHomogeneous;

/** 
 * Method to resize HBox both width and height
 * @param {Integer}, the integer width value to be passed
 * @param {Integer}, the integer height value to be passed
**/
gtk.HBoxNoneHomogeneous.prototype.resize = function(width, height){
	this._resizeHeight(height);
	this._resizeWidth( width);
};

/** @private */
gtk.HBoxNoneHomogeneous.prototype._handleFilledElement = function(child, width){
    child["childWidget"].getNode().style.left = child["padding"] + "px";
    child["childWidget"].resizeWidthComposite(width - child["padding"] * 2);  
};

/** @private */
gtk.HBoxNoneHomogeneous.prototype._handleNoneFilledElement = function(child, width, childWidth){
    var left = (width - childWidth)/ 2;
    child["childWidget"].getNode().style.left = left + "px";
    child["childWidget"].resizeWidthComposite(childWidth);
};

/** @private */
gtk.HBoxNoneHomogeneous.prototype._handleExpandedElement = function(child, extraWidth){
    var childWidth = child["childWidget"].getMinWidthComposite();
    var width = childWidth + extraWidth + child["padding"] * 2;
    child["element"].style.width = width +"px";

    if(child["fill"]) this._handleFilledElement(child, width);
    else this._handleNoneFilledElement(child, width, childWidth);
};

/** @private */
gtk.HBoxNoneHomogeneous.prototype._handleNoneExpandedElement = function(child){
    var childWidth = child["childWidget"].getMinWidthComposite();
    child["childWidget"].getNode().style.left = child["padding"] + "px";
};

/** @private */
gtk.HBoxNoneHomogeneous.prototype._resizeChildrenWidth = function(extraWidth){
    var child;
    for(var i = 0, len = this._children.length; i < len; i++){
        child = this._children[i];
        child["space"].style.width  = this._spacing + "px";
        child["space"].style.left  = this._padding + "px";
        child["element"].style.left = this._padding + "px";
        
        if(child["expand"]) this._handleExpandedElement(child, extraWidth);
        else this._handleNoneExpandedElement(child);
    }
    this._hideLastSpace();
};

/** @private */
gtk.HBoxNoneHomogeneous.prototype._getNumberOfExpandedChildren = function(){
    var numberOfExpandedChildren = 0;
    for(var i = 0, len = this._children.length; i < len; i++){
        if(this._children[i]["expand"]) numberOfExpandedChildren++;
    }
    return numberOfExpandedChildren;
};

/** 
 * Method to get/return minimum composite width
 * @return {Integer}, the minimum composite width
**/
gtk.HBoxNoneHomogeneous.prototype.getMinWidthComposite = function(){
    var maxChildWidth = 0;
    for(var i = 0, len = this._children.length; i < len; i++){
        maxChildWidth += this._children[i]["childWidget"].getMinWidthComposite() + (this._children[i]["padding"] * 2);
    }
    var paddings = this._padding * 2;
    var spacings =this._spacing * (len-1);

	var minWidth 	  = maxChildWidth + spacings + paddings;
	minWidth		  = this._minWidth < minWidth ? minWidth : this._minWidth; 
    return minWidth;
};

/** 
 * Method to resize composite width of the widget
 * @param {Integer}, the composite width to be set
**/
gtk.HBoxNoneHomogeneous.prototype.resizeWidthComposite = function(width){
    var maxChildWidth = this.getMinWidthComposite();
    if(width > maxChildWidth){
        var offsetWidth = width - maxChildWidth;
        var extraWidth = offsetWidth / this._getNumberOfExpandedChildren();
        this._resizeWidth(width);
        this._resizeChildrenWidth(extraWidth);
    }else{
        this._resizeWidth(maxChildWidth);
        this._resizeChildrenWidth(0);
    }
};
