/**
 * Separate the paned into two vertical paned 
 * @constructor
 * @base gtk.Paned
 */
gtk.HPaned = function() {
	gtk.Paned.apply(this);
	
    var self = this;
	this._dividerWidth    = 6;
	this._paned1Width     = (this._width / 2) - (this._dividerWidth / 2);
	this._paned2Width     = (this._width / 2) - (this._dividerWidth / 2);
	this._isPaned1Restore = true;
	this._isPaned2Restore = true;
	this._isDrawer        = false;
	this._drawerWidth     = 6;
	this._isDrawed        = false;
	this._isDrawedToLeft  = false;
	this._isDrawerLeftEnabled = false;
	this._isDrawerRightEnabled = false;
	this._leftAnchored      = 0;
	this._drawerEventId = null;
	this._isDragged = false;
	
	/** @private */
	this._render = function(){
		var domObj = this._domObj,
			domObjStyle = this._domObj.style,
			jsgtkDom = jsgtk.Dom;
		
		domObj.className += " gtk_hPaned";
		domObjStyle.width = this._width + 'px';
		domObjStyle.height = this._height + 'px';

			this._paned1 = jsgtkDom.create({'tag': 'div', 'class': 'gtk_paned1'});
			var paned1Style = this._paned1.style;
			paned1Style.width = this._paned1Width + 'px';
			paned1Style.height = this._height + 'px';
		
			this._domDivider = jsgtkDom.create({'tag': 'div', 'class': 'gtk_dom_divider'});
			    this._dom = jsgtkDom.create({'append': this._domDivider, 'tag': "div", 'class': 'gtk_paned_divider_image'});
			    
			this._drawer = jsgtkDom.create({"append": domObj, "tag": "div", "class": "gtk_hPaned_drawer"});
			    this._restoreButton = jsgtkDom.create({"append": this._drawer, "tag": "div", "class": "gtk_hPaned_drawer_restore_icon"});
			var drawerStyle = this._drawer.style;
			drawerStyle.width = this._drawerWidth + "px";
			drawerStyle.height= this._height + "px";
			jsgtkDom.hideElement(this._drawer);
			
			this._draw = jsgtkDom.create({"append": domObj, "tag": "div", "class": "gtk_hPaned_draw"});
			jsgtkDom.hideElement(this._draw);
			
			var domDividerStyle = this._domDivider.style;
			domDividerStyle.width = this._dividerWidth + 'px';
			domDividerStyle.height =  this._height+ "px";
			domDividerStyle.left = this._paned1Width + 'px';
			
			this._ghostDivider = this._domDivider.cloneNode(false);
			this._ghostDivider.className = "gtk_paned_ghost_divider";
		
			this._paned2 = jsgtk.Dom.create({'tag': 'div', 'class': 'gtk_paned2'});
			var paned2Style = this._paned2.style;
			paned2Style.width = this._paned1Width + 'px';
			paned2Style.height = this._height + 'px';

		domObj.appendChild(this._paned1);
		domObj.appendChild(this._domDivider);
		domObj.appendChild(this._paned2);
		domObj.appendChild(this._ghostDivider);
	};

	this._render();
	
	this._hBehaviour(0, this._width - this._dividerWidth);

	jsgtk.Event.addDomListener(this._drawer, "click", function(e){
        self._isDrawed = false;
        if(self._isDrawedToLeft){
            self._child1.show();
            self._paned1.appendChild(self._child1.getNode());
            self.restoreStatus(self._child1, true);
        }else{
            self._child2.show();            
            self._paned2.appendChild(self._child2.getNode());
            self.restoreStatus(self._child2, true);
        }
        
        jsgtk.Dom.hideElement(self._drawer);
	});
	
};

gtk.HPaned.prototype = new gtk.Paned();
gtk.HPaned.prototype.constructor = gtk.HPaned;

/**
 * set position of the left paned x pixel from the left
 * @param {integer} position number of pixel from the left
 */
gtk.HPaned.prototype.setPosition = function(position){
    var widget1Width = this._child1.getMinWidthComposite();
    this._leftAnchored = (widget1Width < position)? position : widget1Width;
};

/**
 * anble drawer to the right. When user click on the drawer, the right paned will be shrunk 
 */
gtk.HPaned.prototype.enableDrawerRight = function(){
    this._enableDrawer(this._child2, "gtk_enabled_drawer_right");
	this._isDrawerRightEnabled = true;
};

/**
 * anble drawer to the right. When user click on the drawer, the left paned will be shrunk
 */
gtk.HPaned.prototype.enableDrawerLeft = function(){
    this._enableDrawer(this._child1, "gtk_enabled_drawer_left");
    this._isDrawerLeftEnabled = true;
};

/** @private */
gtk.HPaned.prototype._enableDrawer = function(widget, className){
    var self = this;
    //this._dividerWidth = this._drawerWidth;
    this._domDivider.className += " gtk_dom_divider_drawer";
    this._domDivider.style.width = 5 + "px";
    
   	this._dom.className = className;
    
    var jsgtkEvent = jsgtk.Event;
    /** @private */
    this._drawerEventId = jsgtkEvent.addDomListener(this._ghostDivider, "click", function(e){
        self._isDrawed = true;
        self.autoResize(widget, true);
        jsgtkEvent.stopBubble(e);
    });
    
    /** @private */
    jsgtkEvent.addDomListener(this._dom, "click", function(e){
        self._isDrawed = true;
        self.autoResize(widget, true);
        jsgtkEvent.stopBubble(e);
    });
    
    this.resize();
    
    /** @private */
    jsgtkEvent.addDomListener(this._ghostDivider, "mouseover", function(e){ 
        if(self._isDragged){
            jsgtk.Event.addDomListener(
                self._drawerEventId["obj"],
                self._drawerEventId["type"],
                self._drawerEventId["callback"]
            );   
        }
        jsgtkEvent.stopBubble(e);
    });
};

/** @private */
gtk.HPaned.prototype._resizeChild1 = function(){
	var width = this._width - this._drawerWidth;
	var paned1     = this._paned1;
	var paned2     = this._paned2;

    if(this._isDrawer){
        this._child1.resizeWidthComposite(this._width - this._drawerWidth);
	    this._child2.hide();
	    
        paned2.style.width = this._drawerWidth + "px";
        paned1.style.width = width + "px";
	    
        this._drawer.style.right = "0px";
        this._isDrawedToLeft = false;
        
        this._restoreButton.className += " draw_right";
        jsgtk.Dom.showElement(this._drawer);
    }else{
        this._child1.resizeWidthComposite(this._width);
	    this._child2.hide();
	    paned2.style.width = '0px';  
    } 
	this._isPaned2Restore = false;
};

/** @private */
gtk.HPaned.prototype._resizeChild2 = function(){
	var width = this._width - this._drawerWidth;
	var paned1     = this._paned1;
	var paned2     = this._paned2;

    if(this._isDrawer){
        this._child2.resizeWidthComposite(width);
        this._child1.hide();
        
        paned1.style.width = this._drawerWidth + "px";
        paned2.style.width = width + "px";
        
        this._isDrawedToLeft = true;
        
        this._restoreButton.className += " draw_left";
        jsgtk.Dom.showElement(this._drawer);   
    }else{
        this._child2.resizeWidthComposite(this._width);
        this._child1.hide();
        paned1.style.width = '0px';
    }
    this._isPaned1Restore = false;
};

/** 
 * shrinken the widget inside each paned
 * @param {gtk.Widget} widget will be shrinken along with its paned
 * @param {boolean} isDrawer if drawer enabled, will show the pooler after paned was shrunk
 */
gtk.HPaned.prototype.autoResize = function(widget, isDrawer){
	this._isDrawer = isDrawer === undefined ? false : isDrawer;

	if(widget._domObj.parentNode.className == "gtk_paned1" )
	    this._resizeChild2();

	if(widget._domObj.parentNode.className == "gtk_paned2" )
	    this._resizeChild1();
	
	jsgtk.Dom.hideElement(this._domDivider);
    jsgtk.Dom.hideElement(this._ghostDivider);

	if(!this._isPaned1Restore && !this._isPaned2Restore){
        jsgtk.Dom.hideElement(this._drawer);
		this.emit('childrendminimize', this);
	}	
};

/** @private */
gtk.HPaned.prototype._restoreWidget1 = function(){
    var drawerWidth = 0;
    if(this._isDrawerRightEnabled){
        if(this._isDrawed){
            jsgtk.Dom.showElement(this._drawer);
            drawerWidth = this._drawerWidth;   
        }
    }
    if(this._isPaned2Restore){
		this._paned1.style.width = this._paned1Width + "px";
		this._child1.resizeWidthComposite(this._paned1Width);

		this._paned2.style.width = this._paned2Width + "px";
		this._child2.resizeWidthComposite(this._paned2Width);

		this._domDivider.style.display = 'block';
		if(!this._isDrawerRightEnabled)this._ghostDivider.style.display = 'block';

	}else{
		this._paned1.style.width = this._width - drawerWidth + "px";
		this._child1.resizeWidthComposite(this._width - drawerWidth);
	}
	this._isPaned1Restore = true;
};

/** @private */
gtk.HPaned.prototype._restoreWidget2 = function(){

    var drawerWidth = 0;
    if(this._isDrawerLeftEnabled){
        if(this._isDrawed){
            jsgtk.Dom.showElement(this._drawer);
            drawerWidth = this._dividerWidth;   
        }
    }
    if(this._isPaned1Restore){
	    this._paned2.style.width = this._paned2Width + "px";
	    this._child2.resizeWidthComposite(this._paned2Width);

	    this._paned1.style.width = this._paned1Width + "px";
	    this._child1.resizeWidthComposite(this._paned1Width);

	    this._domDivider.style.display = 'block';
	    if(!this._isDrawerLeftEnabled)this._ghostDivider.style.display = 'block';
    }else{
	    this._paned2.style.width = this._width - drawerWidth + "px";
	    this._child2.resizeWidthComposite(this._width - drawerWidth);
    }
    this._isPaned2Restore = true;
};

/** 
 * Restore the shrunk widget and its paned
 * @param {gtk.Widget} widget will be shrinken along with its paned
 */
gtk.HPaned.prototype.restoreStatus = function(widget){
	if(widget === this._child1) this._restoreWidget1();
	if(widget === this._child2) this._restoreWidget2();
		
	if(this._isPaned1Restore || this._isPaned2Restore) {
		this.emit('childrestore', this);
	}
};

/** @private */
gtk.HPaned.prototype._resizePaned1Height = function(){
	if(this._height > this._child1.getHeight())
		this._child1.resizeHeightComposite(this._height);
	else this._resizeHeight(this._child1.getHeight());
};

/** @private */
gtk.HPaned.prototype._resizePaned1Width = function(isInit){

	if(isInit) this._initModifyPaned1Width();
	else this._modifyPaned1Width();	

	this._resizeWidth(this._paned1Width, this._paned2Width);
	this._leftMinX = this._child1.getMinWidthComposite();

	this._hBehaviour( this._leftMinX, this._width - this._dividerWidth);
};

/** @private */
gtk.HPaned.prototype._resizePaned2Height = function(){
	if(this._height > this._child2.getHeight())
	    this._child2.resizeHeightComposite(this._height);
	else{
		this._resizeHeight(this._child2.getHeight());
		this._child1.resizeHeightComposite(this._child2.getHeight());
	}
};

/** @private */
gtk.HPaned.prototype._resizePaned2Width = function(isInit){

	if(isInit) this._initModifyPaned2Width();
	else this._modifyPaned2Width();

	this._resizeWidth(this._paned1Width, this._paned2Width);
	this._rightMinX = this._child2.getMinWidthComposite();
	var maxX = this._width - this._dividerWidth - this._rightMinX;

	this._hBehaviour(this._child1.getMinWidthComposite(), maxX);
};

/** @private */
gtk.HPaned.prototype._initModifyPaned1Width = function(){
	if(this._paned1Width > this._child1.getWidth())
		this._child1.resize(this._paned1Width, this._child1.getHeight());
	else{
		this._paned1Width = this._child1.getWidth();
	}
};

/** @private */
gtk.HPaned.prototype._initModifyPaned2Width = function(){
	if(this._paned2Width > this._child2.getWidth()){
	    this._child2.resizeWidthComposite(this._paned2Width);
	}
	else{
		this._paned2Width = this._child2.getWidth();		
	}
};

/** @private */
gtk.HPaned.prototype._modifyPaned1Width = function(width){
    if(typeof width != 'undefined') this._child1.resizeWidthComposite(width);	
	else this._child1.resizeWidthComposite(this._paned1Width);	
};

/** @private */
gtk.HPaned.prototype._modifyPaned2Width = function(width){
    if(typeof width != 'undefined') this._child2.resizeWidthComposite(width);
	else{
	    this._child2.resizeWidthComposite(this._paned2Width);
    	this._paned2Width = this._child2.getWidth();   
	}
};

/** @private */
/** @ignore */
gtk.HPaned.prototype._hBehaviour = function(minX, maxX){
    
	var self = this;
	
	/** @ignore */
	jsgtk.Util.DomDraggableAdapter.hDragInit(this._ghostDivider, minX, maxX);	
    
	/** @ignore */
    this._ghostDivider.onDragStart = function(){
        self._isDragged = false;
        this.style.background = "#DFE8F6";
        this.style.opacity = "0.9";
        if(this.firstChild) this.removeChild(this.firstChild);
    };

	/** @ignore */
    this._ghostDivider.onDrag = function(){
        if(self._drawerEventId) jsgtk.Event.removeListener(self._drawerEventId);
    }; 
    
	/** @ignore */
	this._ghostDivider.onDragEnd = function(x, y){
		var width1 = x;
		var width2 = self._width - width1 - self._dividerWidth;

        self.setPosition(width1);
		self._resizePaned12Dom(width1, width2);

		self._child1.resizeWidthComposite(width1);
		self._child2.resizeWidthComposite(width2);
		
		self._isDragged = true;
		
        this.style.background = "";
        this.style.opacity = "0.9"; 
	};
};

/** @private */
gtk.HPaned.prototype._resizePaned12Dom = function(width1, width2){
	this._paned1Width = width1;
	this._paned2Width = width2;
	this._paned1.style.width = width1 + 'px';
	this._paned2.style.width = width2 + 'px';
};

/** @private */
gtk.HPaned.prototype._resizeWidth = function(paned1Width, paned2Width){
	this._width = paned1Width + paned2Width + this._dividerWidth;
	this._domObj.style.width = this._width + 1 + 'px';
		this._paned1.style.width = paned1Width + 'px';
		this._ghostDivider.style.left = paned1Width + 'px';
		this._paned2.style.width = paned2Width + 'px';
};

/** @private */
gtk.HPaned.prototype._resizeHeight = function(height){
	this._height = height;
	this._domObj.style.height = this._height + 'px';
		this._paned1.style.height = this._height + 'px';
		this._domDivider.style.height = this._height + 'px';
		this._ghostDivider.style.height = this._height + 'px';
		this._paned2.style.height = this._height + 'px';
		
		this._drawer.style.height = this._height + "px";
};

/** @private */
gtk.HPaned.prototype.getMinWidthComposite = function(){
	var widget1Width = this._child1.getMinWidthComposite();
	var widget2Width = this._child2.getMinWidthComposite();
	var minWidth = widget1Width + widget2Width + this._dividerWidth;
	minWidth = minWidth > this._minWidth ? minWidth : this._minWidth;
	
	return minWidth;
};

/** @private */
gtk.HPaned.prototype.getMinHeightComposite = function(){
	var widget1Height = this._child1.getMinHeightComposite();
	var widget2Height = this._child2.getMinHeightComposite();
	var minHeight = this._minHeight > widget1Height ? this._minHeight : widget1Height;
	minHeight = minHeight > widget2Height ? minHeight : widget2Height;

	return minHeight;
};

/** @private */
gtk.HPaned.prototype.resizeWidthComposite = function(width){

	var minWidthComposite = this.getMinWidthComposite();
	var minWidth = width > minWidthComposite ? width : minWidthComposite;
	
	var offsetWidth = (minWidth - this.getWidth()) / 2 ;
	
	this.setWidth(minWidth);
	
	var width1 = this._paned1Width + offsetWidth;
	var width2 = this._paned2Width + offsetWidth;
 
	if(this._isDrawedToLeft){
	    this._modifyPaned2Width(minWidth - this._drawerWidth);
	    this._resizeWidth(0, minWidth);  
	}
	else{
	    this._modifyPaned1Width(minWidth - this._drawerWidth);
        this._resizeWidth(minWidth, 0);
	}
    
	if(
		this._paned2Width === this._child2.getMinWidthComposite() || 
		this._paned1Width === this._child1.getMinWidthComposite() ||
		width1			   <  this._child1.getMinWidthComposite() ||
		width2			   <  this._child2.getMinWidthComposite()
 	){
		if(this._paned2Width === this._child2.getMinWidthComposite() ||
		   width2			  < this._child2.getMinWidthComposite()
		){
		    this._paned1Width = (this._paned1Width + offsetWidth * 2) - 1;
		    this._paned2Width = (this._child2.getMinWidthComposite()) - 1;   
		}
		if(this._paned1Width === this._child1.getMinWidthComposite() ||
		   width1			  <  this._child1.getMinWidthComposite()		
		){
		    this._paned2Width = (this._paned2Width + offsetWidth * 2) - 1;
		    this._paned1Width = (this._child1.getMinWidthComposite()) - 1; 
		}
	}else{
		this._paned1Width = this._paned1Width + offsetWidth - 1;
		this._paned2Width = this._paned2Width + offsetWidth - 1;
	}
	
	if(this._leftAnchored !== 0){
	    var widget1Width = this._child1.getMinWidthComposite();
	    var widget2Width = this._child2.getMinWidthComposite();
	    var width = this._leftAnchored + widget2Width;
	    this._paned1Width = width < minWidth ? this._leftAnchored : minWidth - widget2Width;
		this._paned2Width = minWidth - this._paned1Width;
	}
	
    this._paned1Width -= this._dividerWidth / 2;
    this._paned2Width -= this._dividerWidth / 2;
	
	if(this._isPaned1Restore && this._isPaned2Restore) {
        this.resize();
	}
};

/** @private */
gtk.HPaned.prototype.resizeHeightComposite = function(height){
	var minHeightComposite = this.getMinHeightComposite();
	var minHeight = height > minHeightComposite ? height : minHeightComposite;

	this.setHeight(minHeight);
			
	this._child1.resizeHeightComposite(minHeight);
	this._child2.resizeHeightComposite(minHeight);
	
    if(this._isDrawedToLeft) this._resizePaned1Height();
    else this._resizePaned2Height();
    
	if(this._isPaned1Restore && this._isPaned2Restore) {
        this.resize();
	}
};
