/**
 * @constructor
 * @base gtk.Widget
 */
gtk.Image = function(imagePath, imageWidth, imageHeight) {
	gtk.Widget.apply(this);
	var self = this;

	/** 'clicked' signal */
	this.IMAGE_CLICKED = "clicked";
	/** 'mouseover' signal */
	this.IMAGE_MOUSEOVER = "mouseover";
	/** 'mouseout' signal */
	this.IMAGE_MOUSEOUT = "mouseout";
	/** 'closed' signal */
	this.IMAGE_CLOSED = "closed";

	this._size         = null;
	this._dom          = null;
	this._defaultImage = "images/icons/ajax_loader.gif";

	this._width        = this._minWidth = imageWidth || 16;
	this._height       = this._minHeight = imageHeight || 16;

    this._attribute;
	
	/** @private */
	this._render = function(){
		this._domObj.className += " gtk_image";
			this.header = jsgtk.Dom.create({"append": this._domObj, "tag": "div", "class": "gtk_image_header"});
				this.headerDom = jsgtk.Dom.create({"append": this.header, "tag": "div", "class": "gtk_image_header_dom"});		
					this.closeDom = jsgtk.Dom.create({"append":this.headerDom, "tag": "div","className": "gtk_image_header_close_button"});

			var table = jsgtk.Dom.create({append: this._domObj, tag: "table", "className": "gtk_image_table"});
				var tr = jsgtk.Dom.create({append: table, tag: "tr"});
					var td = jsgtk.Dom.create({append: tr, tag: "td", "className": "gtk_image_table_td"});
						this._dom = jsgtk.Dom.create({append: td, tag: "img", src: this._defaultImage});
        
            this._disableLayer = jsgtk.Dom.create({append: this._domObj, tag: "div", 'class': "gtk_image_div_disable"});

		this._invisibleOverlay = jsgtk.Dom.create({"append": this._domObj, "tag": "div", "className": "gtk_image_invisible_overlay"});		

		if(imagePath !== undefined){
		    this.setFromFile(imagePath, imageWidth, imageHeight);
		}
	};

	this._render();
	
	jsgtk.Event.addDomListener(this._dom, "click", function(){
		self.emit("clicked");
	});

	jsgtk.Event.addDomListener(this._dom, "mouseover", function(){
		self.emit("mouseover");
	});

	jsgtk.Event.addDomListener(this._dom, "mouseout", function(){
		self.emit("mouseout");
	});
};

gtk.Image.prototype = new gtk.Widget();
gtk.Image.prototype.constructor = gtk.Image;
/**
* Set Image src from given file
* @param {String} filename which is assigned when we call the method.
*/	
gtk.Image.prototype.setFromFile = function(fileName, imageWidth, imageHeight){
    var file = fileName || this._defaultImage;
	this._dom.src = file;
    
    this.resetMinWidth(imageWidth || this._minWidth);
    this.resetMinHeight(imageHeight || this._minHeight);

    this.resizeWidthComposite(this._minWidth);
    this.resizeHeightComposite(this._minHeight);
};
/**
* when mouseover the image, the frame will be visible
*/	
gtk.Image.prototype.enableDelete = function(){
	gtk.ImageAnimator.init(this);
};
/** @private */
gtk.Image.prototype.resize = function(width, height){
   this._resizeWidth(width, height);
	this._resizeHeight(width, height);
};

/** @private */
gtk.Image.prototype.resetWidth = function(width){
	this._dom.setAttribute("width", width || this._minWidth);
	this.resetMinWidth(width || this._minWidth);
};

/** @private */
gtk.Image.prototype.resetHeight = function(height){
	this._dom.setAttribute("height", height || this._minHeight);
	this.resetMinHeight(height || this._minHeight);
};

/** 
* reset the size of image
* @param {number} width
* @param {number} height
*/
gtk.Image.prototype.resetSize = function(width, height){
	 if(width !== null) this.resetWidth(width);
	 if(height !== null) this.resetHeight(height);
};

/** 
* reset to the defualt size
*/
gtk.Image.prototype.resetDefaultSize = function(){
	var actualImageSize = this._getActualImageSize();
	this.resetSize(actualImageSize[0], actualImageSize[1]);
};
/** @private */
gtk.Image.prototype._getActualImageSize = function(){
	var imageNode = this.getChildNode();
	var imageObj = new Image();
		 imageObj.src = imageNode.src;
	return [imageObj.width, imageObj.height];
};
/** @private */
gtk.Image.prototype._resizeWidth = function(width){
	this._width = width;
    this._domObj.style.width = width + "px";
		this._dom.width = this._minWidth;
};
/** @private */
gtk.Image.prototype._resizeHeight = function(height){
	this._height = height;
    this._domObj.style.height = height + "px";
		this._dom.height = this._minHeight;
};
/**
 * Set attributes
 * @param {array} array of attributes object
 */
gtk.Image.prototype.setAttribute = function(attribute){
	this._attribute = attribute;
	this._applyAttribute();
};
/** 
 * Return actual image dom
 * @return {object} dom
 */
gtk.Image.prototype.getChildNode = function(){
	return this._dom;
};
/**
 * Assign property disabled to image
 */
gtk.Image.prototype.disable = function(){
    jsgtk.Dom.showElement(this._disableLayer);
};

/**
 * Assign property enabled to image
 */
gtk.Image.prototype.enable = function(){
    jsgtk.Dom.hideElement(this._disableLayer);
};

/** @private */
gtk.Image.prototype._applyAttribute = function(){
	for(var el in this._attribute)
		this._dom.style[el] = this._attribute[el];
};

gobject.signalNew('closed', gtk.Image, [], null, null);
gobject.signalNew('clicked', gtk.Image, [], null, null);
gobject.signalNew('mouseout', gtk.Image, [], null, null);
gobject.signalNew('mouseover', gtk.Image, [], null, null);

/**@constructor*/
gtk.ImageAnimator = {
	_timeoutId1: null,
	_timeoutId2: null,
	_isHeaderActive: false,
	_isImageMousovered: false,
	_imageObj: null,
	init: function(imageObj){
		this._imageObj = imageObj;

		jsgtk.Dom.showElement(this._imageObj._invisibleOverlay);

		var self = this;

		jsgtk.Event.addDomListener(this._imageObj._invisibleOverlay, "mouseover", function(e){
			self._handleImageMouseovered();
			jsgtk.Event.stopBubble(e);
		});

		jsgtk.Event.addDomListener(this._imageObj._invisibleOverlay, "mouseout", function(e){
			self._handleImageMouseOut();
			jsgtk.Event.stopBubble(e);
		});

		jsgtk.Event.addDomListener(this._imageObj.header, "mouseover", function(e){
			clearTimeout(self._timeoutId2);
			jsgtk.Event.stopBubble(e)
		});

		jsgtk.Event.addDomListener(this._imageObj.header, "mouseout", function(e){
			self._handleImageMouseOut();
			jsgtk.Event.stopBubble(e)
		});

		jsgtk.Event.addDomListener(this._imageObj._invisibleOverlay, "click", function(){
			self._imageObj.emit("clicked");
		});

		jsgtk.Event.addDomListener(this._imageObj.closeDom, "click", function(){
			self._imageObj.emit("closed");
		});
	},
	_handleImageMouseovered: function(){
		clearTimeout(this._timeoutId2);
		if(this._isHeaderActive) return;
		var self = this;
		self._imageObj._domObj.className += " gtk_image_enable_delete";
		this._timeoutId1 = window.setTimeout(function(){
			jsgtk.Dom.showElement(self._imageObj.header);
			self._imageObj.headerDom.style.top = 18 + "px";
			var i = 18;
			var intervalId = window.setInterval(function(){
				i -= 4;
				self._imageObj.headerDom.style.top = i + "px";
				if(i < 0){
					self._imageObj.headerDom.style.top = 0 + "px";

					self._isHeaderActive = true;
					self._isImageMousovered = true;

	 				clearInterval(intervalId);
				}
			}, 40);
		}, 200);
	},
	_handleImageMouseOut: function(){
		var self = this;
		clearTimeout(this._timeoutId1);
		self._timeoutId2 = window.setTimeout(function(){
			var i = 0;
			var intervalId = window.setInterval(function(){
				i += 4;
				self._imageObj.headerDom.style.top = i + "px";
				if(i >18){
					self._imageObj.headerDom.style.top = 18 + "px";
					jsgtk.Dom.hideElement(self._imageObj.header);

					self._isHeaderActive = false;
					self._isImageMousovered = false;

					self._imageObj._domObj.className = "gtk_image";
					clearInterval(intervalId);
				}
			}, 40);
		}, 50);	
	}
};
