/**
* @constructor
* @base gtk.Widget
*/
gtk.JustifiedLabel = function(labelValues) {
	gtk.Widget.apply(this);

	/** 'clicked' signals*/
 	this.JUSTIFIEDLABEL_CLICKED = "clicked";
	/** 'focused' signals*/
	this.JUSTIFIEDLABEL_FOCUSED = "focused";
	/** 'mouseover' signals*/
	this.JUSTIFIEDLABEL_MOUSEOVER = "mouseover";
	/** 'blurred' signals*/
	this.JUSTIFIEDLABEL_BLURRED = "blurred";
	/** 'mouseout' signals*/
	this.JUSTIFIEDLABEL_MOUSEOUT = "mouseout";
	/** 'refreshed' signals*/
	this.JUSTIFIEDLABEL_REFRESHED = "refreshed";
	/** 'cleared' signals*/
	this.JUSTIFIEDLABEL_CLEARED = "cleared";

	this._labelValues    = labelValues;
	this._width          = this._minWidth  = 16;
	this._height         = this._minHeight = 16;
	this._rowHeight      = 20;
	this._seperatorWidth = 80;
	this._applyYellow = false;

	this._render = function(){
		this._domObj.className   += " gtk_justified_label";
	};
	
	this._render();
};

gtk.JustifiedLabel.prototype = new gtk.Widget();
gtk.JustifiedLabel.prototype.constructor = gtk.JustifiedLabelgtk;

/** @private */
gtk.JustifiedLabel.prototype._clearChild = function(){
    this._domObj.innerHTML = "";  
};

/**
 * Set the width of dote separator
 * @param {number} width
 */
gtk.JustifiedLabel.prototype.setSeparatorWidth = function(width){
    this._seperatorWidth = width;
};

/**
 * Set the model
 * @param {gtk.LabelStore} model
 */
gtk.JustifiedLabel.prototype.setModel = function(model){
	this._clearChild();
	new gtk.JustifiedLabelController(this, model);
    this._resize();
	this.emit("refreshed");
};

/**
 * clear model and refresh
 */
gtk.JustifiedLabel.prototype.clearModel = function(){
    this._clearChild();
    this._width =  this._minWidth = 0;
    this._height = this._minHeight = 0;
    this.emit("cleared");
};

/** @private */
gtk.JustifiedLabel.prototype.constructLabelRows = function(data){
    this._labelValues = data;
    var domObj = this._domObj;
    this._rowNode;
    
    for(var i = 0, len = data.length; i < len; i++){
        var leftLabel = data[i]["leftLabel"];
        var rightLabel = data[i]["rightLabel"];
        var tabindex = data[i]["tabindex"];
        
        
        var rowNode = jsgtk.Dom.create({"append": domObj, "tag": "div", "class": "gtk_justified_label_row"});
        this._rowNode = rowNode;
        rowNode.style.height = this._rowHeight + "px";
            
            var dotsSeperator  = jsgtk.Dom.create({"append": rowNode, "tag": "div", "class": "gtk_justified_label_row_dot_separator_node"});
            var leftLabelNode  = jsgtk.Dom.create({"append": rowNode, "tag": "div", "class": "gtk_justified_label_row_left_node"});
                leftLabelNode.innerHTML = leftLabel.getText();
                
            var rightLabelNode = jsgtk.Dom.create({"append": rowNode, "tag": "div", "class": "gtk_justified_label_row_right_node"});
                rightLabelNode.innerHTML = rightLabel.getText();
                
        this._handleMouseOver(rowNode, rightLabel.getText());
        if(tabindex !=null)this._setTabIndex(rowNode,tabindex);
    } 
    this._resize();
};

/** @private */
gtk.JustifiedLabel.prototype._setTabIndex = function(row, index){
    row.tabIndex = index;
};

/** @private */
gtk.JustifiedLabel.prototype._handleMouseOver = function(rowNode, rightLabel){
    var self = this;
    var jsgtkEvent = jsgtk.Event;
    
    jsgtkEvent.addDomListener(rowNode, "mouseover", function(e){
        self.emit("mouseover", rightLabel);
        self._setYellowBorderLeft(rowNode);
        jsgtkEvent.stopBubble(e);
    });
    
    jsgtkEvent.addDomListener(rowNode, "mouseout", function(){
    	  self.emit("mouseout", rightLabel);
        self._resetYellowBorderLeft(rowNode);
    });
           
    jsgtkEvent.addDomListener(rowNode, "click", function(e){
        self.emit("clicked", rightLabel);
        jsgtkEvent.stopBubble(e);
    });
    
    jsgtkEvent.addDomListener(rowNode, "focus", function(e){
        self.emit("focused", rightLabel);
        self._setYellowBorderLeft(rowNode);
        jsgtkEvent.stopBubble(e);
    });
    
    jsgtkEvent.addDomListener(rowNode, "blur", function(){
        self._resetYellowBorderLeft(rowNode);
        self.emit("blurred", rightLabel);
    });

};

/** @private */
gtk.JustifiedLabel.prototype._getMaxRowWidth = function(){
    var labelValues = this._labelValues;
    var maxWidth = labelValues[0]["leftLabel"].getMinWidthComposite() + 
                   this._seperatorWidth + 
                   labelValues[0]["rightLabel"].getMinWidthComposite();
                   
    for(var i = 1, len = this._labelValues.length; i < len; i++){
        var width = labelValues[i]["leftLabel"].getMinWidthComposite() + 
                    this._seperatorWidth + 
                    labelValues[i]["rightLabel"].getMinWidthComposite();
        maxWidth = width > maxWidth ? width : maxWidth;
    }
    return maxWidth;
};

/** @private */
gtk.JustifiedLabel.prototype._resizeWidth = function(width){
    this._minWidth = this._width = width;
    this._domObj.style.width = width + "px";
};

/** @private */
gtk.JustifiedLabel.prototype._setYellowBorderLeft = function(rowNode){
    rowNode.style.borderLeft = "4px solid #FFCC00";
    rowNode.childNodes[0].style.marginLeft = "8px";
    rowNode.childNodes[1].style.marginLeft = "8px";
};

/** @private */
gtk.JustifiedLabel.prototype._resetYellowBorderLeft = function(rowNode){
    rowNode.style.borderLeft = "0";
    rowNode.childNodes[0].style.marginLeft = "0";
    rowNode.childNodes[1].style.marginLeft = "0";
};

/** @private */
gtk.JustifiedLabel.prototype._resizeHeight = function(height){
    this._minHeight = this._height = height;
    this._domObj.style.height = height + "px";
};

/** @private */
gtk.JustifiedLabel.prototype._resize = function(){
    var width  = this._getMaxRowWidth();
    this._resizeWidth(width);
    
    var height = this._labelValues.length * this._rowHeight;
    this._resizeHeight(height);
};

/** @private */
gtk.JustifiedLabel.prototype.getMinWidthComposite = function(){
    return this._minWidth;  
};

/** @private */
gtk.JustifiedLabel.prototype.getMinHeightComposite = function(){
    return this._minHeight;  
};

/**
 * Set attributes
 * @param {array} array of attributes object style
 */
gtk.JustifiedLabel.prototype.setAttribute = function(attribute){
	this._attribute = attribute;
	this._applyAttribute();
};

/** @private */
gtk.JustifiedLabel.prototype._applyAttribute = function(){
    var childLength = this._domObj.childNodes.length;
    
	for(el in this._attribute){
	    if(el == "backgroundColor"){
	        for(var i = 0; i < childLength; i++){
	            this._domObj.style[el] = this._attribute[el];
                this._domObj.childNodes[i].childNodes[1].style[el] = this._attribute[el];
                this._domObj.childNodes[i].childNodes[2].style[el] = this._attribute[el];
            }
	    }else{
            this._domObj.style[el] = this._attribute[el];  
        }         
	}
};

/** @private */
gtk.JustifiedLabel.prototype.setWidth = function(width){
    this._width = this._minWidth  = width;
    this._resizeWidth(width);
};

/** @private */
gtk.JustifiedLabel.prototype.resizeWidthComposite = function(width){
    this._width = width;
    this._resizeWidth(width);
};

/** @private */
gtk.JustifiedLabel.prototype.resetMinWidth = function(width){
    this._width = this._minWidth  = width;
    this._resizeWidth(width);
};

/** @private */
gtk.JustifiedLabel.prototype.resetMinHeight = function(height){
    this._height = this._minHeight  = height;
    this._resizeHeight(height);
};

/** @private */
gtk.JustifiedLabel.prototype.resizeHeightComposite = function(height){
    this._height = height;
    this._resizeHeight(height);
};

gobject.signalNew('clicked', gtk.JustifiedLabel, [], null, null);
gobject.signalNew('focused', gtk.JustifiedLabel, [], null, null);
gobject.signalNew('mouseover', gtk.JustifiedLabel, [], null, null);
gobject.signalNew('blurred', gtk.JustifiedLabel, [], null, null);
gobject.signalNew('mouseout', gtk.JustifiedLabel, [], null, null);
gobject.signalNew('refreshed', gtk.JustifiedLabel, [], null, null);
gobject.signalNew('cleared', gtk.JustifiedLabel, [], null, null);
