/**
 * @constructor
 * @base gtk.Widget
 * @param {gtk.ListStor} model
 * @link gtk.ListStor
 */
gtk.ListView = function(model){
	gtk.Widget.apply(this);

	/** 'rowselected' signal */
	this.LISTVIEW_ROWSELECTED = 'rowselected';
	/** 'rowclicked' signal */
	this.LISTVIEW_ROWCLICKED = 'rowclicked';
	/** 'rowover' signal */
	this.LISTVIEW_ROWOVER = 'rowover';
	/** 'rowout' signal */
	this.LISTVIEW_ROWOUT = 'rowout';
	/** 'rowdoubleclicked' signal */
	this.LISTVIEW_ROWDOUBLECLICKED = 'rowdoubleclicked';

	this._width            = this._minWidth = 400;
	this._height           = this._minHeight = 100;
    this._titleHeight      = 19;
    this._cellHeight       = 18;
	this._bodyHeight       = this._height - this._cellHeight;
	this._columnsWidth     = [];

	this._headerVisible    = true;
	this._leftPadding      = 4;
	this._model            = model || null;
	this._selectedRow      = null;
	this._emptyRows        = [];
	this._rowIndex         = -1;
	this._border           = 2;
	this._selectedRowPath  = "";
	this._hilightedRowNode    = null;
	this._parentWidth      = this._width;
	this._valueMember      = "";
	this._createTooltip    = false;
	this._selectedRowIndex = 0;
	
	this._render = function(){	
		this._domObj.className   += " gtk_listview";
            this._tableDom = jsgtk.Dom.create({"append": this._domObj, "tag": "TABLE", "BORDER": "0","CELLPADDING": "6", "class": "gtk_listview_table"});
                this._headerDom = jsgtk.Dom.create({"append": this._tableDom, "tag": "THEAD", "class": "gtk_listview_table_head"});
                this._bodyDom   = jsgtk.Dom.create({"append": this._tableDom, "tag": "TBODY", "class": "gtk_listview_table_body"});
                this._footerDom = jsgtk.Dom.create({"append": this._tableDom, "tag": "TFOOT", "class": "gtk_listview_table_footer"});
            
            this._vGhost = jsgtk.Dom.create({"append": this._domObj, "tag": "div", "class": "gtk_listview_vghost"});
            jsgtk.Dom.hideElement(this._vGhost);
            
            this._headerDomGhost = jsgtk.Dom.create({"append": this._domObj, "tag": "div", "class": "gtk_listview_table_head_ghost"});
            jsgtk.Dom.hideElement(this._headerDomGhost);
                    
        this._hiddenTextBox = jsgtk.Dom.create({"append":  document.body, "tag": "input", "type": "text",  "class": "gtk_listview_hiddent_textbox"});
    
        this._grayOutBackground   = jsgtk.Dom.create({"append": this._domObj, "tag": "div", "class": "gtk_gray_out"});
        this._loadingImgContainer = jsgtk.Dom.create({"append": this._domObj, "tag": "div", "class": "gtk_loading_bar_container_listview"});
            this._loadingImg      = jsgtk.Dom.create({"append": this._loadingImgContainer, "tag": "div", "class": "gtk_loading_bar_listview"});
	        
        jsgtk.Dom.hideElement(this._grayOutBackground);
        jsgtk.Dom.hideElement(this._loadingImgContainer);
	};
	this._render();
	
	this._model = model;
	new gtk.ListViewController(this, this._model);
	
	//enable table sortable
	this._enableSortable();
	
	this._keyControllEvent();
};

gtk.ListView.prototype = new gtk.Widget();
gtk.ListView.prototype.constructor = gtk.ListView;

/**
 * Show loading bar
 */
gtk.ListView.prototype.showLoading = function(){
    jsgtk.Dom.showElement(this._grayOutBackground);
    jsgtk.Dom.showElement(this._loadingImgContainer);
};


/**
 * Hide loading bar
 */
gtk.ListView.prototype.hideLoading = function(){
    jsgtk.Dom.hideElement(this._grayOutBackground);
    jsgtk.Dom.hideElement(this._loadingImgContainer);
};

/**
 * Return number of none empty row
 * @return {integer} 
 */
gtk.ListView.prototype.getNumberOfActiveRow = function(){
    var rowNodes = this._bodyDom.childNodes, count = 0;
    for(var i = 0, len = rowNodes.length; i < len; i++){
        var emptyRowId = new RegExp("^" + "empty_");
        if(!rowNodes[i].id.match(emptyRowId)) count++;
    }
    return count;
};

/**
 * set the next row of current selected row selected
 */
gtk.ListView.prototype.setNextSelectedRow = function(){
    var nextRow = this._hilightedRowNode ? this._hilightedRowNode.nextSibling : this._bodyDom.firstChild;
    this._rowIndex = this._getIndex(nextRow);
    this._hilightCell(this._rowIndex);
    this._emitSelectedRowSignalToController(nextRow.id);
};

/**
 * set selected row by index
 * @param {integer} index
 */
gtk.ListView.prototype.setSelectedRowByIndex = function(index){
    var childLen = this._model.children.length;
    this._rowIndex = index > childLen ? childLen-1 : index;
    this._hilightCell(this._rowIndex);
    var path = this._bodyDom.childNodes[this._rowIndex].id;
    this._emitSelectedRowSignalToController(path);
    this.emit("rowclicked", this.getSelectedRow());
};

/**
 * set selected row by key value
 * @param {any type} key
 * @param {any type} value
 */
gtk.ListView.prototype.setSelectedRowByValue = function(valueMember, value){
    this.emit("searchcontenttocontroller", valueMember, value);
    this.emit("rowclicked", this.getSelectedRow());
};

/**
 * filter row by key value
 * @param {any type} key
 * @param {any type} value
 */
gtk.ListView.prototype.filterRowsByValue = function(valueMember, value){
    this.emit("filterrowstocontroller", valueMember, value);
};

/** @private */
gtk.ListView.prototype.hilightSelectedRow = function(path){
    this._rowIndex = this._getRowIndexById(path);
    this._hilightCell(this._rowIndex);
    
    this._scrollTableBodyByIndex(this._rowIndex);
};

/**
 * set model for gtk.ListView
 * @param {gtk.ListStor} model
 */
gtk.ListView.prototype.setModel = function(model){
    this._model = model;
    this._refreshView();
    this.emit("refreshmodel", model);
    
    this.resizeHeightComposite(this._height + 2);
    this.resizeWidthComposite(this._parentWidth);    
};

/** @private */
gtk.ListView.prototype._getRowById = function(id){
    var rows = this._bodyDom.childNodes;
    for(var i = 0, len = rows.length; i < len; i++){
        if(rows[i].id === id) return rows[i];
    }
    return rows[0];
};

/** @private */
gtk.ListView.prototype._getRowIndexById = function(id){
    var rows = this._bodyDom.childNodes;
    for(var i = 0, len = rows.length; i < len; i++){
        if(rows[i].id === id) return i;
    }
    return 0;
};

/** @private */
gtk.ListView.prototype.getCellHeight = function(){
    return this._cellHeight;
};

/** 
 * enale row selection by using arrow key
 */
gtk.ListView.prototype.activateArrowKey = function(){
    this._hiddenTextBox.focus();
};

/** @private */
gtk.ListView.prototype._scrollTableBodyByIndex = function(rowIndex){
    var bodyHeight = this._height - this._cellHeight;
    var currentScrollTop = rowIndex * this._cellHeight;
    var offsetTop = currentScrollTop - bodyHeight;
    this._bodyDom.scrollTop += offsetTop + 36;
};

/** @private */
gtk.ListView.prototype._refreshView = function(){
    this._headerDom.innerHTML = "";
    this._bodyDom.innerHTML = "";
    this._hilightedRowNode = null;
};

/** @private */
gtk.ListView.prototype._hilightCell = function(index){
    if(this._hilightedRowNode) jsgtk.Dom.applyAttributes(this._hilightedRowNode, {"backgroundColor": ""});
    this._hilightedRowNode = this._bodyDom.childNodes[index];
    jsgtk.Dom.applyAttributes(this._hilightedRowNode, {"backgroundColor": "#FFFF66"});
    
    if(index !== this._selectedRowIndex){
        var selectedRowNode = this._bodyDom.childNodes[this._selectedRowIndex];
        jsgtk.Dom.applyAttributes(selectedRowNode, {"backgroundColor": ""});
    }
};

/** @private */
gtk.ListView.prototype._hilightCellByArrowKey = function(index){
    if(this._hilightedRowNode){
        jsgtk.Dom.applyAttributes(this._hilightedRowNode, {"backgroundColor": ""});
    } 
    this._hilightedRowNode = this._bodyDom.childNodes[index];
    jsgtk.Dom.applyAttributes(this._hilightedRowNode, {"backgroundColor": "#FFFFCC"});
    
    if(index !== this._selectedRowIndex){
        var selectedRowNode = this._bodyDom.childNodes[this._selectedRowIndex];
        jsgtk.Dom.applyAttributes(selectedRowNode, {"backgroundColor": "#FEFA89"});
    }
};

/** @private */
gtk.ListView.prototype._emitSelectedRowSignalToController = function(path){
    var emptyRowId = new RegExp("^" + "empty_");

    if(!path.match(emptyRowId) && this._selectedRowPath !== path) {
        this.emit("_rowselectedtocontroller", path);
        this._selectedRowPath = path;
    }
};

/** @private */
gtk.ListView.prototype._enableSortable = function(){
    jsgtk.Dom.SortTable(this._tableDom);
};

/** @private */
gtk.ListView.prototype.updateRowValue = function(path, value){
    var childNodes = this._getRowById(path).childNodes;
    for(var i = 0, len = childNodes.length; i < len; i++){
        childNodes[i].firstChild.innerHTML = value[i];
    }
};

/** @private */
gtk.ListView.prototype._constructRowColumns = function(tr, columns, columnsData){
    for(var j = 0, len = columns; j < len; j++){
        var td = jsgtk.Dom.create({"append": tr, "tag": "TD", "class": "gtk_listview_table_body_td"});
            var cell = jsgtk.Dom.create({"append": td, "tag": "DIV", "class": "gtk_listview_table_body_td_cell"});
            if(columnsData[j]) cell.innerHTML = columnsData[j];
            cell.style.height =  this._cellHeight + "px";
            if( j === len - 1) jsgtk.Dom.hideElement(cell);
    }
};

/** @private */
gtk.ListView.prototype._constructBodyRow = function(index, path, columns, columnsData){
    var tr = jsgtk.Dom.create({"tag": "TR", "class": "gtk_listview_table_body_tr", "id": path});

    if(index % 2 === 0) tr.className += " gtk_listview_table_body_tr_even";
    else tr.className += " gtk_listview_table_body_tr_odd";
    
    this._constructRowColumns(tr, columns, columnsData);
            
    return tr;
};

/** @private */
gtk.ListView.prototype._storeEmptyRows = function(tr, path){
    var emptyRowId = new RegExp("^" + "empty_");
    if(path.match(emptyRowId)) this._emptyRows.push(tr);
};

/** @private */
gtk.ListView.prototype._resetSelectedRowPath = function(){
    this._selectedRowPath = "";
    
};

/** @private */
gtk.ListView.prototype.renderEmptyRows = function(children, columns){
    this.constructBody(children, columns);
};

/** @private */
gtk.ListView.prototype.renderRows = function(children, columns){
    this.constructBody(children, columns);
};

/** @private */
gtk.ListView.prototype.constructBody = function(children, columns){
    this._resetSelectedRowPath();
	//construct body
	var docFragment = document.createDocumentFragment();
	for(var i = 0, len = children.length; i < len; i++){
	    var path = children[i]["path"];
	    
	    var tr = this._constructBodyRow(i, path, columns, children[i]["rows"]);
        docFragment.appendChild(tr);
        
	    this._handleRowEvent(tr, path);
	    
	    this._storeEmptyRows(tr, path);
	}
	
	this._bodyDom.appendChild(docFragment);
	this._enableSortable();
};

/** @private */
gtk.ListView.prototype.refreshRows = function(){
    this._bodyDom.innerHTML = "";
    this._hilightedRowNode = null;
};

/** @private */
gtk.ListView.prototype.clearEmptyRows = function(){
    for(var i = 0, len = this._emptyRows.length; i < len; i++){
        if(this._emptyRows[i].parentNode) this._bodyDom.removeChild(this._emptyRows[i]);
    }
    this._emptyRows = [];   
};

/** @private */
gtk.ListView.prototype.constructHeader = function(headers){
	//construct header
	var tr = jsgtk.Dom.create({"append": this._headerDom, "tag": "TR", "class": "gtk_listview_table_head_tr"});
	
	var docFragment = document.createDocumentFragment();
	
	for(var i = 0, len = headers.length; i < len; i++){
	    var td = jsgtk.Dom.create({"append": docFragment, "tag": "TD", "width": headers[i].width, "height": this._titleHeight, "class": "gtk_listview_table_head_td"});
	        var div = jsgtk.Dom.create({"append": td, "tag": "div", "width": headers[i].width, "class": "gtk_listview_table_head_td_div"});

                var textSpan = jsgtk.Dom.create({"append": div, "tag": "span", "text": headers[i].title, "style" : "float: left;", "class": "gtk_listview_table_head_td_div_span"});
	            var sortArrow = jsgtk.Dom.create({"append": div, "tag": "div", "style" : "float: left;", "class": "gtk_listview_table_head_td_div_sortarrow arrowdown"});
	            jsgtk.Dom.hideElement(sortArrow);
	            
	            var vbar = jsgtk.Dom.create({"append": div, "tag": "div", "class": "gtk_listview_table_head_td_div_vbar"});
	            vbar.style.left = headers[i].width - this._leftPadding + "px";
	            
	            this._handleColumnResizeEvent(i, vbar);
	            this._handleCellEvent(i, div, sortArrow);
	            
	            if( i === len - 1){
	                jsgtk.Dom.hideElement(vbar);
                    div.removeChild(sortArrow);
	            }
    }
    tr.appendChild(docFragment);
};

/** @private */
gtk.ListView.prototype._getIndex = function(row){
    var rows = this._bodyDom.childNodes;
    for(var i = 0, len = rows.length; i < len; i++){
        if(rows[i] === row) return i;
    }
};

/** @private */
gtk.ListView.prototype._increaseColumnWidth = function(offsetWidth){

    //the minWidth of the second last column
    var secondLastColumnMinColWidth = this._model.getColumnsWidth()[this._columnsWidth.length - 2];

    //the width to make second last column resize
    var width = this._columnsWidth[this._columnsWidth.length - 2] - offsetWidth;
    width = width < secondLastColumnMinColWidth ? secondLastColumnMinColWidth : width;

    if(width > secondLastColumnMinColWidth){
        this._columnsWidth[this._columnsWidth.length - 2] = width;
        this._resizeWidth(this._width);
    }else this._resizeWidth(this._width + offsetWidth);
};

/** @private */
gtk.ListView.prototype._decreaseColumnWidth = function(offsetWidth){
    this._columnsWidth[this._columnsWidth.length - 2] -= offsetWidth;
    this._resizeWidth(this._width);
};

/** @private */
gtk.ListView.prototype._isSecondLastColumn = function(index){
    return index === this._columnsWidth.length - 2;
};

/** @private */
gtk.ListView.prototype._updateColumWidth = function(index, offsetWidth){

    //update column to be resized
    this._columnsWidth[index] += offsetWidth;
    
    if(this._isSecondLastColumn()) {
        this._resizeWidth(this._width + offsetWidth);
        return;
    }
    
    if(offsetWidth > 0)
        this._increaseColumnWidth(offsetWidth);
    else
        this._decreaseColumnWidth(offsetWidth);
};

/** @private */
gtk.ListView.prototype._getGhostPosition = function(index){
    var left = 0;
    for(var i = 0; i <= index; i++){
        left += this._columnsWidth[i];
    }
    return left;
};

/** 
 * Return model of gtk.ListView
 * @return {gtk.ListStor} model
 */
gtk.ListView.prototype.getModel = function(){
	return this._model;
};

/** @private */
gtk.ListView.prototype.resetMinHeight = function(minHeight){
    this._minHeight = minHeight;
    this._resizeHeight(minHeight);
};

/** @private */
gtk.ListView.prototype.resize = function(width, height){
    this._resizeWidth(width);
    this._resizeHeight(height);
};

/** @private */
gtk.ListView.prototype.setMinColumnsWidth = function(columnsWidth, width){
    this._columnsWidth = [];

    for(var i = 0, len = columnsWidth.length; i < len; i++){
        this._columnsWidth.push(columnsWidth[i]);   
    }
    this._minWidth = width;
    this._width = this._minWidth;
};

/** @private */
gtk.ListView.prototype.getMinWidthComposite = function(){
    return this._minWidth;
};

/** @private */
gtk.ListView.prototype._resizeHeight = function(height){
    this._height = height;
    this._domObj.style.height = this._height + "px";
    this._vGhost.style.height = this._height + "px";
    
    this._tableDom.style.height = this._height + "px";
    this._bodyHeight = this._height - this._cellHeight;
    this._bodyDom.style.height = this._bodyHeight + "px";
};

/** @private */
gtk.ListView.prototype._resizeHeaderWidth = function(){
    var headerNodes = this._headerDom.firstChild.childNodes;
    for(var i = 0, len = this._columnsWidth.length; i < len; i++){
        headerNodes[i].width = this._columnsWidth[i];
        headerNodes[i].firstChild.style.width = this._columnsWidth[i] + "px";
        headerNodes[i].firstChild.lastChild.style.left = this._columnsWidth[i] - this._leftPadding + "px";
    }  
};

/** @private */
gtk.ListView.prototype._resizeWidth = function(width){
    this._width = width > this._width ? width : this._width;
    this._domObj.style.width = this._width - 2 + "px";
    this._resizeHeaderWidth();
};

/** @private */
gtk.ListView.prototype.setSelectedRow = function(data){
    this._selectedRow = data;
};

/**
 * Return the selected row's value
 * @return {object}
 */
gtk.ListView.prototype.getSelectedRow = function(data){
    return this._selectedRow;
};

/** @private */
gtk.ListView.prototype.setParentWidth = function(width){
    this._parentWidth = width;
};

/** @private */
gtk.ListView.prototype.resizeHeightComposite = function(height){
    if(height >= this._minHeight){
        this._resizeHeight(height - 2);
        this.emit("_renderemptyrows", height - this._cellHeight, this._cellHeight);
    }
};

/** @private */
gtk.ListView.prototype.resizeWidthComposite = function(width){
    if(width >= this._minWidth){

        var offsetWidth = Math.ceil((width - this._width) / (this._columnsWidth.length - 1));

        for(var i = this._columnsWidth.length - 1; i--;) this._columnsWidth[i] += offsetWidth;
        
        this._width = width;
        this._resizeWidth(width);
    }
};

/** @private */
gtk.ListView.prototype._handleRowEvent = function(row, path){
    var self = this;
    var jsgtkEvent = jsgtk.Event;
    jsgtkEvent.addDomListener(row, "click", function(e){
    
        //focus the hidden text box, so arrow key event can be captured
        self.activateArrowKey();
        self._emitSelectedRowSignalToController(path);
        self.hideTooltip();
        
        //get index of selected row
        self._rowIndex = self._getIndex(this);
        self._selectedRowIndex = self._rowIndex;
        
        if(!self._isEmptyRowClicked(self._rowIndex)){
            self.emit("rowclicked", self.getSelectedRow());
            self._hilightCell(self._rowIndex);
        }
        jsgtkEvent.stopBubble(e);
    });
   jsgtkEvent.addDomListener(row, "mouseover", function(e){
        var rowOver = e.target.parentNode.parentNode;
        self.emit("rowover",e,rowOver);
        jsgtkEvent.stopBubble(e);
    });
    jsgtkEvent.addDomListener(row, "mouseout", function(e){
        self.emit("rowout");
        jsgtkEvent.stopBubble(e);
    });
    
    jsgtkEvent.addDomListener(row, "dblclick", function(e){
        self._selectedRowIndex = self._rowIndex;
        self.hideTooltip();
        
        if(!self._isEmptyRowClicked(self._rowIndex)){
            self.emit("rowdoubleclicked", self.getSelectedRow());
        }
        jsgtkEvent.stopBubble(e);
    });
};

/** @private */
gtk.ListView.prototype._isEmptyRowClicked = function(selectedIndex){
    var isEmptyRow = false;
    if(this._model !== null){
        var LastChildIndex = this._model.children.length - 1;
        if(LastChildIndex < selectedIndex){
            isEmptyRow = true;
        }
    }
    return isEmptyRow;
};

/** @private */
gtk.ListView.prototype._showVGhost = function(index){
    var left = this._getGhostPosition(index);
    this._vGhost.style.left = left - 4 + "px";
    jsgtk.Dom.showElement(this._vGhost);
};

/** @private */
gtk.ListView.prototype._handleColumnResizeEvent = function(i, dom){
    var j = i;
    var self = this;
    var jsgtkDom =  jsgtk.Dom;
        
    jsgtk.Event.addDomListener(dom, "mouseover", function(){

        self._showVGhost(j);

        var minX = self._getGhostPosition(j - 1) + self._model.getColumnsWidth()[j];
        
        //init dragable vGhost
       jsgtk.Util.DomDraggableAdapter.hDragInit(self._vGhost, minX, null);	
        
        //the value will be changed after onDragEnd()
        var currentX;
        
        self._vGhost.onDragStart = function(x){
            self._vGhost.className = "gtk_listview_vghost_visible";
            currentX = x;
            
            self._headerDomGhost.style.cursor = "col-resize";
            jsgtkDom.showElement(self._headerDomGhost);
        };
        
        self._vGhost.onDragEnd = function(x, y){
            self._vGhost.className = "gtk_listview_vghost";
            var offsetWidth = x - currentX;
            
            //update the column
            self._updateColumWidth(j, offsetWidth);
            currentX = x;
            
            jsgtkDom.hideElement(self._vGhost);
            
            jsgtkDom.hideElement(self._headerDomGhost);
	    };
    });
};

/** @private */
gtk.ListView.prototype._handleCellEvent = function(i, cell, sortArrow){
    var index = i;
    var jsgtkEvent = jsgtk.Event;
    var jsgtkDom =  jsgtk.Dom;
    
    jsgtkEvent.addDomListener(cell, "mouseover", function(e){
       jsgtkDom.showElement(sortArrow);
        jsgtkEvent.stopBubble(e);
    });
    
    jsgtkEvent.addDomListener(cell, "mouseout", function(e){
        jsgtkDom.hideElement(sortArrow);
        jsgtkEvent.stopBubble(e);
    });
    
    var isDecending = true;
    jsgtkEvent.addDomListener(cell, "click", function(e){
        if(isDecending){
            sortArrow.id = "gtk_listview_table_head_td_div_sortarrow_arrowup";
            isDecending = false;
        }else{
            sortArrow.id = "gtk_listview_table_head_td_div_sortarrow_arrowdown";
            isDecending = true;
        }
    });
};

/** @private */
gtk.ListView.prototype._keyControllEvent = function(){
    var self = this;
    var jsgtkEvent = jsgtk.Event;
    
	jsgtkEvent.addDomListener(this._hiddenTextBox, "keyup", function(e){
	    var topIndex = self._model.children.length;
        if(e.keyCode === 13){
	        var path = self._bodyDom.childNodes[self._rowIndex].id;
	        self._emitSelectedRowSignalToController(path);
	        self._hilightCell(self._rowIndex);
	        self._selectedRowIndex = self._rowIndex;
	        
	        if(!self._isEmptyRowClicked(self._rowIndex)){
                self.emit("rowclicked", self.getSelectedRow());
            }	        
	    }
	    
	    if(e.keyCode === 40){
            if(self._rowIndex === topIndex - 1) self._rowIndex = -1;
            self._rowIndex++;
            
            if(!self._isEmptyRowClicked(self._rowIndex)){
                self._hilightCellByArrowKey(self._rowIndex);
            }
        }
        
        if(e.keyCode === 38){
            if(self._rowIndex <= 0) self._rowIndex = topIndex;
            self._rowIndex--;
            
            if(!self._isEmptyRowClicked(self._rowIndex)){
                self._hilightCellByArrowKey(self._rowIndex);
            }
        }
        self._scrollTableBodyByIndex(self._rowIndex);

        jsgtkEvent.stopBubble(e);
    });  
};

gobject.signalNew('refreshmodel', gtk.ListView, [], null, null);
gobject.signalNew('_rowselectedtocontroller', gtk.ListView, [], null, null);
gobject.signalNew('_renderemptyrows', gtk.ListView, [], null, null);
gobject.signalNew('searchcontenttocontroller', gtk.ListView, [], null, null);
gobject.signalNew('filterrowstocontroller', gtk.ListView, [], null, null);
gobject.signalNew('rowselected', gtk.ListView, [], null, null);
gobject.signalNew('rowclicked', gtk.ListView, [], null, null);
gobject.signalNew('rowover', gtk.ListView, [], null, null);
gobject.signalNew('rowout', gtk.ListView, [], null, null);
gobject.signalNew('rowdoubleclicked', gtk.ListView, [], null, null);
