gtk.ListViewController = function(view, data){
	this._view = view;
	this._model = data;
	this._emptyRows = [];
	var self = this;
	
	this._render = function(data){
	    this._model = data;
	    
        //set one last colum to be reserved for empty
        this._model.appendToHeader( {"title": "", "width": 15} );

        var headers = this._model.getHeaders();
        var children = this._model.getChildren();
        
        this._headerMashup = this._constructHeaderData(headers);
        var bodyMashup = this._mashUpData(children);
			
        this._view.constructHeader(this._headerMashup);
        
        this._view.renderRows(bodyMashup, this._headerMashup.length);

        var width  = this._model.getWidths();

        this._view.setMinColumnsWidth(this._model.getColumnsWidth(), width);

        this._view.resize(width, this._view.getHeight());
        
        //render empty rows
        this._view.connect("_renderemptyrows", function(height, cellHeight){
        
            self._view.clearEmptyRows();
            
            self._emptyRows = self._constructEmptyRowsData(self._model.getChildren().length, height, cellHeight);

            self._view.renderEmptyRows(self._emptyRows, self._headerMashup.length);
        });
	
	    this._model.connect("rowsupdated", function(rows){
            var bodyMashup = self._mashUpData(rows);
            
            self._view.renderRows(bodyMashup, self._headerMashup.length);
            
            if(bodyMashup.length >= self._emptyRows.length){
                
                //empty the rows
                self._view.clearEmptyRows();
                self._emptyRows = [];
            }else{
                
                self._view.clearEmptyRows();
                self._emptyRows.splice(0, bodyMashup.length);

                self._view.renderEmptyRows(self._emptyRows, self._headerMashup);
            }
        });
        
	};
	
	this._render(this._model);
	
	this._view.connect("refreshmodel", function(model){
	    self._render(model);
	});
	
	this._view.connect("_rowselectedtocontroller", function(path){
	    var model = self._model.getChildByPath(path);
	    self._view.setSelectedRow(model);
        self._view.emit("rowselected", model);
	});
	
	this._view.connect("searchcontenttocontroller", function(valueMember, value){
	    self._setHilightRowToView(valueMember, value);
	});
	
	this._view.connect("filterrowstocontroller", function(valueMember, value){
	     var filteredChildren = self._model.filter(value, valueMember);
         var bodyMashedup = self._mashUpData(filteredChildren);
         
         self._view.refreshRows();
         self._view.renderRows(bodyMashedup, self._headerMashup.length);

        self._emptyRows = self._constructEmptyRowsData(filteredChildren.length + 1, self._view.getHeight(), self._view.getCellHeight());
        self._view.renderEmptyRows(self._emptyRows, self._headerMashup.length);
        
	    self._setHilightRowToView(valueMember, value);
	});
	
	this._model.connect("rowupdated", function(path, value){
        self._view.updateRowValue(path, value);
	});
};

gtk.ListViewController.prototype._setHilightRowToView = function(valueMember, value){
    var model = this._model.search(valueMember, value);
    this._view.setSelectedRow(model);
    this._view.hilightSelectedRow(model.getPath());
};

gtk.ListViewController.prototype._constructHeaderData = function(headers){
    var headerMashup = [];
    for(var i = 0, len = headers.length; i < len; i++){
        headerMashup.push({"title": headers[i]["title"], "width": headers[i]["width"]});
    }
    return headerMashup;
};

gtk.ListViewController.prototype._mashUpData = function(children){
    var bodyMashup = [];
    for(var i = 0, len = children.length; i < len; i++){
        bodyMashup.push({"rows": children[i].getValue()["value"], "path": children[i].getPath()});
    }
    return bodyMashup;
};

gtk.ListViewController.prototype._constructEmptyRowsData = function(numberOfChildren, height, cellHeight){
    var rowsHeight = numberOfChildren * cellHeight;
    var emptyRows = [];
    if(rowsHeight < height){
        rows = Math.ceil( (height - rowsHeight) / cellHeight) - 1;
        for(var i = 0; i < rows; i++){
            emptyRows.push({"rows": " ", "path": "empty_" + new UUID()});
        }
    }
    return emptyRows;
};
