/**
 * @constructor
 * @base gtk.Widget
 */
gtk.MultiPanelContainer = function() {
	gtk.Widget.apply(this);

	this._width           = this._minWidth  = 20;
    this._height          = this._minHeight = 20;
    this._loadingImgWidth = 150;
    this._activeChild     = null;
    this._children        = [];
	
	/** @private */
	this._render = function(){
		var jsgtkDom = jsgtk.Dom;
		var domObj = this._domObj;
		domObj.className   += " gtk_MultiPanelContainer";
		    this._grayText = jsgtk.Dom.create({"append": domObj, "tag": "table", "class": "gtk_MultiPanelContainer_table"});
		        var tr = jsgtk.Dom.create({"append": this._grayText, "tag": "tr"});
                    var td = jsgtk.Dom.create({"append": tr, "tag": "td", "class": "gtk_MultiPanelContainer_tr_gray_text","text": "Text"});
            
            this._loadingDom = jsgtk.Dom.create({"append": domObj, "tag": "table", "class": "gtk_MultiPanelContainer_loading"});
		        var tr = jsgtk.Dom.create({"append": this._loadingDom, "tag": "tr"});
                    var td = jsgtk.Dom.create({"append": tr, "tag": "td", "class": "gtk_MultiPanelContainer_tr_loading"});
                        this._loadingImg = jsgtk.Dom.create({"append": td, "tag": "div", "class": "gtk_MultiPanelContainer_tr_loaing_img"});
            jsgtk.Dom.hideElement(this._loadingDom);
	};
	
    this._render();
};

gtk.MultiPanelContainer.prototype = new gtk.Widget();
gtk.MultiPanelContainer.prototype.constructor = gtk.MultiPanelContainer;

/**
 * show loading bar
 */
gtk.MultiPanelContainer.prototype.showLoading = function(){
    jsgtk.Dom.showElement(this._loadingDom);
    jsgtk.Dom.hideElement(this._grayText);
};

/**
 * hide loading bar
 */
gtk.MultiPanelContainer.prototype.hideLoading = function(){
    jsgtk.Dom.showElement(this._grayText);
    jsgtk.Dom.hideElement(this._loadingDom);
};

/** @private */
gtk.MultiPanelContainer.prototype._resetMinWidthMinHeight = function(childWidget){
    var childWidth = childWidget.getMinWidthComposite();
    this._minWidth = childWidth > this._minWidth ? childWidth : this._minWidth;
    
    var childHeight = childWidget.getMinHeightComposite();
    this._minHeight = childHeight > this._minHeight ? childHeight : this._minHeight;
};

/**
 * Set text inside gtk.MultiPanelContainer
 * @param {string} text
 */
gtk.MultiPanelContainer.prototype.setText = function(text){
    this._grayText.rows[0].firstChild.innerHTML = text;
};

/**
 * Return gtk.Widget whic is active inside gtk.MultiPanelContainer
 * @param {gtk.Widget || null} gtk.Widget
 */
gtk.MultiPanelContainer.prototype.getActiveChild = function(){
    var isActive = false;
    for(var i = 2, len = this._domObj.childNodes.length; i < len; i++){
        var node = this._domObj.childNodes[i].firstChild;
        if(node.style.display !== "none") isActive = true;
    }
    if(isActive) return this._activeChild;
    else return null;

}

/**
 * Remove gtk.Widget (child) from gtk.MultiPanelContainer
 * @param {gtk.Widget} childWiget
 */
gtk.MultiPanelContainer.prototype.removeChild = function(childWidget){
    for(var i = 0, len = this._children.length; i < len; i++){
        var child = this._children[i];
        if(child["childWidget"] === childWidget) {
            child["holder"].parentNode.removeChild(child["holder"]);
            this._children.splice(i, 1);
            return;
        }
    }
};

/**
 * Append gtk.Widget to gtk.MultiPanelContainer
 * @param {gtk.Widget} childWidget
 */
gtk.MultiPanelContainer.prototype.appendChild = function(childWidget){
    this._resetMinWidthMinHeight(childWidget);
    
    var holder = this._createChildHolder();
    holder.appendChild(childWidget.getNode());
    
    this.setActiveChild(childWidget);
    
    var childObject = {
        "holder": holder,
        "childWidget": childWidget
    };
    
    this._children.push(childObject);
    this._domObj.appendChild(holder);
    
    this.resizeWidthComposite(this._width);
    this.resizeHeightComposite(this._height);
    
    if(this._children.length > 1) this._minimizePreviousChild();
    
    var self = this;
    childWidget.connect("restored", function(panel){
        for(var i = self._children.length; i--;){
            var child = self._children[i];
            if(child["childWidget"] ===  panel){
                self.setActiveChild(panel);
                child["holder"].style.zIndex = 2;
                jsgtk.Dom.showElement(child["holder"]);
            }else{
                child["childWidget"].minimize();
                child["holder"].style.zIndex = 1;
            }
        }        
    });
    
    childWidget.connect("closed", function(panel){
        self.removeChild(panel)
    });
};

/**
 * Set the gtk.Widget (child) active
 * @param {gtk.Widget} childWidget
 */
gtk.MultiPanelContainer.prototype.setActiveChild = function(childWidget){
    this._activeChild = childWidget;
};

/** @private */
gtk.MultiPanelContainer.prototype._minimizePreviousChild = function(){
    this._children[this._children.length - 2]["childWidget"].minimize();
    jsgtk.Dom.hideElement(this._children[this._children.length - 2]["holder"]);
};

/** @private */
gtk.MultiPanelContainer.prototype._getIndexOfChild = function(childWidget){
    for(var i = 0, len = this._children.length; i < len; i++){
        var child = this._children[i];
        if(child["childWidget"] === childWidget) return i;   
    }
    return null;
};

/**
 * Replace existing childWidget with new childWidget
 * @param {gtk.Widget} newChildWidget
 * @param {gtk.Widget} oldChildWidget
 */
gtk.MultiPanelContainer.prototype.replaceChild = function(newChildWidget, oldChildWidget){
    var index = this._getIndexOfChild(oldChildWidget);
    if(!index) return;
    
    this._domObj.removeChild(this._children[index]["holder"]);
    
    this._children.splice(index, 1);
    this.appendChild(newChildWidget);
};

/**
 * Return children widget
 * @return {array} children widget
 */
gtk.MultiPanelContainer.prototype.getChildren = function(){
    var children = []
    for(var i = 0, len = this._children.length; i < len; i++){
        children.push(this._children[i]["childWidget"]);
    }
    return children;  
};

/** @private */
gtk.MultiPanelContainer.prototype._positionLoadingImg = function(width){
    this._loadingImg.style.left = width / 2 - this._loadingImgWidth / 2 + "px";
}

/** @private */
gtk.MultiPanelContainer.prototype._resizeChildrenWidth = function(width){
    for(var i = this._children.length; i--;){
        var child = this._children[i];
        child["childWidget"].resizeWidthComposite(width);
        child["holder"].style.width = width + "px";
    }
};

/** @private */
gtk.MultiPanelContainer.prototype._resizeChildrenHeight = function(height){
    for(var i = this._children.length; i--;){
        var child = this._children[i];
        child["childWidget"].resizeHeightComposite(height);
        child["holder"].style.height = height + "px";
    }
};

/** @private */
gtk.MultiPanelContainer.prototype._createChildHolder = function(){
    var holder = jsgtk.Dom.create({"tag": "div", "class": "gtk_multiPanelContainer_widget_holder"});
    return  holder;
};

/** @private */
gtk.MultiPanelContainer.prototype.resizeWidthComposite = function(width){
    var minWidth = this._minWidth < width ? width : this._minWidth;
    this._resizeWidth(minWidth);
    this._positionLoadingImg(minWidth);
    if(this._children.length > 0) this._resizeChildrenWidth(minWidth);
};

/** @private */
gtk.MultiPanelContainer.prototype.resizeHeightComposite = function(height){
    var minHeight = this._minHeight < height ? height : this._minHeight;
    this._resizeHeight(minHeight);
    if(this._children.length > 0) this._resizeChildrenHeight(minHeight);
};
