/**
 * @constructor
 * @base gtk.Widget
 */
gtk.NoteBook = function() {
	gtk.Widget.apply(this);
	
	/** 'tabchanged' signal */
	this.NOTEBOOK_TABCHANGED = "tabchanged";
	
	this._minWidth        = 200;
	this._minHeight       = 200;
	this._width           = this._minWidth;
	this._height          = this._minHeight;
	this._tabHeight       = 32;
    this._hBarHeight      = 3;
    this._loadingImgWidth = 16;
    this._extraWidthRight = 10;
    this._border          = 2;
    this._vspaceWidth     = 2;
    this._numberTabs      = 0;
	this._tabs = new gtk.Tabs(this._width, this._tabHeight, this._vspaceWidth);

    var contentsHeight = this._height - this._tabHeight - this._hBarHeight;
	this._contents = new gtk.Contents(this._width, contentsHeight);
	
	this._render = function(){
		this._domObj.className += " gtk_notebook";
		this._domObj.style.width = this._width + 'px';
		this._domObj.style.height = this._height + 'px';
		    var hBar = jsgtk.Dom.create({'tag' : 'div', 'className': 'gtk_hbar'});
		this._appendChild(this._contents, this._tabs, hBar);
	};
	this._initEvent();
	this._render();
};

gtk.NoteBook.prototype = new gtk.Widget();
gtk.NoteBook.prototype.constructor = gtk.NoteBook;

/** @private */
gtk.NoteBook.prototype._initEvent = function(){
    var self = this;
    this._tabs.connect('active', function(id){
		var content = self._contents.activateContent(id);
		self.emit("tabchanged", self.getActiveTab());
	});
    jsgtk.Event.addDomListener(this._domObj, 'keyup', function(e){
	    if(e.keyCode == gtk.RIGHT_ARROW_KEY){
		    var activeTabIndex = self._tabs.getIndex(self._tabs._activeTab);
            if(activeTabIndex < self.getNumberTabs()-1)self.setActiveTab(activeTabIndex+1);
	    }
	    else if(e.keyCode == gtk.LEFT_ARROW_KEY){
	       var activeTabIndex = self._tabs.getIndex(self._tabs._activeTab);
            if(activeTabIndex > 0)self.setActiveTab(activeTabIndex -1);
	    }
	});
	jsgtk.Event.addDomListener(this._domObj, 'focus', function(e){
	   self._tabs._setFocus(self._tabs._activeTab);
	});
	jsgtk.Event.addDomListener(this._domObj, 'blur', function(e){
	   self._unFocus(self._tabs._activeTab);
	});
};

/**
 * Turn border on || off
 * @param {boolean}
 */
gtk.NoteBook.prototype.setBorderVisible = function(visible){
    if(visible) this._contents.showBorder();
    else this._contents.hideBorder();
};


/**
 * Update the content of selected tab
 * @param {object} pageObj {"content": gtk.Widget, "padding": 5}
 */
gtk.NoteBook.prototype.updatePage = function(pageObj){
	var index = this._tabs.getIndex(pageObj['tab']);
	this._contents.updateContent(pageObj["content"], index);
	
	this._contents.activate(index);
	
	this.resizeWidthComposite(this._width);
	this.resizeHeightComposite(this._height);
};

/**
 * Appends a page to the notebook 
 * @param {object} tabObj {"tab": label1, "padding": 55}
 * @param {object} contentObj {"content": content2, "padding": 5}
 */
gtk.NoteBook.prototype.appendPage = function(tabObj, contentObj){
	contentObj["content"] = contentObj["content"] || new gtk.Blank();

    var id = new UUID();
	tabObj["id"] = id;
    contentObj["id"] = id;

	this._contents.addChild(contentObj);
	this._tabs.addTab(tabObj);
	
	this._tabs.activateDefault();
	this._contents.activateDefault();

	this.resizeWidthComposite(this._width);
	this.resizeHeightComposite(this._height);
	
    return {
        "tab": tabObj["tab"],
        "content": contentObj["content"],
		"isBlanked": contentObj.type === gtk.BLANK_WIDGET ? true : false
    };
};

/**
 * Insert a page to the notebook at index
 * @param {object} tabObj {"tab": label1, "padding": 55}
 * @param {object} contentObj {"content": content2, "padding": 5}
 * @param {integer} index
 */
gtk.NoteBook.prototype.insertAt = function(tabObj, contentObj,index){

	contentObj["content"] = contentObj["content"] || new gtk.Blank();

    var id = new UUID();
	tabObj["id"] = id;
    contentObj["id"] = id;
    
	this._contents.insertAt(contentObj, index);
	this._tabs.insertAt(tabObj, index);
	
	this.resizeWidthComposite(this._width);
	this.resizeHeightComposite(this._height);
	
   return {
        "tab": tabObj["tab"],
        "content": contentObj["content"],
		"isBlanked": contentObj.type === gtk.BLANK_WIDGET ? true : false
    };
};

/**
 * Remove tabe by index
 * @param {intger} index of tab to be removed
 */
gtk.NoteBook.prototype.removeTab = function(index){
    var len = this._tabs.getChildren().length;
    if(len == 1) jsgtk.Util.debug("The tab can not be removed");
    if(index < 0 || index >= len) jsgtk.Util.debug("The index out of range");

    this._tabs.removeTab(index);
    this._contents.removeChild(index);
    
    if(index === 0) this._activatePreviousTab(index + 1);
    if(this._tabs.getActiveTab() === this._tabs.getChildren()[index]) this._activatePreviousTab(index);
    else this._activateActiveTab();
};

/** @private */
gtk.NoteBook.prototype.setTabindex = function(index){
    this._tabs.getNode().tabIndex = index;
};

/**
 * Return number of tabs
 * @return {intger}
 */
gtk.NoteBook.prototype.getNumberTabs = function(){
    return this._tabs._tabs.length;
};

/** @private */
gtk.NoteBook.prototype.showLoading = function(activeTab){
    this._tabs.showLoading(activeTab["tab"]);
};

/** @private */
gtk.NoteBook.prototype.hideLoading = function(activeTab){
    this._tabs.hideLoading(activeTab["tab"]);
};

/** @private */
gtk.NoteBook.prototype._activateActiveTab = function(){
    this._tabs.activateByObject(this._tabs.getActiveTab());
    this._contents.activateByObject(this._contents.getActiveContent());
};

/** @private */
gtk.NoteBook.prototype._activatePreviousTab = function(index){
    this._tabs.activate(index - 1);
    this._contents.activate(index -1);
};

/**
 * activate the tab by tab
 * @param {gtk.Widget} tab
 */
gtk.NoteBook.prototype.setActiveTabByTab = function(tab){
    var index = this._tabs.getIndex(tab);
    this.setActiveTab(index);
};

/**
 * activate the tab by index
 * @param {integer} index
 */
gtk.NoteBook.prototype.setActiveTab = function(index){
    this._tabs.activate(index);
    this._contents.activate(index);
    this.emit("tabchanged", this.getActiveTab());
};

/** @private */
gtk.NoteBook.prototype.setFocusTab = function(index){
    this._tabs.setFocus(index);
};

/**
 * Return all tabs objects
 * @return {array} array of tabs of object
 */
gtk.NoteBook.prototype.getTabs = function(){
    var tabs = [];
    for(var i = 0, len = this._tabs.getChildren().length; i < len; i++){
        tabs.push({
            "tab": this._tabs.getChildren()[i]["tab"],
            "content": this._contents.getChildren()[i]["content"],
			"isBlanked": this._contents.getChildren()[i]["content"].type === gtk.BLANK_WIDGET ? true : false
        });
    }
    return tabs;
};

/**
 * Return active tab
 */
gtk.NoteBook.prototype.getActiveTab = function(){
    return {
        "tab": this._tabs.getActiveTab(),
        "content": this._contents.getActiveContent(),
		"isBlanked": this._contents.getActiveContent().type === gtk.BLANK_WIDGET ? true : false
    };
};

/** @private */
gtk.NoteBook.prototype._appendChild = function(contents, tabs, hBar){
	this._domObj.appendChild(tabs._domObj);
	this._domObj.appendChild(hBar);
	this._domObj.appendChild(contents._domObj);
};


/** @private */
gtk.NoteBook.prototype._resizeHeight = function(height){
    this._domObj.style.height = height - this._border + 'px';
	this._height = height;
};

/** @private */
gtk.NoteBook.prototype._resizeWidth = function(width){
	this._domObj.style.width = width - this._border + 'px';
	this._width = width;
};

gtk.NoteBook.prototype._getLoadingImgWidths = function(){
    var children = this._tabs.getChildren();
    var width = 0;
    for(var i = 0, len = children.length; i < len; i++){
        pageObj = children[i];
        width += pageObj["padding"] > this._loadingImgWidth ? pageObj["padding"] : this._loadingImgWidth;
    }
    return width;
};

/** @private */
gtk.NoteBook.prototype.getMinWidthComposite = function(){
	var minWidth = this._minWidth;

	var tabsWidth = this._tabs.getMinWidthComposite();
	var contentsWidth = this._contents.getMinWidthComposite();

	if(tabsWidth > minWidth) minWidth = tabsWidth;
	if(contentsWidth > minWidth) minWidth = contentsWidth;
    this._getLoadingImgWidths();
	return minWidth + this._extraWidthRight + this._getLoadingImgWidths();
};

/** @private */
gtk.NoteBook.prototype.getMinHeightComposite = function(){
	var minHeight = this._minHeight - this._tabs.getHeight();
	var contentsHeight = this._contents.getMinHeightComposite();

	if(contentsHeight > minHeight) minHeight = contentsHeight;

	return minHeight + this._tabs.getHeight();
};

/** @private */
gtk.NoteBook.prototype.resizeHeightComposite = function(height){
	var minHeightComposite = this.getMinHeightComposite();
	var minHeight = height > minHeightComposite ? height : minHeightComposite;
	this._contents.resizeHeightComposite(minHeight - this._tabs.getHeight() - this._border);

    this._resizeHeight(minHeight);
};

/** @private */
gtk.NoteBook.prototype.resizeWidthComposite = function(width){
	var minWidthComposite = this.getMinWidthComposite();
	var minWidth = width > minWidthComposite ? width : minWidthComposite;

	this._tabs.resizeWidthComposite(minWidth - 1);
	this._contents.resizeWidthComposite(minWidth - this._border);

    this._resizeWidth(minWidth);
};

gobject.signalNew('tabchanged', gtk.NoteBook, [], null, null);
