/**
 * PopupWindow is a class to create pop-up window to appear on the top most of the screen.
 * There are five types: Window Default, Window Confirm, Window Warning, Window Info, and Window Error or Critical
 * @param {String} title The header title of the pop-up window
 * @param {String} textMessage The body text/content of the pop-up window
 * @param {String} type It must be a constant variable to create pop-up window: 
 * <br>These are the constant values to be passed:
 * <br> - gtk.WINDOW_CONFIRM The flag constant value to define as Window Confirm
 * <br> - gtk.WINDOW_WARNING The flag constant value to define as Window Warning
 * <br> - gtk.WINDOW_INFO The flag constant value to define as Window Info
 * <br> - gtk.WINDOW_ERROR The flag constant value to define as Window Error
 * <br> - gtk.WINDOW_Default The flag constant value to define as Window Default
 * @constructor
 */
gtk.PopupWindow = function(title, textMessage, type){    
    switch (type){
		case gtk.WINDOW_CONFIRM: 
			return new gtk.PopupWindowConfirm(title, textMessage);
			break;
		case gtk.WINDOW_WARNING: 
			return new gtk.PopupWindowWarning(title, textMessage);
			break;
		case gtk.WINDOW_INFO: 
			return new gtk.PopupWindowInfo(title, textMessage);
			break;
		case gtk.WINDOW_ERROR: 
			return new gtk.PopupWindowError(title, textMessage);
			break;
		default:
			return new gtk.PopupWindowDefault(title, textMessage);
    }
};
