/**
 * PopupWindowConfirm is a class to create pop-up window type Confirmation
 * @param {String} title The header title of the pop-up window
 * @param {String} textMessage The body text/content of the pop-up window
 * @constructor
 * @base gtk.PopupWindowDefault
 * @extends gtk.PopupWindowDefault
 */
gtk.PopupWindowConfirm = function(title, textMessage){
    gtk.PopupWindowDefault.apply(this, [title, textMessage]);
    var self = this;
    this._iconWidth = 95;
    this._cancelBtnHandler = null;
    /** It allows user to connect to a registered signal event "cancelled" when click Cancel button */
    this.POPUPWINDOWCONFIRM_CANCELLED = "";
    
    this._render = function(){
        
        var imageDiv = jsgtk.Dom.create({"tag": "div", "className": "gtk_popup_window_confirm_image"});
        this._body.insertBefore(imageDiv,this._bodyDiv);
        this._bodyDiv.style.margin = "0px 0px 0px 66px";
        this._footerButtonCancel = jsgtk.Dom.create({"tag": "div", "className": "gtk_popup_window_footer_buttonConcel"});
    }; 
    this._render();
};

gtk.PopupWindowConfirm.prototype = new gtk.PopupWindowDefault();
gtk.PopupWindowConfirm.prototype.constructor = gtk.PopupWindowConfirm;

/**
 * A method to display the pop-up window after it is instantiated
 */
gtk.PopupWindowConfirm.prototype.draw = function(){
        this._drawOkButton();
        this._drawCancelButton();
        this._resizePopupWindow();
        this._setCenterScreen();
        this._disableBackground();
        this._initEvent();
        this._attachCancelBtnEvent();
        document.body.appendChild(this._domObj);
};

/** @private */
gtk.PopupWindowConfirm.prototype._attachCancelBtnEvent = function(){
    var self = this;     
    this._cancelBtnHandler = jsgtk.Event.addDomListener(this._footerButtonCancel, "click", function(){
		self.emit("cancelled");
        self.close();
    });
};

/** @private */
gtk.PopupWindowConfirm.prototype._drawCancelButton = function(){
        this._footerButtonCancel.innerHTML = "";
        var buttonCancel = new gtk.Button("Cancel");
            buttonCancel.resizeWidthComposite(75);
            buttonCancel.resizeHeightComposite(22);

        this._footerButtonCancel.appendChild(buttonCancel.getNode());
        this._footer.insertBefore(this._footerButtonCancel, this._footerButtonOk);
};

/**
 * A method to close/remove pop-up window
 */
gtk.PopupWindowConfirm.prototype.close = function(){
    if(!this._domObj.parentNode) return;
    
    this._domObj.parentNode.removeChild(this._domObj);
    this._disableDiv.parentNode.removeChild(this._disableDiv);
    this._detachOkBtnHandler();
    this._detachCancelBtnHandler();
    this._detachCloseBtnHandler();
};

/** @private */
gtk.PopupWindowConfirm.prototype._detachCancelBtnHandler = function(){   
    jsgtk.Event.removeListener(this._cancelBtnHandler);
    this._cancelBtnHandler = null;
};

gobject.signalNew("cancelled", gtk.PopupWindowConfirm, null, null, []);
