/**
 * PopupWindowDefault is a class to create pop-up window type Default
 * @param {String} title The header title of the pop-up window
 * @param {String} textMessage The body text/content of the pop-up window
 * @constructor
 * @base gtk.Widget
 * @extends gtk.Widget
 */
gtk.PopupWindowDefault = function(title, textMessage){
    gtk.Widget.apply(this);
    
    this._offsetX;
    this._offsetY;
    this._title     = title;
    this._bodyText  = textMessage;
    
    this._maxWidth  = 678;
    this._maxHeight = 600;
    this._width     = this._minWidth = 380;
    this._height    = this._minHeight = 130;
    this._iconWidth = 0;
    this._okBtnHandler = null;
    this._closeBtnHandler;

    /** It allows user to connect to a registered signal event "ok" when click OK button */
    this.POPUPWINDOWDEFAULT_OK = "";
    /** It allows user to connect to a registered signal event "closed" when click close button */
    this.POPUPWINDOWDEFAULT_CLOSED = "";
        
    this._render = function(){
        
        this._domObj.className += " gtk_popup_window";
        this._domObj.style.position = "absolute";
            this._header =  jsgtk.Dom.create({"append" : this._domObj, "tag": "div", "className": "gtk_popup_window_header"});
                this._titleDiv = jsgtk.Dom.create({"tag": "div", "className": "gtk_popup_window_title", "append": this._header});
                this._titleDiv.innerHTML = this._title;
                this._closeButton = jsgtk.Dom.create({"tag": "div", "className": "gtk_popup_window_button_close", "append": this._header});
            
            this._body = jsgtk.Dom.create({"append" : this._domObj, "tag": "div", "className": "gtk_popup_window_body"});
                this._bodyDiv = jsgtk.Dom.create({"tag": "div", "className": "gtk_popup_window_body_text", "append": this._body});
                    this._bodyDiv.innerHTML = this._bodyText;
            
            this._footer = jsgtk.Dom.create({"append": this._domObj, "tag": "div", "className": "gtk_popup_window_footer"});
            this._footerButtonOk = jsgtk.Dom.create({"tag": "div", "className": "gtk_popup_window_footer_buttonOk", "append": this._footer});
            
            this._titleDiv.style.width = this._width - 22 + "px";
    };
    
    this._render();
};

gtk.PopupWindowDefault.prototype = new gtk.Widget();
gtk.PopupWindowDefault.prototype.constructor = gtk.PopupWindowDefault;

/** 
 * A method to set title to the header of the pop-up window 
 * @param {String} title The title of the pop-up window to be set
 * type {String}
 */
gtk.PopupWindowDefault.prototype.setTitle = function(title){
    this._title = title;  
    this._resizePopupWindow();
};

/**
 * A method to set content to the body of the pop-up window
 * @param {String} textMessage The text content of the body of the pop-up window to be set
 * type {String}
 */
gtk.PopupWindowDefault.prototype.setBody = function(textMessage){
    this._bodyText = textMessage;
    this._bodyDiv.innerHTML = this._bodyText;
    this._resizePopupWindow();
};

/**
 * A method to set focus onto the pop-up window
 */
gtk.PopupWindowDefault.prototype.setFocus = function(){
    this._domObj.focus();
};

/**
 * A method to display the pop-up window after it is instantiated
 */
gtk.PopupWindowDefault.prototype.draw = function(){
        this._drawOkButton();
        this._resizePopupWindow();
        this._setCenterScreen();
        this._disableBackground();
        this._initEvent();
        document.body.appendChild(this._domObj);
};

/** @private */
gtk.PopupWindowDefault.prototype._drawOkButton = function(){
    this._footerButtonOk.innerHTML = "";
    var buttonOk = new gtk.Button("OK");
    buttonOk.resizeWidthComposite(75);
    buttonOk.resizeHeightComposite(22);
    this._footerButtonOk.appendChild(buttonOk.getNode());
};

/** @private */
gtk.PopupWindowDefault.prototype._calculateHeight = function(dom){
    var tempDom = dom.cloneNode(true);
    tempDom.style.display = "none";
    tempDom.style.position = "absolute";
  
    var height = jsgtk.Util.fullHeight(tempDom);
  
    return height;    
};

/** @private */
gtk.PopupWindowDefault.prototype._initEvent = function(){
    var self = this;
    var jsgtkEvent = jsgtk.Event;
       
    jsgtkEvent.addDomListener(this._header, "mouseover", function(e){
        self._enableDrag(); 
        jsgtkEvent.stopBubble(e);
    });
    
    jsgtkEvent.addDomListener(this._header, "mouseout", function(e){
        self._disableDrag();
    });
        
    this._closeBtnHandler = jsgtkEvent.addDomListener(this._closeButton, "mouseup", function(e){
        self.emit("closed");
        self.close();
    });
    
    this._okBtnHandler = jsgtkEvent.addDomListener(this._footerButtonOk, "mouseup", function(){
        self.emit("ok");
        self.close();
    });
};

/** @private */
gtk.PopupWindowDefault.prototype._detachOkBtnHandler = function(){   
    jsgtk.Event.removeListener(this._okBtnHandler);
    this._okBtnHandler = null;
};

/** @private */
gtk.PopupWindowDefault.prototype._detachCloseBtnHandler = function(){   
    jsgtk.Event.removeListener(this._closeBtnHandler);
    this._closeBtnHandler = null;
};

/**
 * A method to close/remove pop-up window
 */
gtk.PopupWindowDefault.prototype.close = function(){
    if(!this._domObj.parentNode) return;
    this._domObj.parentNode.removeChild(this._domObj);
    this._disableDiv.parentNode.removeChild(this._disableDiv);
    this._detachOkBtnHandler();
    this._detachCloseBtnHandler();
    
};

/** @private */
gtk.PopupWindowDefault.prototype._resizeWidth = function(width){
    this._width = width;
    this._domObj.style.width = width + "px";
        this._header.style.width = width + "px";
        this._bodyDiv.style.width = width - this._iconWidth + "px"; 
        this._titleDiv.innerHTML = this._title;
        this._titleDiv.style.width = this._width - 32 + "px";
};

/** @private */
gtk.PopupWindowDefault.prototype._resizeHeight = function(height){
    this._height = height;
    this._domObj.style.height = height + "px";
        this._bodyDiv.style.height = height - 62 + "px";
};

/** @private */
gtk.PopupWindowDefault.prototype._getMaxHeight = function(){
    var height = jsgtk.Util.calculateTextWidthHeight(this._bodyText)[1] + 62;
    if(height < this._minHeight) height = this._minHeight;
    return height;
};

/** @private */
gtk.PopupWindowDefault.prototype._getMaxWidth = function(){
    var width = jsgtk.Util.calculateTextWidthHeight(this._bodyText)[0];
    if(width > this._maxWidth - 20) width = this._maxWidth;
    else if(width < this._minWidth) width = this._minWidth;
    return width;
};

/** @private */
gtk.PopupWindowDefault.prototype._resizePopupWindow = function(){
    this._resizeWidth(this._getMaxWidth());
    this._resizeHeight(this._getMaxHeight());
};

/** @private */
gtk.PopupWindowDefault.prototype._calculateOffsetCenter = function(){
    var jsgtkUtil = jsgtk.Util;

    var windowWidth = jsgtkUtil.getWindowWidth();
    var windowHeight = jsgtkUtil.getWindowHeight();
        
    var scrollY = jsgtkUtil.getScrollY();
    var scrollX = jsgtkUtil.getScrollX();
    
    var offsetX =(windowWidth - this._width)/2 + scrollX;
    var offsetY =(windowHeight - this._height)/2 + scrollY; 
    return [offsetX, offsetY];
};

/** @private */
gtk.PopupWindowDefault.prototype._setCenterScreen = function(){
    var offset = this._calculateOffsetCenter();
    var domObjStyle = this._domObj.style;
    domObjStyle.top = offset[1] + "px";
    domObjStyle.left= offset[0] + "px";
};

/** @private */
gtk.PopupWindowDefault.prototype._disableBackground = function(){
    this._disableDiv = jsgtk.Dom.create({"tag": "div", "className": "gtk_disable_body"});
    var widthHeight = jsgtk.Util.getPageSize();
    
    var disableDivStyle = this._disableDiv.style;
    disableDivStyle.width = widthHeight[0] + "px";
    disableDivStyle.height = widthHeight[1] + "px";
    disableDivStyle.opacity = 0.3;
    disableDivStyle.backgroundColor = "#ADADAD";
    
    document.body.appendChild(this._disableDiv);
};

/** @private */
gtk.PopupWindowDefault.prototype._getMinimizedTitle = function(titleDom, minimizedWidth){
    
    var tmpNode = titleDom.cloneNode(true);
    var title = tmpNode.innerHTML;
 
    var titleWidth = jsgtk.Util.fullWidth(tmpNode);
    if( titleWidth < minimizedWidth){
        return title;
    }
    
    var charWidth = 0;
    var title = title.replace(/\s+/g,'`');

    for(var i = 0, len = title.length; i < len; i++){
        var character = title.substring(i, i+1);
        tmpNode.innerHTML = character;
        charWidth += jsgtk.Util.fullWidth(tmpNode);

        if(charWidth >= minimizedWidth){
            break;
        }
    }

    var minimizedTitle = title.substring(0, i);
    return minimizedTitle.split("`").join(" ") + "...";
};

/** @private */
gtk.PopupWindowDefault.prototype._enableDrag = function(){
    var minX = 1, maxX =jsgtk.Util.getWindowWidth()-this._width - 17;
    var minY = 0, maxY = jsgtk.Util.getWindowHeight()-this._height - 17;
    jsgtk.Util.DomDraggableAdapter.dragInit(this._domObj, minX, maxX, minY, maxY);
};

/** @private */
gtk.PopupWindowDefault.prototype._disableDrag = function(){
    var minX = maxX = parseInt(this._domObj.style.left);
    var minY = maxY = parseInt(this._domObj.style.top);
    jsgtk.Util.DomDraggableAdapter.dragInit(this._domObj, minX, maxX, minY, maxY);
};

gobject.signalNew("ok", gtk.PopupWindowDefault, null, null, []);
gobject.signalNew("closed", gtk.PopupWindowDefault, null, null, []);
