/**
 * RadioButton is a class to create a radio button
 * @constructor
 * @base gtk.Widget
 * @param {gtk.RadioButton} group The group name that it belongs to
 * @param {string} label The text of radio button
 */
gtk.RadioButton = function(group, label){
    gtk.Widget.apply(this);
    
    this._activated = false;
    this._name = group ? group.getName() : new UUID().id;
	this._addToGroup(this);
	
    this._imageSize   = 16;
    this._space       = 4;
    this._imageClassUnchecked  = "gtk_radio_button_img_unchecked";
    this._imageClassChecked    = "gtk_radio_button_img_checked"
    this._isFocus  = false;
    this._labelWidget = new gtk.Label(label);
    
    /** @private */
    this._render = function(){
        this._domObj.className += " gtk_radio_button";
            this._radioImgLabelContainer = jsgtk.Dom.create({"append": this._domObj, "tag": "div", "class": "gtk_radio_button_img_label_container"});
                this._radioImage = jsgtk.Dom.create({"append": this._radioImgLabelContainer, "tag": "div", "className": this._imageClassUnchecked});                
                this._radioLabel = jsgtk.Dom.create({"append": this._radioImgLabelContainer, "tag": "div", "class": "gtk_radio_button_label"});
                this._radioLabel.appendChild(this._labelWidget.getNode());
                
        this._width  = this._minWidth  = this._imageSize + this._labelWidget.getMinWidthComposite() + this._space;
        this._height = this._minHeight = this._labelWidget.getMinHeightComposite();
    };
    this._render();
    
    this._initEvent();
    
    this.resizeWidthComposite(this._width);
    this.resizeHeightComposite(this._height);
};

gtk.RadioButton.prototype = new gtk.Widget();
gtk.RadioButton.prototype.constructor = gtk.RadioButton;

/** @private */
gtk.RadioButton.group = [];

/** @private */
gtk.RadioButton.prototype._initEvent = function(){
    var self = this;
    var jsgtkEvent = jsgtk.Event;
    
    jsgtkEvent.addDomListener(this._domObj, "mouseup", function(){
            self._unMarkeOther(self);
            self._unFocusOther(self)
            self._select();
            self.emit("toggled");
    });
    jsgtkEvent.addDomListener(this._domObj, 'keyup', function(e){
	    if(e.keyCode == gtk.ENTER_KEY){
            self._unMarkeOther(self);
            self._unFocusOther(self)
            self.setFocus(self);
            self._select();
	    }
	});
	jsgtkEvent.addDomListener(this._domObj, "focus", function(){
        self.setFocus(self);
    });
    jsgtkEvent.addDomListener(this._domObj, "blur", function(){
        self._unFocusOther(self)
    });
};

/**
 * A method to set the tab index value onto the gtk.RadioButton
 * @param {integer} index The integer index value to be set
 */
gtk.RadioButton.prototype.setTabindex = function(index){
	this._domObj.tabIndex = index;
};

/** @private */
gtk.RadioButton.prototype._unFocusOther = function(btn){
	var group = gtk.RadioButton.group;
    for(var i = group.length; i--;){
        if(group[i].name === btn.getName()) {
            for(var j = group[i].children.length; j--;){
                this._unFocus(group[i].children[j]);
            }
            return;
        }
    }
};

/** @private */
gtk.RadioButton.prototype._unFocus = function(btn){
	btn.getNode().firstChild.lastChild.className = "gtk_radio_button_label"
};

/**
 * A method to set focus onto the gtk.RadioButton
 */
gtk.RadioButton.prototype.setFocus = function(){
	this._domObj.firstChild.lastChild.className += " gtk_radio_focus"
};

/**
 * A method to set active onto the radio button
 * @param {Boolean} isActive The state to define whether it is true or false
 */
gtk.RadioButton.prototype.setActive = function(isActive){
	this._activated = isActive;
};

/**
 * A method to get/return the state of radio button whether it is activated or not
 * @return {Boolean} The state as true or false
 * @type Boolean
 */
gtk.RadioButton.prototype.getActive = function(){
	return this._activated;
};

/** @private */
gtk.RadioButton.prototype._select = function(){
    this._radioImage.className = this._imageClassUnchecked + " " + this._imageClassChecked;
	this.setActive(true);
};

/** @private */
gtk.RadioButton.prototype._deSelect = function(){
    this._radioImage.className = this._imageClassUnchecked;
	this.setActive(false);
};

/**
 * A method to get the group
 * @return {gtk.RadioButton} The group name as an object of gtk.RadioButton
 * @type gtk.RadioButton
 */
gtk.RadioButton.prototype.getGroup = function(){
    var group = gtk.RadioButton.group;
    var tmpGroup = [];
    for(var i = group.length; i--;){
        if(group[i].getName() === this.getName()) tmpGroup.push(group[i]);
    }
    return tmpGroup;
};

/**
 * A method to get the name of group that it belongs to
 * @return {String} The name of the gtk.RadioButton belongs to
 * @type String
 */
gtk.RadioButton.prototype.getName = function(){
    return this._name;
};

/**
 * A method to set the name of group that it is to belongs to
 * @param {String} name The name of the group
 */
gtk.RadioButton.prototype.setName = function(name){
    this._name = name;
};

/** @private */
gtk.RadioButton.prototype._addToGroup = function(group){
    var radioGroup = gtk.RadioButton.group;
    if(radioGroup != []){
        for(var i = radioGroup.length; i--;){
            if(radioGroup[i].name == group._name){
                radioGroup[i].children.push(group);
                return;
            }
        }
    }
    var groupObj = {name: group._name, children: [group]};
    radioGroup.push(groupObj);
};

/** @private */
gtk.RadioButton.prototype._unMarkeOther = function(btn){
    var group = gtk.RadioButton.group;
    for(var i = group.length; i--;){
        if(group[i].name === btn.getName()) {
            for(var j = group[i].children.length; j--;){
                group[i].children[j]._deSelect();
            }
            return;
        }
    }
};

/** @private */
gtk.RadioButton.prototype._resizeHeight = function(height){
    this._height = height;
    this._domObj.style.height = height + "px";
        this._radioImgLabelContainer.style.height = this._minHeight + "px";
};

/** @private */
gtk.RadioButton.prototype._resizeWidth = function(width){
    this._width = width;
    this._domObj.style.width = width + "px";
        this._radioImgLabelContainer.style.width = width + "px";
};

/** 
 * A method to resize its composite width of gtk.Radiobutton 
 * @param {Integer} width The integer value as width to be set
 */
gtk.RadioButton.prototype.resizeWidthComposite = function(width){
    var minWidth = this._minWidth < width ? width : this._minWidth;
    this._resizeWidth(minWidth);
};

/** 
 * A method to resize its composite height of gtk.Radiobutton 
 * @param {Integer} height The integer value as height to be set
 */
gtk.RadioButton.prototype.resizeHeightComposite = function(height){
    var minHeight = this._minHeight < height ? height : this._minHeight;
    this._resizeHeight(minHeight);
    this._positionCheckBox();
};

/** @private */
gtk.RadioButton.prototype._positionCheckBox = function(height){
    this._radioImgLabelContainer.style.top = this._height / 2 - this._minHeight / 2 + "px";
};

/** It allows user to connect to a registered signal event "toggled" when click on the radio button */
gobject.signalNew("toggled", gtk.RadioButton, null, null, []);
