/**
* @constructor
* @base gtk.Widget
* @param {Interger} columns The numbers of columns that slider can contain
* @param {Interger} spacing The value of spacing to set between each columns
*/
gtk.Slider = function(columns, spacing) {
	gtk.Widget.apply(this);
	
    var self = this;
    
	this._width      = this._minWidth  = 300;
	this._height     = this._minHeight = 200;

	this._visibleColumns    = columns || 1;
	this._spacing    = spacing || 1;
	this._marginLeft = 50;
	
	this._slideWidth = this._getSlideWidth(this._width);
	
	this._slides     = [];
	this._currentLeft= 0;
	this._arrowWidth = 27;
	this._arrowHeight= 37;
	this._slideWidths;

	//used to prevent user from sponteneouse click on navigator button
	this._isMoving = false;
	
	/** @private */
	this._render = function(){
		this._domObj.className   += " gtk_slider";
		    this._viewport = jsgtk.Dom.create({"append": this._domObj, "tag": "div", "class": "gtk_slides_viewport", "style": "position: relative;"});
		    this._viewport.style.left  = this._marginLeft + "px";
		    this._viewport.style.overflowY = "hidden";
		        this._slidesContainer = jsgtk.Dom.create({"append": this._viewport, "tag": "div", "class": "gtk_slides_container"});
		        this._slidesContainer.style.height = this._height + "px";
		        
        this._leftArrow = jsgtk.Dom.create({"append": this._domObj, "tag": "div", "class": "gtk_slider_leftarrow"});
        this._leftArrowDisabled = jsgtk.Dom.create({"append": this._domObj, "tag": "div", "class": "gtk_slider_leftarrow_disabled"});
        this._positionArrowLeft(this._leftArrow);
        this._positionArrowLeft(this._leftArrowDisabled);
        jsgtk.Dom.hideElement(this._leftArrow);
        
        this._rightArrow = jsgtk.Dom.create({"append": this._domObj, "tag": "div", "class": "gtk_slider_rightarrow"});
        this._rightArrowDisabled = jsgtk.Dom.create({"append": this._domObj, "tag": "div", "class": "gtk_slider_rightarrow_disabled"});
        this._positionArrowRight(this._rightArrow);
        this._positionArrowRight(this._rightArrowDisabled);
        jsgtk.Dom.hideElement(this._rightArrowDisabled);

 		this.resize(this._width, this._height);
	};
	
    this._render();
    
    jsgtk.Event.addDomListener(this._leftArrow, "click", function(e){

        //do nothing when the slides are still moving
        if(!self._isMoving && self._currentLeft !== 0)self._moveRight();
	    jsgtk.Event.stopBubble(e);
	}, false);

    jsgtk.Event.addDomListener(this._rightArrow, "click", function(e){
    
        //do nothing when the slides are still moving
        if(!self._isMoving){
            var len = self._slides.length >= self._visibleColumns ? self._visibleColumns : 1;
            var left = self._slideWidth * (self._slides.length - len);
            var isMovable = left + self._currentLeft;
            if(isMovable >= 0) self._moveLeft();
        }
        jsgtk.Event.stopBubble(e);
	}, false);	
};

gtk.Slider.prototype = new gtk.Widget();
gtk.Slider.prototype.constructor = gtk.Slider;

/**
* Append gtk.Widget(any type)
* @param {gtk.Widget} widget
* @param {boolean} isCloseButtonVisible
*/
gtk.Slider.prototype.appendChild = function(widget, closeButtonVisible){

    var self = this;
    var isCloseButtonVisible = closeButtonVisible === undefined || closeButtonVisible === null ? false : closeButtonVisible;
    var scroller = this._getScroller(widget);
    
    var slide = this._constructSlide(scroller);
    this._resizeElement(slide, this._slideWidth, this._height - 12);
    
    var space = this._constructSpace();
    this._resizeElement(space, this._spacing, this._height);
    
    var closeButton = slide.lastChild;
    
    this._slides.push({
        "slide"  : slide,
        "space"  : space,
        "widget" : scroller 
    });
    scroller.resizeWidthComposite(this._slideWidth);
    scroller.resizeHeightComposite(this._height - 12);
    
    var slideWidth = this._slideWidth + this._spacing + 2;
    this._setSlideWidths(slideWidth * (this._slides.length));
    
    this.resize(this._width, this._height);
    
    var jsgtkDom = jsgtk.Dom;
    var jsgtkEvent = jsgtk.Event;
    
    //hide arrow when the slides still less then the given columns
    if(this._slides.length <= this._visibleColumns) {
        jsgtkDom.hideElement(this._rightArrow);
        jsgtkDom.showElement(this._rightArrowDisabled);   
    }else{
        jsgtkDom.hideElement(this._rightArrowDisabled);
        jsgtkDom.showElement(this._rightArrow);
    }
    
    jsgtk.Event.addDomListener(slide, "mouseover", function(e){
        if(isCloseButtonVisible) 
        	jsgtkDom.showElement(this.lastChild);
        jsgtkEvent.stopBubble(e);
    });
    
    jsgtk.Event.addDomListener(slide, "mouseout", function(e){
        jsgtkDom.hideElement(this.lastChild);
        jsgtkEvent.stopBubble(e);
    });
    
    jsgtk.Event.addDomListener(closeButton, "mousedown", function(e){
        self._removeSlide(slide, space);
        if(self._slides.length <= self._visibleColumns){
            self._disableArrow();
        }
        jsgtkEvent.stopBubble(e);
    });
};

/** @private */
gtk.Slider.prototype.resetMinWidth = function(width){
	this._isMinWidthChanged = width ? true : false;
	this._minWidth = width;
    this._width = width;
    this.resizeWidthComposite(this._width);
};

/** @private */
gtk.Slider.prototype.resetMinHeight = function(height){
	this._isMinHeightChanged = height ? true : false;
	this._minHeight = height;
    this._height = height;
    this.resizeHeightComposite(this._height);
};

/**
* clear all gtk.Widget inside gtk.Slider
*/
gtk.Slider.prototype.clearSlides = function(){
    this._slides = [];
    this._slidesContainer.innerHTML = "";
    this._disableArrow();
    this._viewport.scrollLeft = 0;
};

/** @private */
gtk.Slider.prototype._disableArrow = function(){
	var domUtil = jsgtk.Dom;
    domUtil.hideElement(this._rightArrow);
    domUtil.showElement(this._rightArrowDisabled);
    
    domUtil.hideElement(this._leftArrow);
    domUtil.showElement(this._leftArrowDisabled);
};

/** @private */
gtk.Slider.prototype._removeSlide = function(slide, space){
    var self = this;
    for(var i = this._slides.length; i--;){
        if(this._slides[i].slide === slide) {
            this._slides.splice(i, 1);
            break;
        }
    }

    var width = parseInt(slide.style.width);
    var intervalID = window.setInterval(function(){
        width -= 20;
        slide.style.width = width + "px";
        if(width <= 0) {            
            var slideWidth = self._slideWidth + self._spacing + 2;
            
            self._setSlideWidths(slideWidth * (self._slides.length));
            
            self._slidesContainer.style.width = self._slideWidths + "px";
            
            var left = self._currentLeft - slideWidth;
            
            var slideParent = slide.parentNode;
            
            //hack to solve the left positioning of slide container for last slide removing
            if(slideParent.lastChild === space) self._viewport.scrollLeft -= 5;

            if(left >0) self._currentLeft = left;
            
            slideParent.removeChild(space);
            slideParent.removeChild(slide);

            clearInterval(intervalID);
        }
    }, 5);    
};

/** @private */
gtk.Slider.prototype._setSlideWidths = function(width){
    this._slideWidths = width;
};

/** @private */
gtk.Slider.prototype._resizeSlides = function(width, height){
    var slide;
    
    this._slideWidth = this._getSlideWidth(width);    
    
    var slideWidth = this._slideWidth + this._spacing + 2;
    this._setSlideWidths(slideWidth * (this._slides.length));
    
    for(var i = 0, len = this._slides.length; i<len; i++){
        slide = this._slides[i];
        this._resizeElement(slide["slide"], this._slideWidth, height - 13);
        this._resizeElement(slide["space"], this._spacing, height);
        slide["widget"].resizeWidthComposite(this._slideWidth);
        slide["widget"].resizeHeightComposite(height - 13);
    }
    this.resize(width, height);
    
    this._positionArrowLeft(this._leftArrow);
    this._positionArrowLeft(this._leftArrowDisabled);
    this._positionArrowRight(this._rightArrow);
    this._positionArrowRight(this._rightArrowDisabled);
};

/** @private */
gtk.Slider.prototype._getSlideWidth = function(width){
    width = ( (width - this._marginLeft * 2) - ( (this._visibleColumns - 1) * this._spacing) - this._visibleColumns * 2) / this._visibleColumns;
    return width;
};

/** @private */
gtk.Slider.prototype.resize = function(width, height){
    this._resizeWidth(width);
    this._resizeHeight(height);
};

/** @private */
gtk.Slider.prototype._resizeElement = function(element, width, height){
    element.style.width = width + "px";
    element.style.height= height + "px";
};

/** @private */
gtk.Slider.prototype._constructSpace = function(){
    var space = jsgtk.Dom.create({"append": this._slidesContainer, "tag": "div", "style": "float: left;"});
    return space;
};

/** @private */
gtk.Slider.prototype._constructSlide = function(scrolledWindow){    
    var slide = jsgtk.Dom.create({"append": this._slidesContainer, "tag": "div", "class": "gtk_slider_slide", "style": "float: left;"});
    slide.appendChild(scrolledWindow._domObj);
	var closeButton = jsgtk.Dom.create({"append": slide, "tag": "div", "class": "gtk_slider_close_button"});
	jsgtk.Dom.hideElement(closeButton);
    return slide;
};

/** @private */
gtk.Slider.prototype._getScroller = function(widget){
    var scroller = new gtk.Scroller();
    scroller.show();
    scroller.appendChild(widget, true);
    scroller.disableHScrolled();
    
    return scroller;
};

/** @private */
gtk.Slider.prototype._moveRight = function(){
    var self = this;
    var left = this._currentLeft - (this._slideWidth + this._spacing + 2);

    var intervalID = window.setInterval(function(){
        self._currentLeft -= 20;
        self._isMoving = true;
        self._viewport.scrollLeft = self._currentLeft;

        if(self._currentLeft <= left){
            self._viewport.scrollLeft = left;
            self._currentLeft = left;
            clearInterval(intervalID);
            self._enableRightArrow();
            if(self._viewport.scrollLeft === 0) self._disableRightArrow();
            self._isMoving = false;
        }
    }, 10);
};

/** @private */
gtk.Slider.prototype._moveLeft = function(){

    var self = this;
    var left = this._currentLeft + this._slideWidth + this._spacing + 2;

    var intervalID = window.setInterval(function(){
        self._currentLeft += 20;
        self._viewport.scrollLeft = self._currentLeft;
        self._isMoving = true;
        if(self._currentLeft >= left){
            self._viewport.scrollLeft = left;
            self._currentLeft = left;
            
            self._enableLeftArrow();
            
            var actualSlideWidth = self._slideWidth + self._spacing + 2;
            var width = Math.floor(self._slideWidths - self._viewport.scrollLeft);
            if(width <= actualSlideWidth * self._visibleColumns) self._disableLeftArrow();
            clearInterval(intervalID);
            
            self._isMoving = false;
        }
    }, 10);
};

/** @private */
gtk.Slider.prototype._positionArrowLeft = function(leftArrow){
	var leftArrowStyle = leftArrow.style;
    leftArrowStyle.width  = this._arrowWidth + "px";
    leftArrowStyle.height = this._arrowHeight + "px";
    leftArrowStyle.top = this._height / 2 - this._arrowHeight/2 + "px";
    leftArrowStyle.left = "16px";
};

/** @private */
gtk.Slider.prototype._positionArrowRight = function(rightArrow){
	var rightArrowStyle = rightArrow.style;
    rightArrowStyle.width  = this._arrowWidth + "px";
    rightArrowStyle.height = this._arrowHeight + "px";
    rightArrowStyle.top = this._height / 2 - this._arrowHeight/2 + "px";
    rightArrowStyle.right = "8px";
};

/** @private */
gtk.Slider.prototype._enableRightArrow = function(){
	var jsgtkDom = jsgtk.Dom;
    jsgtkDom.hideElement(this._rightArrowDisabled);
    jsgtkDom.showElement(this._rightArrow);
};

/** @private */
gtk.Slider.prototype._disableRightArrow = function(){
	var jsgtkDom = jsgtk.Dom;
    jsgtkDom.hideElement(this._leftArrow);
    jsgtkDom.showElement(this._leftArrowDisabled);
};

/** @private */
gtk.Slider.prototype._enableLeftArrow = function(){
	var jsgtkDom = jsgtk.Dom;
    jsgtkDom.hideElement(this._leftArrowDisabled);
    jsgtkDom.showElement(this._leftArrow);  
};

/** @private */
gtk.Slider.prototype._disableLeftArrow = function(){
	var jsgtkDom = jsgtk.Dom;
    jsgtkDom.showElement(this._rightArrowDisabled);
    jsgtkDom.hideElement(this._rightArrow);
};

/** @private */
gtk.Slider.prototype._resizeWidth = function(width){
    this._width = width;
    this._domObj.style.width  = this._width + "px";
    this._viewport.style.width = this._width  - this._marginLeft * 2 + 6 + "px";
    this._slidesContainer.style.width = this._slideWidths + "px";
};

/** @private */
gtk.Slider.prototype._resizeHeight= function(height){
    this._height = height;
    this._viewport.style.height= this._height + 25 + "px";
    this._domObj.style.height = this._height + "px";
    this._slidesContainer.style.height = this._height + "px";
};

/** @private */
gtk.Slider.prototype.getMinHeightComposite = function(){
    return this._minHeight;  
};

/** @private */
gtk.Slider.prototype.getMinWidthComposite = function(){
    return this._minWidth;  
};

/** @private */
gtk.Slider.prototype.resizeWidthComposite = function(width){
	var minWidthComposite = this.getMinWidthComposite();
	var minWidth = width > minWidthComposite ? width : minWidthComposite;
	this._resizeSlides(minWidth, this._height);
};

/** @private */
gtk.Slider.prototype.resizeHeightComposite = function(height){
	var minHeightComposite = this.getMinHeightComposite();
	var minHeight = height > minHeightComposite ? height : minHeightComposite;
	this._resizeSlides(this._width, minHeight);
};
