/**
 * A widget with an entry which is able to create many tags
 * @param {Boolean} allowLabel The setting of label. If true, the labels can be displayed inside smart entry
 * @constructor
 */
gtk.SmartEntry = function(allowLabel){
    gtk.Widget.apply(this);
    
    var self 				= this;
    this._minWidth 		= this._width = 150;
    this._minHeight 		= this._height = 75;
    this._entryWidth 	= 50;
    this._entry 			= null;
    this._allowLabel 	= allowLabel || false;
    this._isAllowOneTag = false;
    this._filter 			= false;
    this._eventStorage 	= [];
    this._keyEvent;
    this._selectedIndex = -1;
    this._index 			= 0;
    this._tagValue 		= "";
    this._textBoxError 	= false;
    
	this._render = function(){
	    this._domObj.className += " gtk_smart_entry";
	    this._domObj.style.width = this._width + "px";
	    this._domObj.style.height = this._height + "px";
	    this._areaDom = jsgtk.Dom.create({"tag": "div", append: this._domObj, "className": "gtk_smart_entry_area"});
	    this._holder = jsgtk.Dom.create({"tag": "span", append: this._areaDom});
	    
	    if(!this._allowLabel) this._createEntry();

	    this.hide();
	};	

	this._render();
	
	jsgtk.Event.addDomListener(document.body, "click", function(){
	    self._hideDropDown();
	}, false);
};

gtk.SmartEntry.prototype = new gtk.Widget();
gtk.SmartEntry.prototype.constructor = gtk.SmartEntry;

/** @private */
gtk.SmartEntry.prototype._hideDropDown = function(){
    var dropDown = this._domObj.getElementsByClassName("gtk_smart_entry_drop_down_holder");
    for(var i = 0, len = dropDown.length; i < len; i++){
        dropDown[i].style.display = "none";
    }
};

gtk.SmartEntry.prototype.showError = function(isOnError){
	 this._textBoxError = (typeof isOnError !== 'undefined' && typeof isOnError === 'boolean') ? isOnError : this._textBoxError;
    
    if(this._textBoxError) this._domObj.className += " gtk_smart_entry_onerror";
    else this._domObj.className = "gtk_widget gtk_smart_entry_default";
};

/** @private */
gtk.SmartEntry.prototype._createEntry = function(){
    var self = this;
   
    var entryDiv = jsgtk.Dom.create({"tag": "div", append: this._areaDom, "className": "gtk_smart_entry_input_holder"});    
        this._entry = jsgtk.Dom.create({"tag": "input", append: entryDiv, "className": "gtk_smart_entry_input"});

    entryDiv.style.width = "32px";
    
    this._bindEntryLostFocusEvent();
    
    this._bindEntryKeypressEvent(this._entry);
    
    jsgtk.Event.addDomListener(this._areaDom, "click", function(e){
        self._entry.style.visibility = "visible";
	    self._entry.focus();
	    jsgtk.Event.stopBubble(e);
	});
};

/** @private */
gtk.SmartEntry.prototype._bindEntryLostFocusEvent = function(textBox){
    var self = this;
    var entry = textBox || this._entry;
    
    jsgtk.Event.addDomListener(entry, "blur", function(){
        if(self._isAllowOneTag){
            self._createTailTag(self._holder);
            return;
        }
        
        self._unSelectTags();
        
        self._createTag(self._holder);
    }, false);
};

/** @private */
gtk.SmartEntry.prototype._unSelectTags = function(){
    var tags = this._holder.childNodes;
        
    for(var i = 0, len = tags.length; i < len; i++){
        if(tags[i].selected == true){
            tags[i].selected = false;
            tags[i].className = "gtk_smart_entry_tag_holder";
        }
    }
};

/** @private */
gtk.SmartEntry.prototype._bindEntryKeypressEvent = function(entry){
    var self = this;
    var tempSpan = document.createElement("span");
    var tempWidth;
    
    jsgtk.Event.addDomListener(entry, "keypress", function(e){
        switch(e.keyCode){
            case gtk.ENTER_KEY:
                self._bindEntryEnterKeyEvent(entry);
                break;
            case gtk.BACKSPACE_KEY:
                self._bindEntryBackspaceKeyEvent(entry);
                break;
            case gtk.DELETE_KEY:
                self._bindEntryDeleteKeyEvent(entry, e);
                break;
            default:
                if(!self._isAllowOneTag){
                    tempSpan.innerHTML = self._entry.value;
                    tempWidth = self._calculateTextWidth(tempSpan) + self._entryWidth - 2;
                    self._entry.style.width = (tempWidth) < self._width - 2? tempWidth + "px" : self._width - 2 + "px";
                }
        }
        
        if((e.which == gtk.A_KEY || e.which == gtk.CAP_A_KEY) && e.ctrlKey){
            self._bindEntryCtrlAKeyEvent(entry);
        }
    },false);
};

/** @private */
gtk.SmartEntry.prototype._bindEntryCtrlAKeyEvent = function(entry){
    if(entry.value != "") return;
    var tags = this._holder.childNodes;
    var len = tags.length;
    for(var i = 0; i < len; i++){
    	if(tags[i].tagName == "INPUT") continue;
        if(tags[i].childNodes[0].className == "gtk_smart_entry_fixed_tag") continue;
        tags[i].className += " gtk_smart_entry_tag_selected";
        tags[i].selected = true;
    }
    this._entry.style.visibility = "hidden";
};

/** @private */
gtk.SmartEntry.prototype._bindEntryBackspaceKeyEvent = function(entry){
        var tags = this._holder.childNodes;

//        if(!entry.previousSibling) return;
        var len = tags.length;
        var tag = (this._tagValue) ? entry.previousSibling : tags[len-1];

        if(len > 0 && entry.value == "" && tag.childNodes[0].className != "gtk_smart_entry_fixed_tag"){
        
            if(tag.selected){
                tag.parentNode.removeChild(tag);
                entry.style.visibility = "visible";
                this._unSelectTags();
                
            }
            else{
            
                tag.selected = true;
                tag.className += " gtk_smart_entry_tag_selected";
                entry.style.visibility = "hidden";
            }
        }
        return; 
};

/** @private */
gtk.SmartEntry.prototype._bindEntryDeleteKeyEvent = function(entry, e){
    var tags = this._holder.childNodes;
    var len = tags.length;

    if(len > 0 && entry.value == ""){
    	
        tags = this._holder.childNodes;
        var length = this._holder.childNodes.length;
        for(var i = length-1 ; i >= 0 ; i--){
            if(tags[i].selected||tags[i].tagName == "INPUT") this._holder.removeChild(tags[i]);
        }
        this._tagValue = "";
        this._entry.style.visibility = "visible";
        this._entry.focus();
    }
};

/** @private */
gtk.SmartEntry.prototype._bindEntryEnterKeyEvent = function(entry){
    if(this._isAllowOneTag){
        this._entry.blur();
        return;
    }
    this._tagValue? this._createTag(this._holder, true, entry) : this._createTag(this._holder);
};

/** @private */
gtk.SmartEntry.prototype._createTag = function(holder, isAppendToHolder, textBox){
    if(this._entry.value || textBox){
        var self = this;
        
        var tagValue = textBox ? textBox.value : this._entry.value;

        var tagHolder = jsgtk.Dom.create({"tag": "div", "className": "gtk_smart_entry_tag_holder"});
        var tag  = jsgtk.Dom.create({"tag": "span", "append": tagHolder , "text": tagValue});
        var closeButton = jsgtk.Dom.create({"tag": "div", append: tagHolder, "className": "gtk_smart_entry_close"});
        
        if(this._tagValue){
        	holder.replaceChild(tagHolder, textBox);
        	this._tagValue = "";
        }else{
        	holder.appendChild(tagHolder);
        }
        
        jsgtk.Event.addDomListener(closeButton, "click", function(e){
            tagHolder.parentNode.removeChild(tagHolder);
            jsgtk.Event.stopBubble(e);
        }, false);
        
        var entry = jsgtk.Dom.create({"tag": "input", "className": "gtk_smart_entry_input"});
        
        this._bindClickEventToTag(holder, tagHolder, entry);
                
        this._entry.value = "";
        this._entry.style.width = this._entryWidth + "px";
        window.setTimeout(function(){self._entry.focus();}, 1);
        return tagHolder;
    }
};

/** @private */
gtk.SmartEntry.prototype._bindClickEventToTag = function(holder, tagHolder, entry){
	var self = this;	
	jsgtk.Event.addDomListener(tagHolder, "click", function(e){
    	if(self._allowLabel) return;
    	self._tagValue = tagHolder.childNodes[0].innerHTML;
    	holder.replaceChild(entry, tagHolder);
    	entry.focus();

    	jsgtk.Event.addDomListener(entry, "blur", function(){
    		if(entry.value == ""){
    			entry.value = self._tagValue;
    			self._createTag(holder, true, entry);
    			return;
    		}else{
    			self._createTag(holder, true, entry);
    		}
    	});
    	self._bindEntryKey(entry);

		jsgtk.Event.stopBubble(e);
    });
};

gtk.SmartEntry.prototype._bindEntryKey = function(entry){
	this._bindEntryKeypressEvent(entry);
};

/**
 * Method to create a smartEntry fixed tag
 * @param {String} fixedTag the value of fixed tag to be created
 */
gtk.SmartEntry.prototype.createFixedTag = function(fixedTag){
    var tagHolder = jsgtk.Dom.create({"tag": "div", "append": this._holder, "className": "gtk_smart_entry_tag_holder"});
    var tag  = jsgtk.Dom.create({"tag": "span", "append": tagHolder , "text": fixedTag, "className": "gtk_smart_entry_fixed_tag"});
};

/** @private */
gtk.SmartEntry.prototype._showInput = function(tagHolder){
    if(tagHolder.childNodes[2]) tagHolder.removeChild(tagHolder.childNodes[2]);
    var entryDiv = tagHolder.childNodes[1];

    tagHolder.childNodes[0].style.display = "none";
    entryDiv.style.display = "";
    entryDiv.value = "";

    window.setTimeout(function(){entryDiv.focus();}, 0);
    var self = this;

    jsgtk.Event.addDomListener(entryDiv, "blur", function(){
        if(self._tagValue && entryDiv.value == "") entryDiv.value = self._tagValue;
        self._bindInputLostFocusEvent(entryDiv, tagHolder);
        self._tagValue = "";
    }, false);

    return entryDiv;
};

/** @private */
gtk.SmartEntry.prototype._bindInputLostFocusEvent = function(entryDiv, tagHolder){
    if(entryDiv.value){
        if(tagHolder.childNodes[2]) tagHolder.removeChild(tagHolder.childNodes[2]);
        var tagDiv = jsgtk.Dom.create({"append": tagHolder,"tag": "div", "className": "gtk_smart_entry_tag_holder"});
        var tag  = jsgtk.Dom.create({"tag": "span", "append": tagDiv , "text": entryDiv.value});
        var closeButton = jsgtk.Dom.create({"tag": "div", append: tagDiv, "className": "gtk_smart_entry_close"});
        
        entryDiv.style.display = "none";
        
        jsgtk.Event.addDomListener(closeButton, "click", function(){
            tagHolder.removeChild(tagDiv);
            tagHolder.childNodes[0].style.display = "";
        }, false);
    }
    else{
        tagHolder.childNodes[0].style.display = "";
        tagHolder.childNodes[1].style.display = "none";
        
        if(tagHolder.childNodes[2]) tagHolder.removeChild(tagHolder.childNodes[2]);
    }  
    
};

/** @private */
gtk.SmartEntry.prototype._createTagHolder = function(tagHolder, value){
    var tagDiv = jsgtk.Dom.create({"tag": "div", "className": "gtk_smart_entry_tag_holder"});
    var tag  = jsgtk.Dom.create({"tag": "span", "append": tagDiv , "text": value});
    var closeButton = jsgtk.Dom.create({"tag": "div", append: tagDiv, "className": "gtk_smart_entry_close"});

    jsgtk.Event.addDomListener(closeButton, "click", function(){
        tagDiv.parentNode.childNodes[0].style.display = "";
        tagDiv.parentNode.removeChild(tagDiv);
    }, false);
    
    var entry = jsgtk.Dom.create({"tag": "input", "className": "gtk_smart_entry_input"});
    this._bindClickEventToTag(this._holder, tagDiv, entry);
    
    return tagDiv;
};

/** @private */
gtk.SmartEntry.prototype._createKeypressEventForInput = function(entryDiv, tagHoldersss, index){
    var self = this;
    var tagHolders = this._holder.childNodes;

    jsgtk.Event.addDomListener(entryDiv, "keypress", function(e){
        if(e.keyCode == gtk.ENTER_KEY){
            entryDiv.blur();
            return;
        }
        
        if((e.which == gtk.A_KEY || e.which == gtk.CAP_A_KEY) && e.ctrlKey){
            if(entryDiv.value != "") return;
            var tags = self._holder.childNodes;
            var len = tags.length;
            for(var i = 0; i < len; i++){
                if(tags[i].childNodes[0].className == "gtk_smart_entry_fixed_tag" || !tags[i].childNodes[2]) continue;
                if(tags[i].childNodes[2].tagName == "DIV") {
                    tags[i].childNodes[2].className += " gtk_smart_entry_tag_selected";
                    tags[i].selected = true;
                }
            }
            return;
        }
    
        if(e.keyCode == gtk.TAB_KEY && tagHolders[index+1]){
            var nextEntryDiv = self._showInput(tagHolders[index + 1]);

            self._createKeypressEventForInput(nextEntryDiv, tagHolders, index+1);

            window.setTimeout(function(){nextEntryDiv.focus();}, 0);
        }

//TODO: fix bug for tabkey event after backspace delete 
        if(e.keyCode == gtk.BACKSPACE_KEY && entryDiv.parentNode.previousSibling && entryDiv.value == ""){
            if(entryDiv.parentNode.previousSibling){
                var previousSibling = entryDiv.parentNode.previousSibling;
                if(previousSibling.selected){
                    if(previousSibling.childNodes[2].selected == true)  previousSibling.removeChild(previousSibling.childNodes[2]);
                    
                    for(var i = 0; i < tagHolders.length; i++){
                        tagHolders[i].selected = false;
                    }
                    
                    var input = self._showInput(previousSibling);

                    previousSibling.childNodes[1].focus();
                }
                else{
                    if(previousSibling.childNodes[2]){
                        previousSibling.selected = true;
                        previousSibling.childNodes[2].className += " gtk_smart_entry_tag_selected";
                    }
                }
            }
        }
        
        if(e.keyCode == gtk.DELETE_KEY && entryDiv.parentNode.previousSibling){
            if(!entryDiv.parentNode.previousSibling.selected) 
                return;
            var j;
            for(var i = self._holder.childNodes.length - 1; i >= 0; i--){
                if(self._holder.childNodes[i].selected){
                    self._holder.childNodes[i].removeChild(self._holder.childNodes[i].childNodes[2]);
                    self._holder.childNodes[i].selected = false;
                    self._holder.childNodes[i].childNodes[0].style.display = "";
                    j = i;
                } 
            
            }
            var previousSibling = entryDiv.parentNode.previousSibling;            
            
            entryDiv.blur();
            
            for(var i = 0; i < tagHolders.length; i++){
                if(tagHolders[i] == previousSibling)
                    break;
            }
            
            var input = self._showInput(self._holder.childNodes[j]);
//            self._createKeypressEventForInput(input, tagHolders, i-1);
            self._holder.childNodes[j].focus();
//            previousSibling.childNodes[1].focus();
        }
    });
};

/**
 * Set the fixed tag in the smart entry. Fixed tag cannot be deleted
 * @param {String} arguments Label string to be set
 */

gtk.SmartEntry.prototype.setLabel = function(){
    this._smartEntryLabelHolder = [];
    var self = this;
    for(var i = 0, len = arguments.length; i < len; i++){
        var label = arguments[i];
        var tagHolder = jsgtk.Dom.create({"tag": "div", "append": this._holder, "className": "gtk_smart_entry_label_holder"});
        var labelSpan  = jsgtk.Dom.create({"tag": "span", "append": tagHolder , "text": label, "className": "gtk_smart_entry_label"});
        var input = jsgtk.Dom.create({"append": tagHolder, "tag": "input", "type": "text", "value": "", "className": "gtk_smart_entry_input"});
        input.style.display = "none";
    }
    
    var tagHolders = this._holder.childNodes;
    this._smartEntryLabelHolder = tagHolders;
    
    for(var i = 0, len = tagHolders.length; i< len; i++){
        if(tagHolders[i].childNodes[0].className == "gtk_smart_entry_fixed_tag") continue;
        (function(){
            var j = i;
            jsgtk.Event.addDomListener(tagHolders[j], "click", function(){
                if(tagHolders[j].childNodes[2]) self._tagValue = tagHolders[j].childNodes[2].childNodes[0].innerHTML;
                var entryDiv = self._showInput(tagHolders[j]);
                self._index = j;

                self._createKeypressEventForInput(entryDiv, tagHolders, self._index);
                
                jsgtk.Event.addDomListener(entryDiv, "blur", function(){
                    self._createTagHolder(tagHolders[j], entryDiv.value);
                }, false);

            }, false);
        })();    
    } 
};

/**
 * Set the minimum size of SmartEntry
 * @param {Integer} width the minimum width to be set
 * @param {Integer} height the minimum height to be set
 */
gtk.SmartEntry.prototype.setSize = function(width, height){
    this._width = this._minWidth = width;
    this._height = this._minHeight = height;
    this._domObj.style.width = this._width + "px";
    this._domObj.style.height = this._height + "px";
    
    this.emit('bubble', this._width, this._height);
};

gtk.SmartEntry.prototype.setDropDown = function(filterable, dropDown){
    var self = this;
    var fragment = document.createDocumentFragment();
    var width = this._width;
    var dom = null;
    
    if(filterable){
        for(var i = 0, len = dropDown.length; i < len; i ++){
            dom = jsgtk.Dom.create({"tag":"div", "append": fragment, "text": dropDown[i]});
            var textWidth = this._calculateTextWidth(dom);
            if(width < textWidth) width = textWidth;
            
            (function(){
                var j = i;
                jsgtk.Event.addDomListener(dom, "click", function(){
                    self._createTagFromDropDown(dropDown[j]);
                    dropDownDom.style.display = "none";
                    self._entry.focus();
                });
            })();
        }
        
        var dropDownDom = this._createDropDown(fragment, width);

        jsgtk.Event.addDomListener(this._entry, "keyup", function(e){ 
            if(e.keyCode == gtk.UP_ARROW_KEY || e.keyCode == gtk.DOWN_ARROW_KEY) return;
            var dropDownChildren = dropDownDom.childNodes;
            var re = new RegExp("^" + self._entry.value.toLowerCase());
            var result = null;
            var results = [];
            
            if(self._domObj.childNodes[2]) self._domObj.removeChild(self._domObj.childNodes[2]);

            for(var k = 0, len = dropDownChildren.length; k < len; k++){
                var child = dropDownChildren[k];
                result = child.innerHTML.toLowerCase().match(re);
                if(result){
                    results.push(child.cloneNode(true));
                }
            }
            
            self._displayDropDownResult(results, dropDownDom);
        }, false);
        
        var list = self._domObj.childNodes;
        var dropDownList = list[list.length - 1].childNodes;
        
        jsgtk.Event.addDomListener(this._areaDom, "click", function(){
            dropDownDom.style.display = "";
            if(self._domObj.childNodes[2]) self._domObj.removeChild(self._domObj.childNodes[2]);
                self._bindDropDownMouseEvent(dropDownList);
                self._bindDropDownKeydownEvent(dropDownList, dropDown, list);
        }, false);
    }
    else{
        for(var i = 0, len = dropDown.length; i < len; i ++){
            dom = jsgtk.Dom.create({"tag":"div", "append": fragment, "text": dropDown[i]});
            var textWidth = this._calculateTextWidth(dom);
            if(width < textWidth) width = textWidth;
            
            (function(){
                var j = i;
                jsgtk.Event.addDomListener(dom, "click", function(){
                    self._createTagFromDropDown(dropDown[j]);
                    dropDownDom.style.display = "none";
                });
            })();
        }
        
        var dropDownDom = this._createDropDown(fragment, width);
        
        jsgtk.Event.addDomListener(this._areaDom, "click", function(){
            dropDownDom.style.display = "";
            self._entry.style.display = "";
            self._entry.maxLength = "0";
        }, false);
    
        var list = self._domObj.childNodes;
        var dropDownList = list[list.length - 1].childNodes;
        
        this._bindDropDownMouseEvent(dropDownList);
        this._bindDropDownKeydownEvent(dropDownList, dropDown, list);
    }
};

/** @private */
gtk.SmartEntry.prototype._bindDropDownMouseEvent = function(dropDownList){
    var self = this;
    
    for(var i = 0, len = dropDownList.length; i < len; i++){
        (function(){
            var j = i;
            var eventId = jsgtk.Event.addDomListener(dropDownList[j], "mouseover", function(){
                if(self._selectedIndex != -1) dropDownList[self._selectedIndex].className = "";
                self._selectedIndex = j;
                dropDownList[self._selectedIndex].className = "gtk_smart_entry_drop_down";            
            });
            
            self._eventStorage.push(eventId);
        })();
    }
};

/** @private */
gtk.SmartEntry.prototype._bindDropDownKeydownEvent = function(dropDownList, dropDown, list){
    var self = this;
    var len = dropDownList.length;
    self._selectedIndex = -1;
    
    for(var i = 0; i<len; i++){
        dropDownList[i].className = "";
    }
    
    if(this._keyEvent) jsgtk.Event.removeListener(this._keyEvent);
    
    this._keyEvent = jsgtk.Event.addDomListener(this._entry, "keydown", function(e){         
        if(e.keyCode == gtk.DOWN_ARROW_KEY){
            if(self._selectedIndex < len - 1){
                if(self._selectedIndex != -1) dropDownList[self._selectedIndex].className = "";
                self._selectedIndex++;
                dropDownList[self._selectedIndex].className = "gtk_smart_entry_drop_down";
            } 
        }
        else if(e.keyCode == gtk.UP_ARROW_KEY){
            if(self._selectedIndex > 0) {
                dropDownList[self._selectedIndex].className = "";
                self._selectedIndex--;
                dropDownList[self._selectedIndex].className = "gtk_smart_entry_drop_down";
            }
        }
        
        else if(e.keyCode == gtk.ENTER_KEY){
        	if(!dropDownList[self._selectedIndex]) return;
            dropDownList[self._selectedIndex].className = "";
            self._entry.value = dropDown[self._selectedIndex];
            self._selectedIndex = -1;
            self._entry.blur();
            window.setTimeout(function(){
//                self._entry.blur();
                list[list.length - 1].style.display = "none";
            }, 2);
        }
    }, false);
};

/** @private */
gtk.SmartEntry.prototype._createDropDown = function(fragment, width){
    var self = this;
    var dropDownDom = jsgtk.Dom.create({"tag": "div", "className": "gtk_smart_entry_drop_down_holder"});
    dropDownDom.appendChild(fragment);
    dropDownDom.style.width = width + "px";
    dropDownDom.style.display = "none";
    this._domObj.appendChild(dropDownDom);
    
    return dropDownDom;
};

/** @private */
gtk.SmartEntry.prototype._createTagFromDropDown = function(value){
    var tagHolder = jsgtk.Dom.create({"tag": "div", "append": this._holder, "className": "gtk_smart_entry_tag_holder"});
    var tag  = jsgtk.Dom.create({"tag": "span", "append": tagHolder , "text": value});
    var closeButton = jsgtk.Dom.create({"tag": "div", append: tagHolder, "className": "gtk_smart_entry_close"});
    
    jsgtk.Event.addDomListener(closeButton, "click", function(e){
        tagHolder.parentNode.removeChild(tagHolder);
        jsgtk.Event.stopBubble(e);
    }, false);
    
    this._entry.focus();
};

/** @private */
gtk.SmartEntry.prototype._calculateTextWidth = function(dom){
    var tempDom = dom.cloneNode(true);
    tempDom.style.display = "none";
    document.body.appendChild(tempDom);
    return jsgtk.Util.fullWidth(tempDom);
};

/** @private */
gtk.SmartEntry.prototype._displayDropDownResult = function(results, dropDownDom){
    var self = this;
    if(results){
        dropDownDom.style.display = "none";
        var resultsDom = jsgtk.Dom.create({"tag":"div", "append": this._domObj, "className": "gtk_smart_entry_drop_down_holder"});
        resultsDom.style.width = this._width + "px";
        
        for(var i = 0, len = this._eventStorage.length; i < len; i++){
            jsgtk.Event.removeListener(this._eventStorage[i]);
        }
        
        this._eventStorage = [];
        var resultList = [];
        for(var i = 0, len = results.length; i < len; i++){
            resultsDom.appendChild(results[i]);
            resultList.push(results[i].innerHTML);
            (function(){
                var j = i;
                jsgtk.Event.addDomListener(results[j], "click", function(){
                    var parent = self._domObj.childNodes[0].childNodes[0];
                    parent.removeChild(parent.childNodes[parent.childNodes.length - 1]);
                    self._createTagFromDropDown(results[j].innerHTML);
                    self._domObj.removeChild(self._domObj.childNodes[2]);
                });
            })();
        }
        this._selectedIndex = -1;
        this._bindDropDownMouseEvent(resultsDom.childNodes);
        this._bindDropDownKeydownEvent(resultsDom.childNodes, resultList, this._domObj.childNodes);
    }
};

/**
 * Add the tail to smartEntry
 * @param {String} rightTail The value of right tail to be set
 * @param {String} label (Optional) The value of label to be set in entry before tail
 */
gtk.SmartEntry.prototype.setTailLabel = function(rightTail, label){
    var self = this;
    this._isAllowOneTag = true;
    this.createFixedTag(rightTail);
    this._holder.childNodes[0].className = "gtk_smart_entry_right_label_holder";    
    if(label){
        this._entry.style.display = "none";
        var tagHolder = jsgtk.Dom.create({"tag": "div", "append": this._holder, "className": "gtk_smart_entry_label_holder"});
        var labelSpan  = jsgtk.Dom.create({"tag": "span", "append": tagHolder , "text": label, "className": "gtk_smart_entry_label"});
        tagHolder.style.cssFloat = "right";
        
        jsgtk.Event.addDomListener(this._areaDom, "click", function(){
            if(labelSpan.style.display == "none"){
                var tag = self._holder.getElementsByClassName("gtk_smart_entry_tag_with_tail")[0];
                if(tag){
                    var value = tag.childNodes[0].innerHTML;
                    tag.parentNode.removeChild(tag);
                    self._entry.style.display = "";
                    self._entry.value = value;
                    self._entry.focus();
                }

                return;
            } 
            self._entry.style.display = "";
            self._entry.style.position = "absolute";
            self._entry.style.top = "3px";
            labelSpan.style.display = "none";
            self._entry.focus();
        });
    }
    
    var tempWidth = this._calculateTextWidth(this._holder.childNodes[0]);
    this._entry.style.width = this._width - tempWidth - 6 + "px";
    this._entry.style.textAlign = "right";
};

/** @private */
gtk.SmartEntry.prototype._createTailTag = function(holder){
    var self = this;
    if(this._entry.value){

        this._entry.blur();
        var tagHolder = jsgtk.Dom.create({"tag": "div", "append": holder, "className": "gtk_smart_entry_tag_with_tail"});
        var tag  = jsgtk.Dom.create({"tag": "span", "append": tagHolder , "text": this._entry.value});
                
        this._entry.value = "";
        this._entry.style.display = "none";

    }else{
        this._entry.style.display = "none";
        self._holder.childNodes[1].childNodes[0].style.display = "";
    }
};

/**
 * Set the input value of smartEntry with tail
 * @param {String} value The value to be set as smartEntry tag
 */

gtk.SmartEntry.prototype.setTailInput = function(value){
    this._entry.value = value;
    if(this._holder.childNodes[1]) this._holder.childNodes[1].childNodes[0].style.display = "none";
    this._createTailTag(this._holder);
    this._entry.style.position = "absolute";
    this._entry.style.top = "3px";
};

/**
 * set the entry to accept only number
 * @param {Integer} index The index of entry to be set. set index = -1 to set all the input in smart entry to accept only number
 */
gtk.SmartEntry.prototype.setAcceptNumber = function(index){
    var self = this;
    
    if(index == -1){
        this._bindEntryNumberOnlyEvent(this._entry);
        return;
    }
    
    if(this._allowLabel){
        var indexDom = this._holder.childNodes[index];
        if(indexDom && indexDom.firstChild.className != "gtk_smart_entry_fixed_tag")
            var eventId = jsgtk.Event.addDomListener(indexDom, "click", function(){
                self._bindEntryNumberOnlyEvent(indexDom.childNodes[1]);
            }, false);
    }      
};

/** @private */
gtk.SmartEntry.prototype._bindEntryNumberOnlyEvent = function(entry){
    var self = this;
    var value = "";
    jsgtk.Event.addDomListener(entry, "keypress", function(e){
        if (e.charCode > 31 && (e.charCode < 48 || e.charCode > 57)){
            window.setTimeout(function(){entry.value = value;}, 1);
        }
        else{
            window.setTimeout(function(){value = entry.value;}, 1);
        }
    });
    return;
};

/**
 * set the entry to accept limited characters
 * @param {Integer} index The index of entry to be set. set index = -1 to apply to all the input in smart entry
 * @param {Integer} limitedChar The limited number of characters which is abled to be type to entry
 */
gtk.SmartEntry.prototype.setLimitCharacter = function(index, limitedChar){
    var self = this;
    
    if(index == -1){
        this._entry.setAttribute("maxlength", limitedChar);
        return;
    }
    
    if(this._allowLabel){
        var indexDom = this._holder.childNodes[index];
        if(indexDom && indexDom.firstChild.className != "gtk_smart_entry_fixed_tag")
            var eventId = jsgtk.Event.addDomListener(indexDom, "click", function(){
                indexDom.childNodes[1].setAttribute("maxLength", limitedChar);
            }, false);
    }
};

/**
 * Get the values of smartEntry tag
 * @return The array of tag values
 */
gtk.SmartEntry.prototype.getValues = function(){
    var tags = this._holder.childNodes;
    var values = [];
    
    if(this._allowLabel){
        for(var i = 0, len = tags.length; i < len; i++){
            if(tags[i].childNodes[0].className == "gtk_smart_entry_fixed_tag"){
                values.push(tags[i].childNodes[0].innerHTML);
                continue;
            }
            values.push(tags[i].childNodes[2] ? tags[i].childNodes[2].childNodes[0].innerHTML : "");
        }
    }
    else{
        if(this._isAllowOneTag){
            var value = "";
            if(tags[2])  value = tags[2].childNodes[0].innerHTML;
            values = value + tags[0].childNodes[0].innerHTML;
        }
        else{
            for(var i =0, len = tags.length; i < len; i++){
                values.push(tags[i].childNodes[0].innerHTML);
            }
        }
    }    
    return values;
};

/**
 * Create tag in smartEntry to a specific order
 * @param {integer} index The index of tag to be created
 * @param {String} value The value of tag to be created
 */
gtk.SmartEntry.prototype.createTag = function(index, value){
    var tagHolder = this._holder.childNodes[index];
    if(this._allowLabel){
        if(!tagHolder) return;  
        if(tagHolder.childNodes[2]) tagHolder.removeChild(tagHolder.childNodes[2]);
        var tagDiv = this._createTagHolder(tagHolder, value);
        tagHolder.childNodes[0].style.display = "none";
        tagHolder.appendChild(tagDiv);
    }
    else{
        if(tagHolder){
            if(tagHolder.childNodes[0].className == "gtk_smart_entry_fixed_tag") return;
            tagHolder.childNodes[0].innerHTML = value;
        }
        else if(index == 0){
            var tagDiv = this._createTagHolder(tagHolder, value);
            this._holder.appendChild(tagDiv);
        }
        else if(this._holder.childNodes[index-1]){
                var tagDiv = this._createTagHolder(tagHolder, value);
                this._holder.appendChild(tagDiv);
        }
    }
};

gtk.SmartEntry.prototype._createLabelTag = function(tagHolder, value){
    if(tagHolder.childNodes[2]) {
        tagHolder.removeChild(tagHolder.childNodes[2]);
    }
    var tagDiv = this._createTagHolder(tagHolder, value);
    tagHolder.childNodes[0].style.display = "none";
    tagHolder.appendChild(tagDiv);
};

/**
 * Create tags in smartEntry with the specified valued
 * @param {Array} values The value of tags to be created
 */
gtk.SmartEntry.prototype.createTags = function(values){
    if(this._allowLabel){
        for(var i = 0, len = values.length; i < len; i++){
            this._createLabelTag(this._holder.childNodes[i], values[i]);
        }    
    }
    else{
        for(var i = 0, len = values.length; i < len; i++){
            var tagDiv = this._createTagHolder("", values[i]);
            this._holder.appendChild(tagDiv);
        }
    }
    
};

/**
 * Delete the specified tag in smartEntry
 * @param {Integer} index The index of tag to be deleted
 */
gtk.SmartEntry.prototype.deleteTag = function(index){
    var tagHolder = this._holder.childNodes[index];
    if(!tagHolder) return;
    if(this._allowLabel){
        if(tagHolder.childNodes[2]){
            tagHolder.removeChild(tagHolder.childNodes[2]);
            tagHolder.childNodes[0].style.display = "";
        }
    }
    else{
        this._holder.removeChild(tagHolder);
    }
};
