/**
 * A widget with an entry which is able to create many tags
 * @param {Boolean} allowLabel The setting of label. If true, the labels can be displayed inside smart entry
 * @constructor
 */
gtk.SmartEntryDefault = function(allowLabel){
    gtk.Widget.apply(this);
    
    var self = this;
    this._minWidth 		= this._width 	= 150;
    this._minHeight 		= this._height = 75;
    this._entryWidth 	= 50;
    this._textBoxError 	= false;
	
	/** @private */
	this._render = function(){
	    this._domObj.className += " gtk_smart_entry_default";
	    this._domObj.style.width = this._width + "px";
	    this._domObj.style.height = this._height + "px";
	    this._holder = jsgtk.Dom.create({"tag": "div", append: this._domObj, className: "gtk_smart_entry_default_tag_holder"});
		this._entry = jsgtk.Dom.create({"tag": "input", append: this._domObj, "className": "gtk_smart_entry_default_input"});
		this._entry.style.width = this._entryWidth + "px";

	};	

	this._render();
	this._initEvents();
};

gtk.SmartEntryDefault.prototype = new gtk.Widget();
gtk.SmartEntryDefault.prototype.constructor = gtk.SmartEntryDefault;

/** @private */
gtk.SmartEntryDefault.prototype._initEvents = function(){
	var self = this;
	jsgtk.Event.addDomListener(this._domObj, "click", function(){
		self._entry.focus();
	});
	
	this._bindEntryLostFocusEvent();
    
    this._bindEntryKeypressEvent(this._entry);
};

gtk.SmartEntryDefault.prototype.showError = function(isOnError){
	 this._textBoxError = (typeof isOnError !== 'undefined' && typeof isOnError === 'boolean') ? isOnError : this._textBoxError;
    
    if(this._textBoxError) this._domObj.className += " gtk_smart_entry_onerror";
    else this._domObj.className = "gtk_widget gtk_smart_entry_default";
};

/** @private */
gtk.SmartEntryDefault.prototype._bindEntryLostFocusEvent = function(){
    var self = this;
    
    jsgtk.Event.addDomListener(this._entry, "blur", function(){        
        self._createTag();
    }, false);
};

/** @private */
gtk.SmartEntryDefault.prototype._bindEntryKeypressEvent = function(entry){
	var self = this;
    var tempSpan = document.createElement("span");
    var tempWidth;
    
    jsgtk.Event.addDomListener(entry, "keypress", function(e){
        switch(e.keyCode){
            case gtk.ENTER_KEY:
                self._bindEntryEnterKeyEvent();
                break;
            case gtk.BACKSPACE_KEY:
                self._bindEntryBackspaceKeyEvent();
                break;
            case gtk.DELETE_KEY:
                self._bindEntryDeleteKeyEvent(e);
                break;
            case gtk.TAB_KEY:
            	self._bindEntryTabKeyEvent(e);
            default:
            	if(!self._isAllowOneTag){
		            tempSpan.innerHTML = self._entry.value;
		            tempWidth = self._calculateWidth(tempSpan) + self._entryWidth;
		            self._entry.style.width = (tempWidth) < self._width - 2? tempWidth + "px" : self._width - 2 + "px";
	            }
        }
        
        if((e.which == gtk.A_KEY || e.which == gtk.CAP_A_KEY) && e.ctrlKey){
            self._bindEntryCtrlAKeyEvent(entry);
        }
    });
};

/** @private */
gtk.SmartEntryDefault.prototype._bindEntryEnterKeyEvent = function(entry){
    this._createTag();
};

/** @private */
gtk.SmartEntryDefault.prototype._bindEntryBackspaceKeyEvent = function(){
	var tags = this._holder.childNodes;
    var len = tags.length;
    if(len == 0) return;
    var entry = this._entry;
    var tag = tags[len-1];

    if(len > 0 && entry.value == "" && tag.childNodes[0].className != "gtk_smart_entry_default_fixed_tag"){
        if(tag.selected){
            tag.parentNode.removeChild(tag);
            entry.style.visibility = "visible";
            this._unSelectTags();
        }
        else{
            tag.selected = true;
            tag.className += " gtk_smart_entry_default_tag_selected";
            entry.style.visibility = "hidden";
        }
    }
};

/** @private */
gtk.SmartEntryDefault.prototype._bindEntryDeleteKeyEvent = function(e){
    var tags = this._holder.childNodes;
    var len = tags.length;
    var entry = this._entry;

    if(len > 0 && entry.value == ""){
        var length = this._holder.childNodes.length;
        for(var i = length-1 ; i >= 0 ; i--){
            if(tags[i].selected) this._holder.removeChild(tags[i]);
        }
        
        entry.style.visibility = "visible";
        entry.focus();
    }
};

gtk.SmartEntryDefault.prototype._bindEntryTabKeyEvent = function(e){
	this._createTag();
	this._entry.value = "";
	jsgtk.Event.stopDefault(e);
};

/** @private */
gtk.SmartEntryDefault.prototype._bindEntryCtrlAKeyEvent = function(){
	var entry = this._entry;
    if(entry.value != "") return;
    var tags = this._holder.childNodes;
    var len = tags.length;
    for(var i = 0; i < len; i++){
    	if(tags[i].tagName == "INPUT") continue;
        if(tags[i].childNodes[0].className == "gtk_smart_entry_default_fixed_tag") continue;
        tags[i].className += " gtk_smart_entry_default_tag_selected";
        tags[i].selected = true;
    }
    this._entry.style.visibility = "hidden";
};

/** @private */
gtk.SmartEntryDefault.prototype._unSelectTags = function(){
    var tags = this._holder.childNodes;
        
    for(var i = tags.length; i--;){
        if(tags[i].selected == true){
            tags[i].selected = false;
            tags[i].className = "gtk_smart_entry_tag_holder";
        }
    }
};

/** @private */
gtk.SmartEntryDefault.prototype._createTag = function(value){
	var tagValue = value || this._entry.value;
	if(tagValue){
        var self = this;

        var tag = jsgtk.Dom.create({"tag": "div", "className": "gtk_smart_entry_default_tag"});
        	var spanTag  = jsgtk.Dom.create({"tag": "span", "append": tag , "text": tagValue});
        	var closeButton = jsgtk.Dom.create({"tag": "div", append: tag , "className": "gtk_smart_entry_default_close"});
        
    	this._holder.appendChild(tag);
        
        jsgtk.Event.addDomListener(closeButton, "click", function(e){
            self._holder.removeChild(tag);
            jsgtk.Event.stopBubble(e);
        }, false);
                
        this._entry.value = "";
        this._entry.style.width = this._entryWidth + "px";
    }
};

/** @private */
gtk.SmartEntryDefault.prototype._calculateWidth = function(dom){
    var tempDom = dom.cloneNode(true);
    tempDom.style.display = "none";
    return jsgtk.Util.fullWidth(tempDom);
};

/**
 * Set the minimum size of SmartEntry
 * @param {Integer} width the minimum width to be set
 * @param {Integer} height the minimum height to be set
 */
gtk.SmartEntryDefault.prototype.setSize = function(width, height){
    this._width = this._minWidth = width;
    this._height = this._minHeight = height;
    var domObjStyle = this._domObj.style;
    domObjStyle.width = this._width + "px";
    domObjStyle.height = this._height + "px";
    
    this.emit('bubble', this._width, this._height);
};

/**
 * Method to create a smartEntry fixed tag
 * @param {String} fixedTag the value of fixed tag to be created
 */
gtk.SmartEntryDefault.prototype.createFixedTag = function(fixedTag){
    var tagHolder = jsgtk.Dom.create({"tag": "div", "append": this._holder, "className": "gtk_smart_entry_default_tag"});
    var tag  = jsgtk.Dom.create({"tag": "span", "append": tagHolder , "text": fixedTag, "className": "gtk_smart_entry_default_fixed_tag"});
};

/**
 * Create tag in smartEntry to a specific order
 * @param {integer} index The index of tag to be created
 * @param {String} value The value of tag to be created
 */
gtk.SmartEntryDefault.prototype.createTag = function(index, value){
    var tagHolder = this._holder.childNodes[index];
    
    if(tagHolder){
        if(tagHolder.childNodes[0].className == "gtk_smart_entry_default_fixed_tag") return;
        tagHolder.childNodes[0].innerHTML = value;
    }
    else if(index == 0){
        var tagDiv = this._createTag(value);
    }
    else if(this._holder.childNodes[index-1]){
        this._createTag(value);
    }
};

/**
 * Get the values of smartEntry tag
 * @return The array of tag values
 */
gtk.SmartEntryDefault.prototype.getValues = function(){
    var tags = this._holder.childNodes;
    var values = [];

    if(this._isAllowOneTag){
        var value = "";
        if(tags[2])  value = tags[2].childNodes[0].innerHTML;
        values = value + tags[0].childNodes[0].innerHTML;
    }
    else{
        for(var i = 0, len = tags.length; i < len; i++){
            values.push(tags[i].childNodes[0].innerHTML);
        }
    }

    return values;
};

/**
 * set the entry to accept limited characters
 * @param {Integer} limitedChar The limited number of characters which is abled to be type to entry
 */
gtk.SmartEntryDefault.prototype.setLimitCharacter = function(limitedChar){
    this._entry.setAttribute("maxlength", limitedChar);
};

/**
 * set the entry to accept only number
 * @param {Integer} index The index of entry to be set. set index = -1 to set all the input in smart entry to accept only number
 */
gtk.SmartEntryDefault.prototype.setAcceptNumber = function(index){
    var self = this;
    var value = "";
    var entry = this._entry;
    jsgtk.Event.addDomListener(entry, "keypress", function(e){
        if (e.charCode > 31 && (e.charCode < 48 || e.charCode > 57)){
        	entry.value = value;
            window.setTimeout(function(){entry.value = value;}, 0);
        }
        else{
            window.setTimeout(function(){value = entry.value;}, 0);
        }
    });
};

/**
 * Add the tail to smartEntry
 * @param {String} rightTail The value of right tail to be set
 * @param {String} label (Optional) The value of label to be set in entry before tail
 */
gtk.SmartEntryDefault.prototype.setTailLabel = function(rightTail, label){
    var self = this;
    this._isAllowOneTag = true;
    this.createFixedTag(rightTail);
    this._holder.childNodes[0].className = "gtk_smart_entry_default_right_label";
    if(label){
        this._entry.style.display = "none";
        var tagHolder = jsgtk.Dom.create({"tag": "div", "append": this._holder, "className": "gtk_smart_entry_default_label"});
        var labelSpan  = jsgtk.Dom.create({"tag": "span", "append": tagHolder , "text": label, "className": "gtk_smart_entry_default_label"});
        tagHolder.style.cssFloat = "right";
//        
//        jsgtk.Event.addDomListener(this._areaDom, "click", function(){
//            if(labelSpan.style.display == "none"){
//                var tag = self._holder.getElementsByClassName("gtk_smart_entry_tag_with_tail")[0];
//                if(tag){
//                    var value = tag.childNodes[0].innerHTML;
//                    tag.parentNode.removeChild(tag);
//                    self._entry.style.display = "";
//                    self._entry.value = value;
//                    self._entry.focus();
//                }

//                return;
//            } 
//            self._entry.style.display = "";
//            self._entry.style.position = "absolute";
//            self._entry.style.top = "3px";
//            labelSpan.style.display = "none";
//            self._entry.focus();
//        });
    }
    
    var tempWidth = this._calculateWidth(this._holder.childNodes[0]);
    this._entry.style.width = this._width - tempWidth - 6 + "px";
    this._entry.style.textAlign = "right";
    this._entry.style.marginTop = "3px";

};

/**
 * Set the minimum size of SmartEntry
 * @param {Integer} width the minimum width to be set
 * @param {Integer} height the minimum height to be set
 */
gtk.SmartEntryDefault.prototype.setSize = function(width, height){
    this._width = this._minWidth = width;
    this._height = this._minHeight = height;
    var domObjStyle = this._domObj.style;
    domObjStyle.width = this._width + "px";
    domObjStyle.height = this._height + "px";
    
    this.emit('bubble', this._width, this._height);
};
