/**
 * A widget with an entry which is able to create many tags
 * @constructor
 * @base gtk.Widget
 */
gtk.SmartEntryDropDown = function(){
    gtk.Widget.apply(this);
    /** 'smartentrychanged' signal */
	this.SMARTENTRYDROPDOWN_SMARTENTRYCHANGED = "smartentrychanged";

    this._minWidth 		= this._width 	= 150;
    this._minHeight 		= this._height = 10;
    this._entryWidth 	= 100;
    this._textBoxError	= false;
    this._ismouseMoved  = false;
    this._isParentSignalHandled = false;

	this._render = function(){
	    this._domObj.className += " gtk_smart_entry_drop_down_widget";
	    	this._holder = jsgtk.Dom.create({tag: "div", append: this._domObj, className: "gtk_smart_entry_drop_down_area"});
	    		this._tagHolder = jsgtk.Dom.create({tag: "div", append: this._holder, className: "gtk_smart_entry_drop_down_tag_group"});
	
			    this._entryHolder = jsgtk.Dom.create({tag: "div", append: this._holder, className: "gtk_smart_entry_drop_down_input_holder"});
				    this._labelEntry = jsgtk.Dom.create({tag: "span", append: this._entryHolder, className: "gtk_smart_entry_drop_down_label"});
				    this._entry = jsgtk.Dom.create({tag: "input", append: this._entryHolder, className: "gtk_smart_entry_drop_down_input"});

		this.__dropDown = jsgtk.Dom.create({tag: "div", "append": document.body, className: "gtk_smart_entry_drop_down_widget_holder"});
			jsgtk.Dom.hideElement(this.__dropDown);
	};

	this._render();
};

gtk.SmartEntryDropDown.prototype = new gtk.Widget();
gtk.SmartEntryDropDown.prototype.constructor = gtk.SmartEntryDropDown;
/**
 * Set mode to gtk.SmartEntryDropDown
 * @param {gtk.TreeStore} mode
 */
gtk.SmartEntryDropDown.prototype.setModel = function(model){
	this._controller.setModel(model);
};
/**
 * Show Error
 * @param {boolean} isOnError
 */
gtk.SmartEntryDropDown.prototype.showError = function(isOnError){
	 this._textBoxError = (typeof isOnError !== 'undefined' && typeof isOnError === 'boolean') ? isOnError : this._textBoxError;
    
    if(this._textBoxError) this._domObj.className += " gtk_smart_entry_onerror";
    else this._domObj.className = "gtk_widget gtk_smart_entry_drop_down_widget";
};
/** @private */
gtk.SmartEntryDropDown.prototype._renderDropDown = function(children, parentNode){
	var rowNodes = this._dropDownRendere._constructRowNodes(children, parentNode);
	this._appendDropDown(rowNodes);
};
/** @private */
gtk.SmartEntryDropDown.prototype._renderTag = function(){
	var tag = this._tagRendere._createTag(null, this._dropDownRendere.getSelectedRowNode());
	this._tagHolder.appendChild(tag);
};
/** @private */
gtk.SmartEntryDropDown.prototype._resetEntry = function(){
	this._entry.focus();
	this._entry.value = "";
	this._entry.style.width = this._entryWidth + 5 + "px";
};
/** @private */
gtk.SmartEntryDropDown.prototype._clearDropDown = function(){
	this.__dropDown.innerHTML = "";
};
/** @private */
gtk.SmartEntryDropDown.prototype._handleRowNodeSelected = function(){
	var selectedRowNode = this._dropDownRendere.getSelectedRowNode();
	this._entry.value = selectedRowNode.firstChild.nodeValue;
	this._renderTag();
	this.__hideDropDown();
	this._resetEntry();
	this.__showLabel();
};
/** @private */
gtk.SmartEntryDropDown.prototype._handleParentSignal = function(){
    var self = this;
    this._isParentSignalHandled = true;
    var parentWidget = this.getParent();
    parentWidget.connect("scrolled", function(){
        self.__hideDropDown();
    });
};
/** @private */
gtk.SmartEntryDropDown.prototype._resetDropDownPosition = function(){
    var top = jsgtk.Util.pageY(this._domObj);
    var left = jsgtk.Util.pageX(this._domObj);

    var header = parseInt(this._holder.offsetHeight);

    var scrollY = this._getNearestScrollerY();

    this.__dropDown.style.top = top + header + 5 - scrollY + "px";
    this.__dropDown.style.left = left + 1 + "px";
};
/** @private */
gtk.SmartEntryDropDown.prototype._getNearestScrollerY = function(){
    var scrollY = 0;
    var parentNode = this._domObj.parentNode;
    while(parentNode != null){
        if(/gtk_scroller_base/.test(parentNode.className)){
            scrollY = parentNode.scrollTop;
            this._addEventMouseWheel(parentNode);
            break;
        }
        parentNode = parentNode.parentNode;
    }
    return scrollY;
};
/** @private */
gtk.SmartEntryDropDown.prototype._addEventMouseWheel = function(scrollObj){
    var self = this;
    jsgtk.Event.addDomListener(scrollObj, 'scroll', function(e){
	    self.__hideDropDown();
	}, false);
};
/** @private */
gtk.SmartEntryDropDown.prototype._initEvent = function(){
	var self = this;

	jsgtk.Event.addDomListener(this._domObj, "click", function(e){
		self.setFocus();
		if(!self._isParentSignalHandled) self._handleParentSignal();
		jsgtk.Event.stopBubble(e);
	});
	this.__bindInputEvent();
	
	jsgtk.Event.addDomListener(this.__dropDown, "click", function(e){
		var node = e.target;
		if(jsgtk.Util.isMatched("gtk_smart_entry_drop_down_element", node.className))
			self._handleRowNodeSelected();

		jsgtk.Event.stopBubble(e);
	});
	
	jsgtk.Event.addDomListener(this.__dropDown, "mouseover", function(e){

	    if(!self._ismouseMoved) return;
	    
		var node = e.target;
		if(jsgtk.Util.isMatched("gtk_smart_entry_drop_down_element", node.className))
			self._dropDownRendere.hilightRowNode(node);
		self._ismouseMoved = false;
	});
	
	jsgtk.Event.addDomListener(this.__dropDown, "mousemove", function(e){
        self._ismouseMoved = true;
	});
	
	jsgtk.Event.addDomListener(document.body, "click", function(e){
		self.__hideEntry();
		self.__hideDropDown();
	});
};
/** @private */
gtk.SmartEntryDropDown.prototype._hilightRow = function(path){
	this._dropDownRendere.hilightRowById(path);
};
/**
 * Set gtk.SmartEntryDropDown focus
 */
gtk.SmartEntryDropDown.prototype.setFocus = function(){
	this.__showEntry();

	this._tagRendere.unSelectTags();
	this._dropDownRendere._deHilightSelectedRow();

	this._entry.focus();
}
/** @private */
gtk.SmartEntryDropDown.prototype.__showEntry = function(){
	this._entryHolder.style.display = "inline-block";
	this._entry.parentNode.style.visibility = "visible";
};
/** @private */
gtk.SmartEntryDropDown.prototype.__hideEntry = function(){
	this._entry.parentNode.style.visibility = "hidden";
};
/** @private */
gtk.SmartEntryDropDown.prototype._appendDropDown = function(dropDown){
	this.__hideLoading();
	this.__dropDown.appendChild(dropDown);
};
/** @private */
gtk.SmartEntryDropDown.prototype.__showLoading = function(){
	this.__dropDown.className += " gtk_smart_entry_drop_down_loading";
};
/** @private */
gtk.SmartEntryDropDown.prototype.__hideLoading = function(){
	this.__dropDown.className = "gtk_smart_entry_drop_down_widget_holder";
};
/** @private */
gtk.SmartEntryDropDown.prototype._isCtrlAKeyPressed = function(e){
	return (e.which == gtk.A_KEY || e.which == gtk.CAP_A_KEY) && e.ctrlKey;
};
/** @private */
gtk.SmartEntryDropDown.prototype.__fireDownArrowKey = function(){
	this._dropDownRendere.hilightNextRowNode();
};
/** @private */
gtk.SmartEntryDropDown.prototype.__fireUpArrowKey = function(){
	this._dropDownRendere.hilightPreviousRowNode();
};
/** @private */
gtk.SmartEntryDropDown.prototype.__fireLeftArrowKey = function(){
	if(!this._tagRendere.isTagExisted()) return;

	var tag = this._tagRendere.isPreviousTagExisted();
	if(!tag) return;
	
	if(this._tagRendere.isFixedTag(tag)) return;

	this._tagRendere.unSelectTag();
	this._tagRendere.selectTag(tag || this._tagRendere.getSelectedTag());

	this.__hideEntry();
    this.__hideDropDown();
};
/** @private */
gtk.SmartEntryDropDown.prototype.__fireRightArrowKey = function(){
	if(!this._tagRendere.isTagExisted() || !this._tagRendere.getSelectedTag()) return;

	var nextTag = this._tagRendere.isNextTagExisted();

	if(!nextTag){
		this.__showEntry();
		return;
	} 
	
	this._tagRendere.unSelectTag();
	this._tagRendere.selectTag(nextTag || this._tagRendere.getSelectedTag());
};
/** @private */
gtk.SmartEntryDropDown.prototype.__fireEnterKey = function(){
	if(this._dropDownRendere.getSelectedRowNode()){
		this._handleRowNodeSelected();
	
		this._dropDownRendere._deHilightSelectedRow();
	}
};
/** @private */
gtk.SmartEntryDropDown.prototype.__fireBackSpace = function(value){
    var tags = this._tagHolder.childNodes;

	var entry = this._entry;
	if(entry.value == "") this.__showLabel();

    var len = tags.length;
    if(len < 1) return;

    var tag = tags[len-1];

    if(len > 0 && value == "" && tag.childNodes[0].className != "gtk_smart_entry_drop_down_fixed_tag"){

        if(tag.selected){
			
            tag.parentNode.removeChild(tag);
			this.__showEntry();
        	this._tagRendere.unSelectTags();
        }
        else{
    		this._tagRendere.unSelectTags();
            tag.selected = true;
            tag.className += " gtk_smart_entry_drop_down_tag_selected";
			this.__hideEntry();
            this.__hideDropDown();
        }
    }
    return;
};
/** @private */
gtk.SmartEntryDropDown.prototype.__fireDeleteKey = function(){
	var tags = this._tagHolder.childNodes;
    var len = tags.length;
    var entry = this._entry;

    if(len > 0 && entry.value == ""){
        for(var i = len-1 ; i >= 0 ; i--){
        	var tag = tags[i];
            if(tag.selected){
				this._tagHolder.removeChild(tag);
			}
        }
		this._tagRendere.unSelectTag();
        this.__showEntry();
        entry.focus();
    }
};
/** @private */
gtk.SmartEntryDropDown.prototype._selectAllTags = function(){
	this._tagRendere.selectTags();
	this.__hideEntry();
	this.__hideDropDown();
};
/** @private*/
gtk.SmartEntryDropDown.prototype.__showDropDown = function(){
	jsgtk.Dom.showElement(this.__dropDown);
};
/** @private*/
gtk.SmartEntryDropDown.prototype.__hideDropDown = function(){
	jsgtk.Dom.hideElement(this.__dropDown);
};
/** @private */
gtk.SmartEntryDropDown.prototype.__hideLabel = function(){
	this._labelEntry.style.visibility = "hidden";
};
/** @private */
gtk.SmartEntryDropDown.prototype.__showLabel = function(){
	this._labelEntry.style.visibility = "";
};
/** @private */
gtk.SmartEntryDropDown.prototype.getValues = function(){
	var children = this._holder.childNodes[0].childNodes,
	len = children.length;
	var values = [];
	for(var i = 0; i < len; i++){
		values.push(children[i].childNodes[0].innerHTML);
	}
	return values;	
};
/** @private */
gtk.SmartEntryDropDown.prototype.getItems = function(){
	var children = this._holder.childNodes[0].childNodes,
	len = children.length;
	var values = [];
	for(var i = 0; i < len; i++){
		values.push(children[i]["valueObject"]);
	}
	return values;
};
/**
 * Method to create a smartEntry fixed tag
 * @param {String} fixedTag the value of fixed tag to be created
 */
gtk.SmartEntryDropDown.prototype.createFixedTag = function(obj){
    var tagHolder = jsgtk.Dom.create({"tag": "div", "append": this._tagHolder, "className": "gtk_smart_entry_drop_down_tag_holder"});
    var tag  = jsgtk.Dom.create({"tag": "span", "append": tagHolder , "text": obj.value, "className": "gtk_smart_entry_drop_down_fixed_tag"});
	tagHolder.object = obj;    
};
/**
 * Create tag in smartEntry to a specific order
 * @param {integer} index The index of tag to be created
 * @param {String} value The value of tag to be created
 */
gtk.SmartEntryDropDown.prototype.createTag = function(obj){
	var tag = this._tagRendere._createTag(obj["display"]);
	this._tagHolder.appendChild(tag);
    tag.valueObject = obj;
};
/**
 * Set text to be displayed inside entry
 * @param {string} label
 */
gtk.SmartEntryDropDown.prototype.setLabel = function(label){
	this._labelEntry.innerHTML = "";
	this._labelEntry.appendChild(document.createTextNode(label));
	this.__updateEntryWidth();
};
/** @private */
gtk.SmartEntryDropDown.prototype.__updateEntryWidth = function(){
	var tempSpan = this._labelEntry.cloneNode(true);
	var width = jsgtk.Util.fullWidth(tempSpan);
	if(width < 100) width = 100;
	this._entryWidth = width;
	this._entry.style.width = width + 10 + "px";
	this._entry.parentNode.style.width = width + 10 + "px";
};
/** @private */
gtk.SmartEntryDropDown.prototype.resizeWidthComposite = function(width){
	this._width = width;
	this._domObj.style.width = width + 'px';
	this.__dropDown.style.width = width - 2 + "px";
};
/** @private */	
gtk.SmartEntryDropDown.prototype.resizeHeightComposite = function(height){
	this._height = height;
	this._domObj.style.height = height - 5 + 'px';
};
gobject.signalNew("_rendersubnodes", gtk.SmartEntryDropDown, null, null, []);
gobject.signalNew("_setselectedvalues", gtk.SmartEntryDropDown, null, null, []);
gobject.signalNew("_search", gtk.SmartEntryDropDown, null, null, []);
gobject.signalNew("smartentrychanged", gtk.SmartEntryDropDown, null, null, []);
