/**
 * A widget with an entry which is able to create many tags
 * @param {Boolean} allowLabel The setting of label. If true, the labels can be displayed inside smart entry
 * @constructor
 */
gtk.SmartEntryDropDownLabel = function(){
    gtk.Widget.apply(this);
    
    var self 					= this;
    this._minWidth 			= this._width 	= 125;
    this._minHeight 			= this._height = 50;
    this._entryWidth 		= 100;
    this.__entryDropDown 	= [];
    this._textBoxError 		= false;
	 this._selectedItem 		= null;
    this._index 				= 0;
    
    /** @private */
	this._render = function(){
	    this._domObj.className += " gtk_smart_entry_drop_down_label_widget";
	    var domObjStyle = this._domObj.style;
	    domObjStyle.width = this._width + "px";
	    domObjStyle.height = this._height + "px";
	    this.__controller = new gtk.SmartEntryDropDownLabelController(this);
	};	

	this._render();
};

gtk.SmartEntryDropDownLabel.prototype = new gtk.Widget();
gtk.SmartEntryDropDownLabel.prototype.constructor = gtk.SmartEntryDropDownLabel;

gtk.SmartEntryDropDownLabel.prototype.constructView = function(model, label, index){
    model? this.__controller._setModel(this, model, label, this._width) : this.__controller._createView(this, label, this._width);
};

gtk.SmartEntryDropDownLabel.prototype.showError = function(isOnError){
	 this._textBoxError = (typeof isOnError !== 'undefined' && typeof isOnError === 'boolean') ? isOnError : this._textBoxError;
    
    if(this._textBoxError) this._domObj.className += " gtk_smart_entry_onerror";
    else this._domObj.className = "gtk_widget gtk_smart_entry_drop_down_label_widget";
};

gtk.SmartEntryDropDownLabel.prototype._appendDropDown = function(dropDown){
	this._domObj.appendChild(dropDown);
};

gtk.SmartEntryDropDownLabel.prototype._appendEntry = function(entryTag){
	var entryTagDom = entryTag._getDom();
	this._domObj.appendChild(entryTagDom);
};

gtk.SmartEntryDropDownLabel.prototype.createFixedTag = function(obj){
    var tagHolder = jsgtk.Dom.create({"tag": "div", "append": this._domObj, "className": "gtk_smart_entry_drop_down_tag_holder"});
    var tag  = jsgtk.Dom.create({"tag": "span", "append": tagHolder , "text": obj.value, "className": "gtk_smart_entry_drop_down_label_fixed_tag"});
	tagHolder.object = obj;
};

gtk.SmartEntryDropDownLabel.prototype.createTag = function(obj, index){
	this.__controller._createTag(obj, index);
};

gtk.SmartEntryDropDownLabel.prototype.getItems = function(){
	var children = this._domObj.childNodes,
		len = children.length;
	var items = [];
	var value = "";
	for(var i = 0; i < len; i++){
		var child = children[i];
		var object = null;
		if(/gtk_entry_tag/.test(child.className)){
			value = /gtk_entry_tag_holder/.test(child.lastChild.className) ? child.lastChild.firstChild.firstChild.nodeValue : "";
			object = {value: value};
		}
		else if(/gtk_drop_down/.test(child.className)){
			object = child.firstChild.lastChild.object;
			if(!object) object = {value: ""};
		}else{
			object = child.object;
		}
		items.push(object);
	}
	return items;
};

gtk.SmartEntryDropDownLabel.prototype.resizeWidthComposite = function(width){
	this.resize(width, this._height);
};

gtk.SmartEntryDropDownLabel.prototype.resize = function(width, height){
	this._resizeWidth(width);
	this._resizeHeight(height);
};

gtk.SmartEntryDropDownLabel.prototype._resizeWidth = function(width){
	this._domObj.style.width = width + 'px';
	this._height = width;
	this.__controller._updateChildrenWidth(width);
};

