/**
 * Factory of all gtk.SmartEntry
 * @param {string} type of gtk.SmartEntry : gtk.SMART_ENTRY_DROPDOWN_FIXED_MODEL, gtk.SMART_ENTRY_DROPDOWN_DYNAMIC_MODEL
 * @constructor
 */
gtk.SmartEntryFactory = function(type){
	var smartEntry;
	switch (type){
		case gtk.SMART_ENTRY_DROPDOWN_FIXED_MODEL: 
			smartEntry = new gtk.SmartEntryDropDownFacade(new gtk.SmartEntryDropDownFixedModel());
			break;
		case gtk.SMART_ENTRY_DROPDOWN_DYNAMIC_MODEL:
			smartEntry = new gtk.SmartEntryDropDownFacade(new gtk.SmartEntryDropDownDynamicModel());
			break;
	}
	return smartEntry;
};

/**
 * SmartEntryDropDownFacade is an assistant class to create Smart Entry
 * @param {string} smartEntryDropDown The object of smart entry dropdown
 * @constructor
 * @base gtk.Object
 */
gtk.SmartEntryDropDownFacade = function(smartEntryDropDown){
    gtk.Object.apply(this);
    
    var vscroller = new gtk.VScroller();
    vscroller.getNode().style.border = "1px solid #D5D5D5";    
    vscroller.appendChild(smartEntryDropDown, true);
    
    vscroller.setModel       = function(model){smartEntryDropDown.setModel(model);};
    vscroller.showError      = function(isOnError){smartEntryDropDown.showError(isOnError);};
    vscroller.getValues      = function(){return smartEntryDropDown.getValues();}
    vscroller.getItems       = function(){return smartEntryDropDown.getItems()};
    vscroller.createFixedTag = function(obj){smartEntryDropDown.createFixedTag(obj);};
    vscroller.createTag      = function(obj){smartEntryDropDown.createTag(obj);};
    vscroller.setLabel       = function(text){smartEntryDropDown.setLabel(text);};
    vscroller.setFocus       = function(){smartEntryDropDown.setFocus();};
    
    smartEntryDropDown.connect("smartentrychanged", function(value){
        vscroller.emit("smartentrychanged", value);
    })
    
    return vscroller;
};

gobject.signalNew("smartentrychanged", gtk.Scroller, null, null, []);
