/**
 * A widget with an entry and displayed label which is able to create many tags
 * @param {Boolean} allowLabel The setting of label. If true, the labels can be displayed inside smart entry
 * @constructor
 */
gtk.SmartEntryLabel = function(){
    gtk.Widget.apply(this);
    
    var self 				= this;
    this._minWidth 		= this._width = 150;
    this._minHeight 		= this._height = 75;
    this._entryWidth 	= 50;
    this._entry 			= null;
    this._eventStorage 	= [];
    this._keyEvent;
    this._selectedIndex = -1;
    this._textBoxError 	= false;
    
    /** @private */
	this._render = function(){
	    this._domObj.className += " gtk_smart_entry_label_widget";
	   	var domObjStyle = this._domObj.style;

	    domObjStyle.width = this._width + "px";
	    domObjStyle.height = this._height + "px";
	};	

	this._render();	
};

gtk.SmartEntryLabel.prototype = new gtk.Widget();
gtk.SmartEntryLabel.prototype.constructor = gtk.SmartEntryLabel;

/**
 * Set the fixed tag in the smart entry. Fixed tag cannot be deleted
 * @param {String} arguments Label string to be set
 */
gtk.SmartEntryLabel.prototype.setLabel = function(){
    this._smartEntryLabelHolder = [];
    var self = this;

    for(var i = 0, len = arguments.length; i < len; i++){
        var label = arguments[i];
        var tag = jsgtk.Dom.create({"tag": "div", "append": this._domObj, "className": "gtk_smart_entry_label_tag"});
        var labelSpan  = jsgtk.Dom.create({"tag": "span", "append": tag , "text": label, "className": "gtk_smart_entry_label_label"});
        var input = jsgtk.Dom.create({"append": tag, "tag": "input", "type": "text", "value": "", "className": "gtk_smart_entry_label_input"});
        input.style.display = "none";
    }
    
    var tags = this._domObj.childNodes;
    this._smartEntryLabelHolder = tags;
    
    for(var i = 0, len = tags.length; i< len; i++){
        if(tags[i].childNodes[0].className == "gtk_smart_entry_label_fixed_tag") continue;
        (function(){
            var j = i;
            jsgtk.Event.addDomListener(tags[j], "click", function(){
            	if(tags[j].childNodes[1].style.display != "none") return;
            	var tagValue = "";
                if(tags[j].childNodes[2]) tagValue = tags[j].childNodes[2].childNodes[0].innerHTML;
                var entry = self._showInput(tags[j], tagValue);
                self._index = j;

                self._createKeypressEventForInput(entry, tags, self._index);
               	self._bindEntryKeyupEvent(entry);
            }, false);
        })();    
    } 
};

gtk.SmartEntryLabel.prototype.showError = function(isOnError){
	 this._textBoxError = (typeof isOnError !== 'undefined' && typeof isOnError === 'boolean') ? isOnError : this._textBoxError;
    
    if(this._textBoxError) this._domObj.className += " gtk_smart_entry_onerror";
    else this._domObj.className = "gtk_widget gtk_smart_entry_label_widget";
};

/** @private */
gtk.SmartEntryLabel.prototype._showInput = function(tagHolder, tagValue){
    if(tagHolder.childNodes[2]) tagHolder.removeChild(tagHolder.childNodes[2]);
    var entry = tagHolder.childNodes[1];
	var label = tagHolder.childNodes[0];

    label.style.display = "";
    entry.style.display = "";
    entry.style.position = "absolute";
    entry.value = "";

    window.setTimeout(function(){entry.focus();}, 0);
    var self = this;
    

    jsgtk.Event.addDomListener(entry, "blur", function(){
        if(tagValue && entry.value == "") entry.value = tagValue;
        self._bindInputLostFocusEvent(entry, tagHolder);
        tagValue = "";
    });

    return entry;
};

/** @private */
gtk.SmartEntryLabel.prototype._createKeypressEventForInput = function(entry){
    var self = this;
    var tags = this._domObj.childNodes;
    var tempSpan = document.createElement("span");
    var tempWidth;

    jsgtk.Event.addDomListener(entry, "keypress", function(e){
        if(e.keyCode == gtk.ENTER_KEY){
            entry.blur();
            return;
        }
        
        if((e.which == gtk.A_KEY || e.which == gtk.CAP_A_KEY) && e.ctrlKey){
            if(entry.value != "") return;
            var len = tags.length;
            for(var i = 0; i < len; i++){
            	var tag = tags[i].childNodes;
                if(tag[0].className == "gtk_smart_entry_label_fixed_tag" || !tags[i].childNodes[2]) continue;
                if(tag[2].tagName == "DIV") {
                    tag[2].className += " gtk_smart_entry_label_tag_selected";
                    tags[i].selected = true;
                }
            }
            return;
        }
        var previousTag = entry.parentNode.previousSibling;
        
        if((e.keyCode == gtk.BACKSPACE_KEY) && !(entry.value || !previousTag)){
        	if(previousTag.selected){
        		previousTag.selected = false;
        		var previousTagChildren = previousTag.childNodes;
        		previousTag.removeChild(previousTagChildren[2]);
        		previousTagChildren[0].style.display = "";
        		var previousEntry = previousTagChildren[1];
        		previousEntry.style.display = "";
        		previousEntry.value = "";
        		previousEntry.style.position = "absolute";
        		previousEntry.focus();
        	}else{
        		if(previousTag.childNodes[2]){
        			previousTag.selected = true;
        			previousTag.childNodes[2].className += " gtk_smart_entry_label_tag_selected";
    			}
        	}
        	return;
        }
        
        if(e.keyCode == gtk.DELETE_KEY){
        	for(var i = tags.length; i--;){
        		var tag = tags[i];
        		if(tag.selected){
        			tag.selected = false;
        			var tagChildren = tag.childNodes;
        			tag.removeChild(tagChildren[2]);
		    		tagChildren[0].style.display = "";
		    		tagChildren[1].value = "";
        		}
        	}
        }
    
		if(e.keyCode == gtk.TAB_KEY){
			var nextTag = entry.parentNode.nextSibling;
			if(!nextTag) return;
			var tagValue = "";
			if(nextTag.childNodes[2]) tagValue = nextTag.childNodes[2].childNodes[0].innerHTML;

//TODO: problem with recursive, (might be solve by add event when input is created)

            var nextEntryDiv = self._showInput(nextTag, tagValue);
            self._createKeypressEventForInput(nextEntryDiv, tags);
	  		self._bindEntryKeyupEvent(nextEntryDiv);
            window.setTimeout(function(){nextEntryDiv.focus();}, 0);
            return;
        }
        
        tempSpan.innerHTML = entry.value;
        
        tempWidth = self._calculateWidth(tempSpan) + self._entryWidth - 5;
//        if(tempWidth < self._entryWidth) tempWidth = self._entryWidth;
        entry.style.width = (tempWidth) < self._width - 2? tempWidth + "px" : self._width - 2 + "px";
    });
};

/** @private */
gtk.SmartEntryLabel.prototype._bindEntryKeyupEvent = function(entry){
	jsgtk.Event.addDomListener(entry, "keyup", function(e){
		if(entry.value != "" && entry.previousSibling.style.display != "none"){
				entry.previousSibling.style.display = "none";
				entry.style.position = "relative";
		}
	
		if(entry.value == "" && entry.previousSibling.style.display == "none"){
			entry.style.position = "absolute";
			entry.previousSibling.style.display = "";
			entry.focus();
		
		}
    	
    });
};

/** @private */
gtk.SmartEntryLabel.prototype._createTag = function(tagHolder, value){
    var tagDiv = jsgtk.Dom.create({"tag": "div", "className": "gtk_smart_entry_label_tag_holder"});
    var tag  = jsgtk.Dom.create({"tag": "span", "append": tagDiv , "text": value});

    var closeButton = jsgtk.Dom.create({"tag": "div", append: tagDiv, "className": "gtk_smart_entry_label_close"});

    jsgtk.Event.addDomListener(closeButton, "click", function(){
        var label = tagDiv.parentNode.childNodes[0];
        label.style.display = "";
        label.nextSibling.style.position = "absolute";
        
        tagDiv.parentNode.removeChild(tagDiv);
    }, false);
    
    return tagDiv;
};

/** @private */
gtk.SmartEntryLabel.prototype._bindInputLostFocusEvent = function(entry, tagHolder){
    if(entry.value){
        if(tagHolder.childNodes[2]) tagHolder.removeChild(tagHolder.childNodes[2]);
        var tag = this._createTag("", entry.value);
        tagHolder.appendChild(tag);
        
        entry.style.display = "none";
        entry.previousSibling.style.display = "none";
    }
    else{
    	var tagChildren = tagHolder.childNodes;
        tagChildren[0].style.display = "";
        tagChildren[1].style.display = "none";
        
        if(tagChildren[2]) tagHolder.removeChild(tagChildren[2]);
    }  
};

/**
 * Method to create a smartEntry fixed tag
 * @param {String} fixedTag the value of fixed tag to be created
 */
gtk.SmartEntryLabel.prototype.createFixedTag = function(fixedTag){
    var tagHolder = jsgtk.Dom.create({"tag": "div", "append": this._domObj, "className": "gtk_smart_entry_label_tag_holder"});
    var tag  = jsgtk.Dom.create({"tag": "span", "append": tagHolder , "text": fixedTag, "className": "gtk_smart_entry_label_fixed_tag"});
};

/**
 * Create tags in smartEntry with the specified valued
 * @param {Array} values The value of tags to be created
 */
gtk.SmartEntryLabel.prototype.createTags = function(values){
    for(var i = 0, len = values.length; i < len; i++){
        this._createLabelTag(this._domObj.childNodes[i], values[i]);
    }        
};

/** @private */
gtk.SmartEntryLabel.prototype._createLabelTag = function(tagHolder, value){
    if(tagHolder.childNodes[2]) {
        tagHolder.removeChild(tagHolder.childNodes[2]);
    }
    var tagDiv = this._createTag(tagHolder, value);
    
    tagHolder.childNodes[0].style.display = "none";
    tagHolder.appendChild(tagDiv);
};

/**
 * Delete the specified tag in smartEntry
 * @param {Integer} index The index of tag to be deleted
 */
gtk.SmartEntryLabel.prototype.deleteTag = function(index){
    var tagHolder = this._domObj.childNodes[index];
    if(!tagHolder) return;

    if(tagHolder.childNodes[2]){
        tagHolder.removeChild(tagHolder.childNodes[2]);
        tagHolder.childNodes[0].style.display = "";
    }

};

/**
 * set the entry to accept only number
 * @param {Integer} index The index of entry to be set. set index = -1 to set all the input in smart entry to accept only number
 */
gtk.SmartEntryLabel.prototype.setAcceptNumber = function(index){
    var self = this;

    var indexDom = this._domObj.childNodes[index];
    if(indexDom && indexDom.firstChild.className != "gtk_smart_entry_fixed_tag")
        jsgtk.Event.addDomListener(indexDom, "click", function(){
            self._bindEntryNumberOnlyEvent(indexDom.childNodes[1]);
        });
};

/** @private */
gtk.SmartEntryLabel.prototype._bindEntryNumberOnlyEvent = function(entry){
    var self = this;
    var value = "";
    jsgtk.Event.addDomListener(entry, "keypress", function(e){
        if (e.charCode > 31 && (e.charCode < 48 || e.charCode > 57)){
            window.setTimeout(function(){entry.value = value;}, 0);
        }
        else{
            window.setTimeout(function(){value = entry.value;}, 0);
        }
    });
    return;
};

/**
 * set the entry to accept limited characters
 * @param {Integer} index The index of entry to be set. set index = -1 to apply to all the input in smart entry
 * @param {Integer} limitedChar The limited number of characters which is abled to be type to entry
 */
gtk.SmartEntryLabel.prototype.setLimitCharacter = function(index, limitedChar){
    var self = this;
    
    var indexDom = this._domObj.childNodes[index];
    if(indexDom && indexDom.firstChild.className != "gtk_smart_entry_fixed_tag")
        var eventId = jsgtk.Event.addDomListener(indexDom, "click", function(){
            indexDom.childNodes[1].setAttribute("maxLength", limitedChar);
        }, false);
};

/** @private */
gtk.SmartEntryLabel.prototype._calculateWidth = function(dom){
    var tempDom = dom.cloneNode(true);
    tempDom.style.display = "none";
    document.body.appendChild(tempDom);
    return jsgtk.Util.fullWidth(tempDom);
};
