gtk.TableBase = function(rows, columns, resizeable, spacing) {
	gtk.Widget.apply(this);

	this._resizeable     = resizeable === undefined  ? false : resizeable;
	this._rows           = rows || 0;
	this._columns        = columns || 0;
	this._spacing        = spacing || 0;
	this._width          = this._minWidth = 10;
	this._height         = this._minHeight = 10;
	this._maxChildWidth  = this._width / this._columns;
	this._maxChildHeight = this._height/ this._rows;
	this._widths         = new Array(this._columns);
	this._heights        = new Array(this._rows);
	this._render = function(){
	    var domObj            = this._domObj;
		domObj.className     += " gtk_table";
		domObj.style.width    = this._width + 'px';
		domObj.style.height   = this._height + 'px';
		domObj.style.position = "relative";
	};
	
	this._render();
	this._initEvent();
};

gtk.TableBase.prototype = new gtk.Widget();
gtk.TableBase.prototype.constructor = gtk.TableBase;

/** @private */
gtk.TableBase.prototype._initEvent = function(){
	var self = this;
    jsgtk.Event.addDomListener(this._domObj, "click", function(e){
   		self.emit("clicked");
	}, false);

};
/** @private */
gtk.TableBase.prototype._initWidthsHeights = function(){
	for(var i = 0, len = this._widths.length; i < len; i++) this._widths[i]  = {"width": this._maxChildWidth + this._spacing, "resizeable": true};
	for(var i = 0, len = this._heights.length; i < len; i++) this._heights[i] = {"height": this._maxChildHeight + this._spacing, "resizeable": true};
};
/** @private */
gtk.TableBase.prototype._setMaxChildWidthHeightHomo = function(child){
    var childWidth       = child.minWidth + 2   * child.xpadding;
    var childHeight      = child.minHeight + 2  * child.ypadding;
    this._maxChildWidth  = this._maxChildWidth  * child.xcells > childWidth ? this._maxChildWidth : childWidth / child.xcells;
    this._maxChildHeight = this._maxChildHeight * child.ycells > childHeight? this._maxChildHeight: childHeight / child.ycells;
};
/** @private */
gtk.TableBase.prototype._resetWidths = function(child, left, right, xoptions){
    var extraWidth = 0;
    for(var i = left; i < right; i++){
        extraWidth += this._widths[i]["width"];
    }
    
    //compare withs of predefined and the width of widget
    var childWidthComposite = child.getMinWidthComposite() + this._spacing;
    var biggerWidth = (extraWidth < childWidthComposite ? childWidthComposite : extraWidth);
    
    //generate offsetWidth to be added to the array withs
    offsetWidth = (biggerWidth - extraWidth) / (right - left);
    
    for(var i = left; i < right; i++){
        this._widths[i]["width"]      += offsetWidth;
        this._widths[i]["resizeable"]  = xoptions["resizeable"] === undefined ? true : xoptions["resizeable"];
    }
};
/** @private */
gtk.TableBase.prototype._resetHeights = function(child, top, bottom, yoptions){
    var extraHeight = 0;
    for(var i = top; i < bottom; i++){
        extraHeight += this._heights[i]["height"];
    }
    
    //compare heights of predefined and the height of widget
    var childHeightComposite = child.getMinHeightComposite() + this._spacing;
    var biggerHeight = (extraHeight < childHeightComposite ? childHeightComposite : extraHeight);
    
     //generate offsetHeight to be added to the array heights
    offsetHeight = (biggerHeight - extraHeight) / (bottom - top);
    for(var i = top; i < bottom; i++){
        this._heights[i]["height"]     += offsetHeight;
        this._heights[i]["resizeable"]  = yoptions["resizeable"] === undefined ? true : yoptions["resizeable"];
    }
};
/** @private */
gtk.TableBase.prototype._initChildren = function(rows, columns){
    var children = new Array(columns);
    for(var i = 0, len = this._columns; i < len; i++){
        children[i] = new Array(rows);
    }
    
    for(var i = 0, len = this._columns; i < len; i++){
        for(var j = 0, length = this._rows; j < length; j++){
            children[i][j] = {
                widget   : null, 
                minWidth : null, 
                minHeight: null,
                width    : null,
                height   : null,
                xoptions : null,
                yoptions : null,
                xpadding : null,
                ypadding : null,
                xcells   : null,
                ycells   : null,
                left     : null,
                right    : null,
                top      : null,
                bottom   : null
            };
        }
    }
    return children;
};
/** @private */
gtk.TableBase.prototype.resize = function(width, height){
    this._resizeWidth(width);
    this._resizeHeight(height);
};
/** @private */
gtk.TableBase.prototype._getWidths = function(){
    var width = 0;
    for(var i = 0; i < this._columns; i++){
        width += this._widths[i]["width"];
    }
    return width;
};
/** @private */
gtk.TableBase.prototype._getHeights = function(){
    var height = 0;
    for(var i = 0; i < this._rows; i++){
        height += this._heights[i]["height"];
    }
    return height;
};
/** @private */
gtk.TableBase.prototype._getResizeableColumns = function(){
    var columns = 0;
    for(var i = this._widths.length; i--;){
        if(this._widths[i]["resizeable"]) columns += 1;
    }
    return columns;
};
/** @private */
gtk.TableBase.prototype._getResizeableRows = function(){
    var rows = 0;
    for(var i = this._heights.length; i--;){
        if(this._heights[i]["resizeable"]) rows += 1;
    }
    return rows;
};
gobject.signalNew("clicked", gtk.TableBase, null, null, []);
