/**
* @constructor
* @param {integer} rows number of row of table
* @param {integer} columns number of column of table
* @param {boolean} resizeable
* <br> - resizeable resolve to true will make all children expand to fill table
* <br> - resizeable resolbe to false all children will be packed to the most top and most left of table
* @param {integer} spacing between each child
*/
gtk.TableNoneHomogenoues = function(rows, columns, resizeable, spacing) {
	gtk.TableBase.call(this, rows, columns, resizeable, spacing);

	this._render = function(){
	    var domObj            = this._domObj;
		domObj.className     += " gtk_table_none_homogenouse";
        this._children        = this._initChildren(this._rows, this._columns);

		//init widths and heights
		this._initWidthsHeights();
	};
	
	this._render();
};

gtk.TableNoneHomogenoues.prototype = new gtk.TableBase();
gtk.TableNoneHomogenoues.prototype.constructor = gtk.TableNoneHomogenoues;

/**
 * Attach each gtk.Widget to gtk.Table
 * @param {gtk.Widget} child
 * @param {intger} left x1
 * @param {integer} right x2
 * @param {integer} top y1
 * @param {integer} bottom y2
 * @param {object} xoptions {align: gtk.JUSTIFY_LEFT || gtk.JUSTIFY_RIGHT, fill: false || true}
 * @param {object} yoptions {align: gtk.JUSTIFY_TOP || gtk.JUSTIFY_BOTTOM, fill: false || true}
 * @param {integer} xpadding
 * @param {integer} ypadding
 */
gtk.TableNoneHomogenoues.prototype.attach = function(child, left, right, top, bottom, xoptions, yoptions, xpadding, ypadding){

    var xcells = 1;
    
    //Occupy the the colspan, more then 1 column
    for(var i = left + 1; i < right; i++){
        this._children[i][top].minWidth   = child.getMinWidthComposite();
        this._children[i][top].minHeight  = child.getMinHeightComposite();
        xcells += 1;
    }
     //Occupy the rowspan, more then 1 row
    if(left + 1 < right){
        for(var i = left; i < right; i++){
            var ycells = 1;
            for(var j = top + 1; j < bottom; j++){
                this._children[i][j].minWidth   = child.getMinWidthComposite();
                this._children[i][j].minHeight  = child.getMinHeightComposite();
                ycells += 1;
            }
        }
    }else{
    
        //Occupy the rows
        var ycells = 1;
        for(var i = top + 1; i < bottom; i++){
            this._children[left][i].minWidth   = child.getMinWidthComposite();
            this._children[left][i].minHeight  = child.getMinHeightComposite();
            ycells += 1;
        }
    }
    
    //Occupy column
    var targetChild = this._children[left][top];
    targetChild.widget   = child;
    targetChild.xoptions = xoptions;
    targetChild.yoptions = yoptions;
    targetChild.xpadding = xpadding;
    targetChild.ypadding = ypadding;
    targetChild.xcells   = xcells;
    targetChild.ycells   = ycells;
    targetChild.left     = left;
    targetChild.right    = right;
    targetChild.top      = top;
    targetChild.bottom   = bottom;
    targetChild.minWidth = child.getMinWidthComposite();
    targetChild.minHeight= child.getMinHeightComposite();
    
    //reset widths and heights
    this._resetWidths(child, left, right, xoptions);
    this._resetHeights(child, top, bottom, yoptions);

    this._placeChildrenNoneHomo();
    
    var maxWidth  = this._getWidths();            
    var maxHeight = this._getHeights();
    
    this.resize(maxWidth, maxHeight);
    this._minWidth = maxWidth;
    this._minHeight= maxHeight;
};

/** @private */
gtk.TableNoneHomogenoues.prototype._placeChildrenNoneHomo = function(){
    var child;
    this._domObj.innerHTML = "";
    var docFragment = document.createDocumentFragment();
    for(var i = 0; i < this._rows; i++){
        for(var j = 0; j < this._columns; j++){
            child = this._children[j][i];
            var width = 0;
            var height = 0;
            if(child.widget){
                var childWrapper = jsgtk.Dom.create({"append": docFragment, "tag": "div", "style": "position: absolute;"});
                
                //the width of the widget wrapper
                for(var k = child.left; k < child.right; k++){
                    width += this._widths[k]["width"];
                }
                var left = 0; //how far from the left has to place the wrapper
                for(var k = 0; k < child.left; k++){
                    left += this._widths[k]["width"];
                }
                
                //the height of the widget wrapper
                for(var k = child.top; k < child.bottom; k++){
                    height += this._heights[k]["height"];
                }
                var top = 0;//how far from the top has to place the wrapper
                for(var k = 0; k < child.top; k++){
                    top += this._heights[k]["height"];
                }
                
                childWrapper.style.width  = width + "px";
                childWrapper.style.height  = height + "px";
                childWrapper.style.left = left + "px";
                childWrapper.style.top = top + "px";
                
                childWrapper.appendChild(child.widget._domObj);
                
                //position the widget inside the wrapper bases on its options properties
                this._positionWidgetByXoptionsNoneHomo(child, width);
                this._positionWidgetByYoptionsNoneHomo(child, height);
            }
        }
    }
    this._domObj.appendChild(docFragment);
};
/** @private */
gtk.TableNoneHomogenoues.prototype._positionWidgetByXoptionsNoneHomo = function(child, width){
    var xpadding = child.xpadding;
    var spacing = child.right === this._columns ? 0 : this._spacing;
    if(child.xoptions.fill){
        child.widget._domObj.style.left = xpadding + "px";
        child.widget.resizeWidthComposite(width - xpadding * 2 - spacing);
    }else{
        var left;
        switch(child.xoptions.align){
            case gtk.JUSTIFY_LEFT :
                child.widget._domObj.style.left = child.xpadding + "px";
                break;
            case gtk.JUSTIFY_RIGHT :
                left = width - (child.minWidth + child.xpadding) - spacing;
                child.widget._domObj.style.left = left + "px";
                break;
            case gtk.JUSTIFY_CENTER :
                left = ( (width-spacing) / 2) - (child.minWidth / 2);
                child.widget._domObj.style.left = left + 'px';
                break;
        }
        child.widget.resizeWidthComposite(child.minWidth);
    }
};
/** @private */
gtk.TableNoneHomogenoues.prototype._positionWidgetByYoptionsNoneHomo = function(child, height){
    var ypadding = child.ypadding;
     var spacing = child.bottom === this._rows ? 0 : this._spacing;
    if(child.yoptions.fill){
        child.widget._domObj.style.top = ypadding + "px";
        child.widget.resizeHeightComposite(height - ypadding * 2 - spacing);
    }else{
        var top;
        switch(child.yoptions.align){
            case gtk.JUSTIFY_TOP :
                child.widget._domObj.style.top = child.ypadding + "px";
                break;
            case gtk.JUSTIFY_BOTTOM :
                top = height - (child.minHeight + child.ypadding) - spacing;
                child.widget._domObj.style.top = top + "px";
                break;
            case gtk.JUSTIFY_CENTER :
                top = ( (height-spacing) / 2) - (child.minHeight / 2);
                child.widget._domObj.style.top = top + 'px';
                break;
        }
        child.widget.resizeHeightComposite(child.minHeight);
    }    
};
/** @private */
gtk.TableNoneHomogenoues.prototype._placeChildrenHomo = function(childWidth, childHeight){
    var children = this._children;
    this._domObj.innerHTML = "";
    var docFragment = document.createDocumentFragment();
    for(var i = 0; i < this._columns; i++){
        for(var j = 0; j < this._rows; j++){
            var child = children[i][j];
            if(child.widget){
                var childWrapper = jsgtk.Dom.create({'append': docFragment, 'tag': 'div', 'style': 'position: absolute;'});
                childWrapper.style.width  = childWidth  * child.xcells + 'px';
                childWrapper.style.height = childHeight * child.ycells + 'px';
                childWrapper.style.top    = child.top * childHeight + 'px';
                childWrapper.style.left   = child.left * childWidth + 'px';
                childWrapper.appendChild(child.widget._domObj);

                var width  = this._positionWidgetByXoptionsHomo(child, childWidth);
                var height =this._positionWidgetByYoptionsHomo(child, childHeight);
                             
                child.widget.resizeWidthComposite(width - 2);
                child.widget.resizeHeightComposite(height - 2);
            }
        }
    }
    this._domObj.appendChild(docFragment);
};
/** @private */
gtk.TableNoneHomogenoues.prototype._positionWidgetByYoptionsHomo = function(child, childHeight){
    var height;
    var spacing = child.bottom === this._rows ? 0 : this._spacing;
    if(child.yoptions.fill){
        child.widget._domObj.style.top = child.ypadding + 'px';
        height  = childHeight  * child.ycells - child.ypadding * 2 - spacing;
    }else{
        var top;
        height  = child.widget.getMinHeightComposite();
        switch(child.yoptions.align){
            case gtk.JUSTIFY_TOP :
                child.widget._domObj.style.top = child.ypadding + "px";
                break;
            case gtk.JUSTIFY_BOTTOM :
                top = (childHeight * child.ycells) - (height + child.ypadding);
                child.widget._domObj.style.top = top + "px";
                break;
            case gtk.JUSTIFY_CENTER :
                top = ( (childHeight * child.ycells) / 2) - (height / 2);
                child.widget._domObj.style.top = top + "px";
                break;
        }
    }
    return height;
};
/** @private */
gtk.TableNoneHomogenoues.prototype._positionWidgetByXoptionsHomo = function(child, childWidth){
    var width;
    var spacing = child.right === this._columns ? 0 : this._spacing;
    if(child.xoptions.fill){
        child.widget._domObj.style.left = child.xpadding + 'px';
        width  = childWidth * child.xcells - child.xpadding * 2 - spacing;
    }else{
        var left;
        width  = child.widget.getMinWidthComposite();
        switch(child.xoptions.align){
            case gtk.JUSTIFY_LEFT :
                child.widget._domObj.style.left = child.xpadding + "px";
                break;
            case gtk.JUSTIFY_RIGHT :
                left = (childWidth * child.xcells) - (width + child.xpadding);
                child.widget._domObj.style.left = left + "px";
                break;
            case gtk.JUSTIFY_CENTER :
                left = ( (childWidth * child.xcells) / 2) - (width / 2);
                child.widget._domObj.style.left = left + 'px';
                break;
        }
    }
    return width;
};
/** @private */
gtk.TableNoneHomogenoues.prototype.getMinWidthComposite = function(){
    return this._minWidth;
};
/** @private */
gtk.TableNoneHomogenoues.prototype.getMinHeightComposite = function(){
    return this._minHeight;
};
/** @private */
gtk.TableNoneHomogenoues.prototype.resizeWidthComposite = function(width){
    var widthComposite = this.getMinWidthComposite();
   	widthComposite     = width > widthComposite ? width : widthComposite;

    var columns = this._getResizeableColumns();
    var offsetWidth = (widthComposite - this._getWidths()) / columns;
    for(var i = 0; i < this._columns; i++){
        if(this._widths[i]["resizeable"]) this._widths[i]["width"] += offsetWidth;
    }
    if(this._resizeable) this._placeChildrenNoneHomo();
    this.resize(widthComposite, this._height);
};
/** @private */
gtk.TableNoneHomogenoues.prototype.resizeHeightComposite = function(height){
   	var heightComposite = this.getMinHeightComposite();
   	heightComposite     = height > heightComposite ? height : heightComposite;

    var rows = this._getResizeableRows();
    var offsetHeight = (heightComposite - this._getHeights()) / rows;
    for(var i = 0; i < this._rows; i++){
        if(this._heights[i]["resizeable"]) this._heights[i]["height"] += offsetHeight;
    }
    if(this._resizeable) this._placeChildrenNoneHomo();
    this.resize(this._width, heightComposite);
};
