/**
 * A widget that displays the contents of a gtk.TextBuffer. A class inherit from gtk.Container
 * @constructor
 */
gtk.TextView = function(){
	gtk.Widget.apply(this);
	/** 'focused' signal */
	this.TEXTVIEW_FOCUSED = "focused";
	/** 'blurred' signal */
	this.TEXTVIEW_BLURRED = "blurred"
	/** 'enterkeypressed' signal */
	this.TEXTVIEW_ENTERKEYPRESSED = "enterkeypressed";

	this._editable 		= true;
	this._justification = "left";
	this._focus 		= false;
	this._showError		= true;
	
	this._minWidth 		= this._width 	= 100;
	this._minHeight 	= this._height 	= 50;
	
	/** @ private */
	this._render = function(){
	    var className = this._domObj.className;
		this._domObj = jsgtk.Dom.create({"tag": "textarea"});
		this._domObj.className = className + ' gtk_text_view';
	};
	
	this._render();
	
	this._initEvent();
};

gtk.TextView.prototype = new gtk.Widget();
gtk.TextView.prototype.constructor = gtk.TextView;
/** @private */
gtk.TextView.prototype._initEvent = function(){
	var self = this;
	var jsgtkEvent = jsgtk.Event;
	jsgtkEvent.addDomListener(this._domObj, "focus", function(){
	    self._focus = true;
	    self.emit("focused", self._focus);
	});
	
	jsgtkEvent.addDomListener(this._domObj, "blur", function(){
	    self._focus = false;
	    self.emit("blurred", self._focus);
	});
};
/**
 * Sets the "editable" property to the value of setting. If setting is True the text in textview is editable by default
 * @param {boolean} setting If True the text is editable by default
 */
gtk.TextView.prototype.setEditable = function(setting){
    this._editable = setting;
    
    setting? this._domObj.removeAttribute("readonly"):this._domObj.setAttribute("readonly", "readonly");
};
/**
 * Show error
 * @param {boolean} isOnError
 */
gtk.TextView.prototype.showError = function(isOnError){
	this._showError = (typeof isOnError !== 'undefined' && typeof isOnError === 'boolean') ? isOnError : true;
    
    if(this._showError) this._domObj.className += " gtk_text_view_onerror";
    else this._domObj.className = "gtk_widget gtk_container gtk_text_view";
};
/**
 * Returns the value of the "editable" property
 * @return True if text is editable by default
 */
gtk.TextView.prototype.getEditable = function(){
    return  this._editable;
};
/**
 * Sets the "justification" property to the value of justification. The value of justification must be one of: gtk.JUSTIFY_LEFT, gtk.JUSTIFY_RIGHT, gtk.JUSTIFY_CENTER or gtk.JUSTIFY_FILL
 * @param {String} the text justification
 */
gtk.TextView.prototype.setJustification = function(justification){
    this._justification = justification;
    this._domObj.style.textAlign = justification;
};
/**
 * Returns the value of the "justification" property. the default justification of paragraphs in text_view. The value of "justification" must be one of: gtk.JUSTIFY_LEFT, gtk.JUSTIFY_RIGHT, gtk.JUSTIFY_CENTER or gtk.JUSTIFY_FILL.
 * @return The default justification
 */
gtk.TextView.prototype.getJustification = function(){
    return this._justification;
};
/**
 * Retrieve the value inside TextView
 * @return The value of TextView
 */
gtk.TextView.prototype.getValue = function(){
    return this._domObj.value;
};
/**
 * Set the default value inside TextView
 * @param {String} value The value to be set into TextView 
 */
gtk.TextView.prototype.setValue = function(value){
    this._domObj.value = value;
};
/** @private */
gtk.TextView.prototype.resize = function(width, height){
    this._resizeWidth(width);
    this._resizeHeight(height);
};
/**
 * Allow user to create custom event when enterkey is pressed
 */
gtk.TextView.prototype.allowEnterKey = function(){
    var self = this;
    jsgtk.Event.addDomListener(this._domObj, "keypress", function(e){
	    if(e.keyCode == gtk.ENTER_KEY){
	        self.emit("enterkeypressed", self._domObj.value);
	        jsgtk.Event.stopDefault(e);
	    }
	});
};
/**
 * Focus the cursor on the textview
 */
gtk.TextView.prototype.focus = function(){
    this._domObj.focus();
};
/** @private */
gtk.TextView.prototype._resizeWidth = function(width){
    this._domObj.style.width = width - 3 + 'px';
    this._width = width;    
};
/** @private */
gtk.TextView.prototype._resizeHeight = function(height){
    this._domObj.style.height = height - 4 + 'px';
    this._height = height;
};
/** @private */
gtk.TextView.prototype.resizeWidthComposite = function(width){
    this.setWidth(width);
    this.resize(this._width, this._height);
};
/** @private */
gtk.TextView.prototype.resizeHeightComposite = function(height){
    this.setHeight(height);
    this.resize(this._width, this._height);
};
gobject.signalNew("focused", gtk.TextView, null, null, []);
gobject.signalNew("blurred", gtk.TextView, null, null, []);
gobject.signalNew("enterkeypressed", gtk.TextView, null, null, []);
