/**
 * TreeIter is an assistant class to create model or structure of a model
 * @constructor
 * @base gtk.Object
 */
gtk.TreeIter = function(path){
	gtk.Object.apply(this);

	this._modelType = gtk.TREE_ITER;
 	this.path = path ? path : "0";
 	this.hasNode = false;
 	this.children = [];
 	this._value;
};

gtk.TreeIter.prototype = new gtk.Object();
gtk.TreeIter.prototype.constructor = gtk.TreeIter; 

gtk.TreeIter.prototype.getChldBy = function(valueMember, value){
    var strValue = String(this._value[valueMember]).toLowerCase();
    var strToCompare = String(value).toLowerCase();

	if(strValue === strToCompare) return this;
	for(var i = this.children.length; i--;){
		return this.children[i].getChldBy(valueMember, value);
	}
};

gtk.TreeIter.prototype.setRow = function(value){
	this._value = value;
};

gtk.TreeIter.prototype.getValue = function(){
	return this._value;
};

gtk.TreeIter.prototype.getModelType = function(){
	return this._modelType;
};

/**
 * Returns a copy of the treeiter.
 * return {object} treeiter
 */
gtk.TreeIter.prototype.copy = function(){
 	return this;
};

gtk.TreeIter.prototype.getChildByPath = function(path){
    var paths = path.split(":");
  
    var child = this._getChildByPath( parseInt(paths[0]) );

    paths.splice(0,1);
    if(paths.length === 1) return child;
    
    return child.getChildByPath(paths.join(":"));   
};

gtk.TreeIter.prototype._getChildByPath = function(path){
	for(var i = this.children.length; i--;){
		var childPath = this.children[i].path.split(":");
		if(parseInt(childPath[childPath.length - 2]) === path) return this.children[i];
	}
	return null;
};

gtk.TreeIter.prototype.updateValue = function(value){
    this._value["value"] = value;
    this.emit("valueupdated", this.path, value);
};

gtk.TreeIter.prototype.getPath = function(){
	return this.path;
};

/**
* getChildren() method
* @param {array} children: array of children of gtk.TreeIter type
*/
gtk.TreeIter.prototype.getChildren = function(){
	return this.children;
};

/**
* add() method
* @param{object} child: object of of gtk.TreeIter type
*/
gtk.TreeIter.prototype.add = function(child){
	this.children.push(child);
};

/*
* remove() method : remove a child
* @param {object} child: child of gtk.TreeIter type
*/
gtk.TreeIter.prototype.remove = function(child){
	for(var i = this.children.length; i--;){
		if(this.children[i] === child){
			this.children.splice(i, 1);
			break;
		}
	}
};

/**
* getChild() method
* @param {integer} index : index of child
* @return {object} child: child of gtk.TreeIter type
*/
gtk.TreeIter.prototype.getChild = function(index){
	return this.children[index] || null;
};

/** @private */
gtk.TreeIter.prototype._search = function(text){
	var re = new RegExp("^" + text.toLowerCase());
    
    var children = this.getChildren();
    var len = children.length;
    if(len > 0){
		for(var i = 0 ; i < len; i++){
			var child = children[i];
	    	this._child = child._search(text);
	    	if(this._child) return this._child;
		}
	}
	else{
    	var row = this.getValue()["display"];
	    var result = row.toLowerCase().match(re);
		if(result)
			return this;
	}
};

gobject.signalNew('valueupdated', gtk.TreeIter, [], null, null);
