/**
 * class gtk.TreeView
 */
gtk.TreeView = function(model){
	gtk.Widget.apply(this);
		
	this.__width             = 	this.__minWidth = 100;
	this.__height            = this.__minHeight  = 100;
	this.__nodeImgWidth      = 15;
	this.__model             = model || null;
	this.__hilightedCell     = null;
	this.__selectedNode      = null;
	this.__selectedNodePath  = "";
	this.__expandedNodePath  = "";
	this.__isLoading         = false;
	
	this.__render = function(){	
		this._domObj.className   += " gtk_treeview";
		this._domObj.style.width  = this.__width + 'px';
		this._domObj.style.height = this.__height + 'px';
	};
	this.__render();
	new gtk.TreeViewController(this);
};

gtk.TreeView.prototype = new gtk.Widget();
gtk.TreeView.prototype.constructor = gtk.TreeView;

gtk.TreeView.prototype.setModel = function(model){
    this._domObj.innerHTML = "";
    this.__model = model;
    this.__selectedNodePath  = "";
	this.emit("_modelreset", this.__model);
};

gtk.TreeView.prototype.getModel = function(){
    return this.__model;  
};

gtk.TreeView.prototype.showLoading = function(node){
    var expandCollapseNode = node.firstChild.firstChild;

    for(var i = 0, len = expandCollapseNode.childNodes.length; i < len; i++)
        jsgtk.Dom.hideElement(expandCollapseNode.childNodes[i]);

    jsgtk.Dom.showElement(expandCollapseNode.lastChild);
    
    this.__isLoading = true;
};

gtk.TreeView.prototype.hideLoading = function(node){
    var expandCollapseNode = node.firstChild.firstChild;

    for(var i = 0, len = expandCollapseNode.childNodes.length; i < len; i++)
        jsgtk.Dom.showElement(expandCollapseNode.childNodes[i]);
        
    jsgtk.Dom.hideElement(expandCollapseNode.lastChild);
    
    this.__isLoading = false;
};

gtk.TreeView.prototype.getSelectedNode = function(){
    return this.__selectedNode;
};

gtk.TreeView.prototype.deleteNode = function(selectedNode){
//	this.deselectNode();
	this.emit("_deletednode", selectedNode);
//	selectedNode["domNode"].parentNode.removeChild(selectedNode["domNode"]);
};

gtk.TreeView.prototype.deselectNode = function(){
    if(this.__hilightedCell) jsgtk.Dom.applyAttributes(this.__hilightedCell, {backgroundColor : ""});
    this.__hilightedCell = null;
};

gtk.TreeView.prototype.getMinWidthComposite = function(){
    return this.__minWidth;  
};

gtk.TreeView.prototype.getMinHeightComposite = function(){
    return this.__minHeight;
};

gtk.TreeView.prototype.resizeWidthComposite = function(width){
	var minWidth = width > this.__minWidth ? width : this.__minWidth;
	this._resizeWidth(minWidth);
};

gtk.TreeView.prototype.resizeHeightComposite = function(height){
	var minHeight = height > this.__minHeight ? height : this.__minHeight;
	this._resizeHeight(minHeight);
};

gtk.TreeView.prototype.__showExpandableImg = function(img){
    img.className += " hasChild";
};

gtk.TreeView.prototype.__constructNode = function(nodeProperties){
	var nodeConstructor = new gtk.NodeConstructor(nodeProperties);
	nodeConstructor.render();
    return nodeConstructor.node;
};

gtk.TreeView.prototype.__clearTreeNodes = function(){
	this._domObj.innerHTML = "";
};

gtk.TreeView.prototype.__clearSubNodes = function(parentNode){
    var firstNode = parentNode.childNodes[0];
    parentNode.innerHTML = "";
    parentNode.appendChild(firstNode);
};

gtk.TreeView.prototype._constructNodes = function(parentNode, nodeDatas, isFirstNode){

    var self = this;
		
	var maxNodeWidth = this.__getMaxTextWidth(nodeDatas);
    var setNodeWidth = function(node){
        node.style.width = maxNodeWidth + self.__nodeImgWidth + 10 + nodeDatas[i]["emptyBoxes"].length *  self.__nodeImgWidth + "px";
    }
    
    var appendSubNodes = function(parentNode, node, emptyBoxes){
        for(var j = 0, len = emptyBoxes.length; j < len; j++){
            var className = emptyBoxes[j] ? "gtk_treeview_vdot_last_branche" : "gtk_treeview_vdot_branche";
            var vDot = jsgtk.Dom.create({"append": parentNode, "tag": "div", "class": className});
        }
        parentNode.appendChild(node);
    };

    var docFragment = document.createDocumentFragment();
    
    for(var i = 0, len = nodeDatas.length; i < len; i++){
        var isParent = false;
        var isLastNode = i === len - 1 ? true : false;
        
        if(nodeDatas[i].hasChildNodes){
            isParent = true;
        }

		var nodeProperties = {
			"text": nodeDatas[i].display,
			"isFirstNode": isFirstNode,
			"isLastNode": isLastNode,
			"index": i,
			"isParent": isParent,
			"id": nodeDatas[i].path
		};        

        var node = this.__constructNode(nodeProperties);
        setNodeWidth(node, nodeDatas);
            
        if(nodeDatas[i].hasChildNodes){
            var expandCollapseImg = node.firstChild.firstChild.firstChild
            this.__showExpandableImg(expandCollapseImg);
        }
        
        appendSubNodes(docFragment, node, nodeDatas[i]["emptyBoxes"]);

        nodeDatas[i]["emptyBoxes"].push(isLastNode);

        this.__handleNodeClick(node, nodeDatas[i].path);
             
        this.__handleExpandCollapse(node, nodeDatas[i].path, nodeDatas[i]["emptyBoxes"]);
    }
    
    this.__appendToTree(parentNode, docFragment);
};

gtk.TreeView.prototype._refreshTreeNodes = function(nodeDatas){
	this.__clearTreeNodes();
	this._constructNodes(null, nodeDatas, true);
};

gtk.TreeView.prototype._constructTreeNodes = function(parentNode, nodeDatas, isFirstNode){
    if(parentNode) this.__clearSubNodes(parentNode);
	this._constructNodes(parentNode, nodeDatas, isFirstNode);
};

gtk.TreeView.prototype.__appendToTree = function(parentNode, docFragment){
     if(parentNode)
        parentNode.appendChild(docFragment);
    else
        this._domObj.appendChild(docFragment);
}

gtk.TreeView.prototype.__renderSubNodes = function(node, path, emptyBoxes){
    this.emit("rendersubnodes", node, path, emptyBoxes);  
};

gtk.TreeView.prototype.__collapse = function(node){
    var textImgContainerNode = node.firstChild;
    var imgNode = textImgContainerNode.firstChild.firstChild;
    this.__hideSibbling(textImgContainerNode);
    
    imgNode.style.backgroundPosition = "";
};

gtk.TreeView.prototype.__expand = function(node, path, emptyBoxes){
    var textImgContainerNode = node.firstChild;
    var imgNode = textImgContainerNode.firstChild.firstChild;
        
    if(textImgContainerNode.parentNode.childNodes.length <= 1)
        this.__renderSubNodes(node, path, emptyBoxes);
    else 
        this.__showSubNodes(textImgContainerNode);
    
    imgNode.style.backgroundPosition = "-386px 1px";
};

gtk.TreeView.prototype.__handleExpandCollapse = function(node, path, emptyBoxes){
    var self = this;
    var isExpanded = false;
    var expandCollapseNode = node.firstChild.firstChild;
    jsgtk.Event.addDomListener(expandCollapseNode, "mousedown", function(e){
        
        if(self.__isLoading) return;
        
        if(self.__expandedNodePath !== path){
            self.__expandedNodePath = path;
            self.emit("_nodeexpandedtocontroller", path, node);
        }

        if(!isExpanded){
            self.__expand(node, path, emptyBoxes);
            isExpanded = true;
        }else{
            self.__collapse(node);
            isExpanded = false;
        }
        
        jsgtk.Event.stopBubble(e);
    });
};

gtk.TreeView.prototype.__handleNodeClick = function(node, path){
    var self = this;
    var textImgContainerNode = node.firstChild;
    jsgtk.Event.addDomListener(textImgContainerNode, "mousedown", function(e){

        self._hilightNode(textImgContainerNode);
 		
        if(self.__selectedNodePath !== path){
		
            self.__selectedNodePath = path;
            self.emit("_nodeselectedtocontroller", path, node);
        }    

        jsgtk.Event.stopBubble(e);
        
    });
};


gtk.TreeView.prototype.__getMaxTextWidth = function(data){ 
    var tmpDom = jsgtk.Dom.create({"append": document.body, "tag": "div"});
    
    var getWidthOfDom = function(text){
        var span = jsgtk.Dom.create({"append": tmpDom, "tag": "span", "style" : "position:absolute;"});
	    var text = jsgtk.Dom.create({"append": span, "tag": "text", "text": text});
	    return span.clientWidth;
    };
    
    var maxWidth = 0;
    for(var i = 0, len = data.length; i < len; i++){
        var textWidth = getWidthOfDom(data[i].display);
        maxWidth = maxWidth < textWidth ? textWidth : maxWidth;
    }
    document.body.removeChild(tmpDom);
    return maxWidth;
};

gtk.TreeView.prototype.__showSubNodes = function(node){
    var parentNode = node.parentNode;
    for(var i = 1, len = parentNode.childNodes.length; i < len; i++)
        jsgtk.Dom.showElement(parentNode.childNodes[i]);
};

gtk.TreeView.prototype.__hideSibbling = function(node){
    var parentNode = node.parentNode;
    for(var i = 1, len = parentNode.childNodes.length; i < len; i++)
        jsgtk.Dom.hideElement(parentNode.childNodes[i]);
};

/** @private */
gtk.TreeView.prototype._setSelectedNode = function(dataObject){
    this.__selectedNode = dataObject;
};

gtk.TreeView.prototype._hilightNode = function(node){
    if(this.__hilightedCell) jsgtk.Dom.applyAttributes(this.__hilightedCell, {backgroundColor : ""});
    this.__hilightedCell = node;
    jsgtk.Dom.applyAttributes(this.__hilightedCell, {backgroundColor : "#FFFF66"});
};

gtk.TreeView.prototype._selectNode = function(path){
	var node = this.__getNodeByPath(path, this._domObj, 0);
};

gtk.TreeView.prototype.__getNodeByPath = function(path, parentNode, i){
	var node = parentNode.childNodes[path.length/2-i];
	if(path.length/2 - i){
		node = this.__getNodeByPath(path, parentNode, i+1);
		return node;
	}else{
		return node;
	}
};

gobject.signalNew('rendersubnodes', gtk.TreeView, [], null, null);
gobject.signalNew('_nodeexpandedtocontroller', gtk.TreeView, [], null, null);
gobject.signalNew('_nodeselectedtocontroller', gtk.TreeView, [], null, null);
gobject.signalNew('nodeexpanded', gtk.TreeView, [], null, null);
gobject.signalNew('nodeselected', gtk.TreeView, [], null, null);
gobject.signalNew('_modelreset', gtk.TreeView, [], null, null);
gobject.signalNew('_deletednode', gtk.TreeView, [], null, null);

gtk.NodeConstructor = function(nodeProperties){
	this._nodeProperties = nodeProperties;
	
	this.render = function(){
		this._constructNode();
		this._constructVdotsHdotsContainer();
		this._constructLoadingImage();
		this._constructNodeText();
	};

	this._constructNode = function(){
		this.node = jsgtk.Dom.create({"tag": "div", "class": "gtk_treeview_node", "id": this._nodeProperties["id"]});
        this._container = jsgtk.Dom.create({"append": this.node, "tag": "div", "class": "gtk_treeview_node_row"});
            this._dotContainer = jsgtk.Dom.create({"append": this._container, "tag": "div", "class": "gtk_treeview_dot_container"});
                jsgtk.Dom.create({"append": this._dotContainer, "tag": "div", "class": "gtk_treeview_node_image"});
	};    

	this._getVdotsHdotsClassName = function(){
		classNameVdots = nodeProperties["isLastNode"] ? "gtk_treeview_node_vdot_last" : "gtk_treeview_node_vdot";
        classNameHdots = nodeProperties["isLastNode"] ? "gtk_treeview_node_hdot_last" : "gtk_treeview_node_hdot";

		if(nodeProperties["isFirstNode"] && nodeProperties["index"] === 0){
            classNameVdots = "gtk_treeview_node_vdot_first";
            classNameHdots = "gtk_treeview_node_hdot_first";
        }
        
        if(nodeProperties["isFirstNode"] && nodeProperties["isLastNode"] && nodeProperties["index"] === 0){
		    classNameVdots = "gtk_treeview_node_vdot_first_and_last";
		    classNameHdots = "gtk_treeview_node_hdot_first_and_last";
		}
		return [classNameVdots, classNameHdots]
	};

	this._constructVdotsHdotsContainer = function(){
		var vdotsHdotsClassNames = this._getVdotsHdotsClassName();                         
		jsgtk.Dom.create({"append": this._dotContainer, "tag": "div", "class": vdotsHdotsClassNames[0]});
		jsgtk.Dom.create({"append": this._dotContainer, "tag": "div", "class": vdotsHdotsClassNames[1]});
	};

	this._constructLoadingImage = function(){
		var loadingImg = jsgtk.Dom.create({"append": this._dotContainer, "tag": "div", "class": "gtk_treeview_node_image_loading"});
		jsgtk.Dom.hideElement(loadingImg);
	};

	this._constructNodeText = function(){
		jsgtk.Dom.create({"append": this._container, "tag": "div", "text": this._nodeProperties["text"], "class": "gtk_treeview_node_text"});
	};
};
