/*
* class gtk.TreeViewController
*/
gtk.TreeViewController = function(view){
	this._view = view;
	this._selectedNodeData = null;
	
	this._viewSignals = [];
	
	var self = this;
	
	this._view.connect("_modelreset", function(model){
		self.setModel(model);
	});
};

gtk.TreeViewController.prototype.setModel = function(model){
	this._model = model;

    this._selectedNodeData = {
        "parentNode": null,
        "emptyBoxes": []
    };

	this._constructTreeNode(this._model, true);

	this._initSignals();
};

gtk.TreeViewController.prototype._initSignals = function(){
	var self = this;
	
	if(this._viewSignals.length > 0) this._disconnectSignalFromView();
	
	this._model.connect("nodesappended", function(data){
		self._constructTreeNode(data);
	});
	
	this._viewSignals.push(this._view.connect("_nodeexpandedtocontroller", function(path, node){
		self._selectedNodeData["parentNode"] = node.parentNode;
	    var model = self._model.getChildByPath(path);
	    self._view._setSelectedNode({"domNode": node, "model": self._model.getChildByPath(path), "display": model.getValue()});
	    self._view.emit("nodeexpanded", model.getValue());
	}));
	
	this._viewSignals.push(this._view.connect("_nodeselectedtocontroller", function(path, node){
		self._selectedNodeData["parentNode"] = node.parentNode;
		var model = self._model.getChildByPath(path);
	    self._view._setSelectedNode({"domNode": node, "model": self._model.getChildByPath(path), "display": model.getValue()});
        self._view.emit("nodeselected", model.getValue());
	}));
	
	this._viewSignals.push(this._view.connect("rendersubnodes", function(node, path, emptyBoxes){
        var data = self._model.getChildByPath(path);        

       	self._selectedNodeData = {
            "parentNode": node,
            "emptyBoxes": emptyBoxes
        };
		self._constructTreeNode(data);		        
	}));

	this._viewSignals.push(this._view.connect("_deletednode", function(selectedNode){
		var parentModel = self._removeChildModel(selectedNode["model"]);

		if(parentModel.getModelType() === gtk.TREESTORE) 
			self._refreshTreeNode(parentModel);
		else 
			self._constructTreeNode(parentModel);
		
	}));
};

gtk.TreeViewController.prototype._disconnectSignalFromView = function(){
	for(var i = this._viewSignals.length; i--;){
		this._view.disconnect(this._viewSignals[i]);
	}
};

gtk.TreeViewController.prototype._removeChildModel = function(childModel){
	var parentModelPath = childModel.path.substring(0, childModel.path.length - 2);

	var parentModel = this._getParentModel(parentModelPath)
	
	parentModel.remove(childModel);

	return parentModel;
};

gtk.TreeViewController.prototype._getParentModel = function(parentModelPath){
	return parentModelPath === "" ? this._model : this._model.getChildByPath(parentModelPath);
};

gtk.TreeViewController.prototype._refreshTreeNode = function(model){
	this._selectedNodeData = {
		"parentNode": null,
		"emptyBoxes": []
	};
	var mashupData = this._mashupModel(model.getChildren(), this._selectedNodeData["emptyBoxes"]);
    this._view._refreshTreeNodes(mashupData);
};

gtk.TreeViewController.prototype._constructTreeNode = function(data, isFirstChild){
	var mashupData = this._mashupModel(data.getChildren(), this._selectedNodeData["emptyBoxes"]);
    this._view._constructTreeNodes(this._selectedNodeData["parentNode"], mashupData, isFirstChild);
};

gtk.TreeViewController.prototype._mashupModel = function(nodes, emptyBoxes){
    var data = [], node, hasChild;
    for(var i = 0, len = nodes.length; i < len; i++){
        node = nodes[i];
        hasChild = (node.getChildren().length  > 0) || node.hasNode ? true :  false;
        data.push({"display": node.getValue()["display"], "hasChildNodes": hasChild, "path": node.getPath(), "emptyBoxes": [].concat(emptyBoxes)});
    }
    return data;
};
