/**
 * UnorderList is a class to create an ordered or unordered list
 * @base gtk.Widget
 * @constructor
 */
gtk.UnorderList = function(){
	gtk.Widget.apply(this);
	this._minWidth = this._width = 150;
	this._minHeight = this._height = 16;
	
	/** A registered signal name 'clicked' */
	this.UNORDERLIST_CLICKED = '';
	
	this._render = function(){
	    this._domObj.className += " gtk_unorderlist";
	        this.__ul = jsgtk.Dom.create({tag: "ul", append: this._domObj});
	};
	
	this._domObj.style.width = this._width + "px";
	this._domObj.style.height = this._height + "px";
	
	this._render();
	this._initEvents();
};

gtk.UnorderList.prototype = new gtk.Widget();
gtk.UnorderList.prototype.constructor = gtk.UnorderList;

/**
 * A method to set modal data or reset modal data
 * @param {Object} model The model to be set as gtk.ListModel
 */ 
gtk.UnorderList.prototype.setModel = function(model){
    this.__model = model;
    this.__constroller = new gtk.UnorderListController(this, model);
};

/** @private */
gtk.UnorderList.prototype._renderList = function(model, parent){
    var docFragment = parent || document.createDocumentFragment();
    if(model.children != null){
        var children = model.children;
        var childLength = children.length;
    }else{
        var children = model;
        var childLength = children.length;;
    }
    
    for(var i = 0;i < childLength; i++){ 
        var child = children[i]; 
        if(child.children == null){
            var li = jsgtk.Dom.create({tag: "li", style: child.style, text: child.value, append: docFragment});
        }
        else{
            var li = jsgtk.Dom.create({tag: "li", style: child.style, text: child.value, append: docFragment});
            var ul = jsgtk.Dom.create({tag: "ul", style: child.style, append: li, style: "list-style-type: " + child.type});
            this._renderList(child.children, ul);
        }
    };
   
    return docFragment;
};

/** @private */
gtk.UnorderList.prototype._initEvents = function(){
    var _self = this;
    jsgtk.Event.addDomListener(this.__ul, 'click', function(){
        _self.emit("clicked", _self._getItemsValue());
    });
};

/** @private */
gtk.UnorderList.prototype._appendList = function(list){
    if(this.__ul.childNodes) this.__ul.innerHTML = "";
    this.__ul.appendChild(list);
};

/**
 * A method to set type orderedlist or unorderedlist
 * @param {String} type The type style of list
 */ 
gtk.UnorderList.prototype.setListType = function(type){
    this.__ul.style.listStyleType = type;
};

/**
 * A method to set line height or spacing between each list
 * @param {Integer} height The line height of paragraph or sentence
 */ 
gtk.UnorderList.prototype.setLineHeight = function(height){
    this.__ul.style.lineHeight = height + "px";
    this._calculateDomHeight();
    this._resize();
};

/** @private */
gtk.UnorderList.prototype._getItemsValue = function(){
    var childObj = this.__ul.childNodes;
    var itemLength = childObj.length;
    var itemsValue = [];
    for(var i = 0; i< itemLength; i++){
        itemsValue.push(childObj[i].textContent);
    }
    return itemsValue;
};

/** @private */
gtk.UnorderList.prototype._calculateDomHeight = function(){
    var temp = this._domObj.childNodes[0].cloneNode(true);
    temp.style.width = this._width - 14 + "px";
    this._height = this._minHeight = jsgtk.Util.fullHeight(temp);
    return this._height;
};

/**
 * A method to get minimum width of the list
 * @return {Integer} The minimum width of the list
 * @type Integer
 */ 
gtk.UnorderList.prototype.getMinWidthComposite = function(){
    return this._minWidth;
};

/**
 * A method to get minimum height of the list
 * @return {Integer} The minimum height of the list
 * @type Integer
 */
gtk.UnorderList.prototype.getMinHeightComposite = function(){
    return this._minHeight;
};

/** @private */
gtk.UnorderList.prototype._resizeHeight = function(height){
    this._domObj.style.height = height + "px";
};

/** @private */
gtk.UnorderList.prototype._resizeWidth = function(width){
    this._domObj.style.width = width + "px";
};

/** @private */
gtk.UnorderList.prototype._resize = function(){
    this._resizeWidth(this._width);
    this._resizeHeight(this._height);
};

/**
 * A method to resize composite width of the list
 * @param {Integer} width The composite width to be set
 */
gtk.UnorderList.prototype.resizeWidthComposite = function(width){
    this._width = this._minWidth = this._width > width ? this._width : width;
    this._calculateDomHeight();
    this._resize();
};

/**
 * A method to resize composite height of the list
 * @param {Integer} width The composite height to be set
 */
gtk.UnorderList.prototype.resizeHeightComposite = function(height){
    this._height = this._minHeight = this._height > height ? this._height : height;
    this._resize();
};

/**
 * A method to set change to pointer or hand
 */ 
gtk.UnorderList.prototype.setPointerCursor = function(){
    this.__ul.style.cursor = "pointer";
    this.__ul.style.cursor = "hand";
};

gobject.signalNew("clicked", gtk.UnorderList, null, null, []);
