gtk.VBoxHomogeneous = function(spacing, padding) {
	gtk.VBox.call(this, spacing, padding);
	
	this._render = function(){
	    var domObj = this._domObj;
		domObj.className += " vbox_homogeneous";
	};
	
	this._render();
};

gtk.VBoxHomogeneous.prototype = new gtk.VBox();
gtk.VBoxHomogeneous.prototype.constructor = gtk.VBoxHomogeneous;

gtk.VBoxHomogeneous.prototype.resize = function(width, height){
	this._resizeHeight(height);
	this._resizeWidth( width);
};

gtk.VBoxHomogeneous.prototype._handleFilledElement = function(child, height){
    child["childWidget"].getNode().style.top = child["padding"] + "px";
    child["childWidget"].resizeHeightComposite(height - child["padding"] * 2);
};

gtk.VBoxHomogeneous.prototype._handleNoneFilledElement = function(child, height){
    var childMinHeight = child["childWidget"].getMinHeightComposite();
    var top = (height - childMinHeight) / 2;
    child["childWidget"].getNode().style.top = top + "px";
    child["childWidget"].resizeHeightComposite(childMinHeight);
};

gtk.VBoxHomogeneous.prototype._resizeChildrenHeight = function(height){
    var child;
    for(var i = 0, len = this._children.length; i < len; i++){
        child = this._children[i];
        child["space"].style.height   = this._spacing + "px";
        child["space"].style.top   = this._padding + "px";
        child["element"].style.height = height + "px";
        child["element"].style.top    = this._padding + "px";

		this._handleHiddenChild(child);        

        if(child["fill"]) this._handleFilledElement(child, height);
        else this._handleNoneFilledElement(child, height);
    }
    this._hideLastSpace();
};


gtk.VBoxHomogeneous.prototype._getMaxHeight = function(){
    var maxChildHeight = this._children[0]["childWidget"].getMinHeightComposite() + (this._children[0]["padding"] * 2);
    var childHeight;
    for(var i = 1, len = this._children.length; i < len; i++){
        childHeight = this._children[i]["childWidget"].isHidden() ?
					  0 :
					  this._children[i]["childWidget"].getMinHeightComposite() + (this._children[i]["padding"] * 2);

        maxChildHeight = maxChildHeight < childHeight ? childHeight : maxChildHeight;
    }
    return maxChildHeight;
};

gtk.VBoxHomogeneous.prototype.getMinHeightComposite = function(){
    var spacings       = this._spacing * (this._children.length - 1);
    var childrenHeight = this._getMaxHeight() * this._children.length;
    var paddings       = this._padding * 2;

	var minHeight = childrenHeight + spacings + paddings;
	minHeight     = minHeight < this._minHeight ? this._minHeight : minHeight;
    return minHeight;
};

gtk.VBoxHomogeneous.prototype.resizeHeightComposite = function(height){
    var maxChildHeight = this._getMaxHeight();
    var maxHeight      = this.getMinHeightComposite();
    
    if(height > maxHeight){
        var offsetHeight = height - maxHeight - this._padding * 2;
        var extraHeight  = offsetHeight / this._children.length;
        this._resizeHeight(height);
        this._resizeChildrenHeight(maxChildHeight + extraHeight);
    }else{
        this._resizeHeight(maxHeight);
        this._resizeChildrenHeight(maxChildHeight);
    }
};
