gtk.VBoxNoneHomogeneous = function(spacing, padding) {
	gtk.VBox.call(this, spacing, padding);
	
	this._render = function(){
	    var domObj = this._domObj;
		domObj.className += " vbox_none_homogeneous";
	};
	
	this._render();
};

gtk.VBoxNoneHomogeneous.prototype = new gtk.VBox();
gtk.VBoxNoneHomogeneous.prototype.constructor = gtk.VBoxNoneHomogeneous;

gtk.VBoxNoneHomogeneous.prototype.resize = function(width, height){
	this._resizeHeight(height);
	this._resizeWidth( width);
};

gtk.VBoxNoneHomogeneous.prototype._handleFilledElement = function(child, height){
    child["childWidget"].getNode().style.top = child["padding"] + "px";
    child["childWidget"].resizeHeightComposite(height - child["padding"] * 2);  
};

gtk.VBoxNoneHomogeneous.prototype._handleNoneFilledElement = function(child, height, childHeight){
    var top = (height - childHeight)/ 2;
    child["childWidget"].getNode().style.top = top + "px";
    child["childWidget"].resizeHeightComposite(childHeight);  
};

gtk.VBoxNoneHomogeneous.prototype._handleExpandedElement = function(child, extraHeight){
    var childHeight = child["childWidget"].getMinHeightComposite();
    var height = childHeight + extraHeight + child["padding"] * 2;
    child["element"].style.height = height +"px";
    
    if(child["fill"]) this._handleFilledElement(child, height);
    else this._handleNoneFilledElement(child, height, childHeight);
};

gtk.VBoxNoneHomogeneous.prototype._handleNoneExpandedElement = function(child){
    var childHeight = child["childWidget"].getMinHeightComposite();
    child["element"].style.height = childHeight + child["padding"] * 2 +"px";
    child["childWidget"].getNode().style.top = child["padding"] + "px";
};

gtk.VBoxNoneHomogeneous.prototype._resizeChildrenHeight = function(extraHeight){
    var child;
    for(var i = 0, len = this._children.length; i < len; i++){
        child = this._children[i];
        child["space"].style.height = this._spacing + "px";
        child["space"].style.top = this._padding + "px";
        child["element"].style.top    = this._padding + "px";

   		this._handleHiddenChild(child);

        if(child["expand"]) this._handleExpandedElement(child, extraHeight);
        else this._handleNoneExpandedElement(child);
    }
    this._hideLastSpace();
};

gtk.VBoxNoneHomogeneous.prototype._getNumberOfExpandedChildren = function(){
    var numberOfExpandedChildren = 0;
    for(var i = 0, len = this._children.length; i < len; i++){
        if(this._children[i]["expand"]) numberOfExpandedChildren++;
    }
    return numberOfExpandedChildren;
};

gtk.VBoxNoneHomogeneous.prototype.getMinHeightComposite = function(){
    var childrenHeight  = 0;
    for(var i = 0, len  = this._children.length; i < len; i++){
        childrenHeight += this._children[i]["childWidget"].isHidden() ?
						  0 :
						  this._children[i]["childWidget"].getMinHeightComposite() + (this._children[i]["padding"] * 2);
    }
    var paddings        = this._padding * 2;

	var minHeight = childrenHeight + this._spacing * (len-1) + paddings;
	minHeight	  = minHeight < this._minHeight ? this._minHeight : minHeight;

    return minHeight;
};

gtk.VBoxNoneHomogeneous.prototype.resizeHeightComposite = function(height){
    var maxChildHeight = this.getMinHeightComposite();

    if(height > maxChildHeight){
        var offsetHeight    = height - maxChildHeight;
        var extraHeight     = offsetHeight / this._getNumberOfExpandedChildren();
        this._resizeHeight(height);
        this._resizeChildrenHeight(extraHeight);
    }else{
    var offsetHeight    = height - maxChildHeight;
        var extraHeight     = offsetHeight / this._getNumberOfExpandedChildren();
        this._resizeHeight(maxChildHeight);
        this._resizeChildrenHeight(0);
    }
};
