/**
 * VPaned is a class to create vertical pane as a container
 * @base gtk.Paned
 * @constructor
 * @see gtk.Paned
 */
gtk.VPaned = function() {
	gtk.Paned.apply(this);
	
	this._dividerHeight = 6;
	this._paned1Height = (this._height / 2) - (this._dividerHeight/2);
	this._paned2Height = (this._height / 2) - (this._dividerHeight/2);
	this._isPaned1Restore = true;
	this._isPaned2Restore = true;
    this._topAnchored = 0;
	this._render = function(){
		this._domObj.className += " gtk_vPaned";
		this._domObj.style.width = this._width + 'px';
		this._domObj.style.height = this._height + 'px';
		this._domObj.style.position = 'relative';

			this._paned1 = jsgtk.Dom.create({'tag': 'div', 'class': 'gtk_paned1'});
			this._paned1.style.width = this._width + 'px';
			this._paned1.style.height = this._paned1Height + 'px';
		
			this._domDivider = jsgtk.Dom.create({'tag': 'div', 'class': 'gtk_dom_divider'});
				this._dom = jsgtk.Dom.create({'append': this._domDivider, 'tag': "div", 'class': 'gtk_paned_divider_image'});
			
			this._domDivider.style.width = this._width + 'px';
			this._domDivider.style.height = this._dividerHeight + "px";
			this._domDivider.style.top = this._paned1Height + 'px';
			
			this._ghostDivider = this._domDivider.cloneNode(true);
			this._ghostDivider.className = "gtk_paned_ghost_divider";
		
			this._paned2 = jsgtk.Dom.create({'tag': 'div', 'class': 'gtk_paned2'});
			this._paned2.style.width = this._width + 'px';
			this._paned2.style.height = this._paned1Height + 'px';

		this._domObj.appendChild(this._paned1);
		this._domObj.appendChild(this._domDivider);
		this._domObj.appendChild(this._paned2);
		this._domObj.appendChild(this._ghostDivider);
	};

	this._render();	
	
	//set the draggable area
	this._vBehaviour(0, this._height - this._dividerHeight);
};

gtk.VPaned.prototype = new gtk.Paned();
gtk.VPaned.prototype.constructor = gtk.VPaned;

/** 
 * A method to set position of the vertical paned
 * @param {Integer} position The integer value to define the height
 */
gtk.VPaned.prototype.setPosition = function(position){
    var widget1Height = this._child1.getMinHeightComposite();
    this._topAnchored = (widget1Height < position)? position : widget1Height;
};

/** 
 * A method to set autoresize given a widget
 * @param {Dom} widget The Dom widget to be passed
 */
gtk.VPaned.prototype.autoResize = function(widget){
	var paned1 = this._paned1;
	var paned2 = this._paned2;
	this._originalPaned1Height = paned1.style.height;
	this._originalPaned1ChildHeight = paned1.childNodes[0].style.height;
	this._originalPaned2Height= paned2.style.height;
	this._originalPaned2ChildHeight = paned2.childNodes[0].style.height;

	if(widget._domObj.parentNode.className == "gtk_paned1" ){
		this._child2.resizeHeightComposite(this._height);
		this._child1.hide();
		paned1.style.height = '0px';
		this._isPaned1Restore = false;
	}

	if(widget._domObj.parentNode.className == "gtk_paned2" ){
		this._child1.resizeHeightComposite(this._height);
		this._child2.hide();
		paned2.style.height = '0px';
		this._isPaned2Restore = false;
	}
	this._domDivider.style.display = 'none';
	this._ghostDivider.style.display = 'none';
	
	if(!this._isPaned1Restore && !this._isPaned2Restore){
		this.emit('childrendminimize');
	}
	
};

/** 
 * A method to restore its status
 * @param {Dom} widget The Dom widget to be passed
 */
gtk.VPaned.prototype.restoreStatus = function(widget){
	if(widget === this._child1){
		if(this._isPaned2Restore){
			this._paned1.style.height = this._paned1Height + "px";
			this._child1.resizeHeightComposite(this._paned1Height);
            this._child1.show();
            
			this._paned2.style.height = this._paned2Height + "px";
			this._child2.resizeHeightComposite(this._paned2Height);
            this._child2.show();
            
			this._domDivider.style.display = 'block';
			this._ghostDivider.style.display = 'block';	
		}else{
			this._paned1.style.height = this._height + "px";
			this._child1.resizeHeightComposite(this._height);
			this._child1.show();
		}
		this._isPaned1Restore = true;
	}

	if(widget === this._child2){
		if(this._isPaned1Restore){
			this._paned2.style.height = this._paned2Height + "px";
			this._child2.resizeHeightComposite(this._paned2Height);
            this._child2.show();
            
			this._paned1.style.height = this._paned1Height + "px";
			this._child1.resizeHeightComposite(this._paned1Height);
            this._child1.show();
            
			this._domDivider.style.display = 'block';
			this._ghostDivider.style.display = 'block';
		}else{
			this._paned2.style.height = this._height + "px";
			this._child2.resizeHeightComposite(this._height);
			this._child2.show();
		}
		this._isPaned2Restore = true;
	}

	if(this._isPaned1Restore || this._isPaned2Restore) {
		this.emit('childrestore');
	}
};

/** @private */
gtk.VPaned.prototype._resizePaned1Width = function(){
	if(this._width > this._child1.getWidth()) 
		this._child1.resizeWidthComposite(this._width);
	else this._resizeWidth(this._child1.getWidth());
};

/** @private */
gtk.VPaned.prototype._resizePaned1Height = function(isInit){

	if(isInit) this._initModifyPaned1Height();
	else this._modifyPaned1Height();
	
	this._resizeHeight(this._paned1Height, this._paned2Height);
	this._topMinY = this._child1.getMinHeightComposite();

	this._vBehaviour( this._topMinY, this._height - this._dividerHeight);
};

/** @private */
gtk.VPaned.prototype._resizePaned2Width = function(){
	if(this._width > this._child2.getWidth())
		this._child2.resizeWidthComposite(this._width);
	else{
		this._resizeWidth(this._child2.getWidth());
	}
};

/** @private */
gtk.VPaned.prototype._resizePaned2Height = function(isInit){

	if(isInit) this._initModifyPaned2Height();
	else this._modifyPaned2Height();	

	this._resizeHeight(this._paned1Height, this._paned2Height);
	this._bottomMinY = this._child2.getMinHeightComposite();
	var maxY = this._height - this._dividerHeight - this._bottomMinY;

	this._vBehaviour( this._topMinY, maxY);
};

/** @private */
gtk.VPaned.prototype._initModifyPaned1Height = function(){
	if(this._paned1Height > this._child1.getHeight()) 
		this._child1.resizeHeightComposite(this._paned1Height);
	else{
		this._paned1Height = this._child1.getHeight();
	}
};

/** @private */
gtk.VPaned.prototype._initModifyPaned2Height = function(){
	if(this._paned2Height > this._child2.getHeight())
		this._child2.resizeHeightComposite(this._paned2Height);
	else
		this._paned2Height = this._child2.getHeight();
};

/** @private */
gtk.VPaned.prototype._modifyPaned1Height = function(){
	this._child1.resizeHeightComposite(this._paned1Height);
	this._paned1Height = this._child1.getHeight();
};

/** @private */
gtk.VPaned.prototype._modifyPaned2Height = function(){
	this._child2.resizeHeightComposite(this._paned2Height);
	this._paned2Height = this._child2.getHeight();
};

/** @private */
gtk.VPaned.prototype._vBehaviour = function(minY, maxY){
	var self = this;
	jsgtk.Util.DomDraggableAdapter.vDragInit(this._ghostDivider, minY, maxY);
	
	this._ghostDivider.onDragStart = function(){
        this.style.background = "#DFE8F6";
        this.style.opacity = "0.9";
        if(this.firstChild) this.removeChild(this.firstChild);
	};

	this._ghostDivider.onDragEnd = function(x, y){
		
		var topHeight = y;
		var bottomHeight = self._height - topHeight - self._dividerHeight;
        
        self.setPosition(topHeight);
		self._resizePaned12Dom(topHeight, bottomHeight);

		self._child1.resizeHeightComposite(topHeight);
		self._child2.resizeHeightComposite(bottomHeight);
		
        this.style.background = "";
        this.style.opacity = "0.5"; 
	};
};

/** @private */
gtk.VPaned.prototype._resizePaned12Dom = function(topHeight, bottomHeight){
	this._paned1Height = topHeight;
	this._paned2Height = bottomHeight;
	this._paned1.style.height = topHeight + 'px';
	this._paned2.style.height = bottomHeight + 'px';
};

/** @private */
gtk.VPaned.prototype._resizeWidth = function(width){
	this._width = width;
	this._domObj.style.width = this._width + 'px';
		this._paned1.style.width = this._width + 'px';
		this._domDivider.style.width = this._width + 'px';
		this._ghostDivider.style.width = this._width + 'px';
		this._paned2.style.width = this._width + 'px';
};

/** @private */
gtk.VPaned.prototype._resizeHeight = function(paned1Height, paned2Height){
	this._height = paned1Height + paned2Height + this._dividerHeight;
	this._domObj.style.height = this._height + 'px';
		this._paned1.style.height = paned1Height + 'px';
		this._ghostDivider.style.top = paned1Height + 'px';
		this._paned2.style.height = paned2Height + 'px';
};

/** 
 * A method to get or return minimum composite width
 * @return {Integer} The minimum composite width to be passed
 * @type Integer
 */
gtk.VPaned.prototype.getMinWidthComposite = function(){
	var widget1Width = this._child1.getMinWidthComposite();
	var widget2Width = this._child2.getMinWidthComposite();
	var minWidth = this.getSizeRequest()[0] > widget1Width ? this.getSizeRequest()[0] : widget1Width;
	minWidth = minWidth > widget2Width ? minWidth : widget2Width;

	return minWidth;
};

/** 
 * A method to get or return minimum composite height
 * @return {Integer} The minimum composite height to be passed
 * @type Integer
 */
gtk.VPaned.prototype.getMinHeightComposite = function(){
	var widget1Height = this._child1.getMinHeightComposite();
	var widget2Height = this._child2.getMinHeightComposite();
	var minHeight = widget1Height + widget2Height;
	minHeight = minHeight > this.getSizeRequest()[1] ? minHeight : this.getSizeRequest()[1];
	
	return minHeight;
};

/** 
 * A method to resize composite width of the widget and its childres
 * @param {Integer} width The integer value as width to be set
 */
gtk.VPaned.prototype.resizeWidthComposite = function(width){
	var minWidthComposite = this.getMinWidthComposite();
	var minWidth = width > minWidthComposite ? width : minWidthComposite;

	this._resizeWidth(minWidth);
			
	this._child1.resizeWidthComposite(minWidth);
	this._child2.resizeWidthComposite(minWidth);
	
	if(this._isPaned1Restore && this._isPaned2Restore) {
        this.resize();
	}
};

/** 
 * A method to resize composite height of the widget and its childres
 * @param {Integer} width The integer value as height to be set
 */
gtk.VPaned.prototype.resizeHeightComposite = function(height){
	var minHeightComposite = this.getMinHeightComposite();
	var minHeight = height > minHeightComposite ? height : minHeightComposite;

	var offsetHeight = (minHeight - this.getHeight()) / 2 ;
	
	this._resizeHeight(minHeight);
	
	var height1 = this._paned1Height + offsetHeight;
	var height2 = this._paned2Height + offsetHeight;
	    
	if(
		this._paned2Height === this._child2.getMinHeightComposite() || 
		this._paned1Height === this._child1.getMinHeightComposite() ||
		height1 			<  this._child1.getMinHeightComposite() ||
		height2   			<  this._child2.getMinHeightComposite()
	){
		if(this._paned2Height === this._child2.getMinHeightComposite() ||
		   height2 			   <  this._child2.getMinHeightComposite()			
		){
		    this._paned1Height = this._paned1Height + offsetHeight * 2;
		    this._paned2Height = this._child2.getMinHeightComposite();
		}
		
		if(this._paned1Height === this._child1.getMinHeightComposite() ||
		   height1 			   <  this._child1.getMinHeightComposite()				
		){
		    this._paned2Height = this._paned2Height + offsetHeight * 2;
   		    this._paned1Height = this._child1.getMinHeightComposite();
		}
			
	}else{
		this._paned1Height = height1;
		this._paned2Height = height2;
	}
	
	if(this._topAnchored !== 0){
        var widget1Height = this._child1.getMinHeightComposite();
	    var widget2Height = this._child2.getMinHeightComposite();
	    
	    var height = this._topAnchored + widget2Height;
	    this._paned1Height = height < minHeight ? this._topAnchored : minHeight - widget2Height;
		this._paned2Height = minHeight - this._paned1Height;
	}
	
    this._paned1Height -= 2;
	this._paned2Height -= 2;
	
	if(this._isPaned1Restore && this._isPaned2Restore) {
        this.resize();
	}
};
