/**
 * Widget is a main base class of all widgets, this widget inherit from gtk.Object.
 * It is used to create any widget such gtk.Entry, gtk.ComboBox, ... and so on.
 * @constructor
 * @base gtk.Object
 */
gtk.Widget = function() {
	gtk.Object.apply(this);
	
    var self 					= this;
	this._domObj;
	this._minWidth 				= 16;
	this._minHeight 			= 16;
	this._width 				= this._minWidth;
	this._height 				= this._minHeight;
	this._isMinWidthChanged 	= false;
	this._isMinHeightChanged 	= false;
	this._activated 			= false;
	this._tag 					= null;
	this._domObj;
	this._signalName 			= "";
	this._parent 				= null;
	this._tooltipObj 			= null;
	this._enabled				= true;
	this.borderWidth 			= 2 ;

	this._render = function(){
		this._domObj = jsgtk.Dom.create({tag: "div", className: "gtk_widget"});
		var domObjStyle = this._domObj.style;
		domObjStyle.width = this._width + 'px';
		domObjStyle.height = this._height + 'px';
		domObjStyle.boder = "1px";
        domObjStyle.position = "relative";
	};
	
	this._render();
};

gtk.Widget.prototype = new gtk.Object();
gtk.Widget.prototype.constructor = gtk.Widget;

/**
 * A method to show the loading image while loading
 */
gtk.Widget.prototype.showLoading = function(){
    if(!this._loadingImgContainer){
		this._loadingImgContainer = this._constructLoading();
        this._appendLoading();
    }else this._loadingImgContainer.style.display = "table";
};

/**
 * A method to hide loading image inside the widget
 */
gtk.Widget.prototype.hideLoading = function(){
    jsgtk.Dom.hideElement(this._loadingImgContainer);
};

/**
 * A method to enable widget, widget will turn to its normal state
 * @see gtk.Widget#disable
 * @see gtk.Widget#setEnable
 */
gtk.Widget.prototype.enable = function(){};

/**
 * A method to disable widget, widget will be no more functioning
 * @see gtk.Widget#enable
 * @see gtk.Widget#setEnable
 */
gtk.Widget.prototype.disable = function(){};

/**
 * A method to set enabled or disabled
 * @param {Boolean}, true to set enabled, false to set disabled
 * @see gtk.Widget#enable
 * @see gtk.Widget#disable
 */
gtk.Widget.prototype.setEnable = function(isEnabled){
	this._enabled = (typeof isEnabled != 'undefined' && typeof isEnabled == 'boolean') ? isEnabled : true;
    this._enabled ? this.enable() : this.disable();
};

/**
 * A method to set tag onto it
 * @param {String} tag The tag name to be set
 */
gtk.Widget.prototype.setTag = function(tag){
    this._tag = tag;  
};

/**
 * A method to get its tag
 * @return {String} The tag name of the widget
 * @type String
 */
gtk.Widget.prototype.getTag = function(){
    return this._tag;  
};

/**
 * A method to store the reference of widget's parent
 * @param {Object} widget The parent to be set
 */
gtk.Widget.prototype.setParent = function(widget){
    this._parent = widget;  
};

/**
 * A method to get reference of wiget's parent widget
 * @return {Object} The parent object of the widget
 * @type Object
 */
gtk.Widget.prototype.getParent = function(){
    return this._parent;  
};

/**
 * A method to show or hide tooltip on widget. showTooltip(false) = hideTooltip()
 * @param {Boolean} isToShow The boolean value true is to show, false is not
 * @see gtk.Widget#hideTooltip
 * @see gtk.Widget#setTooltip
 */
gtk.Widget.prototype.showTooltip = function(isToShow){
	this._isToShow = (typeof isToShow != 'undefined' && typeof isToShow == 'boolean') ? isToShow : true;
	if(this._tooltipObj !== null){
		this._isToShow ? this._tooltipObj.show() : this._tooltipObj.hide();
	}
};

/**
 * A method to hide tooltip. It is used the same as showTooltip(false)
 * @see gtk.Widget#showTooltip
 * @see gtk.Widget#setTooltip
 */
gtk.Widget.prototype.hideTooltip = function(){
	if(this._tooltipObj !== null) this._tooltipObj.hide();
};

/**
 * A method to set tooltip content into its body
 * @param {String} textMessage The content message inside the tooltip to be display
 * @see gtk.Widget#showTooltip
 * @see gtk.Widget#hideTooltip
 */
gtk.Widget.prototype.setTooltip = function(textMessage){
	var contentMessage = textMessage || "&nbsp;&nbsp;<span style='color: #7F7F7F'>Value is undefined</span>&nbsp;&nbsp;&nbsp;&nbsp;";
	var contentObj = jsgtk.Dom.create({"tag": "div","style": "display: block;"});
		contentObj.innerHTML = contentMessage;

	if(this._tooltipObj === null){
		this._tooltipObj = new gtk.Bubble(contentObj, this._domObj, "", 8);	
		this._tooltipObj.setTooltip();
		this._tooltipObj.hide();
		
	} 
	else {
		this._tooltipObj.addContent(contentObj);
		this._tooltipObj.hide();
	}
};

/**
 * A method to reload or refresh the contents inside the widget
 */
gtk.Widget.prototype.refresh = function(){
	var minWidth = this.getMinWidthComposite();
	minWidth = minWidth < this._width ? this._width : minWidth;
    this.resizeWidthComposite(minWidth);

	var minHeight = this.getMinHeightComposite();
	minHeight = minHeight < this._height ? this._height : minHeight;
    this.resizeHeightComposite(minHeight);
};

/**
 * A method to set title to bold style
 * @param {Boolean} bold The boolean value true is bold and false is not
 */
gtk.Widget.prototype.setBoldTitle = function(bold){
	if(bold){
		this._domTitle.style.fontWeight = "bold";
	} else 	this._domTitle.style.fontWeight = "normal";
};

/**
 * A method to return Dom wiget as an object
 * @return {Object} The Dom object value to be appended to any HTML element
 * @type Object
 */
gtk.Widget.prototype.getNode = function(){
	return this._domObj;
};

/**
 * A method to show the widget
 * @see gtk.Widget#hide
 */
gtk.Widget.prototype.show = function(){
	this._domObj.style.display = "";
};

/**
 * A method to hide the widget
 * @see gtk.Widget#show
 */
gtk.Widget.prototype.hide = function(){
	this._domObj.style.display = "none";
};

/**
 * A method to return true if widget is hidden and vice versa
 * @return {Boolean} The boolean value, true or false
 */
gtk.Widget.prototype.isHidden = function(){
	return !jsgtk.Dom.isShowed(this._domObj);
};

/**
 * A method to emit the "activated" signal on the widget
 * @return {Boolean} The boolean value which is true or false
 * @type Boolean
 */
gtk.Widget.prototype.activate = function(){
	var signal = this.getActivateSignal();

	if(signal){
		this._activated = true;
		this.emit(signal);	
	} 
	else this._activated = false;

	return this._activated;
};
/**
 * A method to set the activate signal name
 * @param {String} signal The signal name to be set. And it must be from any conventional name of JSGTK
 */
gtk.Widget.prototype.setActivateSignal = function(signal){
	this._signalName = signal;
};

/**
 * A method to return the name of the signal that is emitted when the widget is activated.
 * @return {String} The signal name that is registered
 * @type String
 */
gtk.Widget.prototype.getActivateSignal = function(){
	return this._signalName;
};

/**
 * A method to reset the minimum width and height of widget
 * @param {Integer} width  The minimum width value to be set
 * @param {Integer} height The minimum height value to be set
 */
gtk.Widget.prototype.setSizeRequest = function(width, height){
	this._isMinWidthChanged = width ? true : false;
	this._isMinHeightChanged = height ? true : false;
	
	this._minHeight = height;
	this._minWidth = width;
};
/**
 * Reset the mininimum width and minimum height of widget
 * @param {Integer} width  The new minimum width value of the widget
 * @param {Integer} height  The new minimum height value of the widget
 */
gtk.Widget.prototype.resetMinSize = function(minWidth, minHeight){
	this.resetMinWidth(minWidth);
	this.resetMinHeight(minHeight);
};
/**
 * A method to reset the mininimum width of widget
 * @param {Integer} width  The new minimum width value of the widget
 */
gtk.Widget.prototype.resetMinWidth = function(width){
	this._isMinWidthChanged = width ? true : false;
	this._minWidth = width;
    this._width = width;
};
/**
 * A method to reset the mininimum height of widget
 * @param {Integer} height  The new minimum height value in pixel
 */
gtk.Widget.prototype.resetMinHeight = function(height){
	this._isMinHeightChanged = height ? true : false;
	this._minHeight = height;
    this._height = height;
};

/**
 * A method to set the width and height of widget
 * @param {Integer} width  The integer width value to be set
 * @param {Integer} height The integer height value to be set
 */
gtk.Widget.prototype.setSize = function(width, height){
	this._height = height;
	this._width = width;
};

/**
 * A method to set new width to widget
 * @param {Integer} width : new width of widget
 */
gtk.Widget.prototype.setWidth = function(width){
	this._width = width;
};

/**
 * A method to return the current width to widget
 * @return {Integer} The integer width value of the widget
 * @type Integer
 */
gtk.Widget.prototype.getWidth = function(){
	return this._width;
};

/**
 * A method to set new height to the widget
 * @param {Integer} height The integer height value in pixel to be set
 */
gtk.Widget.prototype.setHeight = function(height){
	this._height = height;
};

/**
 * A method to return the current height to widget
 * @return {Integer} The height value in pixel of the widget
 * @type Integer
 */
gtk.Widget.prototype.getHeight = function(){
	return this._height;
};

/**
 * A method to return the minimum width and height of widget
 * @return {Array} The array values of minimum width and height
 * @type Array
 */
gtk.Widget.prototype.getSizeRequest = function(){
	return [this._minWidth, this._minHeight];
};

/**
 * A method to get width and height
 * @return {Array} The array value contains width and height
 * @type Array
 */
gtk.Widget.prototype.sizeRequest = function(){
	return [
		this._width,
		this._height
	];
};

/**
 * A method to return an array of current width and height
 * @param {Array} The array values contains width and height
 * @type Array
 */
gtk.Widget.prototype.getSize = function(){
	return [
		this._width,
		this._height
	];
};

/** @private */
gtk.Widget.prototype._constructLoading = function(){
    var loadingImgContainer = jsgtk.Dom.create({tag: "table", 'class': 'gtk_loading_bar_container_widget'});
    	var tr = jsgtk.Dom.create({append: loadingImgContainer, tag: "tr"});
			var td = jsgtk.Dom.create({append: tr, tag: "td"});
                this._loadingImgFrame = jsgtk.Dom.create({"append": td, "tag": "div", "class": "gtk_loading_frame"});
                    this._loadingImg      = jsgtk.Dom.create({"append": this._loadingImgFrame, "tag": "div", "class": "gtk_loading_bar_widget"});
    return loadingImgContainer;
};

/** @private */
gtk.Widget.prototype._appendLoading = function(){
    var firstChild = this._domObj.firstChild;
    if(firstChild)
        this._domObj.insertBefore(this._loadingImgContainer, firstChild);
    else
        this._domObj.appendChild(this._loadingImgContainer);
};

/** @private */
gtk.Widget.prototype._resizeHeight = function(height){
	this._domObj.style.height = height + 'px';
	this._height = height;
};

/** @private */
gtk.Widget.prototype._resizeWidth = function(width){
	this._domObj.style.width = width + 'px';
	this._width = width;
};

/**
 * A method to return the minimum composite width, if the widget contains children, 
 * the minWidth will be smallest with that the widget and its children can be shrinken
 * @return {Integer} The minimum width value in pixel
 * @type Integer
 */
gtk.Widget.prototype.getMinWidthComposite = function(){
	return this._minWidth;
};

/**
 * A method to return the minimum composite height, if the widget contains children, 
 * the minHeight will be the smallest height that the widget and its children can be shrinken
 * @return {Integer} The minimum height value in pixel
 * @type Integer
 */
gtk.Widget.prototype.getMinHeightComposite = function(){
	return this._minHeight;
};

/**
 * A method to resize the wiget and its children's width
 * @param {Integer} width The integer width value in pixel to be set
 */
gtk.Widget.prototype.resizeWidthComposite = function(width){
	this._width = width;
	this._domObj.style.width = this._width + 'px';
};

/**
 * A method to resize the wiget and its children's heights
 * @param {Integer} height The height in pixel value to be set
 */
gtk.Widget.prototype.resizeHeightComposite = function(height){
	this._height = height;
	this._domObj.style.height = this._height + 'px';
};

/**
 * A method to reset to default composite height
 */
gtk.Widget.prototype.resetHeightComposite = function(){
	var minSize = this.getSizeRequest();
	this.setHeight(minSize[1]);
};

/**
 * A method to reset to default composite width
 */
gtk.Widget.prototype.resetWidthComposite = function(){
	var minSize = this.getSizeRequest();
	this.setWidth(minSize[0]);
};
