/**
 *	Window is a class to create window which can contain a child widget
 * @param {String} type A constant variable value to define type of window.
 * <br>The type of window is optional.
 * <br>These are the constant values to be passed:
 * <br> - gtk.WINDOW_TOPLEVEL The flag constant value to define as top level or top most
 * <br> - gtk.WINDOW_POPUP The flag constant value to define as window popup
 * @extends gtk.WindowBase
 * @base gtk.WindowBase
 * @see gtk.WindowBase
 * @constructor
 */
gtk.Window = function(type) {
	gtk.WindowBase.apply(this);

	this._sideBarsWidth = 10;
	this._state = new gtk.StateWindowRestored(this, null);

	this._childWidget = null;	

	this._headerHeight = 27;
	
	this._render = function(){
	    var domObj = this._domObj;
        domObj.className += " gtk_window_new";
            
            this._ghostWindowDom = jsgtk.Dom.create({"append": domObj, "tag": "div", "class": "gtk_window_ghost_new"});
            jsgtk.Dom.hideElement(this._ghostWindowDom);
            
            this._leftBarDom = jsgtk.Dom.create({"append": domObj, "tag": "div", "class": "resizeable_bar gtk_window_left_bar_new"});
            this._rightBarDom = jsgtk.Dom.create({"append": domObj, "tag": "div", "class": "resizeable_bar gtk_window_right_bar_new"});
            this._topBarDom = jsgtk.Dom.create({"append": domObj, "tag": "div", "class": "resizeable_bar gtk_window_top_bar_new"});
            this._bottomBarDom = jsgtk.Dom.create({"append": domObj, "tag": "div", "class": "resizeable_bar gtk_window_bottom_bar_new"});
            
            this._topLeftBarDom = jsgtk.Dom.create({"append": domObj, "tag": "div", "class": "resizeable_bar_corner gtk_window_top_left_bar_new"});
            this._topRightBarDom = jsgtk.Dom.create({"append": domObj, "tag": "div", "class": "resizeable_bar_corner gtk_window_top_right_bar_new"});
            this._bottomLeftBarDom = jsgtk.Dom.create({"append": domObj, "tag": "div", "class": "resizeable_bar_corner gtk_window_bottom_left_bar_new"});
            this._bottomRightBarDom = jsgtk.Dom.create({"append": domObj, "tag": "div", "class": "resizeable_bar_corner gtk_window_bottom_right_bar_new"});            
	};
	
	this._render();
};

gtk.Window.prototype = new gtk.WindowBase();
gtk.Window.prototype.constructor = gtk.Window;
/**
 * A method to draw or show the window after instantiated
 */
gtk.Window.prototype.draw = function(){
    document.body.appendChild(this._domObj);
};
/**
 * A method to draw a modal layer 
 */
gtk.Window.prototype.drawModal = function(){
    this._showModal();
    document.body.appendChild(this._domObj);
};
/**
 * A method to append or add or replace child into its body
 * @param {Dom} childWidget The child widget or Dom object to be passed
 */
gtk.Window.prototype.appendChild = function(childWidget){
	if(this._childWidget) this._clearChild();

    this._childWidget = childWidget;
    this._bodyDom.appendChild(this._childWidget.getNode());
    
    this.resizeWidthComposite(this.getMinWidthComposite());
    this.resizeHeightComposite(this.getMinHeightComposite());
    
    this._initEvent();
};
/**
 * A method to set maximized the window
 */
gtk.Window.prototype.maximize = function(){
	this.emit("maximized", this);
	
	if(!this.getParent()){
	    this._restoredOption = {
            "width": this._width, 
            "height": this._height, 
            "left": parseInt(this._domObj.style.left),
            "top": parseInt(this._domObj.style.top)
        };
        
        this._state.maximize();
	}
};
/**
 * A method to restore window to its previous state
 */
gtk.Window.prototype.restore = function(){
    this._state.restore();

    var id = this._domObj.id;
    this._domObj.id = "";
    if(this._isWindowRestored(id)) return;
    this.emit('restored', this);

    jsgtk.Dom.showElement(this._domObj);
	jsgtk.Dom.showElement(this._minimizeButton);
    this._resizeDomTitleWidth(this._width - this._inconsWidth);
    
    jsgtk.Dom.hideElement(this._minimizedWindow);
    this._domObj.id = id;
};
/**
 * A method to set minimized the window
 */
gtk.Window.prototype.minimize = function(){
    this.emit('minimized', this);
    
    this._state.minimize();
    
    jsgtk.Dom.hideElement(this._domObj);
	jsgtk.Dom.hideElement(this._minimizeButton);
	
    var holder = this._createMinimizedHolder();
    if(this._isNodeExistInMinimizedHolder(holder)){
        jsgtk.Dom.showElement(this._minimizedWindow);    
        return;   
    }

    this._minimizedWindow = this._createMinimizedWindow();
    holder.appendChild(this._minimizedWindow);    

    this._initMinimizedEvents();
};
/** @private */
gtk.Window.prototype._initEvent = function(){
    var self = this;

    jsgtk.Event.addDomListener(this._bodyDom, "mousedown", function(e){
        jsgtk.Event.stopBubble(e); 
    });    

    self._resizeLeftBar();
    self._resizeRightBar();
    self._resizeTopBar();    
    self._resizeBottomBar();        

    self._resizeTopLeftBar();
    self._resizeTopRightBar();
    self._resizeBottomLeftBar();
    self._resizeBottomRightBar();

    jsgtk.Util.DomDraggableAdapter.dragInit(this._domObj, 1, jsgtk.Util.getWindowWidth()-this._width-17, 0, jsgtk.Util.getWindowHeight()-this._height-17);
	
	/** @private */
	this._domObj.onDrag = function(){
		self._checkAndHideComboDropdown();
    	self._checkAndHideToolTip();
	};
};
/** @private */
gtk.Window.prototype._resizeToLeft = function(x){
    var currentLeft = parseInt(this._domObj.style.left);
    this._domObj.style.left = currentLeft + x + "px";
    this.resizeWidthComposite(parseInt(this._ghostWindowDom.style.width));
    this._resetGhostWidth(parseInt(this._ghostWindowDom.style.width));
};
/** @private */
gtk.Window.prototype._resizeToTop = function(y){
    var currentTop = parseInt(this._domObj.style.top);
    this._domObj.style.top = currentTop + y + "px";
    this.resizeHeightComposite(parseInt(this._ghostWindowDom.style.height));    
    this._resetGhostHeight(parseInt(this._ghostWindowDom.style.height));
};
/** @private */
gtk.Window.prototype._resizeToRight = function(x){
    this.resizeWidthComposite(x);
    this._resetGhostWidth(x);
};
/** @private */
gtk.Window.prototype._resizeToBottom = function(y){
    this.resizeHeightComposite(y);
    this._resetGhostHeight(y);
};
/** @private */
gtk.Window.prototype._resizeLeftBar = function(){
    var self = this;
    var minX = parseInt(window.innerWidth);
    jsgtk.Util.DomDraggableAdapter.hDragInit(self._leftBarDom, -minX, self._width - self._minWidth);
    
    /** @private */
    this._leftBarDom.onDragStart = function(x, y, e){
        jsgtk.Dom.showElement(self._ghostWindowDom); 
        jsgtk.Event.stopBubble(e);
    };
    
    /** @private */
    this._leftBarDom.onDrag = function(x, y, e){
        self._ghostWindowDom.style.left = x + "px";
        self._ghostWindowDom.style.width = self._ghostWidth - x + "px";
    };
    
    /** @private */
    this._leftBarDom.onDragEnd = function(x, y){
        jsgtk.Dom.hideElement(self._ghostWindowDom);
        self._resizeToLeft(x);
        
        self._resizeLeftBar();
    };
};
/** @private */
gtk.Window.prototype._resizeRightBar = function(){
    var isResized = false;
    var self = this;
    var minX = self._minWidth;
    jsgtk.Util.DomDraggableAdapter.hDragInit(self._rightBarDom, minX);

	/** @private */
    this._rightBarDom.onDragStart = function(x, y, e){
        jsgtk.Dom.showElement(self._ghostWindowDom);
        jsgtk.Event.stopBubble(e);
    };

	/** @private */
    this._rightBarDom.onDrag = function(x, y, e){
        self._ghostWindowDom.style.width = x + "px";
        isResized = true;
    };
	
	/** @private */
    this._rightBarDom.onDragEnd = function(x, y){
        jsgtk.Dom.hideElement(self._ghostWindowDom);

        if(!isResized) return;
        
        self._resizeToRight(x);
        
        self._resizeRightBar();
    };
};
/** @private */
gtk.Window.prototype._resizeTopBar = function(){
    var self = this;
    var minY = parseInt(window.innerHeight);
    jsgtk.Util.DomDraggableAdapter.vDragInit(self._topBarDom, -minY, self._height - self._minHeight);
	
	/** @private */
    this._topBarDom.onDragStart = function(x, y, e){
        jsgtk.Dom.showElement(self._ghostWindowDom);
        jsgtk.Event.stopBubble(e);
    };

	/** @private */
    this._topBarDom.onDrag = function(x, y, e){
        self._ghostWindowDom.style.top = y + "px";
        self._ghostWindowDom.style.height = self._ghostHeight - y + "px";
    };
	
	/** @private */
    this._topBarDom.onDragEnd = function(x, y){
        jsgtk.Dom.hideElement(self._ghostWindowDom);
        
        self._resizeToTop(y);
        
        self._resizeTopBar();
    };
};
/** @private */
gtk.Window.prototype._resizeBottomBar = function(){
    var self = this;
    var isResized = false;
    jsgtk.Util.DomDraggableAdapter.vDragInit(self._bottomBarDom, self._minHeight);

	/** @private */
    this._bottomBarDom.onDragStart = function(x, y, e){
        jsgtk.Dom.showElement(self._ghostWindowDom);
        jsgtk.Event.stopBubble(e);
    };

	/** @private */
    this._bottomBarDom.onDrag = function(x, y, e){
        self._ghostWindowDom.style.height = y + "px";
        isResized = true;
    };

	/** @private */
    this._bottomBarDom.onDragEnd = function(x, y){
        jsgtk.Dom.hideElement(self._ghostWindowDom);
        
        if(!isResized) return;
        
        self._resizeToBottom(y);
        
        self._resizeBottomBar();
    };
};
/** @private */
gtk.Window.prototype._resizeTopLeftBar = function(){
    var self = this;
    var minX = parseInt(window.innerWidth);
    var maxX = this._width - this._minWidth;
    var minY = parseInt(window.innerHeight);
    var maxY = this._height - this._minHeight;
    
    jsgtk.Util.DomDraggableAdapter.dragInit(this._topLeftBarDom, -minX, maxX, -minY, maxY);
    
    /** @private */
    this._topLeftBarDom.onDragStart = function(x, y, e){ 
        jsgtk.Dom.showElement(self._ghostWindowDom);
        jsgtk.Event.stopBubble(e);
    };
    
    /** @private */
    this._topLeftBarDom.onDrag = function(x, y, e){
        self._ghostWindowDom.style.left = x + "px";
        self._ghostWindowDom.style.width = self._ghostWidth - x + "px";
        
        self._ghostWindowDom.style.top = y + "px";
        self._ghostWindowDom.style.height = self._ghostHeight - y + "px";
    };
    
    /** @private */
    this._topLeftBarDom.onDragEnd = function(x, y){
        jsgtk.Dom.hideElement(self._ghostWindowDom);
        
        self._resizeToTop(y);
        self._resizeToLeft(x);
   
        self._resizeTopLeftBar();
    };
};
/** @private */
gtk.Window.prototype._resizeTopRightBar = function(){
    var isResized = false;
    var self = this;
    var minX = this._minWidth;
    var maxX = parseInt(window.innerWidth);
    var minY = parseInt(window.innerHeight);
    var maxY = this._height - this._minHeight;
    
    jsgtk.Util.DomDraggableAdapter.dragInit(this._topRightBarDom, minX, maxX, -minY, maxY);
    
    /** @private */
    this._topRightBarDom.onDragStart = function(x, y, e){ 
        jsgtk.Dom.showElement(self._ghostWindowDom);
        jsgtk.Event.stopBubble(e);
    };
    
    /** @private */
    this._topRightBarDom.onDrag = function(x, y, e){
        self._ghostWindowDom.style.width = x + "px";
        
        self._ghostWindowDom.style.top = y + "px";
        self._ghostWindowDom.style.height = self._ghostHeight - y + "px";
        
        isResized = true;
    };
    
    /** @private */
    this._topRightBarDom.onDragEnd = function(x, y){
        jsgtk.Dom.hideElement(self._ghostWindowDom);
        
        if(!isResized) return;
        
        self._resizeToTop(y);
        self._resizeToRight(x);
        
        self._resizeTopRightBar();
    };   
};
/** @private */
gtk.Window.prototype._resizeBottomLeftBar = function(){
    var isResized = false;
    var self = this;
    var minX = parseInt(window.innerWidth);
    var maxX = this._width - this._minWidth;
    var minY = self._minHeight;
    var maxY = parseInt(window.innerHeight);
    
    jsgtk.Util.DomDraggableAdapter.dragInit(this._bottomLeftBarDom, -minX, maxX, minY, maxY);
    
    /** @private */
    this._bottomLeftBarDom.onDragStart = function(x, y, e){ 
        jsgtk.Dom.showElement(self._ghostWindowDom);
        jsgtk.Event.stopBubble(e);
    };
    
    /** @private */
    this._bottomLeftBarDom.onDrag = function(x, y, e){
        self._ghostWindowDom.style.left = x + "px";
        self._ghostWindowDom.style.width = self._ghostWidth - x + "px";
        
        self._ghostWindowDom.style.height = y + "px";
        isResized = true;
    };
    
    /** @private */
    this._bottomLeftBarDom.onDragEnd = function(x, y){
        jsgtk.Dom.hideElement(self._ghostWindowDom);
        
        if(!isResized) return;
        
        self._resizeToLeft(x);
        self._resizeToBottom(y);
        
        self._resizeBottomLeftBar();
    };
};
/** @private */
gtk.Window.prototype._resizeBottomRightBar = function(){
    var isResized = false;
    var self = this;
    var minX = self._minWidth;
    var maxX = parseInt(window.innerWidth);
    var minY = self._minHeight;
    var maxY = parseInt(window.innerHeight);
    
    jsgtk.Util.DomDraggableAdapter.dragInit(this._bottomRightBarDom, minX, maxX, minY, maxY);
    
    /** @private */
    this._bottomRightBarDom.onDragStart = function(x, y, e){ 
        jsgtk.Dom.showElement(self._ghostWindowDom);
        jsgtk.Event.stopBubble(e);
    };
    
    /** @private */
    this._bottomRightBarDom.onDrag = function(x, y, e){
        self._ghostWindowDom.style.width = x + "px";
        self._ghostWindowDom.style.height = y + "px";
        isResized = true;
    };
    
    /** @private */
    this._bottomRightBarDom.onDragEnd = function(x, y){
        jsgtk.Dom.hideElement(self._ghostWindowDom);
        
        if(!isResized) return;
        
        self._resizeToRight(x);
        self._resizeToBottom(y);
        
        self._resizeBottomRightBar();
    };
};
/** @private */
gtk.Window.prototype._resetGhostWidth = function(width){
    this._ghostWidth = width;
};
/** @private */
gtk.Window.prototype._resetGhostHeight = function(height){
    this._ghostHeight = height;
};
/** @private */
gtk.Window.prototype._repositionResizeableBars = function(){
    this._leftBarDom.style.left = "0px";
    this._rightBarDom.style.left = this._width - 3 + "px";
    this._topBarDom.style.top = "0px";
    this._bottomBarDom.style.top = this._height - 3 + "px";
    
    this._topLeftBarDom.style.left = "0px";
    this._topLeftBarDom.style.top = "0px";
    
    this._topRightBarDom.style.left = this._width - 3 + "px";
    this._topRightBarDom.style.top = "0px";
    
    this._bottomLeftBarDom.style.top = this._height - 3 + "px";
    this._bottomLeftBarDom.style.left = "0px";
    
    this._bottomRightBarDom.style.top = this._height - 3 + "px";
    this._bottomRightBarDom.style.left = this._width - 3 + "px";
    
    this._ghostWindowDom.style.left = "0px";
    this._ghostWindowDom.style.top = "0px";
};
/** @private */
gtk.Window.prototype._repositionElements = function(){
    this._repositionResizeableBars();
};
/** @private */
gtk.Window.prototype._resizeWidth = function(width){
    this._width = width;
    this._domObj.style.width = width + "px";
        this._bodyDom.style.width = width - this._sideBarsWidth + "px";
        this._ghostWindowDom.style.width = width + "px";
        
    this._repositionElements();
};
/** @private */
gtk.Window.prototype._resizeHeight = function(height){
    this._height = height;
    this._domObj.style.height = height + "px";
        this._bodyDom.style.height = height - this._headerHeight + "px";
        this._ghostWindowDom.style.height = height + "px";
        
    this._repositionElements();
};
/** @private */
gtk.Window.prototype.resetMinWidth = function(width){
    this._minWidth = width;
    this._ghostWidth = width;
};
/** @private */
gtk.Window.prototype.resetMinHeight = function(height){
    this._minHeight = height;
    this._ghostHeight = height;
};
/** @private */
gtk.Window.prototype.getMinWidthComposite = function(){
    var childMinWidth = this._childWidget.getMinWidthComposite() + this._sideBarsWidth;
//    var minWidth = childMinWidth > this._minWidth ? childMinWidth : this._minWidth;
    this.resetMinWidth(childMinWidth);
    return childMinWidth;
};
/** @private */
gtk.Window.prototype.getMinHeightComposite = function(){
    var childMinHeight = this._childWidget.getMinHeightComposite() + this._headerHeight;
//    var minHeight = childMinHeight > this._minHeight ? childMinHeight : this._minHeight;
    this.resetMinHeight(childMinHeight)
    return childMinHeight;
};
/** @private */
gtk.Window.prototype.resizeWidthComposite = function(width){
    var minWidth = this.getMinWidthComposite();

    minWidth = minWidth < width ? width : minWidth;
    
    this._childWidget.resizeWidthComposite(minWidth - this._sideBarsWidth);
    
    this._resizeWidth(minWidth);
};
/** @private */
gtk.Window.prototype.resizeHeightComposite = function(height){
    var minHeight = this.getMinHeightComposite();
    minHeight = minHeight < height ? height : minHeight;
    
    this._childWidget.resizeHeightComposite(minHeight - this._headerHeight - 2);
    
    this._resizeHeight(minHeight);
};
