/**
* @constructor
* @base gtk.Widget
*/
gtk.WindowBase = function() {
	gtk.Widget.apply(this);
	
	this._minWidth = this._width = this._ghostWidth = this._noneMaximizedWidth = 20;
	this._minHeight = this._height = this._ghostHeight = this._noneMaximizedHeight = 20;
	this._headerHeight = 22;
	this._minimizedWidth      = 150;
	this._inconsWidth         = 62;
	this._minimizedWindow      = null;
	this._childWidget = null;
	this._isCloseButtonActive = false;
	
	this._enableControl = true;
	this._enableCloseButton = true;
	this._enableMaximizeButton = true;
	this._enableMinimizeButton = true;
	
	this._isModal = false;
	var self = this;
	
	
	this._render = function(){
	    var domObj = this._domObj;
        domObj.className += " gtk_window_base";
		domObj.id = new UUID().id;
        domObj.style.position = "absolute";
            
            this._headerDom = jsgtk.Dom.create({"append": domObj, "tag": "div", "class": "gtk_window_header_base"});
            
                this._titleDom = jsgtk.Dom.create({"append": this._headerDom, "text": "Window Title","tag": "div", "class": "gtk_window_title_base"});
                this._minimizeButton = jsgtk.Dom.create({"append": this._headerDom,"tag": "div", "title": "Minimize Window", "class": "gtk_window_minimized_botton_base"});
				this._maximizeButton = jsgtk.Dom.create({"append": this._headerDom,"tag": "div", "title": "Maximize Window", "class": "gtk_window_maximized_botton_base"});				
				this._returnToPreviousSizeButton = jsgtk.Dom.create({"append": this._headerDom,"tag": "div", "title": "Restore Window", "class": "gtk_window_return_size_botton_base"});												
				this._closeButton = jsgtk.Dom.create({"append": this._headerDom,"tag": "div", "title": "Close Window", "class": "gtk_window_close_botton_base"});
            
            this._bodyDom = jsgtk.Dom.create({"append": domObj, "tag": "div", "class": "gtk_window_body_new"});
	};
	
	this._render();

	jsgtk.Dom.hideElement(this._returnToPreviousSizeButton);
	jsgtk.Dom.showElement(this._maximizeButton);

    jsgtk.Event.addDomListener(this._minimizeButton, "mouseup", function(e){
    	self._checkAndHideComboDropdown();
    	self._checkAndHideToolTip();
    	self._checkAndHideSmartEntryDropdown();
        self.minimize();
    }, false);

	jsgtk.Event.addDomListener(this._maximizeButton, "mouseup", function(e){
		self._checkAndHideComboDropdown();
		self._checkAndHideToolTip();
		self._checkAndHideSmartEntryDropdown();
        self.maximize();
    }, false);

	jsgtk.Event.addDomListener(this._returnToPreviousSizeButton, "mouseup", function(e){
		self._checkAndHideComboDropdown();
		self._checkAndHideToolTip();
		self._checkAndHideSmartEntryDropdown();
        self.restore();
    }, false);
       
	this.setCloseButton(true);
};

gtk.WindowBase.prototype = new gtk.Widget();
gtk.WindowBase.prototype.constructor = gtk.WindowBase;

/**
 * A method to enable resizeable of the window. 
 * The window will be resisizeable.
 */
gtk.WindowBase.prototype.enableResize = function(){
	var jsgtkDom = jsgtk.Dom;
	jsgtkDom.showElement(this._leftBarDom);
	jsgtkDom.showElement(this._rightBarDom);
	jsgtkDom.showElement(this._topBarDom);
	jsgtkDom.showElement(this._bottomBarDom);
	jsgtkDom.showElement(this._topLeftBarDom);
	jsgtkDom.showElement(this._topRightBarDom);
	jsgtkDom.showElement(this._bottomLeftBarDom);
	jsgtkDom.showElement(this._bottomRightBarDom);
};

/**
 * A method to disable resizeable of the window. 
 * It will not allow user to resize the window.
 */
gtk.WindowBase.prototype.disableResize = function(){
	var jsgtkDom = jsgtk.Dom;
	jsgtkDom.hideElement(this._leftBarDom);
	jsgtkDom.hideElement(this._rightBarDom);
	jsgtkDom.hideElement(this._topBarDom);
	jsgtkDom.hideElement(this._bottomBarDom);
	jsgtkDom.hideElement(this._topLeftBarDom);
	jsgtkDom.hideElement(this._topRightBarDom);
	jsgtkDom.hideElement(this._bottomLeftBarDom);
	jsgtkDom.hideElement(this._bottomRightBarDom);
};

/**
 * A method to disable and hide all control buttons
 */
gtk.WindowBase.prototype.disableControl = function(){
	var jsgtkDom = jsgtk.Dom;
	jsgtkDom.hideElement(this._closeButton);
	jsgtkDom.hideElement(this._minimizeButton);
	jsgtkDom.hideElement(this._maximizeButton);
	jsgtkDom.hideElement(this._returnToPreviousSizeButton);
	this._enableControl=false;
};

/**
 * A method to enable and display all control buttons
 */
gtk.WindowBase.prototype.enableControl = function(){
    this._enableControl = true;
    if(this._enableMinimizeButton)this.enableMinimizeButton();
    if(this._enableMaximizeButton)this.enableMaximizeButton();
    if(this._enableCloseButton) this.enableCloseButton();
};

/**
 * A method to enable and display close button
 */
gtk.WindowBase.prototype.enableCloseButton = function(){
    if(this._enableControl){
        jsgtk.Dom.showElement(this._closeButton);
        if(this._enableMaximizeButton){
            this._minimizeButton.style.right = "40px"
            this._maximizeButton.style.right =this._returnToPreviousSizeButton.style.right= "24px";
        }else this._minimizeButton.style.right = "24px"
        this._enableCloseButton = true;
    }
};

/**
 * A method to enable and display minimize button
 */
gtk.WindowBase.prototype.enableMinimizeButton = function(){
    if(this._enableControl){
         jsgtk.Dom.showElement(this._minimizeButton);
         if((!this._enableCloseButton)&&(!this._enableMaximizeButton)) this._minimizeButton.style.right = "5px";
         else if(this._enableCloseButton && this._enableMaximizeButton) this._minimizeButton.style.right = "40px";
         else this._minimizeButton.style.right = "24px";
         this._enableMinimizeButton = true;
     }
};

/**
 * A method to enable and display maximize button
 */
gtk.WindowBase.prototype.enableMaximizeButton = function(){
   if(this._enableControl){
       jsgtk.Dom.showElement(this._returnToPreviousSizeButton);
       if(this._enableCloseButton){
            this._minimizeButton.style.right = "40px"
            this._maximizeButton.style.right =this._returnToPreviousSizeButton.style.right= "24px";
       }
       this._enableMaximizeButton = true;
   }
};

/**
 * A method to disable and hide close button
 */
gtk.WindowBase.prototype.disableCloseButton = function(){
    jsgtk.Dom.hideElement(this._closeButton);
    
    if(this._enableMaximizeButton){
        this._maximizeButton.style.right =this._returnToPreviousSizeButton.style.right= "5px";
        this._minimizeButton.style.right = "24px"
    }else this._minimizeButton.style.right = "5px"
    
    this._enableCloseButton = false;
};

/**
 * A method to disable and hide minimize button
 */
gtk.WindowBase.prototype.disableMinimizeButton = function(){
    jsgtk.Dom.hideElement(this._minimizeButton);
    this._enableMinimizeButton = false;
};

/**
 * A method to disable and hide maximize button
 */
gtk.WindowBase.prototype.disableMaximizeButton = function(){
    jsgtk.Dom.hideElement(this._maximizeButton);
    jsgtk.Dom.hideElement(this._returnToPreviousSizeButton);
    
    if(this._enableCloseButton)this._minimizeButton.style.right = "24px"
    else this._minimizeButton.style.right = "5px"
    
    this._enableMaximizeButton = false;
};

/**
 * A method to show maximize header
 */
gtk.WindowBase.prototype.showMaximizedHeader = function(){
	jsgtk.Dom.showElement(this._returnToPreviousSizeButton);
	jsgtk.Dom.hideElement(this._maximizeButton); 
};

/**
 * A method to show restore header 
 */
gtk.WindowBase.prototype.showRestoredHeader = function(){
	jsgtk.Dom.hideElement(this._returnToPreviousSizeButton);
	jsgtk.Dom.showElement(this._maximizeButton);
};

/**
 * A method to show minimize header
 */
gtk.WindowBase.prototype.showMinimizedHeader = function(){
    jsgtk.Dom.showElement(this._returnToPreviousSizeButton);
    jsgtk.Dom.hideElement(this._maximizeButton);  
};

/**
 * A method to set state of the window
 * @param {String} state The constant value to be passed.
 * <br>These are the constant values to be passed:
 * <br> - gtk.WINDOW_RESTORED The flag constant value to set as restored state
 * <br> - gtk.WINDOW_MAXIMIZED The flag constant value to set as maximized state
 * <br> - gtk.WINDOW_MINIMIZED The flag constant value to set as minimixed state
 */
gtk.WindowBase.prototype.setState = function(state){
    this._state = state;
};

/**
 * A method to get or return the current state of window
 * @return {String} The constant state value of the window
 * <br>These are the constant values to be returned:
 * <br> - gtk.WINDOW_RESTORED The flag constant value to set as restored state
 * <br> - gtk.WINDOW_MAXIMIZED The flag constant value to set as maximized state
 * <br> - gtk.WINDOW_MINIMIZED The flag constant value to set as minimixed state
 */
gtk.WindowBase.prototype.getState = function(){
    return this._state;  
};

gtk.WindowBase.prototype.setTitle = function(title){
    this._titleDom.innerHTML = title;
};

/**
 * A method to set the title of the window to be bold
 * @param {Boolean} bold The boolean value true is bold, fasle it not bold
 */
gtk.WindowBase.prototype.setBoldTitle = function(bold){
	if(bold){
		this._titleDom.style.fontWeight = "bold";
	} else 	this._titleDom.style.fontWeight = "normal";
};

/**
 * A method to set position of the window
 * @param {String} position The constant value to be passed.
 * <br>This is the constant value to be passed:
 * <br> - gtk.JUSTIFY_CENTER The flag constant value to set as center screen
 */
gtk.WindowBase.prototype.setPosition = function(position){
    if(position == gtk.JUSTIFY_CENTER) this._positionCenter();
};

/** @private */
gtk.WindowBase.prototype._positionCenter = function(){
    var pageSize = jsgtk.Util.getPageSize();
    var left = (pageSize[0] - this._width )/2;
    var top = (pageSize[1] - this._height )/2;
    this.move(left, top);
};

/** @private */
gtk.WindowBase.prototype._clearChild = function(){
	this._bodyDom.removeChild(this._childWidget.getNode());
};

/**
 * A method to move or set position of window given left and top values in pixel
 * @param {Integer} left The left value in pixel
 * @param {Integer} top The top value in pixel
 */
gtk.WindowBase.prototype.move = function(left, top){
	var domObjStyle = this._domObj.style;
    domObjStyle.top = top + "px";
    domObjStyle.left = left + "px";
};

/** @private */
gtk.WindowBase.prototype._initMinimizedEvents = function(){
    var self = this;
    var jsgtkEvent = jsgtk.Event;

    var restoreButton = this._minimizedWindow.childNodes[3];
    
    jsgtkEvent.addDomListener(restoreButton, "mouseup", function(e){
		self.restore();
		jsgtkEvent.stopBubble(e);
    });
	
	var closeButton = this._minimizedWindow.childNodes[4];
	jsgtkEvent.addDomListener(closeButton, "mouseup", function(e){
	    self._minimizedWindow.parentNode.removeChild(self._minimizedWindow);
	    self.emit("closed", self);
	    self._checkAndRemoveComboDropdown();
	    self._checkAndRemoveToolTip();
	    self._checkAndRemoveSmartEntryDropdown();
	    jsgtkEvent.stopBubble(e);
	});
    
    jsgtkEvent.addDomListener(this._minimizedWindow, "mouseup", function(e){
        self.emit("minimizedmouseovered", self._title);
        self._checkAndHideComboDropdown();
        self._checkAndHideToolTip();
        self._checkAndHideSmartEntryDropdown();
        self.restore();
        jsgtkEvent.stopBubble(e);
    });
    
    jsgtkEvent.addDomListener(this._minimizedWindow, "dblclick", function(e){
        self.restore();
    });
};

/** @private */
gtk.WindowBase.prototype._createMinimizedHolder = function(){
    var id = "gtk_minimized_widget_holder";
    var holder = document.getElementById(id);
    if(!holder) var holder = jsgtk.Dom.create({"append": document.body, "tag": "div", "id": id});
    return holder;
};

/** @private */
gtk.WindowBase.prototype._isNodeExistInMinimizedHolder = function(holder){
    var clonedHolder = holder.cloneNode(true);
    for(var i = 0, len = clonedHolder.childNodes.length; i < len; i++){
        if(this._domObj.id == clonedHolder.childNodes[i].id) return true;
    }
    return false;
};

/** @private */
gtk.WindowBase.prototype._createMinimizedWindow = function(){
    this._resizeDomTitleWidth(this._minimizedWidth - this._inconsWidth);

    var minimizeWindow = this._headerDom.cloneNode(true);
	minimizeWindow.className = "gtk_window_base_minimize";
    minimizeWindow.removeAttribute("style");
    minimizeWindow.style.width = this._minimizedWidth + "px";
    minimizeWindow.id = this._domObj.id;
        
    return minimizeWindow;
};

/** @private */
gtk.WindowBase.prototype._resizeDomTitleWidth = function(width){
    this._titleDom.style.width = width + "px";
};

/** @private */
gtk.WindowBase.prototype._hideCloseButton = function(){
    if(this._closeButton){
		this._closeButton.style.display = "none";
		this._minimizeButton.style.right = "21px";	
		this._maximizeButton.style.right = "5px";	
	}
};

/** @private */
gtk.WindowBase.prototype._showCloseButton = function(){
    var self = this,
        jsgtkEvent = jsgtk.Event, 
        jsgtkDom = jsgtk.Dom;
	
    jsgtkDom.showElement(this._closeButton);
	this._minimizeButton.style.right = "41px";
	
	jsgtkEvent.addDomListener(this._closeButton, "mouseup", function(e){
	    self.emit("closed", self);
	    self._closeModal();
	    self._domObj.parentNode.removeChild(self._domObj);
	    self._domObj = null;
	    self._checkAndRemoveComboDropdown();
	    self._checkAndRemoveToolTip();
	    self._checkAndRemoveSmartEntryDropdown();
	    jsgtkEvent.stopBubble(e);
	});
};

/**
 * A method to hide minimize button
 */
gtk.WindowBase.prototype.hideMinimizeButton = function(){
    this._minimizeButton.style.display = "none";
};

/** @private */
gtk.WindowBase.prototype._checkAndRemoveComboDropdown = function(){
    var comboDropDown = document.getElementsByClassName("gtk_combobox_holder");
    if(comboDropDown !== null){
		var comboDropDownLength = comboDropDown.length;
		if(comboDropDownLength > 0){
			for(var i = 0; i < comboDropDownLength; i++){
				var childTobeRemoved = comboDropDown[0];
				childTobeRemoved.parentNode.removeChild(childTobeRemoved);
			}
		}
	}
};

/** @private */
gtk.WindowBase.prototype._checkAndRemoveSmartEntryDropdown = function(){
    var smartEntryDropDown = document.getElementsByClassName("gtk_smart_entry_drop_down_widget_holder");
    if(smartEntryDropDown !== null){
		var smartEntryDropDownLength = smartEntryDropDown.length;
		if(smartEntryDropDownLength > 0){
			for(var i = 0; i < smartEntryDropDownLength; i++){
				var childTobeRemoved = smartEntryDropDown[0];
				childTobeRemoved.parentNode.removeChild(childTobeRemoved);
			}
		}
	}
};

/** @private */
gtk.WindowBase.prototype._checkAndHideSmartEntryDropdown = function(){
    var smartEntryDropDown = document.getElementsByClassName("gtk_smart_entry_drop_down_widget_holder");
    if(smartEntryDropDown !== null){
		var smartEntryDropDownLength = smartEntryDropDown.length;
		if(smartEntryDropDownLength > 0){
			for(var i = 0; i < smartEntryDropDownLength; i++){
				var childTobeHidden = smartEntryDropDown[i];
				childTobeHidden.style.display = "none";
			}
		}
	}
};

/** @private */
gtk.WindowBase.prototype._checkAndHideComboDropdown = function(){
    var comboDropDown = document.getElementsByClassName("gtk_combobox_holder");
    if(comboDropDown !== null){
    	var comboDropDownLength = comboDropDown.length;
		if(comboDropDownLength > 0){
			for(var i = 0; i < comboDropDownLength; i++){
				var childTobeHidden = comboDropDown[i];
				childTobeHidden.style.display = "none";
			}
		}
    }
};

/** @private */
gtk.WindowBase.prototype._checkAndRemoveToolTip = function(){
    var toolTip = document.getElementsByClassName("gtk_widget gtk_bubble");
    if(toolTip !== null){
		var toolTipLength = toolTip.length;
		if(toolTipLength > 0){
			for(var i = 0; i < toolTipLength; i++){
				var childTobeRemoved = toolTip[0];
				childTobeRemoved.parentNode.removeChild(childTobeRemoved);
			}
		}
	}
};

/** @private */
gtk.WindowBase.prototype._checkAndHideToolTip = function(){
    var toolTip = document.getElementsByClassName("gtk_widget gtk_bubble");
    if(toolTip !== null){
		var toolTipLength = toolTip.length;
		if(toolTipLength > 0){
			for(var i = 0; i < toolTipLength; i++){
				var childTobeHidden = toolTip[i];
				childTobeHidden.style.display = "none";
			}
		}
	}
};

/** @private */
gtk.WindowBase.prototype._isWindowRestored = function(id){
    if(!document.getElementById(id)){
        this._domObj.id = id;
        return true;
    }
    return false;
};

/**
 * A method to set or show close button give a boolean value
 * @param {Boolean} isCloseButtonActive The boolean value true is to show, or false to hide
 */
gtk.WindowBase.prototype.setCloseButton = function(isCloseButtonActive){
	this._isCloseButtonActive = isCloseButtonActive;
	if(this._isCloseButtonActive) this._showCloseButton();
	else this._hideCloseButton();
};

/**
 * A method to close or remove window
 */
gtk.WindowBase.prototype.close = function(){
    this.emit("closed", self);
    this._closeModal();
    this._domObj.parentNode.removeChild(this._domObj);
    this._checkAndRemoveComboDropdown();
    this._checkAndRemoveToolTip();
    this._checkAndRemoveSmartEntryDropdown();
    
};

/** @private */
gtk.WindowBase.prototype._showModal = function(){
    if(!this._isModal)this._createModal();
    this._domObj.style.zIndex = "8001";
};

/** @private */
gtk.WindowBase.prototype._closeModal = function(){
	if(typeof this._modal !== 'undefined')
    	this._modal.parentNode.removeChild(this._modal);
	this._domObj.style.zIndex = "8000";
};

/** @private */
gtk.WindowBase.prototype._hideModal = function(){
    this._modal.style.display = "none";
    this._domObj.style.zIndex = "8000";
};

/** @private */
gtk.WindowBase.prototype._createModal = function(){
    this._modal = jsgtk.Dom.create({"append":document.body, "tag": "div", "id": "gtk_window_modal"});
    var widthHeight = jsgtk.Util.getPageSize();
    this._modal.style.width = widthHeight[0] + "px";
    this._modal.style.height = widthHeight[1] + "px";
    this._isModal = true;
};
gobject.signalNew('maximized', gtk.WindowBase, [], null, null);
gobject.signalNew('minimized', gtk.WindowBase, [], null, null);
gobject.signalNew('minimizedmouseovered', gtk.WindowBase, [], null, null);
gobject.signalNew('restored', gtk.WindowBase, [], null, null);
gobject.signalNew('closed', gtk.WindowBase, [], null, null);
